/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime;

import java.awt.Color;
import java.awt.Graphics;

import javax.swing.JComponent;

/**
 * カラーボックスオブジェクト
 * 
 * @create 2008/01/27
 */
public class ColorBox extends JComponent
{
    /**
     * シリアルバージョンID
     */
    private static final long serialVersionUID = 1L;

    /**
     * 
     */
    private Color current;

    /**
     * 
     */
    private final Color borderTop = new Color(0x80, 0x80, 0x80);

    private final Color borderBottom = new Color(0x80, 0x80, 0x80);

    private final Color borderBackground = new Color(0x00, 0x00, 0x00, 0x00);

    /**
     * コンストラクタ
     */
    public ColorBox()
    {
        this(Color.black);
    }

    /**
     * コンストラクタ
     * 
     * @param color 表示色
     */
    public ColorBox(Color color)
    {
        super();
        this.current = color;
    }

    /**
     * 表示色を変更する
     * 
     * @param color 表示色
     */
    public void changeColor(Color color)
    {
        this.current = color;
        repaint();
    }

    /**
     * 表示色を取得する
     * 
     * @return 表示色
     */
    public Color getColor()
    {
        return (this.current);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.JComponent#paint(java.awt.Graphics)
     * @since 2008/01/15
     */
    public void paint(Graphics g)
    {
        g.setColor(this.borderBackground);
        g.fillRect(0, 0, getBounds().width, getBounds().height);

        // top border
        g.setColor(this.borderTop);
        g.drawLine(0, 0, getBounds().width - 1, 0);
        g.drawLine(0, 0, 0, getBounds().height - 1);
        //
        g.setColor(this.current);
        g.fillRect(1, 1, getBounds().width - 2, getBounds().height - 2);

        // bottom border
        g.setColor(this.borderBottom);
        g.drawLine(0, getBounds().height - 1, getBounds().width - 1, getBounds().height - 1);
        g.drawLine(getBounds().width - 1, 0, getBounds().width - 1, getBounds().height - 1);

        g.dispose();
    }
}
