/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime;

import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.io.PrintWriter;
import java.io.StringWriter;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import jp.nihu.gttime.plugin.PluginResourceManager;

/**
 * エラーの内容を表示するダイアログを初期化する実行クラスです。
 */
public class ErrorDialog extends T2MapDialog
{
    /**
     * シリアルバーションID
     */
    private static final long serialVersionUID = 1L;

    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(ErrorDialog.class, T2Map.LOCALE);

    /**
     * テキストエリアオブジェクト
     */
    private JTextArea jta = null;

    /**
     * JLabelオブジェクト
     */
    private JLabel jl = null;

    /**
     * エラーの内容を表示するダイアログを初期化します。
     * 
     * @param owner 親のフレーム
     * @param modal モーダルを設定します
     */
    public ErrorDialog(Frame owner, boolean modal)
    {
        super(owner, modal);
        setTitle(rmgr.getString("title"));
        setSize(new java.awt.Dimension(450, 385));

        jl = new JLabel();
        getContentPane().add("North", jl);

        JScrollPane jsp = new JScrollPane();
        jta = new JTextArea();
        jsp.getViewport().add(jta);
        getContentPane().add("Center", jsp);

        JPanel bottom = new JPanel();
        bottom.setLayout(new FlowLayout(FlowLayout.RIGHT));
        getContentPane().add("South", bottom);

        JButton button = new JButton();
        Action okAction = new AbstractAction()
        {
            /**
             * シリアルバージョンID
             */
            private static final long serialVersionUID = 1L;

            public void actionPerformed(ActionEvent e)
            {
                ErrorDialog.this.setVisible(false);
            }
        };
        button.setText(rmgr.getString("button.ok"));
        button.addActionListener(okAction);
        bottom.add(button);
    }

    /**
     * エラーメッセージを表示します。
     * 
     * @param e Exception
     */
    public void show(Exception e)
    {
        jl.setText(e.getMessage());
        StringWriter sout = new StringWriter();
        e.printStackTrace(new PrintWriter(sout));
        jta.setText(sout.toString());
        setVisible(true);
    }
}
