package jp.nihu.gttime;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Hashtable;

import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.UIManager;

import jp.nihu.gttime.T2MapMenuBar.OpenActionListener;
import jp.nihu.gttime.data.ProjectException;
import jp.nihu.gttime.data.ProjectIF;
import jp.nihu.gttime.plugin.ImportPlugin;
import jp.nihu.gttime.plugin.OpenPlugin;
import jp.nihu.gttime.plugin.OpenPluginException;
import jp.nihu.gttime.plugin.PluginManager;
import jp.nihu.gttime.plugin.PluginManagerException;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.utility.ExtensionFileFilter;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class FileImportMenu {
	
	private static Log log = LogFactory.getLog(T2MapMenuBar.class);
	
	private static PluginResourceManager rmgr = PluginResourceManager.getManager(T2MapMenuBar.class, T2Map.LOCALE);
	
    private PluginManager pluginManager = null;
    
    private T2Map t2map = null;
    
    private ProjectIF project = null;
    
    private boolean forFileMenu = true;
	
	public FileImportMenu(T2Map t2map, ProjectIF project, boolean forFileMenu) {
		this.t2map = t2map;
		this.pluginManager = T2Map.getPluginManager();
		this.project = project;
		this.forFileMenu = forFileMenu;
	}
	
	public JMenu getOpenImportMenu()
    {
		JMenu menu = getImportMenu(rmgr.getString("menu.file.open.import"), "ImportOpenPlugin");
		JMenuItem item = new JMenuItem(rmgr.getString("menu.file.import.project"));
		menu.add(item);
		ArrayList<Hashtable<String, String>> plugins = pluginManager.getPlugins("OpenPlugin");
        for (int i = 0; i < plugins.size(); i++) {
        	log.info(plugins.get(i).get("class"));
        	if ("jp.nihu.gttime.plugin.io.FileOpenPlugin".equals(plugins.get(i).get("class"))) {
        		item.addActionListener(new OpenActionListener(plugins.get(i).get("class")));
        	}
        }
    	return menu;
    }
	
	/**
     * メニューバーの「ファイル」→「開く」についての動作を実装します。
     */
    class OpenActionListener implements ActionListener
    {
        /**
         * プラグインの文字列
         */
        private String clazz = null;

        /**
         * プラグインの文字列を格納
         * 
         * @param clazz プラグインの文字列
         */
        public OpenActionListener(String clazz)
        {
            this.clazz = clazz;
        }

        /**
         * プロジェクトをセットします。
         * 
         * @param event アクションイベントオブジェクト
         */
        public void actionPerformed(ActionEvent event)
        {
            log.debug(clazz);
            try
            {
                OpenPlugin plugin = (OpenPlugin) pluginManager.newInstance(clazz);
                plugin.init(t2map, pluginManager);
                ProjectIF insertProject = plugin.open();
                if (insertProject != null)
                {
                	//log.debug("DDDDDDDDDDDDDDDDDDDDDDdd " + insertProject.getPath());
                	if (project != null) {
                		project.addProject(insertProject);
                	} else {
                		project = T2Map.getProject();
                		if (project != null) {
                			project.addProject(insertProject);
                		} else {
                			t2map.setProject(insertProject);
                		}
                	}
                	t2map.updateView();
                }
                plugin.processError();
            }
            catch (PluginManagerException e)
            {
                t2map.procError(e);
            }
            catch (OpenPluginException e)
            {
                t2map.procError(e);
            } catch (ProjectException e) {
            	t2map.procError(e);
			}
        }
    }

    /**
     * メニューバーの「ファイル」→「インポート」についてのオブジェクトを取得します。
     * 
     * @return メニューバーの「ファイル」→「インポート」についてのオブジェクト
     */
	public JMenu getImportMenu()
    {
    	return getImportMenu(rmgr.getString("menu.file.import"), "ImportPlugin");
    }
    
    private JMenu getImportMenu(String title, String type) {
    	JMenu menu = null;
        JMenuItem item = null;
        menu = new JMenu(title);

        ArrayList<Hashtable<String, String>> plugins = pluginManager.getPlugins(type);
        
        final ArrayList<ImportPlugin> ps = new ArrayList<ImportPlugin>();
        for (int i = 0; i < plugins.size(); i++) {
        	try {
        		ImportPlugin plugin = (ImportPlugin) pluginManager.newInstance(plugins.get(i).get("class"));
        		plugin.init(t2map, pluginManager);
        		ps.add(plugin);
			} catch (PluginManagerException e1) {
				t2map.procError(e1);
			}
        }

        item = new JMenuItem(rmgr.getString("menu.file.import.layer"));//
        item.addActionListener(new java.awt.event.ActionListener()
        {
        	JFileChooser chooser;
        	
            public void actionPerformed(java.awt.event.ActionEvent e)
            {
            	//ファイルチューザ表示
                UIManager.put("FileChooser.readOnly", Boolean.TRUE);
                chooser = new T2MapFileChooser();

                for (int i = 0; i < ps.size(); i++)
                {
                	ImportPlugin plugin = ps.get(i);
					chooser.setFileFilter(plugin.getExtensionFileFilter());
					JPanel panel = plugin.getJPanel();
					if (panel != null) {
						chooser.setAccessory(panel);
					} else {
						chooser.setAccessory(new JPanel());
					}
                }
                chooser.addPropertyChangeListener(new PropertyChangeListener() {
					
					public void propertyChange(PropertyChangeEvent evt) {
						String prop = evt.getPropertyName();
						if (JFileChooser.FILE_FILTER_CHANGED_PROPERTY.equals(prop)) {
							Object o = evt.getNewValue();
							if (o != null && o instanceof ExtensionFileFilter) {
								ExtensionFileFilter f = (ExtensionFileFilter)evt.getNewValue();
								for (int i = 0; i < ps.size(); i++)
				                {
				                	ImportPlugin plugin = ps.get(i);
									if (plugin.getExtensionFileFilter().getDescription().equals(f.getDescription())) {
										JPanel panel = plugin.getJPanel();
										if (panel != null) {
											chooser.setAccessory(panel);
										} else {
											chooser.setAccessory(new JPanel());
										}
										chooser.getAccessory().updateUI();
									}
				                }
//							} else  {
//								chooser.setAccessory(new JPanel());
//								chooser.updateUI();
							}
						}
					}
				});
                
                int returnVal = chooser.showOpenDialog(t2map);
                if (returnVal == JFileChooser.APPROVE_OPTION)
                {
                	String ext = chooser.getFileFilter().getDescription();
                	for (int i = 0; i < ps.size(); i++)
                    {
                		if (ps.get(i).getExtensionFileFilter().getDescription().equals(ext)) {
                			try
                            {
                                ImportPlugin plugin = ps.get(i);
                                plugin.init(t2map, pluginManager);
                                if (forFileMenu) {
                                	project = T2Map.getProject();
                                }
                                plugin.open(chooser.getSelectedFile(), project);
                                plugin.processError();
                            }
                            catch (OpenPluginException e5)
                            {
                                t2map.procError(e5);
                            }
                			t2map.updateView();
                			break;
                		}
                    }
                }
                UIManager.put("FileChooser.readOnly", Boolean.FALSE);
            }
        });
        menu.add(item);

        return menu;
    }
}
