/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import jp.nihu.gttime.plugin.PluginResourceManager;

/**
 * レコードの色や透明度を変更するダイアログを表示する実行クラスです。
 */
public class NColorChooser extends JDialog
{
    /**
     * シリアルバーションID
     */
    private static final long serialVersionUID = 1L;

    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(NColorChooser.class, T2Map.LOCALE);

    /**
     * JPanelオブジェクト
     */
    private JPanel jContentPane = null;

    /**
     * 赤色のスライダーオブジェクト
     */
    private JSlider jSlider = null;

    /**
     * 緑色のスライダーオブジェクト
     */
    private JSlider jSlider1 = null;

    /**
     * 青色のスライダーオブジェクト
     */
    private JSlider jSlider2 = null;

    /**
     * 透明度のスライダーオブジェクト
     */
    private JSlider jSlider3 = null;

    /**
     * 赤色のラベルオブジェクト
     */
    private JLabel jLabel = null;

    /**
     * 緑色のラベルオブジェクト
     */
    private JLabel jLabel1 = null;

    /**
     * 青色のラベルオブジェクト
     */
    private JLabel jLabel2 = null;

    /**
     * 透明度のラベルオブジェクト
     */
    private JLabel jLabel3 = null;

    /**
     * 選択色の表示用パネルオブジェクト
     */
    private JComponent jLabel4 = null;

    /**
     * ボタン用のパネルオブジェクト
     */
    private JPanel jPanel = null;

    /**
     * 赤色のスライダーオブジェクト
     */
    private JSpinner jSpinner1 = null;

    /**
     * 緑色のスライダーオブジェクト
     */
    private JSpinner jSpinner2 = null;

    /**
     * 青色のスライダーオブジェクト
     */
    private JSpinner jSpinner3 = null;

    /**
     * 透明度のスライダーオブジェクト
     */
    private JSpinner jSpinner4 = null;

    /**
     * OKボタンオブジェクト
     */
    private JButton jButton = null;

    /**
     * CANCELボタンオブジェクト
     */
    private JButton jButton1 = null;

    // private JButton jButton2 = null;

    /**
     * カラーオブジェクト
     */
    private Color color = null;

    /**
     * JPanelオブジェクト
     */
    // private ColorBox panel = null;
    /**
     * 選択したカラーを設定します。
     * 
     * @param color 選択したカラー
     */
    public void setColor(Color color)
    {
        this.color = color;
    }

    /**
     * 設定されているカラーを取得します。
     * 
     * @return 取得するカラー
     */
    public Color getColor()
    {
        return color;
    }

    /**
     * 色選択ダイアログを表示します。
     * 
     * @param panel 表示するダイアログ
     */
    public NColorChooser(Color ocolor)
    {
        setLocale(T2Map.LOCALE);
        // this.panel = panel;
        setColor(ocolor);
        setContentPane(getJContentPane());
        this.setResizable(false);
        this.pack();
        setModal(true);
        this.setTitle(rmgr.getString("title"));

        // ダイアログの表示位置を設定。
        Toolkit tk = getToolkit();
        Dimension screenSize = tk.getScreenSize();
        int posX = (screenSize.width / 2) - (getWidth() / 2);
        int posY = (screenSize.height / 2) - (getHeight() / 2);
        setLocation(posX, posY);
    }

    /**
     * jContentPaneを初期化します。
     * 
     * @return JPanel 初期化されたJPanel
     */
    private JPanel getJContentPane()
    {
        if (jContentPane == null)
        {
            jLabel4 = new JPanel();
            GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
            gridBagConstraints7.gridx = 0;
            gridBagConstraints7.gridwidth = 3;
            gridBagConstraints7.gridy = 4;
            gridBagConstraints7.anchor = GridBagConstraints.EAST;
            GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
            gridBagConstraints12.gridx = 0;
            gridBagConstraints12.anchor = GridBagConstraints.EAST;
            gridBagConstraints12.gridwidth = 1;
            gridBagConstraints12.gridy = 3;
            jLabel3 = new JLabel();
            jLabel3.setText(rmgr.getString("label.alfa"));
            GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
            gridBagConstraints11.gridx = 0;
            gridBagConstraints11.anchor = GridBagConstraints.EAST;
            gridBagConstraints11.gridy = 2;
            jLabel2 = new JLabel();
            jLabel2.setText(rmgr.getString("label.blue"));
            GridBagConstraints gridBagConstraints10 = new GridBagConstraints();
            gridBagConstraints10.gridx = 0;
            gridBagConstraints10.anchor = GridBagConstraints.EAST;
            gridBagConstraints10.gridy = 1;
            jLabel1 = new JLabel();
            jLabel1.setText(rmgr.getString("label.green"));
            GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
            gridBagConstraints9.gridx = 0;
            gridBagConstraints9.anchor = GridBagConstraints.EAST;
            gridBagConstraints9.gridy = 0;
            jLabel = new JLabel();
            jLabel.setText(rmgr.getString("label.red"));
            GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
            gridBagConstraints5.fill = GridBagConstraints.BOTH;
            gridBagConstraints5.gridy = 3;
            gridBagConstraints5.weightx = 1.0;
            gridBagConstraints5.ipady = 5;
            gridBagConstraints5.gridx = 1;
            GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
            gridBagConstraints4.fill = GridBagConstraints.BOTH;
            gridBagConstraints4.gridy = 2;
            gridBagConstraints4.weightx = 1.0;
            gridBagConstraints4.ipady = 5;
            gridBagConstraints4.gridx = 1;
            GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
            gridBagConstraints3.fill = GridBagConstraints.BOTH;
            gridBagConstraints3.gridy = 1;
            gridBagConstraints3.weightx = 1.0;
            gridBagConstraints3.ipady = 5;
            gridBagConstraints3.gridx = 1;
            GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
            gridBagConstraints1.fill = GridBagConstraints.BOTH;
            gridBagConstraints1.gridy = 0;
            gridBagConstraints1.weightx = 1.0;
            gridBagConstraints1.ipady = 5;
            gridBagConstraints1.gridx = 1;

            GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
            gridBagConstraints8.fill = GridBagConstraints.BOTH;
            gridBagConstraints8.weightx = 1.0;
            gridBagConstraints8.anchor = GridBagConstraints.CENTER;
            gridBagConstraints8.gridx = 3;
            gridBagConstraints8.gridheight = 4;
            gridBagConstraints8.gridy = 0;

            GridBagConstraints gridBagConstraints13 = new GridBagConstraints();
            gridBagConstraints13.gridy = 3;
            gridBagConstraints13.weightx = 1.0;
            gridBagConstraints13.ipady = 5;
            gridBagConstraints13.gridx = 2;
            GridBagConstraints gridBagConstraints14 = new GridBagConstraints();
            gridBagConstraints14.gridy = 2;
            gridBagConstraints14.weightx = 1.0;
            gridBagConstraints14.ipady = 5;
            gridBagConstraints14.gridx = 2;
            GridBagConstraints gridBagConstraints15 = new GridBagConstraints();
            gridBagConstraints15.gridy = 1;
            gridBagConstraints15.weightx = 1.0;
            gridBagConstraints15.ipady = 5;
            gridBagConstraints15.gridx = 2;
            GridBagConstraints gridBagConstraints16 = new GridBagConstraints();
            gridBagConstraints16.gridy = 0;
            gridBagConstraints16.weightx = 1.0;
            gridBagConstraints16.ipady = 5;
            gridBagConstraints16.gridx = 2;

            jContentPane = new JPanel();
            jContentPane.setLayout(new GridBagLayout());
            jContentPane.add(getJSlider(), gridBagConstraints1);
            jContentPane.add(getJSlider1(), gridBagConstraints3);
            jContentPane.add(getJSlider2(), gridBagConstraints4);
            jContentPane.add(getJSlider3(), gridBagConstraints5);

            jContentPane.add(getJSpinner1(), gridBagConstraints16);
            jContentPane.add(getJSpinner2(), gridBagConstraints15);
            jContentPane.add(getJSpinner3(), gridBagConstraints14);
            jContentPane.add(getJSpinner4(), gridBagConstraints13);

            jContentPane.add(jLabel, gridBagConstraints9);
            jContentPane.add(jLabel1, gridBagConstraints10);
            jContentPane.add(jLabel2, gridBagConstraints11);
            jContentPane.add(jLabel3, gridBagConstraints12);
            jContentPane.add(getJPanel(), gridBagConstraints7);

            jContentPane.add(jLabel4, gridBagConstraints8);
            jLabel4.setBackground(color);
        }
        return jContentPane;
    }

    /**
     * 赤色のJSpinnerを取得します。
     * 
     * @return JSpinner スピナー
     */
    private JSpinner getJSpinner1()
    {
        if (jSpinner1 == null)
        {
            jSpinner1 = new JSpinner();
            JSpinner.NumberEditor editor = new JSpinner.NumberEditor(jSpinner1, "0");
            SpinnerNumberModel model = new SpinnerNumberModel();
            editor.getTextField().setColumns(3);
            editor.getTextField().setEditable(true);
            editor.getTextField().setHorizontalAlignment(JTextField.LEFT);
            editor.getTextField().setValue(getColor().getRed());
            model.setValue(getColor().getRed());
            model.setMinimum(0);
            model.setMaximum(255);
            model.setStepSize(1);
            jSpinner1.setEditor(editor);
            jSpinner1.setModel(model);
            jSpinner1.addChangeListener(new SValueChangeListener());
        }
        return jSpinner1;
    }

    /**
     * 緑色のJSpinnerを取得します。
     * 
     * @return JSpinner スピナー
     */
    private JSpinner getJSpinner2()
    {
        if (jSpinner2 == null)
        {
            jSpinner2 = new JSpinner();
            JSpinner.NumberEditor editor = new JSpinner.NumberEditor(jSpinner2, "0");
            SpinnerNumberModel model = new SpinnerNumberModel();
            editor.getTextField().setColumns(3);
            editor.getTextField().setEditable(true);
            editor.getTextField().setHorizontalAlignment(JTextField.LEFT);
            editor.getTextField().setValue(getColor().getGreen());
            model.setValue(getColor().getGreen());
            model.setMinimum(0);
            model.setMaximum(255);
            model.setStepSize(1);
            jSpinner2.setEditor(editor);
            jSpinner2.setModel(model);
            jSpinner2.addChangeListener(new SValueChangeListener());
        }
        return jSpinner2;
    }

    /**
     * 青色のJSpinnerを取得します。
     * 
     * @return JSpinner スピナー
     */
    private JSpinner getJSpinner3()
    {
        if (jSpinner3 == null)
        {
            jSpinner3 = new JSpinner();
            JSpinner.NumberEditor editor = new JSpinner.NumberEditor(jSpinner3, "0");
            SpinnerNumberModel model = new SpinnerNumberModel();
            editor.getTextField().setColumns(3);
            editor.getTextField().setEditable(true);
            editor.getTextField().setHorizontalAlignment(JTextField.LEFT);
            editor.getTextField().setValue(getColor().getBlue());
            model.setValue(getColor().getBlue());
            model.setMinimum(0);
            model.setMaximum(255);
            model.setStepSize(1);
            jSpinner3.setEditor(editor);
            jSpinner3.setModel(model);
            jSpinner3.addChangeListener(new SValueChangeListener());
        }
        return jSpinner3;
    }

    /**
     * 透明度のJSpinnerを取得します。
     * 
     * @return JSpinner スピナー
     */
    private JSpinner getJSpinner4()
    {
        if (jSpinner4 == null)
        {
            jSpinner4 = new JSpinner();
            JSpinner.NumberEditor editor = new JSpinner.NumberEditor(jSpinner4, "0");
            SpinnerNumberModel model = new SpinnerNumberModel();
            editor.getTextField().setColumns(3);
            editor.getTextField().setEditable(true);
            editor.getTextField().setHorizontalAlignment(JTextField.LEFT);
            editor.getTextField().setValue(getColor().getAlpha());
            model.setValue(getColor().getAlpha());
            model.setMinimum(0);
            model.setMaximum(255);
            model.setStepSize(1);
            jSpinner4.setEditor(editor);
            jSpinner4.setModel(model);
            jSpinner4.addChangeListener(new SValueChangeListener());
        }
        return jSpinner4;
    }

    /**
     * 赤色のスライダーを取得します。
     * 
     * @return JSlider スライダー
     */
    private JSlider getJSlider()
    {
        if (jSlider == null)
        {
            jSlider = new JSlider(JSlider.HORIZONTAL, 0, 255, getColor().getRed());
            jSlider.setMajorTickSpacing(85);
            jSlider.setMinorTickSpacing(15);
            jSlider.setPaintTicks(true);
            jSlider.setPaintLabels(true);
            jSlider.addMouseMotionListener(new CMouseLitener());
            jSlider.addChangeListener(new CMouseLitener());
        }
        return jSlider;
    }

    /**
     * 緑色のスライダーを取得します。
     * 
     * @return JSlider スライダー
     */
    private JSlider getJSlider1()
    {
        if (jSlider1 == null)
        {
            jSlider1 = new JSlider(JSlider.HORIZONTAL, 0, 255, getColor().getGreen());
            jSlider1.setMajorTickSpacing(85);
            jSlider1.setMinorTickSpacing(15);
            jSlider1.setPaintTicks(true);
            jSlider1.setPaintLabels(true);
            jSlider1.addMouseMotionListener(new CMouseLitener());
            jSlider1.addChangeListener(new CMouseLitener());
        }
        return jSlider1;
    }

    /**
     * 青色のスライダーを取得します。
     * 
     * @return JSlider スライダー
     */
    private JSlider getJSlider2()
    {
        if (jSlider2 == null)
        {
            jSlider2 = new JSlider(JSlider.HORIZONTAL, 0, 255, getColor().getBlue());
            jSlider2.setMajorTickSpacing(85);
            jSlider2.setMinorTickSpacing(15);
            jSlider2.setPaintTicks(true);
            jSlider2.setPaintLabels(true);
            jSlider2.addMouseMotionListener(new CMouseLitener());
            jSlider2.addChangeListener(new CMouseLitener());
        }
        return jSlider2;
    }

    /**
     * 透明度のスライダーを取得します。
     * 
     * @return JSlider スライダー
     */
    private JSlider getJSlider3()
    {
        if (jSlider3 == null)
        {
            jSlider3 = new JSlider(JSlider.HORIZONTAL, 0, 255, getColor().getAlpha());
            jSlider3.setMajorTickSpacing(85);
            jSlider3.setMinorTickSpacing(15);
            jSlider3.setPaintTicks(true);
            jSlider3.setPaintLabels(true);
            jSlider3.addMouseMotionListener(new CMouseLitener());
            jSlider3.addChangeListener(new CMouseLitener());
        }
        return jSlider3;
    }

    /**
     * ボタンのパネルを取得します。
     * 
     * @return JPanel パネル
     */
    private JPanel getJPanel()
    {
        if (jPanel == null)
        {
            GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
            gridBagConstraints2.gridx = 1;
            gridBagConstraints2.gridy = 0;
            GridBagConstraints gridBagConstraints = new GridBagConstraints();
            gridBagConstraints.gridx = 0;
            gridBagConstraints.insets = new Insets(0, 0, 0, 7);
            gridBagConstraints.gridy = 0;
            jPanel = new JPanel();
            jPanel.setLayout(new GridBagLayout());
            jPanel.add(getJButton(), gridBagConstraints);
            jPanel.add(getJButton1(), gridBagConstraints2);
        }
        return jPanel;
    }

    /**
     * OKボタンを取得します。
     * 
     * @return JButton OKボタン
     */
    private JButton getJButton()
    {
        if (jButton == null)
        {
            jButton = new JButton();
            jButton.setText(rmgr.getString("button.apply"));
            jButton.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(java.awt.event.ActionEvent e)
                {
                    // panel.changeColor(jLabel4.getBackground());
                    NColorChooser.this.dispose();
                }
            });
        }
        return jButton;
    }

    /**
     * CANCELボタンを取得します。
     * 
     * @return JButton CANCELボタン
     */
    private JButton getJButton1()
    {
        if (jButton1 == null)
        {
            jButton1 = new JButton();
            jButton1.setText(rmgr.getString("button.cancel"));
            jButton1.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(java.awt.event.ActionEvent e)
                {
                    NColorChooser.this.dispose();
                }
            });
        }
        return jButton1;
    }

    /**
     * 表示する色を設定します。
     */
    class SValueChangeListener implements ChangeListener
    {
        public void stateChanged(ChangeEvent e)
        {
            jSlider.setValue((Integer) jSpinner1.getValue());
            jSlider1.setValue((Integer) jSpinner2.getValue());
            jSlider2.setValue((Integer) jSpinner3.getValue());
            jSlider3.setValue((Integer) jSpinner4.getValue());
            color = new Color((Integer) jSpinner1.getValue(), (Integer) jSpinner2.getValue(), (Integer) jSpinner3.getValue(), (Integer) jSpinner4.getValue());
            setBGColor();
        }
    }

    /**
     * マウスイベントを設定します。
     */
    class CMouseLitener implements MouseMotionListener, ChangeListener
    {
        /**
         * コンポーネント上でマウスのボタンが押してドラッグしたときに呼び出されます。
         * 
         * @param e MouseEvent
         */
        public void mouseDragged(MouseEvent e)
        {
            exec();
        }

        /**
         * コンポーネント上でマウスを移動すると呼び出されます。
         * 
         * @param e MouseEvent
         */
        public void mouseMoved(MouseEvent e)
        {
        }

        /**
         * リスナーの対象の状態が変更されたときに呼び出されます。
         * 
         * @param e ChangeEvent
         */
        public void stateChanged(ChangeEvent e)
        {
            JSlider source = (JSlider) e.getSource();
            if (!source.getValueIsAdjusting())
            {
                exec();
            }
        }

        /**
         * 設定された色を取得します。
         */
        private void exec()
        {
            jSpinner1.setValue(jSlider.getValue());
            jSpinner2.setValue(jSlider1.getValue());
            jSpinner3.setValue(jSlider2.getValue());
            jSpinner4.setValue(jSlider3.getValue());
            color = new Color(jSlider.getValue(), jSlider1.getValue(), jSlider2.getValue(), jSlider3.getValue());
            setBGColor();
        }
    }

    /**
     * 設定された色を表示します。
     */
    private void setBGColor()
    {
        jLabel4.setBackground(color);
        repaint();
    }
}
