/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.Transferable;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.math.BigDecimal;
import java.security.AllPermission;
import java.security.CodeSource;
import java.security.PermissionCollection;
import java.security.Permissions;
import java.security.Policy;
import java.util.ArrayList;
import java.util.Locale;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.SwingUtilities;
import javax.swing.UIDefaults;
import javax.swing.UIManager;

import jp.nihu.gttime.data.DatasetException;
import jp.nihu.gttime.data.DatasetIF;
import jp.nihu.gttime.data.JulianDate;
import jp.nihu.gttime.data.ProjectGts;
import jp.nihu.gttime.data.ProjectIF;
import jp.nihu.gttime.data.Record;
import jp.nihu.gttime.layer.LayerViewPart;
import jp.nihu.gttime.plugin.OpenPluginException;
import jp.nihu.gttime.plugin.PluginManager;
import jp.nihu.gttime.plugin.PluginManagerException;
import jp.nihu.gttime.plugin.PluginPreferences;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.plugin.io.FileOpenPlugin;
import jp.nihu.gttime.plugin.io.ProjectSave;
import jp.nihu.gttime.preferences.GeneralPreferences;
import jp.nihu.gttime.time.TimeAxisIF;
import jp.nihu.gttime.time.TimeLineData;
import jp.nihu.gttime.time.TimeSquarePart;
import jp.nihu.gttime.time.utility.ISO8601Exception;
import jp.nihu.gttime.tree.TreeViewPart;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.h_gis.launcher.IGtLauncher;

/**
 * T2Mapのmainメッソドを含む実行クラスです。
 */
public class T2Map extends JFrame implements ClipboardOwner, IGtLauncher
{
    /**
     * ログ用オブジェクト
     */
    private static final Log log = LogFactory.getLog(T2Map.class);
    
    public static final String RDF_BASE_URI = "http://hutime.org/vocab/";

    public static String APPLICATION_NAME = "GT-Time(dev)";
    
//    public static String APPLICATION_VERSION = "20150831 on Java " + System.getProperty("java.version");
    public static String APPLICATION_VERSION = "20160328";
    
    public static String APPLICATION_COPY = "2015";

    private static String APPLICATION_ICON_FILE_NAME = "marksmall.gif";

    private static int projectType = -1;

    public static int getProjectType()
    {
        return projectType;
    }

    public static void setProjectType(int type)
    {
        projectType = type;
    }

    /**
     * リソースマネージャーオブジェクト
     */
    // private static PluginResourceManager rmgr =
    // PluginResourceManager.getManager(T2Map.class);
    private static PluginResourceManager rmgr = null;

    /**
     * シリアルバーションID
     */
    private static final long serialVersionUID = 1L;

    /**
     * TimeSquarePartオブジェクト
     */
    private TimeSquarePart timeAxis = null;

    /**
     * LayerViewPartオブジェクト
     */
    private LayerViewPart layerPanel = null;

    /**
     * TreeViewPartオブジェクト
     */
    private TreeViewPart tree = null;

    /**
     * T2MapPropertiesオブジェクト
     */
    public static T2MapProperties properties = null;

    /**
     * PreferencesDialogオブジェクト
     */
    public static ArrayList<PluginPreferences> preferences = new ArrayList<PluginPreferences>();

    /**
     * ErrorDialogオブジェクト
     */
    private ErrorDialog error = null;

    /**
     * PluginManagerオブジェクト
     */
    public static PluginManager pluginManager = null;

    /**
     * Projectオブジェクト
     */
    private static ProjectIF project = null;

    /**
     * TimeAxisIFオブジェクト
     */
    private TimeAxisIF data = null;
    
    /**
     * T2MapMenuBarオブジェクト
     */
    private T2MapMenuBar menuBar;

    /**
     * プラグインディレクトリ格納用オブジェクト
     */
    private static String[] PLUGIN_DIRS = null;

    /**
     * フレーム横幅
     */
    private static int FRAME_WIDTH = 0;

    /**
     * フレーム高さ
     */
    private static int FRAME_HEIGHT = 0;

    /**
     * ロケール
     */
    public static Locale LOCALE = null;

    /**
     * PluginManagerオブジェクトを取得します。
     * 
     * @return PluginManagerオブジェクト
     */
    public static PluginManager getPluginManager()
    {
        return pluginManager;
    }

    /**
     * TimeAxisIFオブジェクトを取得します。
     * 
     * @return TimeAxisIFオブジェクト
     */
    public TimeAxisIF getTimeAxisData()
    {
        return data;
    }

    /**
     * LayerViewPartオブジェクトを取得します。
     * 
     * @return LayerViewPartオブジェクト
     */
    public LayerViewPart getLayerPanel()
    {
        return layerPanel;
    }

    /**
     * プロジェクトをセットします。
     * 
     * @param project Projectオブジェクト
     */
    public void updateProject(ProjectIF project)
    {
        ProjectIF[] projects = project.getProjects();
        DatasetIF[] datasets = project.getDatasets();
        if ((projects != null && projects.length > 0) || (datasets != null && datasets.length > 0))
        {
            T2Map.project = project;
            T2Map.project.setRoot(true);
            tree.rebuildLayerList(project);
            setTitle(APPLICATION_NAME + " : " + project.getPath());
        }
        else
        {
            StringBuffer sb = new StringBuffer(rmgr.getString("project.error.message"));
            JOptionPane.showConfirmDialog(this, sb.toString(), rmgr.getString("error"), JOptionPane.PLAIN_MESSAGE);
        }

    }
    
    public void updateView()
    {
//    	ProjectIF[] projects = project.getProjects();
//        DatasetIF[] datasets = project.getDatasets();
//        if ((projects != null && projects.length > 0) || (datasets != null && datasets.length > 0))
//        {
//            tree.rebuildLayerList(project);
//            setTitle(APPLICATION_NAME + " : " + project.getPath());
//        }
//        else
//        {
//            StringBuffer sb = new StringBuffer(rmgr.getString("project.error.message"));
//            JOptionPane.showConfirmDialog(this, sb.toString(), rmgr.getString("error"), JOptionPane.PLAIN_MESSAGE);
//        }
        
    	//tree.rebuildLayerList(T2Map.project);
    	tree.load(T2Map.project);
    }

    /**
     * プロジェクトをセットします。
     * 
     * @param project Projectオブジェクト
     */
    public void setProject(ProjectIF project)
    {
//        ProjectIF[] projects = project.getProjects();
//        DatasetIF[] datasets = project.getDatasets();
//        if ((projects != null && projects.length > 0) || (datasets != null && datasets.length > 0))
//        {
            // 画面を初期化
            getJSplitPane();
            T2Map.project = null;
            T2Map.project = project;
            T2Map.project.setRoot(true);

            try
            {
                BigDecimal sdate = project.getStartDatetime();
                BigDecimal edate = project.getEndDatetime();
                data.setApplicationMin(sdate.subtract(BigDecimal.valueOf(200000000000.0)));
                data.setApplicationMax(edate.add(BigDecimal.valueOf(200000000000.0)));
                // data.setApplicationMin(sdate.subtract(BigDecimal.valueOf(200.0)));
                // data.setApplicationMax(edate.add(BigDecimal.valueOf(200.0)));
                data.setMin(sdate);
                data.setMax(edate);
                data.setInitialMin(sdate);
                data.setInitialMax(edate);
                data.setCurrentMin(sdate);
                data.setCurrentMax(edate);
                data.setInitialCurrentMin(sdate);
                data.setInitialCurrentMax(edate);
            }
            catch (DatasetException e)
            {
                log.fatal(e);
                procError(e);
            }
            catch (ISO8601Exception e)
            {
                log.fatal(e);
                procError(e);
            }

            setVisible(true);

            timeAxis.load(data);
            tree.load(project);
            setTitle(APPLICATION_NAME + " : " + project.getPath());
            
            
            int mc = menuBar.getMenuCount();
            for (int i = 0; i < mc; i++) {
            	JMenu cm = menuBar.getMenu(i);
            	cm.setEnabled(true);
            	int cmc = cm.getItemCount();
            	for (int j = 0; j < cmc; j++) {
            		JMenuItem item = cm.getItem(j);
            		if (item != null) {
            			item.setEnabled(true);
            		}
            	}
            }
//        }
//        else
//        {
//        	
//            StringBuffer sb = new StringBuffer(rmgr.getString("project.error.message"));
//            JOptionPane.showConfirmDialog(this, sb.toString(), rmgr.getString("error"), JOptionPane.PLAIN_MESSAGE);
//        }
    }

    /**
     * Projectオブジェクトを取得します。
     * 
     * @return Projectオブジェクト
     */
    public static ProjectIF getProject()
    {
        return project;
    }

    /**
     * T2Mapクラスのコンストラクタです。
     * 
     * @param e 例外処理
     */
    public void procError(Exception e)
    {
        if (error == null)
        {
            error = new ErrorDialog(this, true);
        }
        error.show(e);
    }

    /**
     * T2MapのLookAndFeelを設定します。
     */
    private void setLookAndFeel()
    {
        Font font = new Font("Dialog", Font.PLAIN, 12);
        UIDefaults defaultTable = UIManager.getLookAndFeelDefaults();
        for (Object o : defaultTable.keySet())
        {
            if (o.toString().toLowerCase(Locale.getDefault()).endsWith("font"))
            {
                UIManager.put(o, font);
            }
        }
        UIManager.put("OptionPane.messageFont", font);
        UIManager.put("OptionPane.buttonFont", font);
        UIManager.put("OptionPane.titleFont", font);

        try
        {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }
        catch (Exception exc)
        {
            System.err.println(rmgr.getString("lookAndFeel.error") + exc);
        }
    }

    /**
     * propertiesファイルからデータ取得し、セットします。
     */
    private void setProperties()
    {
        // APPLICATION_NAME =
        // properties.getProperty("jp.nihu.gttime.T2Map.APPLICATION_NAME");
        // APPLICATION_ICON =
        // properties.getProperty("jp.nihu.gttime.T2Map.APPLICATION_ICON");
        PLUGIN_DIRS = properties.getProperties("jp.nihu.gttime.T2Map.PLUGIN_DIR");
        FRAME_WIDTH = Integer.valueOf(properties.getProperty("jp.nihu.gttime.T2Map.FRAME_WIDTH"));
        FRAME_HEIGHT = Integer.valueOf(properties.getProperty("jp.nihu.gttime.T2Map.FRAME_HEIGHT"));
        String locStr = properties.getProperty("jp.nihu.gttime.T2Map.LOCALE");
        if (locStr != null)
        {
        	log.debug("setting locale.");
//        	System.out.println(locStr);
        	if (locStr.contains("_")) {
        		String[] ls = locStr.split("_");
        		LOCALE = new Locale(ls[0], ls[1]);
        	} else {
        		LOCALE = new Locale(locStr);
        	}
        }
        else
        {
        	log.debug("default locale.");
        	if (Locale.getDefault().getLanguage().equals("ja")) {
        		LOCALE = Locale.getDefault();
        	} else {
        		LOCALE = Locale.ENGLISH;
        	}
        }
        setLocale(LOCALE);
        properties.setLocale(LOCALE);
        log.debug("locale: " + LOCALE.getLanguage());
        //UIManager.getLookAndFeelDefaults().setDefaultLocale(T2Map.LOCALE);
        JOptionPane.setDefaultLocale(T2Map.LOCALE);
    }

    /**
     * 初期化します。<br/>
     * <ul>
     * <li>設定ファイルを読み込みます。</li>
     * <li>設定ダイアログの生成します。</li>
     * <li>プラグインを読込みます。</li>
     * <li>GUIの初期画面を生成します。</li>
     * </ul>
     * 
     * @exception jp.nihu.gttime.T2MapPropertiesException
     * @exception jp.nihu.gttime.plugin.PluginManagerException
     */
    private void initialize()
    {
        try
        {
            // 設定ファイルの読込み
            properties = new T2MapProperties();
            setProperties();
            rmgr = PluginResourceManager.getManager(T2Map.class, getLocale());
            
            APPLICATION_NAME = rmgr.getString("application.name");
            APPLICATION_ICON_FILE_NAME = rmgr.getString("application.icon");

            // 基本設定ダイアログの追加
            preferences.add(new GeneralPreferences());

            // プラグインの読込み
            pluginManager = new PluginManager(PLUGIN_DIRS);

            // フレームの設定
            Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
            if (d.width < FRAME_WIDTH)
            {
                FRAME_WIDTH = d.width;
            }
            if (d.height < FRAME_HEIGHT)
            {
                FRAME_HEIGHT = d.height;
            }
            setSize(FRAME_WIDTH, FRAME_HEIGHT);

            setTitle(APPLICATION_NAME);

            ImageIcon icon = new ImageIcon(getClass().getClassLoader().getResource(APPLICATION_ICON_FILE_NAME));
            if (APPLICATION_NAME.equals("HuTime")) {
            	icon = new ImageIcon(getClass().getClassLoader().getResource(APPLICATION_ICON_FILE_NAME.replace(".gif", ".png")));
            }
            setIconImage(icon.getImage());

            // メニューの生成
            menuBar = new T2MapMenuBar(this);
            setJMenuBar(menuBar);

            // StatusBar statusBar = new StatusBar();
            // statusBar.setMessage("あああああああああああああああ");
            // getContentPane().add(statusBar, BorderLayout.SOUTH);

            // メインパネルの生成
            getJSplitPane();
        }
        catch (T2MapPropertiesException e)
        {
            procError(e);
        }
        catch (PluginManagerException e)
        {
            procError(e);
        }
    }

    // public class StatusBar extends JLabel {
    // private static final long serialVersionUID = 1L;
    //
    // /** Creates a new instance of StatusBar */
    // public StatusBar() {
    // super();
    // super.setPreferredSize(new Dimension(100, 16));
    // setMessage("Ready");
    // }
    //        
    // public void setMessage(String message) {
    // setText(" "+message);
    // }
    // }

    /**
     * 画面を分割します
     * 
     * @return 画面分割後のJSplitPane
     */
    public void getJSplitPane()
    {
        JSplitPane jSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, false);
        setContentPane(jSplitPane);
        // getContentPane().add(jSplitPane, BorderLayout.CENTER);

        JPanel rightComponent = getLayerTimePanel();// 右
        jSplitPane.setRightComponent(rightComponent);

        JPanel leftComponent = new JPanel();// 左
        jSplitPane.setLeftComponent(leftComponent);

        leftComponent.setLayout(new BorderLayout());

        JPanel leftTop = getLeftTopPane();// 左上
        leftComponent.add(leftTop, BorderLayout.NORTH);

        tree = new TreeViewPart();// 左下
        leftComponent.add(tree, BorderLayout.CENTER);

        jSplitPane.setOneTouchExpandable(true);
        jSplitPane.setDividerLocation(leftTop.getPreferredSize().width);
    }

    /**
     * TreeViewPartオブジェクトを取得します
     * 
     * @return TreeViewPartオブジェクト
     */
    public TreeViewPart getTree()
    {
        return tree;
    }

    /**
     * レイヤと時間軸の基になるパネルを取得します。
     * 
     * @return 描画の基になるパネル
     */
    private JPanel getLayerTimePanel()
    {
        JPanel layerTimePanel = new JPanel();
        layerTimePanel.setLayout(new BorderLayout());

        data = new TimeLineData();

        layerPanel = new LayerViewPart();// レイヤパネル
        layerTimePanel.add(layerPanel, BorderLayout.CENTER);

        timeAxis = new TimeSquarePart(data, LOCALE);// 時間軸パネル
        layerTimePanel.add(timeAxis, BorderLayout.SOUTH);

        return layerTimePanel;
    }

    public BigDecimal[][] getDurations()
    {
        return layerPanel.getDurations();
    }

    public void setForcePaint()
    {
        layerPanel.setForcePaint();
    }

    /**
     * mainメソッド
     * 
     * @param args 現在、実行パラメータに対応する機能はありません。
     */
    public static void main(String[] args)
    {
        if (args.length > 0)
        {
            if (args[0] != null && args[0].length() > 0) {
                file = new File(args[0]);
            }
        }
        
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                final T2Map t2map = new T2Map();
                t2map.isStandAlone = true;
                t2map.readProjectFile();
                t2map.setVisible(true);
            }
        });
    }
    
    /**
     * T2Mapクラスのコンストラクタです。
     */
    public T2Map()
    {
        super();
        setLookAndFeel();
        initialize();

        Policy.setPolicy(new Policy()
        {
            public PermissionCollection getPermissions(CodeSource codesource)
            {
                Permissions perms = new Permissions();
                perms.add(new AllPermission());
                return (perms);
            }

            public void refresh()
            {
            }
        });
        
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setLocationRelativeTo(null); // 画面の真ん中に表示
        addWindowListener(new CloseListener());
    }
    
    /**
     * プロジェクトパスを取得
     *
     * @return the lProjectPath
     */
     public String getLProjectPath() {
         if (file != null) {
             return file.getAbsolutePath();
         } else {
             return null;
         }
     }
     
     private boolean isStandAlone = true;
     
     /**
     * プロジェクトパスを設定
     *
     * @param projectPath the lProjectPath to set
     */
     public void setLProjectPath(String projectPath) {
         log.debug("setLProjectPath: " + projectPath);
         isStandAlone = false;
         if (projectPath != null && projectPath.length() > 0) {
             log.debug("set to file");
             file = new File(projectPath);
         }
         readProjectFile();
     }
     
     private void saveOnExit() {
         if (!isStandAlone) {
             log.debug("Save Project");

             int selectVal = JOptionPane.showConfirmDialog(this, rmgr.getString("save.on.exit"), "", JOptionPane.YES_NO_OPTION , JOptionPane.QUESTION_MESSAGE, null);
             
             if (selectVal == JOptionPane.OK_OPTION) {
                 ProjectSave save = new ProjectSave();
                 save.saveProject(project, file.getParentFile(), file.getName());
             }
         } else {
             //DO NOTHING.
             log.debug("Without Save Project");
         }
     }
     
     public class CloseListener extends WindowAdapter {
         public void windowClosing(WindowEvent e) {
             log.debug("Window closing");
             saveOnExit();
             super.windowClosing(e);
         }
     }
     
     public void quit() {
         log.info("exit");
         if (isStandAlone) {
             saveOnExit();
             System.exit(0);
         } else {
             WindowEvent closingEvent = new WindowEvent(this, WindowEvent.WINDOW_CLOSING);
             Toolkit.getDefaultToolkit().getSystemEventQueue().postEvent(closingEvent);
         }
     }
     
     public void readProjectFile() {
         log.info("Start t2map");

         if (file != null && (!file.exists() || !(file.getPath().endsWith(".tms") || file.getPath().endsWith(".gts"))))
         {
             file = null;
             JOptionPane.showMessageDialog(this, rmgr.format("read.file.error", "tms, gts"));
         }

         if (file != null)
         {
             try
             {
                 FileOpenPlugin open = new FileOpenPlugin();
                 open.init(this, pluginManager);
                 log.debug("Open project: " + file.getAbsolutePath());
                 ProjectIF p = open.open(file);
                 setProject(p);
                 getLayerPanel().repaint();
                 log.debug("Display project");
             }
             catch (OpenPluginException e)
             {
                 log.error(e.getMessage(), e);
             }
         }
         else
         {
        	 ProjectIF p = new ProjectGts(null);
             setProject(p);
         }
     }
     
     /*
     public void runT2Map(String projectFilePath) {

        Policy.setPolicy(new Policy()
        {
            public PermissionCollection getPermissions(CodeSource codesource)
            {
                Permissions perms = new Permissions();
                perms.add(new AllPermission());
                return (perms);
            }

            public void refresh()
            {
            }
        });

        final T2Map t2map = this;
        

        File cdir = new File("./");
        log.debug(cdir.getAbsolutePath());
        
        if (projectFilePath != null && projectFilePath.length() > 0)
        {
            file = new File(projectFilePath);
            if (!file.exists() || !(file.getPath().endsWith(".tms") || file.getPath().endsWith(".gts")))
            {
                file = null;
                JOptionPane.showMessageDialog(t2map, rmgr.format("read.file.error", "tms, gts"));
            }
        }

        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                t2map.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
                t2map.setLocationRelativeTo(null); // 画面の真ん中に表示
                t2map.setVisible(true);

                if (file != null)
                {
                    try
                    {
                        FileOpenPlugin open = new FileOpenPlugin();
                        open.init(t2map, pluginManager);
                        ProjectIF p = open.open(file);
                        t2map.setProject(p);
                        t2map.getLayerPanel().repaint();
                    }
                    catch (OpenPluginException e)
                    {
                        log.error(e.getMessage(), e);
                    }
                }

            }
        });
    }
    */

    static File file = null;

    // static File file = null;

    /**
     * クリップボードの内容のオーナではなくなったことを、このオブジェクトに通知します。
     * 
     * @param clp クリップボード
     * @param tras クラスのインタフェース
     */
    public void lostOwnership(Clipboard clp, Transferable tras)
    {
    }

    /**
     * JPanelオブジェクト
     */
    private JPanel jPanel = null;

    /**
     * ALLボタンオブジェクト
     */
    private JButton jButton = null;

    /**
     * NONEボタンオブジェクト
     */
    private JButton jButton1 = null;

    /**
     * JLabelオブジェクト
     */
    private JLabel jLabel = null;

    /**
     * JPanelを初期化します。
     * 
     * @return JPanel 初期化されたJPanel
     */
    private JPanel getLeftTopPane()
    {
        if (jPanel == null)
        {
            jLabel = new JLabel();
            jLabel.setText(rmgr.getString("jlabel.select"));
            jPanel = new JPanel();
            jPanel.setLayout(new FlowLayout());
            // jPanel.setSize(new Dimension(243, 53));
            jPanel.add(jLabel, null);
            jPanel.add(getJButton(), null);
            jPanel.add(getJButton1(), null);
        }
        return jPanel;
    }

    /**
     * ALLボタンを取得します。
     * 
     * @return JButton ALLボタン
     */
    private JButton getJButton()
    {
        if (jButton == null)
        {
            jButton = new JButton();
            jButton.setText(rmgr.getString("jlabel.all"));
            jButton.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(java.awt.event.ActionEvent e)
                {
                    tree.checkAll();
                }
            });
        }
        return jButton;
    }

    /**
     * CANCELボタンを取得します。
     * 
     * @return JButton CANCELボタン
     */
    private JButton getJButton1()
    {
        if (jButton1 == null)
        {
            jButton1 = new JButton();
            jButton1.setText(rmgr.getString("jlabel.none"));
            jButton1.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(java.awt.event.ActionEvent e)
                {
                    tree.uncheckAll();
                }
            });
        }
        return jButton1;
    }

    public void changeInitialMinMax(Record[] records)
    {
        TimeAxisIF data = getTimeAxisData();
        BigDecimal min = data.getInitialMin();
        BigDecimal max = data.getInitialMax();
        for (int i = 0; i < records.length; i++)
        {
            JulianDate js = records[i].getStartDatetime();
            if (js != null)
            {
                BigDecimal rmin = js.getDatetime();
                if (rmin != null && rmin.compareTo(min) < 0)
                {
                    min = rmin;
                }
            }
            JulianDate je = records[i].getEndDatetime();
            if (je != null)
            {
                BigDecimal rmax = je.getDatetime();
                if (rmax != null && rmax.compareTo(max) > 0)
                {
                    max = rmax;
                }
            }
        }
        data.setInitialMin(min);
        data.setInitialMax(max);
    }
}
