/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.Method;

import javax.swing.JFileChooser;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * TMSファイルを開くファイルチューザーを表示するクラスです。
 */
public class T2MapFileChooser extends JFileChooser
{
    /**
     * シリアルバーションID
     */
    private static final long serialVersionUID = 1L;
    
    private static final Log log = LogFactory.getLog(T2MapFileChooser.class);

    /**
     * ファイル選択ダイアログの表示
     */
    public T2MapFileChooser()
    {
        super();
        setLocale(T2Map.LOCALE);
        updateUI();
        File dir = null;
        String dirPath = T2Map.properties.getProperty("jp.nihu.gttime.T2Map.CURRENT_DIR");
        if (dirPath != null && dirPath.length() > 0)
        {
            dir = new File(dirPath);
            if (!dir.exists())
            {
            	String jarDir = new File("").getAbsolutePath();
                dir = new File(jarDir + File.separator +  "SampleData");
            }
            
            if (!dir.isDirectory())
            {
                dir = dir.getParentFile();
            }
        }
        else
        {
            //dir = new File(".");
        	String jarDir = new File("").getAbsolutePath();
            dir = new File(jarDir + File.separator +  "SampleData");
        }
        super.setCurrentDirectory(dir);
    }
    
    /*
    public static String getContextFilePath(String fileName) {
    	return getTCL().getResource(fileName).getPath();
    }
    	 
    private static ClassLoader getTCL() {
    	Method method = null;
    	 
    	try {
    		method = Thread.class.getMethod("getContextClassLoader", new Class[0]);
    		return (ClassLoader)method.invoke(Thread.currentThread(), new Object[0]);
    	} catch (Exception e) {
    		e.printStackTrace();
    	}
    	 
    	return null;
    }
    */
    
    /**
     * 直近にロードしたファイルディレクトリの保存
     */
    public File getSelectedFile()
    {
        File selectedFile = super.getSelectedFile();
        if (selectedFile != null)
        {
            File dir = super.getSelectedFile();
            if (!dir.isDirectory())
            {
                dir = dir.getParentFile();
            }
            T2Map.properties.setProperty("jp.nihu.gttime.T2Map.CURRENT_DIR", dir.getAbsolutePath());
            try
            {
                T2Map.properties.store();
            }
            catch (FileNotFoundException e)
            {
                e.printStackTrace();
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
        }

        return selectedFile;
    }
}
