/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime;

import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Hashtable;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;

import jp.nihu.gttime.data.DatasetIF;
import jp.nihu.gttime.data.ProjectGts;
import jp.nihu.gttime.data.ProjectIF;
import jp.nihu.gttime.layer.LayerViewPart;
import jp.nihu.gttime.layer.TimeDurationSelector;
import jp.nihu.gttime.plugin.AnalizePlugin;
import jp.nihu.gttime.plugin.ImageOutputPlugin;
import jp.nihu.gttime.plugin.LayerPlugin;
import jp.nihu.gttime.plugin.MaskAnalyzePlugin;
import jp.nihu.gttime.plugin.OpenPlugin;
import jp.nihu.gttime.plugin.OpenPluginException;
import jp.nihu.gttime.plugin.PluginManager;
import jp.nihu.gttime.plugin.PluginManagerException;
import jp.nihu.gttime.plugin.PluginPreferences;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.plugin.SavePlugin;
import jp.nihu.gttime.plugin.SearchPlugin;
import jp.nihu.gttime.plugin.TimeMeasureDialog;
import jp.nihu.gttime.preferences.PreferencesDialog;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * t2mapのメニューバークラスです。
 */
public class T2MapMenuBar extends JMenuBar
{
    /**
     * シリアルバーションID
     */
    private static final long serialVersionUID = 0L;

    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(T2MapMenuBar.class, T2Map.LOCALE);

    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(T2MapMenuBar.class);

    /**
     * PluginManagerオブジェクト
     */
    private PluginManager pluginManager = null;

    /**
     * T2Mapオブジェクト
     */
    private T2Map t2map = null;

    /**
     * メニューバーを管理するコンストラクタです。
     * 
     * @param t2map
     */
    public T2MapMenuBar(T2Map t2map)
    {
        log.debug(T2Map.LOCALE);
        this.t2map = t2map;
        this.pluginManager = T2Map.getPluginManager();
        add(getFileMenu());
        add(getAnalizeMenu());
//        add(getEditMenu());
//        add(getViewMenu());
        
        //【将来拡張】
//        add(getToolMenu());
        
        add(getHelpMenuButton());
    }
    
    
    public JMenu getToolMenu()
    {
        JMenu menu = new JMenu();
        menu.setEnabled(false);
        menu.setText("ツール");
        JMenuItem item = null;
        item = new JMenuItem("【将来拡張】時間名辞書");
        menu.add(item);
        item = new JMenuItem("【将来拡張】暦変換");
        menu.add(item);
        return menu;
    }
    
    

    /**
     * メニューバーのフォントを定義します。
     * 
     * @param comp Componentオブジェクト
     */
    public void setFont(Component comp)
    {
        Font cf = comp.getFont();
        comp.setFont(new Font(cf.getName(), Font.PLAIN, cf.getSize()));
    }

    /**
     * メニューバーの「ヘルプ」についてのオブジェクトを取得します。
     * 
     * @return メニューバーの「ヘルプ」についてのオブジェクト
     */
    public JMenu getHelpMenuButton()
    {
        JMenu viewMenu = new JMenu();
        viewMenu.setText(rmgr.getString("menu.help"));
        JMenuItem item = null;
        item = new JMenuItem(rmgr.getString("menu.help.about"));
        item.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent e)
            {
//                StringBuffer sb = new StringBuffer(rmgr.getString("help.word1") + "(Ver."+T2Map.APPLICATION_VERSION+")" + rmgr.getString("changing.line"));
//                sb.append(rmgr.getString("help.copy") + rmgr.getString("changing.line"));
//                sb.append(rmgr.getString("changing.line"));
//                sb.append(rmgr.getString("help.othercopy") + rmgr.getString("changing.line"));

                // JOptionPane.showConfirmDialog(t2map, sb.toString(),
                // rmgr.getString("help.title"), JOptionPane.PLAIN_MESSAGE);

                AboutDialog d = new AboutDialog(t2map, true);
                d.setTitle(rmgr.getString("help.title"));
//                String y = Integer.toString(Calendar.getInstance().get(Calendar.YEAR));
//                String mm = String.format("%02d",Calendar.getInstance().get(Calendar.MONTH)+1);
//                String dd = String.format("%02d",Calendar.getInstance().get(Calendar.DAY_OF_MONTH));
//                d.setVersion(rmgr.getString("help.word1") + "(Ver."+T2Map.APPLICATION_VERSION+")");
//                d.setVersion(rmgr.getString("help.word1") + "(Ver."+y+mm+dd+")");
                d.setVersion(rmgr.getString("help.word1") + "(Ver." + T2Map.APPLICATION_VERSION +")");
                d.setCopyright(rmgr.getString("help.copy").replaceAll("##CURRENT_YEAR##", T2Map.APPLICATION_COPY));
                BufferedReader r = new BufferedReader(new InputStreamReader(this.getClass().getClassLoader().getResourceAsStream("T_LICENSE.txt")));
                StringBuffer sbs = new StringBuffer();
                String l = null;
                try
                {
                    while ((l = r.readLine()) != null)
                    {
                        sbs.append(l).append("\n");
                    }
                }
                catch (IOException e1)
                {
                    e1.printStackTrace();
                }
                d.setThirdParty(sbs.toString());
                d.setVisible(true);
            }
        });
        viewMenu.add(item);

        return viewMenu;
    }

    // /**
    // * メニューバーの「表示」についてのオブジェクトを取得します。
    // *
    // * @return メニューバーの「表示」についてのオブジェクト
    // */
    // private JMenu getViewMenu()
    // {
    // JMenu viewMenu = new JMenu();
    // viewMenu.setText(rmgr.getString("menu.hyoji"));
    // JMenuItem item = null;
    // item = new JMenuItem(rmgr.getString("menu.hyoji.rekihenko"));
    // item.setEnabled(false);
    // viewMenu.add(item);
    // return viewMenu;
    // }

//    /**
//     * メニューバーの「編集」についてのオブジェクトを取得します。
//     * 
//     * @return メニューバーの「編集」についてのオブジェクト
//     */
//    private JMenu getEditMenu()
//    {
//        JMenu editMenu = new JMenu();
//        editMenu.setText(rmgr.getString("menu.henshu"));
//        editMenu.add(getSearchMenu());
//        editMenu.add(getAnalizeMenu());
//
//        JMenuItem item = null;
//
//        editMenu.addSeparator();
//
//        item = new JMenuItem(rmgr.getString("menu.henshu.setei"));
//        item.addActionListener(new java.awt.event.ActionListener()
//        {
//            public void actionPerformed(java.awt.event.ActionEvent e)
//            {
//                // 設定ダイアログの生成
//                PreferencesDialog preferences = new PreferencesDialog();
//                preferences.setT2Map(t2map);
//                int n = T2Map.preferences.size();
//                for (int i = 0; i < n; i++)
//                {
//                    PluginPreferences pref = T2Map.preferences.get(i);
//                    pref.initValue();
//                    preferences.addPreference(pref);
//                }
//                preferences.setVisible(true);
//                preferences.dispose();
//                preferences = null;
//            }
//        });
//        editMenu.add(item);
//        return editMenu;
//    }
    
    
    
    private JMenu getNewMenu()
    {
    	JMenu menu = new JMenu(rmgr.getString("menu.new"));
        JMenuItem item = new JMenuItem(rmgr.getString("menu.new.workspace"));
        item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				log.debug("New Workspace");
				if (saveCurrentProject(true) != JOptionPane.CANCEL_OPTION) {
					ProjectIF p = new ProjectGts(null);
					t2map.setProject(p);
				}
			}
		});
        menu.add(item);
        
        return menu;
    }
    /**
     * メニューバーの「ファイル」についてのオブジェクトを取得します。
     * 
     * @return メニューバーの「ファイル」についてのオブジェクト
     */
    private JMenu getFileMenu()
    {
        JMenu fileMenu = new JMenu();
        fileMenu.setText(rmgr.getString("menu.file"));
        
        fileMenu.add(getNewMenu());
        
        fileMenu.add(getOpenMenu());
        
//        fileMenu.addSeparator();
        
        //【将来拡張】
        fileMenu.add(getSaveMenu());
        
        fileMenu.add(getSaveAsMenu());
        
        fileMenu.addSeparator();
        
        ProjectIF p = T2Map.getProject();
        FileImportMenu fim = new FileImportMenu(t2map, p, true);
        fileMenu.add(fim.getOpenImportMenu());
        
        fileMenu.add(fim.getImportMenu());
        

        JMenuItem item = null;

        
        fileMenu.addSeparator();

        ArrayList<?> v = pluginManager.getPlugins("ImageOutputPlugin");
        if (v != null && !v.isEmpty())
        {
            for (Object o : v)
            {
                if (o instanceof Hashtable)
                {
                    Object name = ((Hashtable<?, ?>) o).get("name");
                    Object clazz = ((Hashtable<?, ?>) o).get("class");
                    if (name != null && clazz != null)
                    {
                        item = new JMenuItem(name.toString());
                        item.setEnabled(false);
                        item.addActionListener(new ImageOutputActionListener(clazz.toString()));
                        fileMenu.add(item);
                    }
                }
            }
        }
        
        
//        fileMenu.addSeparator();
        
        ArrayList<?> vp = pluginManager.getPlugins("PrintOutputPlugin");
        if (vp != null && !vp.isEmpty())
        {
            for (Object o : vp)
            {
                if (o instanceof Hashtable)
                {
                    Object name = ((Hashtable<?, ?>) o).get("name");
                    Object clazz = ((Hashtable<?, ?>) o).get("class");
                    if (name != null && clazz != null)
                    {
                        item = new JMenuItem(name.toString());
                        item.setEnabled(false);
                        item.addActionListener(new ImageOutputActionListener(clazz.toString()));
                        fileMenu.add(item);
                    }
                }
            }
        }
        
        
        fileMenu.addSeparator();
        
        item = new JMenuItem(rmgr.getString("menu.henshu.setei"));
        item.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent e)
            {
                // 設定ダイアログの生成
                PreferencesDialog preferences = new PreferencesDialog();
                preferences.setT2Map(t2map);
                int n = T2Map.preferences.size();
                for (int i = 0; i < n; i++)
                {
                    PluginPreferences pref = T2Map.preferences.get(i);
                    pref.initValue();
                    preferences.addPreference(pref);
                }
                preferences.setVisible(true);
                preferences.dispose();
                preferences = null;
            }
        });
//        item.setEnabled(false);
        fileMenu.add(item);
        
        

        fileMenu.addSeparator();

        item = new JMenuItem(rmgr.getString("menu.file.exit"));
        item.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent e)
            {
                // レイヤ表示順序、画面サイズなど、アプリケーションの現状を保存する。
                //System.exit(0);
                t2map.quit();
            }
        });
        fileMenu.add(item);
        return fileMenu;
    }

    /**
     * メニューバーの「ファイル」→「開く」についてのオブジェクトを取得します。
     * 
     * @return メニューバーの「ファイル」→「開く」についてのオブジェクト
     */
//    private JMenu getOpenMenu()
//    {
//        JMenu menu = null;
//        JMenuItem item = null;
//        menu = new JMenu(rmgr.getString("menu.file.open"));
//        ArrayList<Hashtable<String, String>> plugins = pluginManager.getPlugins("OpenPlugin");
//
//        for (int i = 0; i < plugins.size(); i++)
//        {
//            item = new JMenuItem(plugins.get(i).get("name"));
//            item.addActionListener(new OpenActionListener(plugins.get(i).get("class")));
//            menu.add(item);
//        }
//        if (menu.getItemCount() < 1)
//        {
//            item = new JMenuItem(rmgr.getString("no.item"));
//            menu.add(item);
//        }
//        return menu;
//    }
    
    
    private int saveCurrentProject(boolean withCancel) {
    	int res = JOptionPane.CANCEL_OPTION;
    	ProjectIF cp = T2Map.getProject();
		if (cp != null && cp.getNodes() != null && cp.getNodes().length > 0) {
			// Save Current project.
			
			if (withCancel) {
				res = JOptionPane.showConfirmDialog(t2map, rmgr.getString("menu.alert.save"), rmgr.getString("menu.file.save"), JOptionPane.YES_NO_CANCEL_OPTION);
			} else {
				res = JOptionPane.showConfirmDialog(t2map, rmgr.getString("menu.alert.save"), rmgr.getString("menu.file.save"), JOptionPane.YES_NO_OPTION);
			}
			if (res == JOptionPane.YES_OPTION) {
				log.debug("Save Current project");
				try
                {
					ArrayList<Hashtable<String, String>> plugins = pluginManager.getPlugins("SavePlugin");
			        String clazz = plugins.get(0).get("class");
			        SavePlugin plugin = (SavePlugin) pluginManager.newInstance(clazz);
			        plugin.saveAs(t2map, T2Map.getProject());
                }
                catch (PluginManagerException ex)
                {
                	log.error(ex.getMessage(),ex);
                }
			}
		}
		return res;
    }
    
    private JMenuItem getOpenMenu()
    {
        JMenuItem item = new JMenuItem(rmgr.getString("menu.file.open"));
        ArrayList<Hashtable<String, String>> plugins = pluginManager.getPlugins("OpenPlugin");
        for (int i = 0; i < plugins.size(); i++) {
        	log.info(plugins.get(i).get("class"));
        	if ("jp.nihu.gttime.plugin.io.FileOpenPlugin".equals(plugins.get(i).get("class"))) {
        		item.addActionListener(new OpenActionListener(plugins.get(i).get("class")));
        	}
        }
        return item;
    }

    /**
     * メニューバーの「ファイル」→「開く」についての動作を実装します。
     */
    class OpenActionListener implements ActionListener
    {
        /**
         * プラグインの文字列
         */
        private String clazz = null;

        /**
         * プラグインの文字列を格納
         * 
         * @param clazz プラグインの文字列
         */
        public OpenActionListener(String clazz)
        {
            this.clazz = clazz;
        }

        /**
         * プロジェクトをセットします。
         * 
         * @param event アクションイベントオブジェクト
         */
        public void actionPerformed(ActionEvent event)
        {
            log.debug(clazz);
            try
            {
            	log.debug("Open Workspace");

                OpenPlugin plugin = (OpenPlugin) pluginManager.newInstance(clazz);
                plugin.init(t2map, pluginManager);
                ProjectIF project = plugin.open();
                if (project != null)
                {
                	saveCurrentProject(false);
                    t2map.setProject(project);
                }
                plugin.processError();
            }
            catch (PluginManagerException e)
            {
                t2map.procError(e);
            }
            catch (OpenPluginException e)
            {
                t2map.procError(e);
            }
        }
    }

    public JMenuItem getMenuItem(String name)
    {
        JMenuItem item = null;
        for (int i = 0; i < getMenuCount(); i++)
        {
            JMenu m = this.getMenu(i);
            item = getMenuItem(m, name);
            if (item != null)
            {
                return item;
            }
        }
        return item;
    }

    public JMenuItem getMenuItem(JMenu menu, String name)
    {
        for (int i = 0; i < menu.getItemCount(); i++)
        {
            JMenuItem m = menu.getItem(i);
            if (m.getName().equals(name))
            {
                return m;
            }
            else
            {
                if (m instanceof JMenu)
                {
                    return getMenuItem((JMenu) m, name);
                }
            }
        }
        return null;
    }
    
//    private JMenu getOpenImportMenu()
//    {
//    	return getImportMenu(rmgr.getString("menu.file.open.import"), "ImportOpenPlugin");
//    }
//
//    /**
//     * メニューバーの「ファイル」→「インポート」についてのオブジェクトを取得します。
//     * 
//     * @return メニューバーの「ファイル」→「インポート」についてのオブジェクト
//     */
//    private JMenu getImportMenu()
//    {
//    	return getImportMenu(rmgr.getString("menu.file.import"), "ImportPlugin");
//    }
//    
//    private JMenu getImportMenu(String title, String type) {
//    	JMenu menu = null;
//        JMenuItem item = null;
//        menu = new JMenu(title);
//
//        ArrayList<Hashtable<String, String>> plugins = pluginManager.getPlugins(type);
//        
//        final ArrayList<ImportPlugin> ps = new ArrayList<ImportPlugin>();
//        for (int i = 0; i < plugins.size(); i++) {
//        	try {
//        		ImportPlugin plugin = (ImportPlugin) pluginManager.newInstance(plugins.get(i).get("class"));
//        		plugin.init(t2map, pluginManager);
//        		ps.add(plugin);
//			} catch (PluginManagerException e1) {
//				t2map.procError(e1);
//			}
//        }
//
//        item = new JMenuItem(rmgr.getString("menu.file.import.layer"));//
//        item.addActionListener(new java.awt.event.ActionListener()
//        {
//        	JFileChooser chooser;
//        	
//            public void actionPerformed(java.awt.event.ActionEvent e)
//            {
//            	//ファイルチューザ表示　モックアップ
//                UIManager.put("FileChooser.readOnly", Boolean.TRUE);
//                chooser = new T2MapFileChooser();
//
//                for (int i = 0; i < ps.size(); i++)
//                {
//                	ImportPlugin plugin = ps.get(i);
//					chooser.setFileFilter(plugin.getExtensionFileFilter());
//					JPanel panel = plugin.getJPanel();
//					if (panel != null) {
//						chooser.setAccessory(panel);
//					} else {
//						chooser.setAccessory(new JPanel());
//					}
//                }
//                chooser.addPropertyChangeListener(new PropertyChangeListener() {
//					
//					public void propertyChange(PropertyChangeEvent evt) {
//						String prop = evt.getPropertyName();
//						if (JFileChooser.FILE_FILTER_CHANGED_PROPERTY.equals(prop)) {
//							Object o = evt.getNewValue();
//							if (o != null && o instanceof ExtensionFileFilter) {
//								ExtensionFileFilter f = (ExtensionFileFilter)evt.getNewValue();
//								for (int i = 0; i < ps.size(); i++)
//				                {
//				                	ImportPlugin plugin = ps.get(i);
//									if (plugin.getExtensionFileFilter().getDescription().equals(f.getDescription())) {
//										JPanel panel = plugin.getJPanel();
//										if (panel != null) {
//											chooser.setAccessory(panel);
//										} else {
//											chooser.setAccessory(new JPanel());
//										}
//										chooser.getAccessory().updateUI();
//									}
//				                }
////							} else  {
////								chooser.setAccessory(new JPanel());
////								chooser.updateUI();
//							}
//						}
//					}
//				});
//                
//                int returnVal = chooser.showOpenDialog(t2map);
//                if (returnVal == JFileChooser.APPROVE_OPTION)
//                {
//                	String ext = chooser.getFileFilter().getDescription();
//                	for (int i = 0; i < ps.size(); i++)
//                    {
//                		if (ps.get(i).getExtensionFileFilter().getDescription().equals(ext)) {
//                			try
//                            {
//                                ImportPlugin plugin = ps.get(i);
//                                plugin.init(t2map, pluginManager);
//                                plugin.open(chooser.getSelectedFile());
//                                plugin.processError();
//                            }
//                            catch (OpenPluginException e5)
//                            {
//                                t2map.procError(e5);
//                            }
//                			break;
//                		}
//                    }
//                }
//                UIManager.put("FileChooser.readOnly", Boolean.FALSE);
//            }
//        });
//        menu.add(item);
//        
//        /*
//        ArrayList<Hashtable<String, String>> plugins = pluginManager.getPlugins("ImportPlugin");
//
//        for (int i = 0; i < plugins.size(); i++)
//        {
//            item = new JMenuItem(plugins.get(i).get("name"));
//            item.setName(plugins.get(i).get("class"));
//            item.addActionListener(new ImportActionListener(plugins.get(i).get("class")));
//            menu.add(item);
//            menu.addMouseListener(new ImportMouseListener(item));
//        }
//        
//        if (menu.getItemCount() < 1)
//        {
//            item = new JMenuItem(rmgr.getString("no.item"));
//            menu.add(item);
//        }
//        */
//        
//        return menu;
//    }

//    class ImportMouseListener implements MouseListener
//    {
//        private JMenuItem item = null;
//
//        public ImportMouseListener(JMenuItem item)
//        {
//            this.item = item;
//        }
//
//        public void mouseClicked(MouseEvent e)
//        {
//        }
//
//        public void mouseEntered(MouseEvent e)
//        {
//            if ("jp.nihu.gttime.plugin.io.TMMImportPlugin".equals(item.getName()))
//            {
//                // if (T2Map.getProjectType() == 1)
//                // {
//                // item.setEnabled(false);
//                // }
//                // else
//                // {
//                item.setEnabled(true);
//                // }
//            }
//            else if ("jp.nihu.gttime.plugin.io.GTMImportPlugin".equals(item.getName()))
//            {
//                if (T2Map.getProjectType() == 0)
//                {
//                    item.setEnabled(false);
//                }
//                else
//                {
//                    item.setEnabled(true);
//                }
//            }
//        }
//
//        public void mouseExited(MouseEvent e)
//        {
//        }
//
//        public void mousePressed(MouseEvent e)
//        {
//        }
//
//        public void mouseReleased(MouseEvent e)
//        {
//        }
//    }

//    /**
//     * メニューバーの「ファイル」→「インポート」についての動作を実装します。
//     */
//    class ImportActionListener implements ActionListener
//    {
//        /**
//         * プラグインの文字列
//         */
//        private String clazz = null;
//
//        /**
//         * プラグインの文字列を格納
//         * 
//         * @param clazz プラグインの文字列
//         */
//        public ImportActionListener(String clazz)
//        {
//            this.clazz = clazz;
//        }
//
//        /**
//         * プロジェクトをセットします。
//         * 
//         * @param event アクションイベントオブジェクト
//         */
//        public void actionPerformed(ActionEvent event)
//        {
//            log.debug(clazz);
//            try
//            {
//                ImportPlugin plugin = (ImportPlugin) pluginManager.newInstance(clazz);
//                plugin.init(t2map, pluginManager);
//                plugin.open();
//                plugin.processError();
//            }
//            catch (PluginManagerException e)
//            {
//                t2map.procError(e);
//            }
//            catch (OpenPluginException e)
//            {
//                t2map.procError(e);
//            }
//        }
//    }

    /**
     * メニューバーの「ファイル」→「保存」についてのオブジェクトを取得します。
     * 
     * @return メニューバーの「ファイル」→「保存」についてのオブジェクト
     */
    private JMenuItem getSaveMenu()
    {
        JMenuItem item = new JMenuItem(rmgr.getString("menu.file.save"));
        item.setEnabled(false);
    	ArrayList<Hashtable<String, String>> plugins = pluginManager.getPlugins("SavePlugin");
    	final String clazz = plugins.get(0).get("class");
    	item.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent e)
            {
            	log.debug(clazz);
                try
                {
                    SavePlugin plugin = (SavePlugin) pluginManager.newInstance(clazz);
                    plugin.save(t2map, T2Map.getProject());
                }
                catch (PluginManagerException ex)
                {
                	log.error(ex.getMessage(),ex);
                }
            }
        });
    	add(item);
        return item;
    }
    
    private JMenuItem getSaveAsMenu()
    {
    	FileSaveMenu fsm = new FileSaveMenu(t2map, T2Map.getProject());
    	return fsm.getSaveAsMenu();
    }

    /**
     * メニューバーの「ファイル」→「保存」についての動作を実装します。
     */
    class SaveActionListener implements ActionListener
    {
        /**
         * プラグインの文字列
         */
        private String clazz = null;

        /**
         * プラグインの文字列を格納
         * 
         * @param clazz プラグインの文字列
         */
        public SaveActionListener(String clazz)
        {
            this.clazz = clazz;
        }

        /**
         * 保存を行います。
         * 
         * @param event アクションイベントオブジェクト
         */
//        @SuppressWarnings("static-access")
        public void actionPerformed(ActionEvent event)
        {
            log.debug(clazz);
            try
            {
                SavePlugin plugin = (SavePlugin) pluginManager.newInstance(clazz);
                plugin.saveAs(t2map, T2Map.getProject());
            }
            catch (PluginManagerException e)
            {
            }
        }
    }

//    /**
//     * メニューバーの「編集」→「外部システム検索」についてのオブジェクトを取得します。
//     * 
//     * @return メニューバーの「編集」→「外部システム検索」についてのオブジェクト
//     */
//    private JMenu getSearchMenu()
//    {
//        JMenu menu = null;
//        JMenuItem item = null;
//        menu = new JMenu(rmgr.getString("menu.henshu.out.retrieval"));
//        ArrayList<Hashtable<String, String>> plugins = pluginManager.getPlugins("SearchPlugin");
//
//        for (int i = 0; i < plugins.size(); i++)
//        {
//            item = new JMenuItem(plugins.get(i).get("name"));
//            item.addActionListener(new SearchActionListener(plugins.get(i).get("class")));
//            menu.add(item);
//        }
//        if (menu.getItemCount() < 1)
//        {
//            item = new JMenuItem(rmgr.getString("no.item"));
//            menu.add(item);
//        }
//        return menu;
//    }

    /**
     * メニューバーの「編集」→「外部システム検索」についての動作を実装します。
     */
    class SearchActionListener implements ActionListener
    {
        /**
         * プラグインの文字列
         */
        private String clazz = null;

        /**
         * プラグインの文字列を格納
         * 
         * @param clazz プラグインの文字列
         */
        public SearchActionListener(String clazz)
        {
            this.clazz = clazz;
        }

        /**
         * 外部システム検索を行います。
         * 
         * @param event アクションイベントオブジェクト
         */
        public void actionPerformed(ActionEvent event)
        {
            log.debug(clazz);
            try
            {
                SearchPlugin plugin = (SearchPlugin) pluginManager.newInstance(clazz);
                plugin.init(t2map);
                DatasetIF[] datasets = plugin.search();
                plugin.setDatasets2Project(datasets);
            }
            catch (PluginManagerException e)
            {
            }
        }
    }

    /**
     * メニューバーの「解析」についてのオブジェクトを取得します。
     * 
     * @return メニューバーの「解析」についてのオブジェクト
     */
    private JMenu getAnalizeMenu()
    {
        JMenu menu = new JMenu(rmgr.getString("menu.henshu.in.retrieval"));
        
        menu.addMenuListener(new MenuListener() {
            public void menuSelected(MenuEvent e) {
//                log.debug("menuSelected");
            	LayerViewPart lvp = t2map.getLayerPanel();
                LayerPlugin layer = lvp.getLayer(0);
                if (layer != null) {
                TimeDurationSelector timeDurationSelector = layer.getTimeDurationSelector();
                BigDecimal[][] duls = timeDurationSelector.getDurations();
                if (duls.length > 0)
                {
                	measureMenuItem.setEnabled(true);
                }
                else
                {
                	measureMenuItem.setEnabled(false);
                }
        	}
            }

            public void menuDeselected(MenuEvent e) {
//                log.debug("menuDeselected");
            }

            public void menuCanceled(MenuEvent e) {
//                log.debug("menuCanceled");
            }
        });
        
        menu.setEnabled(false);
        JMenuItem item = null;
        ArrayList<?> v = null;

        // 検索用メニュー設定
        v = pluginManager.getPlugins("AnalizePlugin");
        if (v != null && !v.isEmpty())
        {
            for (Object o : v)
            {
                if (o instanceof Hashtable)
                {
                    Object name = ((Hashtable<?, ?>) o).get("name");
                    Object clazz = ((Hashtable<?, ?>) o).get("class");
                    if (name != null && clazz != null)
                    {
                        item = new JMenuItem(name.toString());
                        item.addActionListener(new AnalizeActionListener(clazz.toString()));
                        menu.add(item);
                    }
                }
            }
        }
        // マスク演算用メニュー設定
        v = pluginManager.getPlugins("MaskAnalyzePlugin");
        if (v != null && !v.isEmpty())
        {
            if (menu.getItemCount() > 0)
            {
                menu.addSeparator();
            }
            for (Object o : v)
            {
                if (o instanceof Hashtable)
                {
                    Object name = ((Hashtable<?, ?>) o).get("name");
                    Object clazz = ((Hashtable<?, ?>) o).get("class");
                    if (name != null && clazz != null)
                    {
                        item = new JMenuItem(name.toString());
                        item.addActionListener(new MaskAnalyzeActionListener(clazz.toString()));
                        menu.add(item);
                    }
                }
            }
        }
        
        // ################################################################# メニューサンプル
//        if (menu.getItemCount() > 0)
//        {
//            menu.addSeparator();
//        }
//        item = new JMenuItem("【将来拡張】統計処理");
//        menu.add(item);
        // #################################################################
        
        // #################################################################
        if (menu.getItemCount() > 0)
        {
            menu.addSeparator();
        }
//        item = new JMenuItem("時間軸計測");
//        menu.add(item);
        createMeasureMenu(menu);
        // #################################################################
        
        if (menu.getItemCount() <= 0)
        {
            item = new JMenuItem(rmgr.getString("no.item"));
            menu.add(item);
        }
        return menu;
    }
    
    /**
     * 計測メニューを管理するメッソドです。
     * 
     * @param menu JPopupMenuオブジェクト
     * @return 計測メニューのオブジェクト
     */
    public void createMeasureMenu(JMenu menu)
    {
        measureMenuItem = new JMenuItem(rmgr.getString("time.axis.measure"));
        menu.add(measureMenuItem);
//        measureMenuItem.setText(rmgr.getString("time.axis.measure"));
        measureMenuItem.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
            	LayerPlugin layer = t2map.getLayerPanel().getLayer(0);
                TimeDurationSelector timeDurationSelector = layer.getTimeDurationSelector();
                BigDecimal[][] duls = timeDurationSelector.getDurations();
                if (duls.length > 0)
                {
                	BigDecimal all = null;
                    for (int i = 0; i < duls.length; i++)
                    {
                        if (all == null)
                        {
                            all = new BigDecimal(0);
                        }
                        all = all.add(duls[i][1].subtract(duls[i][0]));
                    }
                    new TimeMeasureDialog((T2Map) SwingUtilities.getRoot(getParent()), true, all);
                }
            }
        });
        measureMenuItem.setEnabled(false);
    }

    private JMenuItem measureMenuItem = null;

    
    
    
    

    /**
     * メニューバーの「編集」→「内部データ解析・演算・検索」についての動作を実装します。
     */
    class AnalizeActionListener implements ActionListener
    {
        /**
         * プラグインの文字列
         */
        private String clazz = null;

        /**
         * プラグインの文字列を格納
         * 
         * @param clazz プラグインの文字列
         */
        public AnalizeActionListener(String clazz)
        {
            this.clazz = clazz;
        }

        /**
         * 内部データ解析・演算・検索を行います。
         * 
         * @param event アクションイベントオブジェクト
         */
        public void actionPerformed(ActionEvent event)
        {
            log.debug(clazz);
            try
            {
                AnalizePlugin plugin = (AnalizePlugin) pluginManager.newInstance(clazz);
                plugin.init(t2map, T2Map.getProject());

            }
            catch (PluginManagerException e)
            {
            }
        }
    }

    class ImageOutputActionListener implements ActionListener
    {
        private String clazz = null;

        public ImageOutputActionListener(String clazz)
        {
            this.clazz = clazz;
        }

        public void actionPerformed(ActionEvent ae)
        {
            try
            {
                log.debug(clazz);
                ImageOutputPlugin plugin = (ImageOutputPlugin) pluginManager.newInstance(clazz);
                plugin.initPlugin(t2map);
                plugin.output();
            }
            catch (PluginManagerException e)
            {
                log.error(e, e);
            }
        }
    }

    class MaskAnalyzeActionListener implements ActionListener
    {
        private String clazz = null;

        public MaskAnalyzeActionListener(String clazz)
        {
            this.clazz = clazz;
        }

        public void actionPerformed(ActionEvent ae)
        {
            try
            {
                // Log [debug]
                log.debug(clazz);
                MaskAnalyzePlugin plugin = (MaskAnalyzePlugin) pluginManager.newInstance(clazz);
                plugin.initPlugin(t2map);
                plugin.addDataset(plugin.analyze());
            }
            catch (PluginManagerException e)
            {
                // Log [error]
                log.error(e, e);
            }
        }
    }
}
