/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import javax.swing.JButton;
import javax.swing.JProgressBar;

import jp.nihu.gttime.plugin.PluginResourceManager;

public class T2MapProgressBarDialog<Project> extends T2MapDialog
{
    private static final long serialVersionUID = 1L;

    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(T2MapProgressBarDialog.class, T2Map.LOCALE);

    Future<Project> future;

    ExecutorService executor;

    ExecutorService observer;

    /** Creates new form ProgressDialog */
    public T2MapProgressBarDialog(Frame parent, boolean modal, String title, String message)
    {
        super(parent, modal);
        setTitle(title);
        executor = Executors.newSingleThreadExecutor();
        observer = Executors.newSingleThreadExecutor();
        initComponents();
        addWindowListener(new WindowAdapter()
        {
            @Override
            public void windowOpened(WindowEvent e)
            {
                if (future == null)
                {
                    setVisible(false);
                }
                observer.submit(new Runnable()
                {
                    public void run()
                    {
                        while (!future.isDone())
                        {
                            try
                            {
                                Thread.sleep(1000);
                            }
                            catch (InterruptedException ex)
                            {
                                break;
                            }
                        }
                        future = null;
                        setVisible(false);
                    }
                });
            }

            @Override
            public void windowClosing(WindowEvent e)
            {
                if (future != null && !future.isDone() && !future.isCancelled())
                {
                    cancelTask();
                }
            }
        });
    }

    private void initComponents()
    {
        JProgressBar aJProgressBar = new JProgressBar();
        aJProgressBar.setPreferredSize(new Dimension(250, 15));
        aJProgressBar.setIndeterminate(true);
        add(aJProgressBar, BorderLayout.NORTH);

        JButton cancel = new JButton(rmgr.getString("cancel"));
        cancel.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent evt)
            {
                cancelTask();
            }
        });
        add(cancel, BorderLayout.SOUTH);

        setSize(250, 70);
        setResizable(false);
        Toolkit tk = getToolkit();
        Dimension screenSize = tk.getScreenSize();
        int posX = (screenSize.width / 2) - (getWidth() / 2);
        int posY = (screenSize.height / 2) - (getHeight() / 2);
        setLocation(posX, posY);
    }

    public Future<Project> submit(Callable<Project> task)
    {
        if (future != null)
        {
            throw new IllegalStateException("Task already submitted.");
        }
        future = executor.submit(task);
        return future;
    }

    public void cancelTask()
    {
        if (future != null && !future.isDone())
        {
            future.cancel(true);
        }
    }
}
