/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.data;

import java.awt.Color;
import java.awt.Font;
import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.XMLConstants;
import javax.xml.namespace.NamespaceContext;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.data.property.LineIF;
import jp.nihu.gttime.data.property.LineTms;
import jp.nihu.gttime.data.property.PolygonIF;
import jp.nihu.gttime.data.property.PolygonTms;
import jp.nihu.gttime.data.property.PropertiesGts;
import jp.nihu.gttime.data.property.PropertiesIF;
import jp.nihu.gttime.data.property.SymbolIF;
import jp.nihu.gttime.data.property.SymbolTms;
import jp.nihu.gttime.plugin.OpenPlugin;
import jp.nihu.gttime.plugin.OpenPluginException;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.rdf.RDFaReader;
import jp.nihu.gttime.time.utility.ISO8601Exception;
import jp.nihu.gttime.time.utility.JulianGregorianDate;
import jp.nihu.gttime.utility.GTM;
import jp.nihu.gttime.utility.Utilities;
import jp.nihu.gttime.utility.XMLUtil;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.semarglproject.rdf.ParseException;
import org.semarglproject.source.StreamProcessor;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;

public class DatasetGtm implements DatasetIF
{
    /**
     * シリアルバーションID
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(DatasetGtm.class, T2Map.LOCALE);

    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(DatasetGtm.class);

    /**
     * Documentオブジェクト
     */
    private Document doc;

    /**
     * レコード配列のオブジェクト
     */
    private Record[] records;
    
    /**
     * groupsのオブジェクト
     */
    private List<String> groups;
    
    private Model model;
    
    public String getRdfBase() {
    	return rdfBase;
    }
    
    @Override
    public Model getModel() throws ParseException, IOException {
    	if (model == null && getDomDataset() != null) {
//    		String basePath = T2Map.RRF_BASE_URI;
    		String basePath = Utilities.getUri(getRdfBase());
//    		System.out.println("DatasetGtm: " + basePath);
    		model = ModelFactory.createDefaultModel();
    		RDFaReader reader = new RDFaReader();
    		StreamProcessor streamProcessor = reader.getStreamProcessor(model);
    		streamProcessor.process(XMLUtil.getByteInputStream(getDomDataset()), basePath);
    		
    		if (0 < records.length) {
    			Node n = records[0].getNode();
    			if (n != null) {
    				streamProcessor.process(XMLUtil.getByteInputStream(n.getOwnerDocument()), basePath);
    			}
    		}
    	}
    	return model;
    }
    
    public List<String> getGroups() {
        if (groups == null) {
            for (int i = 0; i < records.length; i++) {
                setGroup(records[i].getGroup());
            }
        }
        return groups;
    }
    
    public void setGroup(String group) {
        if (groups == null) {
            groups = new ArrayList<String>();
        }
        if (!groups.contains(group)) {
            groups.add(group);
        }
    }
    
    public void setGroups(List<String> groups) {
        this.groups = groups;
    }

    /**
     * TMMファイルのパスを格納します。
     */
    private String path;

    /**
     * データセットのタイトル
     */
    private String title = null;

    /**
     * データセットの時間範囲終了日
     */
    private BigDecimal endDate = null;

    /**
     * データセットの時間範囲開始日
     */
    private BigDecimal startDate = null;

    /**
     * レイヤタイプオブジェクト
     */
    private String layerType = null;

    private PropertiesIF properties = null;

    public PropertiesIF getProperties()
    {
        return properties;
    }

    public void reloadProperties(String layerName)
    {
        //TODO: Should I reload properties from gtm or set to null?
        //properties = null;
    }

    public void setProperties(PropertiesIF properties)
    {
        this.properties = properties;
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.DatasetIF#applyStyleChanges()
     */
    public void applyStyleChanges(String layerTypeName, String layerStyleName)
    {
        String nshgis = "http://www.h-gis.org/hgis";
        String nsxsi = "http://www.w3.org/2001/XMLSchema-instance";

        Document owner = fragmentOfProject.getOwnerDocument();
        // 新しいスタイル設定記述要素を構築する
        Element newTime = owner.createElementNS(nshgis, "time");
        newTime.setAttribute("version", GTM.getVersion());

        newTime.setAttributeNS(nsxsi, "type", layerStyleName);
        {
            Element layerTypeElement = owner.createElementNS(nshgis, "layerType");
            newTime.appendChild(layerTypeElement);
            layerTypeElement.setTextContent(layerTypeName);
        }

        if (layerStyleName.equals("BarChartLayer.type"))
        {
            applyPolygonChange(nshgis, owner, newTime);
            applySelectedColorChange(nshgis, owner, newTime);
            applyBackgroundColorChange(nshgis, owner, newTime);
            
            applyDetailFontNameChange(nshgis, owner, newTime);
            applyShowGroupNameChange(nshgis, owner, newTime);
            
            applyLayerHeightChange(nshgis, owner, newTime);
            applyDataReductionChange(nshgis, owner, newTime);
        }
        else if (layerStyleName.equals("BarTimeScaleLayer.type"))
        {
            applyPolygonChange(nshgis, owner, newTime);
            applyFontChange(nshgis, owner, newTime);
            applySelectedColorChange(nshgis, owner, newTime);
            applyBackgroundColorChange(nshgis, owner, newTime);
            
            applyDetailFontNameChange(nshgis, owner, newTime);
            applyShowGroupNameChange(nshgis, owner, newTime);
            
            applyDataReductionChange(nshgis, owner, newTime);
        }
        else if (layerStyleName.equals("DefaultLayer.type"))
        {
            applyPolygonChange(nshgis, owner, newTime);
            applyFontChange(nshgis, owner, newTime);
            applySelectedColorChange(nshgis, owner, newTime);
            applyBackgroundColorChange(nshgis, owner, newTime);
            
            applyDetailFontNameChange(nshgis, owner, newTime);
            applyShowGroupNameChange(nshgis, owner, newTime);
            
            applyLayerHeightChange(nshgis, owner, newTime);
            applyDataReductionChange(nshgis, owner, newTime);
        }
        else if (layerStyleName.equals("LineChartLayer.type"))
        {
            applyLineChange(nshgis, owner, newTime);
            applySymbolChange(nshgis, owner, newTime);
            applySelectedColorChange(nshgis, owner, newTime);
            applyBackgroundColorChange(nshgis, owner, newTime);
            
            applyDetailFontNameChange(nshgis, owner, newTime);
            applyShowGroupNameChange(nshgis, owner, newTime);
            
            applyLayerHeightChange(nshgis, owner, newTime);
            applyDataReductionChange(nshgis, owner, newTime);
        }
        else if (layerStyleName.equals("MaskLayer.type"))
        {
            applyPolygonChange(nshgis, owner, newTime);
            applyBackgroundColorChange(nshgis, owner, newTime);
            applyDataReductionChange(nshgis, owner, newTime);
        }
        else if (layerStyleName.equals("PlotChartLayer.type"))
        {
            applySymbolChange(nshgis, owner, newTime);
            applySelectedColorChange(nshgis, owner, newTime);
            applyBackgroundColorChange(nshgis, owner, newTime);
            
            applyDetailFontNameChange(nshgis, owner, newTime);
            applyShowGroupNameChange(nshgis, owner, newTime);
            
            applyLayerHeightChange(nshgis, owner, newTime);
            applyDataReductionChange(nshgis, owner, newTime);
        }
        else if (layerStyleName.equals("TimeScaleLayer.type"))
        {
            applyFontChange(nshgis, owner, newTime);
            applyScaleMainChange(nshgis, owner, newTime);
            applyScaleSubChange(nshgis, owner, newTime);
            applyBackgroundColorChange(nshgis, owner, newTime);
            applyDataReductionChange(nshgis, owner, newTime);
        }

        // applyLineChange(nshgis, owner, newTime);
        // applyPolygonChange(nshgis, owner, newTime);
        // applySymbolChange(nshgis, owner, newTime);
        // applyFontChange(nshgis, owner, newTime);
        // applySelectedColorChange(nshgis, owner, newTime);
        // applyBackgroundColorChange(nshgis, owner, newTime);
        // applyLayerHeightChange(nshgis, owner, newTime);
        // applyScaleMainChange(nshgis, owner, newTime);
        // applyScaleSubChange(nshgis, owner, newTime);
        // applyDataReductionChange(nshgis, owner, newTime);

        // 既存の要素と入れ替える

        NodeList time = fragmentOfProject.getElementsByTagNameNS("http://www.h-gis.org/hgis", "time");
        if (time.getLength() > 0)
        {
            time.item(0).getParentNode().replaceChild(newTime, time.item(0));
        }
        else
        {
            fragmentOfProject.appendChild(newTime);
            NodeList resourceStatus = fragmentOfProject.getElementsByTagNameNS("http://www.h-gis.org/hgis", "resourceStatus");
            if (resourceStatus.getLength() > 0)
            {
                resourceStatus.item(0).appendChild(newTime);
            }
            else
            {
                Element resourceStatusElem = owner.createElementNS("http://www.h-gis.org/hgis", "resourceStatus");
                resourceStatusElem.setAttribute("version", GTM.getVersion());
                resourceStatusElem.appendChild(owner.createElementNS("http://www.h-gis.org/hgis", "common"));
                resourceStatusElem.appendChild(newTime);
            }
        }
    }

    private void applyLineChange(String nshgis, Document owner, Element newTime)
    {
        List<String> groups = getGroups();
        for (int i = 0; i < groups.size(); i++) {
            String group = groups.get(i);
            Element line = owner.createElementNS(nshgis, "line");
            if (group.length() > 0) {
                line.setAttribute("group", group);
            }
            Element thin = owner.createElementNS(nshgis, "thin");
            Element lineColor = owner.createElementNS(nshgis, "lineColor");
            newTime.appendChild(line);
            line.appendChild(thin);
            line.appendChild(lineColor);
            LineIF linep = properties.getLine(group);
            thin.setTextContent(Integer.toString(linep.getThin()));
            lineColor.setTextContent(Utilities.getColor(linep.getLineColor()));
        }
    }

    private void applyPolygonChange(String nshgis, Document owner, Element newTime)
    {
        List<String> groups = getGroups();
        for (int i = 0; i < groups.size(); i++) {
            String group = groups.get(i);
            Element polygon = owner.createElementNS(nshgis, "polygon");
            if (group.length() > 0) {
                polygon.setAttribute("group", group);
            }
            Element fillColor = owner.createElementNS(nshgis, "fillColor");
            Element thin = owner.createElementNS(nshgis, "thin");
            Element lineColor = owner.createElementNS(nshgis, "lineColor");
            newTime.appendChild(polygon);
            polygon.appendChild(fillColor);
            polygon.appendChild(thin);
            polygon.appendChild(lineColor);
            PolygonIF polygonp = properties.getPolygon(group);
            fillColor.setTextContent(Utilities.getColor(polygonp.getFillColor()));
            thin.setTextContent(Integer.toString(polygonp.getThin()));
            lineColor.setTextContent(Utilities.getColor(polygonp.getLineColor()));
        }
    }

    private void applySymbolChange(String nshgis, Document owner, Element newTime)
    {
        List<String> groups = getGroups();
        for (int i = 0; i < groups.size(); i++) {
            String group = groups.get(i);
            Element symbol = owner.createElementNS(nshgis, "symbol");
            if (group.length() > 0) {
                symbol.setAttribute("group", group);
            }
            Element symbolc = owner.createElementNS(nshgis, "symbol");
            Element size = owner.createElementNS(nshgis, "size");
            Element fillColor = owner.createElementNS(nshgis, "fillColor");
            Element thin = owner.createElementNS(nshgis, "thin");
            Element lineColor = owner.createElementNS(nshgis, "lineColor");
            newTime.appendChild(symbol);
            symbol.appendChild(symbolc);
            symbol.appendChild(size);
            symbol.appendChild(fillColor);
            symbol.appendChild(thin);
            symbol.appendChild(lineColor);
            SymbolIF symbolp = properties.getSymbol(group);
            symbolc.setTextContent(symbolp.getSymbol());
            size.setTextContent(Integer.toString(symbolp.getSize()));
            fillColor.setTextContent(Utilities.getColor(symbolp.getFillColor()));
            thin.setTextContent(Integer.toString(symbolp.getThin()));
            lineColor.setTextContent(Utilities.getColor(symbolp.getLineColor()));
        }
    }

    private void applyFontChange(String nshgis, Document owner, Element newTime)
    {
        List<String> groups = getGroups();
        for (int i = 0; i < groups.size(); i++) {
            String group = groups.get(i);
            Element font = owner.createElementNS(nshgis, "font");
            if (group.length() > 0) {
                font.setAttribute("group", group);
            }
            Element fontFamily = owner.createElementNS(nshgis, "font-family");
            Element fontName = owner.createElementNS(nshgis, "font-name");
            Element fontSize = owner.createElementNS(nshgis, "font-size");
            Element fontColor = owner.createElementNS(nshgis, "font-color");
            Element fontStyle = owner.createElementNS(nshgis, "font-style");
            newTime.appendChild(font);
            font.appendChild(fontFamily);
            font.appendChild(fontName);
            font.appendChild(fontSize);
            font.appendChild(fontColor);
            font.appendChild(fontStyle);
            Font fontp = properties.getFont(group);
            fontFamily.setTextContent(fontp.getFamily());
            fontName.setTextContent(fontp.getName());
            fontSize.setTextContent(Integer.toString(fontp.getSize()));
            fontColor.setTextContent(Utilities.getColor(properties.getFontColor(group)));
            fontStyle.setTextContent(Utilities.getFontStyle(fontp.getStyle()));
        }
    }

    private void applySelectedColorChange(String nshgis, Document owner, Element newTime)
    {
        Element selectedColor = owner.createElementNS(nshgis, "selectedColor");
        newTime.appendChild(selectedColor);
        selectedColor.setTextContent(Utilities.getColor(properties.getSelectedColor()));
    }

    private void applyBackgroundColorChange(String nshgis, Document owner, Element newTime)
    {
        Element backgroundColor = owner.createElementNS(nshgis, "backgroundColor");
        newTime.appendChild(backgroundColor);
        backgroundColor.setTextContent(Utilities.getColor(properties.getBackgroundColor()));
    }
    
    private void applyDetailFontNameChange(String nshgis, Document owner, Element newTime)
    {
        Element detailFontName = owner.createElementNS(nshgis, "detailFontName");
        newTime.appendChild(detailFontName);
        detailFontName.setTextContent(properties.getDetailFontName());
    }
    
    private void applyShowGroupNameChange(String nshgis, Document owner, Element newTime)
    {
        Element showGroupName = owner.createElementNS(nshgis, "showGroupName");
        newTime.appendChild(showGroupName);
        showGroupName.setTextContent(Boolean.toString(properties.isShowGroupName()));
    }

    private void applyLayerHeightChange(String nshgis, Document owner, Element newTime)
    {
        Element layerHight = owner.createElementNS(nshgis, "layerHight");
        newTime.appendChild(layerHight);
        layerHight.setTextContent(Integer.toString(properties.getLayerHeight()));
    }

    private void applyScaleMainChange(String nshgis, Document owner, Element newTime)
    {
        Element scaleMain = owner.createElementNS(nshgis, "scaleMain");
        Element fillColor = owner.createElementNS(nshgis, "fillColor");
        Element thin = owner.createElementNS(nshgis, "thin");
        Element lineColor = owner.createElementNS(nshgis, "lineColor");
        newTime.appendChild(scaleMain);
        scaleMain.appendChild(fillColor);
        scaleMain.appendChild(thin);
        scaleMain.appendChild(lineColor);
        PolygonIF scaleMainp = properties.getScaleMain();
        fillColor.setTextContent(Utilities.getColor(scaleMainp.getFillColor()));
        thin.setTextContent(Integer.toString(scaleMainp.getThin()));
        lineColor.setTextContent(Utilities.getColor(scaleMainp.getLineColor()));
    }

    private void applyScaleSubChange(String nshgis, Document owner, Element newTime)
    {
        Element scaleSub = owner.createElementNS(nshgis, "scaleSub");
        Element fillColor = owner.createElementNS(nshgis, "fillColor");
        Element thin = owner.createElementNS(nshgis, "thin");
        Element lineColor = owner.createElementNS(nshgis, "lineColor");
        newTime.appendChild(scaleSub);
        scaleSub.appendChild(fillColor);
        scaleSub.appendChild(thin);
        scaleSub.appendChild(lineColor);
        PolygonIF scaleSubp = properties.getScaleSub();
        fillColor.setTextContent(Utilities.getColor(scaleSubp.getFillColor()));
        thin.setTextContent(Integer.toString(scaleSubp.getThin()));
        lineColor.setTextContent(Utilities.getColor(scaleSubp.getLineColor()));
    }

    private void applyDataReductionChange(String nshgis, Document owner, Element newTime)
    {
        Element dataReduction = owner.createElementNS(nshgis, "dataReduction");
        newTime.appendChild(dataReduction);
        dataReduction.setTextContent(Boolean.toString(properties.isDataReduction()));
        Element dataReductionEnableRecordNum = owner.createElementNS(nshgis, "dataReductionEnableRecordNum");
        newTime.appendChild(dataReductionEnableRecordNum);
        dataReductionEnableRecordNum.setTextContent(Integer.toString(properties.getDataReductionNum()));
    }

    /**
     * プロジェクトXMLのこのデータセットに関する記述がされている要素
     */
    private Element fragmentOfProject = null;

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.DatasetIF#setFragmentDatasetNodeOfProject(org.w3c.dom.Element)
     */
    public void setFragmentDatasetNodeOfProject(Element fragment)
    {
        fragmentOfProject = fragment;
    }
    
    private String getGroupAttrValue(Element elem) {
        String group = elem.getAttribute("group");
        if (group == null) {
            group = "";
        }
        return group;
    }

    public void setProperties(Element time)
    {
        properties = new PropertiesGts();

        // line
        NodeList elems = getElement(time, "http://www.h-gis.org/hgis", "line");
        if (elems != null) {
            for (int i = 0; i < elems.getLength(); i++)
            {
                Element elem = (Element)elems.item(i);
                LineIF line = new LineTms();
                line.setThin(Integer.parseInt(getElementContent(elem, "./hgis:thin/text()")));
                line.setLineColor(getColor(getElementContent(elem, "./hgis:lineColor/text()")));
                String group = getGroupAttrValue(elem);
                properties.setLine(line, group);
            }
        }
        // polygon
        elems = getElement(time, "http://www.h-gis.org/hgis", "polygon");
        if (elems != null) {
            for (int i = 0; i < elems.getLength(); i++) {
                Element elem = (Element)elems.item(i);
                PolygonTms polygon = new PolygonTms();
                polygon.setFillColor(getColor(getElementContent(elem, "./hgis:fillColor/text()")));
                polygon.setThin(Integer.parseInt(getElementContent(elem, "./hgis:thin/text()")));
                polygon.setLineColor(getColor(getElementContent(elem, "./hgis:lineColor/text()")));
                String group = getGroupAttrValue(elem);
                properties.setPolygon(polygon, group);
            }
        }
        // symbol
        elems = getElement(time, "http://www.h-gis.org/hgis", "symbol");
        if (elems != null) {
            for (int i = 0; i < elems.getLength(); i++) {
                Element elem = (Element)elems.item(i);
                if (elem.getParentNode().getLocalName().equals("time")) {
                    SymbolTms symbol = new SymbolTms();
                    symbol.setSymbol(getElementContent(elem, "./hgis:symbol/text()"));
                    symbol.setSize(Integer.parseInt(getElementContent(elem, "./hgis:size/text()")));
                    symbol.setThin(Integer.parseInt(getElementContent(elem, "./hgis:thin/text()")));
                    symbol.setLineColor(getColor(getElementContent(elem, "./hgis:lineColor/text()")));
                    symbol.setFillColor(getColor(getElementContent(elem, "./hgis:fillColor/text()")));
                    String group = getGroupAttrValue(elem);
                    properties.setSymbol(symbol, group);
                }
            }
        }
        // font
        elems = getElement(time, "http://www.h-gis.org/hgis", "font");
        if (elems != null) {
            for (int i = 0; i < elems.getLength(); i++) {
                Element elem = (Element)elems.item(i);
                String name = getElementContent(elem, "./hgis:font-name/text()");
                String styleStr = getElementContent(elem, "./hgis:font-style/text()");
                int style = Utilities.getFontStyle(styleStr);
                int size = Integer.parseInt(getElementContent(elem, "./hgis:font-size/text()").replaceAll("pt", ""));
                Font font = new Font(name, style, size);
                Color fontColor = getColor(getElementContent(elem, "./hgis:font-color/text()"));
                String group = getGroupAttrValue(elem);
                properties.setFont(font, group);
                properties.setFontColor(fontColor, group);
                // log.debug(getElementContent(elem, "./hgis:font-family/text()"));
            }
        }
        // selectedColor
        String selectedColor = getElementContent(time, "./hgis:selectedColor/text()");
        if (selectedColor != null && selectedColor.length() > 0)
        {
            properties.setSelectedColor(getColor(selectedColor));
        }
        // backgroundColor
        String backgroundColor = getElementContent(time, "./hgis:backgroundColor/text()");
        if (backgroundColor != null && backgroundColor.length() > 0)
        {
            properties.setBackgroundColor(getColor(backgroundColor));
        }
        
        // detailFontName
        String detailFontName = getElementContent(time, "./hgis:detailFontName/text()");
        if (detailFontName != null && detailFontName.length() > 0)
        {
            properties.setDetailFontName(detailFontName);
        }
        
        // showGroupName
        String showGroupName = getElementContent(time, "./hgis:showGroupName/text()");
        if (showGroupName != null && showGroupName.length() > 0)
        {
            properties.setShowGroupName(Boolean.parseBoolean(showGroupName));
        }
        
        // layerHight
        String layerHight = getElementContent(time, "./hgis:layerHight/text()");
        if (layerHight != null && layerHight.length() > 0)
        {
            properties.setLayerHeight(Integer.parseInt(layerHight));
        }
        // scaleSub
        elems = getElement(time, "http://www.h-gis.org/hgis", "scaleSub");
        if (elems != null && elems.getLength() > 0)
        {
            Element elem = (Element)elems.item(0);
            PolygonTms scaleSub = new PolygonTms();
            scaleSub.setFillColor(getColor(getElementContent(elem, "./hgis:fillColor/text()")));
            scaleSub.setThin(Integer.parseInt(getElementContent(elem, "./hgis:thin/text()")));
            scaleSub.setLineColor(getColor(getElementContent(elem, "./hgis:lineColor/text()")));
            properties.setScaleSub(scaleSub);
        }
        // scaleMain
        elems = getElement(time, "http://www.h-gis.org/hgis", "scaleMain");
        if (elems != null && elems.getLength() > 0)
        {
            Element elem = (Element)elems.item(0);
            PolygonTms scaleMain = new PolygonTms();
            scaleMain.setFillColor(getColor(getElementContent(elem, "./hgis:fillColor/text()")));
            scaleMain.setThin(Integer.parseInt(getElementContent(elem, "./hgis:thin/text()")));
            scaleMain.setLineColor(getColor(getElementContent(elem, "./hgis:lineColor/text()")));
            properties.setScaleMain(scaleMain);
        }
        // dataReduction
        String dataReduction = getElementContent(time, "./hgis:dataReduction/text()");
        if (dataReduction != null && dataReduction.length() > 0)
        {
            properties.setDataReduction(Boolean.parseBoolean(dataReduction));
        }
        // dataReductionEnableRecordNum
        String dataReductionEnableRecordNum = getElementContent(time, "./hgis:dataReductionEnableRecordNum/text()");
        if (dataReductionEnableRecordNum != null && dataReductionEnableRecordNum.length() > 0)
        {
            properties.setDataReductionNum(Integer.parseInt(dataReductionEnableRecordNum));
        }
    }

    private Color getColor(String colorStr)
    {
        return Utilities.getColor(colorStr);
    }

    private NodeList getElement(Element elem, String namespace, String name)
    {
        NodeList nl = elem.getElementsByTagNameNS(namespace, name);
        if (nl != null)
        {
            return nl;
        }
        return null;
    }

    /**
     * デフォルトのコンストラクタです。
     */
    public DatasetGtm()
    {
    }

    public DatasetGtm(Element fragmentOfProject)
    {
        this.fragmentOfProject = fragmentOfProject;
    }
    
    private static String getPrefix() {
    	if (T2Map.LOCALE.equals(Locale.JAPANESE)) {
    		return "検索結果";
    	} else {
    		return "Search Results";
    	}
    }

    /**
     * 新規データセットを作成する情報を取得します。
     * 
     * @param title データセットのタイトル
     * @param layerType データセットのレイヤタイプ
     * @param records レコードの配列
     * @throws DatasetException
     */
    public DatasetGtm(String title, String layerType, Record[] records) throws DatasetException
    {
    	this(getPrefix() + "(" + title + ")", layerType, "XML for HuTime", records);
    }

    /**
     * 新規データセットを作成する情報を取得します。
     * 
     * @param title title データセットのタイトル
     * @param layerType データセットのレイヤタイプ
     * @param dataFormat データフォーマット
     * @param records レコードの配列
     * @throws DatasetException
     */
    public DatasetGtm(String title, String layerType, String dataFormat, Record[] records) throws DatasetException
    {
        if (records != null && records.length > 0)
        {
            if (layerType == null || layerType.length() <= 0)
            {
                layerType = "DefaultLayer";
            }
            this.records = records;
            String t = Utilities.escapeFileName(title);
            if (t.length() > 10) {
            	t = t.substring(0, 10);
            }
            String newFilename = t + "_" + Utilities.getUniqueString();

            this.path = newFilename + ".gtm";

            String[] replaceKey = { "-T2MAP_TITLE-", "-T2MAP_DESC-", "-T2MAP_DATE-", "-T2MAP_FORMAT-", "-T2MAP_EARLY-", "-T2MAP_LATE-", "-T2MAP_LAYERTYPE-" };
            String sDate = "";
            BigDecimal js = getStartDatetime();
            if (js != null)
            {
                sDate = JulianGregorianDate.toGregorian(getStartDatetime(), "Z");
            }
            String eDate = "";
            BigDecimal je = getEndDatetime();
            if (je != null)
            {
                eDate = JulianGregorianDate.toGregorian(getEndDatetime(), "Z");
            }
            String[] replace = { title, title, newFilename, dataFormat, sDate, eDate, layerType };
            String gtmXml = getDatasetXML();
            Pattern p2 = null;
            Matcher m = null;
            for (int i = 0; i < replace.length; i++)
            {
                p2 = Pattern.compile(replaceKey[i]);
                m = p2.matcher(gtmXml);
                if (m.find())
                {
                    gtmXml = m.replaceFirst(replace[i]);
                }
            }

            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            dbf.setNamespaceAware(true);
            DocumentBuilder db = null;
            try
            {
                db = dbf.newDocumentBuilder();
            }
            catch (ParserConfigurationException e)
            {
                throw new DatasetException(rmgr.format("error.parser.configuration", rmgr.getString("word.kensaku")), e);
            }
            try
            {
                doc = db.parse(new InputSource(new StringReader(gtmXml)));
            }
            catch (SAXException e)
            {
                throw new DatasetException(rmgr.format("error.sax", rmgr.getString("word.kensaku")), e);
            }
            catch (IOException e)
            {
                throw new DatasetException(rmgr.format("error.io", rmgr.getString("word.kensaku")), e);
            }
        }
    }

    /**
     * Datasetの初期化をします。
     * 
     * @return Datasetの初期設定
     */
    private String getDatasetXML()
    {
        StringBuffer sb = new StringBuffer();

        sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");

        sb.append("<gt:gtm version=\""+GTM.getVersion()+"\" xmlns:gt=\"http://www.h-gis.org/gt\" xmlns:hgis=\"http://www.h-gis.org/hgis\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">\n");
        sb.append("<gt:metadata>\n");
        sb.append("<gt:dataVersion>1.0</gt:dataVersion>\n");
        sb.append("<gt:spatial>\n");
        sb.append("<gt:westBoundLongitude></gt:westBoundLongitude>\n");
        sb.append("<gt:eastBoundLongitude></gt:eastBoundLongitude>\n");
        sb.append("<gt:southBoundLatitude></gt:southBoundLatitude>\n");
        sb.append("<gt:northBoundLatitude></gt:northBoundLatitude>\n");
        sb.append("</gt:spatial>\n");
        sb.append("<gt:temporal>\n");
        sb.append("<gt:begin>-T2MAP_EARLY-</gt:begin>\n");
        sb.append("<gt:end>-T2MAP_LATE-</gt:end>\n");
        sb.append("</gt:temporal>\n");
        sb.append("<hgis:hgisMetadata version=\""+GTM.getVersion()+"\">\n");
        sb.append("<hgis:title>-T2MAP_TITLE-</hgis:title>\n");
        sb.append("<hgis:identifier></hgis:identifier>\n");
        sb.append("<hgis:creater></hgis:creater>\n");
        sb.append("<hgis:description>-T2MAP_DESC-</hgis:description>\n");
        sb.append("</hgis:hgisMetadata>\n");
        sb.append("</gt:metadata>\n");
        sb.append("<gt:resource>\n");
        sb.append("<gt:filename>").append(this.path).append("</gt:filename>\n");
        sb.append("<gt:type>application/xml</gt:type>\n");
        sb.append("<gt:detaDefinition>-T2MAP_FORMAT-</gt:detaDefinition>\n");
        sb.append("<gt:spatial type=\"\">\n");
        sb.append("<gt:name></gt:name>\n");
        sb.append("<gt:projection></gt:projection>\n");
        sb.append("<gt:coordinate></gt:coordinate>\n");
        sb.append("<gt:latitudeField>location/x</gt:latitudeField>\n");
        sb.append("<gt:longitudeField>location/y</gt:longitudeField>\n");
        sb.append("</gt:spatial>\n");
        sb.append("<gt:temporal type=\"Term\">\n");
        sb.append("<gt:name></gt:name>\n");
        sb.append("<gt:calendar>Gregorian</gt:calendar>\n");
        sb.append("<gt:beginField>datetime/from</gt:beginField>\n");
        sb.append("<gt:endField>datetime/from</gt:endField>\n");
        sb.append("</gt:temporal>\n");
        sb.append("<gt:record>/records/record</gt:record>\n");
        sb.append("</gt:resource>\n");
        sb.append("<gt:application>\n");
        sb.append("<hgis:hgisApplications version=\""+GTM.getVersion()+"\">\n");
        sb.append("<hgis:common>\n");
        sb.append("</hgis:common>\n");
        sb.append("<hgis:time version=\""+GTM.getVersion()+"\">\n");
        sb.append("<hgis:layerType>-T2MAP_LAYERTYPE-</hgis:layerType>\n");

        sb.append("<hgis:polygon>\n");
        sb.append("<hgis:fillColor>255,200,0,0</hgis:fillColor>\n");
        sb.append("<hgis:thin>1</hgis:thin>\n");
        sb.append("<hgis:lineColor>0,0,0,0</hgis:lineColor>\n");
        sb.append("</hgis:polygon>\n");
        sb.append("<hgis:font>\n");
        sb.append("<hgis:font-family>sans-serif</hgis:font-family>\n");
        sb.append("<hgis:font-name>MS-ゴシック</hgis:font-name>\n");
        sb.append("<hgis:font-size>10pt</hgis:font-size>\n");
        sb.append("<hgis:font-color>0,0,0,0</hgis:font-color>\n");
        sb.append("<hgis:font-style>normal</hgis:font-style>\n");
        sb.append("</hgis:font>\n");

        sb.append("<hgis:selectedColor>255,0,0,0</hgis:selectedColor>\n");
        sb.append("<hgis:layerHight>100</hgis:layerHight>\n");
        sb.append("<hgis:dataReduction>false</hgis:dataReduction>\n");
        sb.append("</hgis:time>\n");
        sb.append("</hgis:hgisApplications>\n");
        sb.append("</gt:application>\n");
        sb.append("</gt:gtm>\n");

        // sb.append("<METADATA>\n");
        // sb.append("<META NAME=\"dc.title\" TMCODE=\"92\" SCHEMECODE=\"7\"
        // CONTENT=\"-T2MAP_TITLE-\"/>\n");
        // sb.append("<META NAME=\"dc.description\" TMCODE=\"24\"
        // SCHEMECODE=\"7\" CONTENT=\"-T2MAP_DESC-\"/>\n");
        // sb.append("<META NAME=\"dc.pulisher\" TMCODE=\"89\" SCHEMECODE=\"7\"
        // CONTENT=\"T2Map\"/>\n");
        // sb.append("<META NAME=\"dc.date\" TMCODE=\"39\" SCHEMECODE=\"14\"
        // CONTENT=\"-T2MAP_DATE-\"/>\n");
        // sb.append("<META NAME=\"dc.type\" TMCODE=\"38\" SCHEMECODE=\"6\"
        // CONTENT=\"Dataset\"/>\n");
        // sb.append("<META NAME=\"dc.format\" TMCODE=\"7\" SCHEMECODE=\"5\"
        // CONTENT=\"-T2MAP_FORMAT-\"/>\n");
        // sb.append("<META NAME=\"dc.language\" TMCODE=\"6\" SCHEMECODE=\"26\"
        // CONTENT=\"Ja\"/>\n");
        // sb.append("<META NAME=\"dc.coverage.x.min\" TMCODE=\"5\"
        // SCHEMECODE=\"27\" CONTENT=\"-180\"/>\n");
        // sb.append("<META NAME=\"dc.coverage.x.max\" TMCODE=\"4\"
        // SCHEMECODE=\"27\" CONTENT=\"180\"/>\n");
        // sb.append("<META NAME=\"dc.coverage.y.min\" TMCODE=\"19\"
        // SCHEMECODE=\"27\" CONTENT=\"-90\"/>\n");
        // sb.append("<META NAME=\"dc.coverage.y.max\" TMCODE=\"16\"
        // SCHEMECODE=\"27\" CONTENT=\"90\"/>\n");
        // sb.append("<META NAME=\"dc.coverage.t.early\" TMCODE=\"15\"
        // SCHEMECODE=\"14\" CONTENT=\"-T2MAP_EARLY-\"/>\n");
        // sb.append("<META NAME=\"dc.coverage.t.late\" TMCODE=\"14\"
        // SCHEMECODE=\"14\" CONTENT=\"-T2MAP_LATE-\"/>\n");
        // sb.append("<META NAME=\"dc.rights\" TMCODE=\"10\" SCHEMECODE=\"7\"
        // CONTENT=\"デフォルト著作権\"/>\n");
        // sb.append("<META NAME=\"dc.subject.domain\" TMCODE=\"8\"
        // SCHEMECODE=\"32\" CONTENT=\"\"/>\n");
        // sb.append("<META NAME=\"ecai.team\" TMCODE=\"168\" SCHEMECODE=\"33\"
        // CONTENT=\"\"/>\n");
        // sb.append("<META NAME=\"tm.Server.Type\" TMCODE=\"90\"
        // SCHEMECODE=\"50\" CONTENT=\"\"/>\n");
        // sb.append("<META NAME=\"tm.Spatial.MapObjType\" TMCODE=\"36\"
        // SCHEMECODE=\"17\" CONTENT=\"Polygon\"/>\n");
        // sb.append("<META NAME=\"tm.Time.TimeVarying\" TMCODE=\"34\"
        // SCHEMECODE=\"18\" CONTENT=\"NoDateFields\"/>\n");
        // sb.append("<META NAME=\"tm.Filter.AccessibleFields\" TMCODE=\"37\"
        // SCHEMECODE=\"7\" CONTENT=\"\"/>\n");
        // sb.append("<META NAME=\"tm.TableData.TableName\" TMCODE=\"49\"
        // SCHEMECODE=\"7\" CONTENT=\"\"/>\n");
        // sb.append("<META NAME=\"tm.TableData.KeyIDfield\" TMCODE=\"51\"
        // SCHEMECODE=\"7\" CONTENT=\"\"/>\n");
        // sb.append("<META NAME=\"tm.TableData.Labelfield\" TMCODE=\"172\"
        // SCHEMECODE=\"7\" CONTENT=\"\"/>\n");
        // sb.append("<META NAME=\"tm.sys.Local.GISType\" TMCODE=\"80\"
        // SCHEMECODE=\"8\" CONTENT=\"xml\"/>\n");
        // sb.append("<META NAME=\"t2map.layertype\"
        // CONTENT=\"-T2MAP_LAYERTYPE-\"/>\n");
        // sb.append("</METADATA>\n");

        return sb.toString();
    }

    public Document getDomDataset()
    {
        return doc;
    }

    public BigDecimal getEndDatetime() throws DatasetException
    {
        if (endDate == null)
        {
            BigDecimal jdate = null;
            try
            {
                if (doc != null)
                {
                    String date = getEndDateNode().getTextContent();
                    if (date != null && date.length() > 0)
                    {
                        jdate = JulianGregorianDate.toJulian(date);
                    }
                }
                Record[] records = getRecords();
                if (records != null)
                {
                    for (int i = 0; i < records.length; i++)
                    {
                        JulianDate jsd = records[i].getEndDatetime();
                        if (jsd != null)
                        {
                            BigDecimal rdate = jsd.getDatetime();
                            if (jdate == null || rdate.compareTo(jdate) == 1)
                            {
                                jdate = rdate;
                            }
                        }
                    }
                }
            }
            catch (ISO8601Exception e)
            {
                throw new DatasetException(e.getMessage(), e);
            }
            endDate = jdate;
        }
        return endDate;
    }

    public String getLayerType()
    {
        if (layerType != null)
        {
            return layerType;
        }
        else
        {
            String layer = null;
            Element elem = getLayerTypeNode();
            if (elem != null)
            {
                layer = getLayerTypeNode().getTextContent();
            }
            else
            {
                layer = "DefaultLayer";
            }
            layerType = layer;
            return layer;
        }
        // String layer = null;
        // Element elem = getLayerTypeNode();
        // if (elem != null)
        // {
        // layer = getLayerTypeNode().getTextContent();
        // }
        // else
        // {
        // layer = "DefaultLayer";
        // }
        // if (layerType != null && !layerType.equals(layer))
        // {
        // return layerType;
        // }
        // else
        // {
        // return layer;
        // }
    }

    public String getPath()
    {
        return path;
    }
    
    public void setPath(String path) {
    	this.path = path;
    }

    public Record[] getRecords()
    {
        return records;
    }
    
    public Record[] getRecords(String group)
    {
        List<Record> list = new ArrayList<Record>();
        for (Record r : records) {
            if (r.getGroup().equals(group)) {
                list.add(r);
            }
        }
        return list.toArray(new Record[0]);
    }

    public Record[] getRecords(BigDecimal s, BigDecimal e)
    {
        ArrayList<Record> v = new ArrayList<Record>();
        int al = records.length;
        for (int i = 0; i < al; i++)
        {
            boolean doAdd = false;
            JulianDate jsdate = records[i].getStartDatetime();
            JulianDate jedate = records[i].getEndDatetime();
            if (jsdate == null && jedate == null)
            {
                doAdd = true;
            }
            else if (jedate == null && e.compareTo(jsdate.getDatetime()) >= 0)
            {
                doAdd = true;
            }
            else if (jsdate == null && s.compareTo(jedate.getDatetime()) <= 0)
            {
                doAdd = true;
            }
            else if (e.compareTo(jsdate.getDatetime()) >= 0 && s.compareTo(jedate.getDatetime()) <= 0)
            {
                doAdd = true;
            }

            if (doAdd)
            {
                v.add(records[i]);
            }
        }
        Record[] rs = new Record[v.size()];
        rs = v.toArray(rs);// v.copyInto(rs);
        return rs;
    }

    public Record[] getHitRecords()
    {
        ArrayList<Record> v = new ArrayList<Record>();
        int al = records.length;
        for (int i = 0; i < al; i++)
        {
            if (records[i].isHit())
            {
                v.add(records[i]);
            }
        }
        Record[] rs = new Record[v.size()];
        rs = v.toArray(rs);// v.copyInto(rs);
        return rs;
    }

    public BigDecimal getStartDatetime() throws DatasetException
    {
        if (startDate == null)
        {
            BigDecimal jdate = null;
            try
            {
                if (doc != null)
                {
                    String date = getBeginDateNode().getTextContent();
                    if (date != null && date.length() > 0)
                    {
                        jdate = JulianGregorianDate.toJulian(date);
                    }
                }
                Record[] records = getRecords();
                if (records != null)
                {
                    for (int i = 0; i < records.length; i++)
                    {
                        JulianDate jsd = records[i].getStartDatetime();
                        if (jsd != null)
                        {
                            BigDecimal rdate = jsd.getDatetime();
                            if (jdate == null || rdate.compareTo(jdate) == -1)
                            {
                                jdate = rdate;
                            }
                        }
                    }
                }
            }
            catch (ISO8601Exception e)
            {
                throw new DatasetException(e.getMessage(), e);
            }
            startDate = jdate;
        }
        return startDate;
    }

    public String getTitle()
    {
        if (title == null)
        {
            title = getTitleNode().getTextContent();
        }
        return title;
    }

    public boolean hasRecords()
    {
        if (records != null && records.length > 0)
        {
            return true;
        }
        return false;
    }

    public void loadDataset(OpenPlugin op, String path, String base) throws DatasetException
    {
        loadDataset(op, path, base, path);
    }
    
    private String absolutePath;
    
    public String getAbsolutePath() {
    	return absolutePath;
    }
    
    public void setAbsolutePath(String absolutePath) {
    	this.absolutePath = absolutePath;
    }
    
    private String rdfBase;

    public void loadDataset(OpenPlugin op, String pathx, String base, String uniquePath) throws DatasetException
    {
        log.debug("loadDataset");
        if (uniquePath.endsWith("tmm"))
        {
            uniquePath = uniquePath.replaceAll(".tmm", ".gtm");
        }
        // TODO: 以下、要検討
        this.path = uniquePath;
        this.absolutePath = op.getAbsolutePath(uniquePath, base);
        
        String realPath = op.getAbsolutePath(pathx, base);
        
        this.rdfBase = realPath;
        
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        DocumentBuilder builder = null;
        try
        {
            builder = factory.newDocumentBuilder();
        }
        catch (ParserConfigurationException e)
        {
            throw new DatasetException(rmgr.format("error.parser.configuration", ""), e);
        }
        try
        {
            doc = builder.parse(op.getDatasetInputStream(realPath));
            NodeList time = doc.getElementsByTagNameNS("http://www.h-gis.org/hgis", "time");
            if (getProperties() == null && time.getLength() > 0)
            {
                setProperties((Element) time.item(0));
            }
        }
        catch (SAXException e)
        {
            throw new DatasetException(rmgr.format("error.sax", realPath), e);
        }
        catch (IOException e)
        {
            throw new DatasetException(rmgr.format("error.io", realPath), e);
        }
        catch (OpenPluginException e)
        {
            throw new DatasetException(e.getMessage(), e);
        }

//        if (base == null)
//        {
//            base = realPath.substring(0, realPath.lastIndexOf(File.separator));
//        }
        
        if (this.absolutePath.startsWith("http://") || this.absolutePath.startsWith("https://")) {
        	base = this.absolutePath.substring(0, this.absolutePath.lastIndexOf("/") + 1);
        } else {
        	base = this.absolutePath.substring(0, this.absolutePath.lastIndexOf(File.separator));
        }
        
        log.info(op + " :: " + getFileNameNode() + " :: " + base);
        String recordFilePath = op.getAbsolutePath(getFileNameNode().getTextContent(), base);
        setRecords(op, recordFilePath);
    }

    /**
     * レコードをセットします。
     * 
     * @param op プラグイン名
     * @param realPath パス
     * @throws DatasetException
     */
    private void setRecords(OpenPlugin op, String realPath) throws DatasetException
    {
        log.debug(realPath);
        try
        {
            String format = getRecordFormatNode().getTextContent();
            records = op.getRecords(realPath, format);
            setGroups(op.getGroups());
        }
        catch (OpenPluginException e)
        {
            throw new DatasetException(e.getMessage(), e);
        }
    }

    public void setDomDataset(Document doc)
    {
        this.doc = doc;
    }

    public void setLayerType(String layerType)
    {
        this.layerType = layerType;
    }

    public void setTitle(String title)
    {
        this.title = title;
        getTitleNode().setTextContent(title);
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
        return getTitle();
    }

    private Element getTitleNode()
    {
        return getElement("/gt:gtm/gt:metadata/hgis:hgisMetadata/hgis:title");
    }

    private Element getLayerTypeNode()
    {
        return getElement("/gt:gtm/gt:application/hgis:hgisApplications/hgis:time/hgis:layerType");
    }

    private Element getBeginDateNode()
    {
        return getElement("/gt:gtm/gt:metadata/gt:temporal/gt:begin");
    }

    private Element getEndDateNode()
    {
        return getElement("/gt:gtm/gt:metadata/gt:temporal/gt:end");
    }

    private Element getRecordFormatNode()
    {
        return getElement("/gt:gtm/gt:resource/gt:detaDefinition");
    }

    private Element getFileNameNode()
    {
        return getElement("/gt:gtm/gt:resource/gt:filename");
    }

    private Element getElement(String xpathStr)
    {
        Element elem = null;
        try
        {
            XPathFactory factory = XPathFactory.newInstance();
            XPath xpath = factory.newXPath();
            xpath.setNamespaceContext(new GtmNamespaceContext());
            XPathExpression expr = xpath.compile(xpathStr);
            elem = (Element) expr.evaluate(doc, XPathConstants.NODE);
        }
        catch (XPathExpressionException e)
        {
            // never catch
            log.fatal("not valid xpath.", e);
        }
        return elem;
    }

    private String getElementContent(Element elem, String xpathStr)
    {
        String str = null;
        try
        {
            XPathFactory factory = XPathFactory.newInstance();
            XPath xpath = factory.newXPath();
            xpath.setNamespaceContext(new GtmNamespaceContext());
            XPathExpression expr = xpath.compile(xpathStr);
            str = (String) expr.evaluate(elem, XPathConstants.STRING);
        }
        catch (XPathExpressionException e)
        {
            // never catch
            log.fatal("not valid xpath.", e);
        }
        return str;
    }

    private class GtmNamespaceContext implements NamespaceContext
    {
        public String getNamespaceURI(String prefix)
        {
            if (prefix == null)
            {
                throw new NullPointerException("Null prefix");
            }
            else if ("gt".equals(prefix))
            {
                return "http://www.h-gis.org/gt";
            }
            else if ("hgis".equals(prefix))
            {
                return "http://www.h-gis.org/hgis";
            }
            else if ("xml".equals(prefix))
            {
                return XMLConstants.XML_NS_URI;
            }
            return XMLConstants.NULL_NS_URI;
        }

        public String getPrefix(String namespaceURI)
        {
            throw new UnsupportedOperationException();
        }

        public Iterator<Object> getPrefixes(String namespaceURI)
        {
            throw new UnsupportedOperationException();
        }
    }

    // private boolean visible = true;

    /**
     * @return the visible
     */
    public boolean isVisible()
    {
        NodeList nl = fragmentOfProject.getElementsByTagNameNS("http://www.h-gis.org/hgis", "isVisible");
        if (nl.getLength() == 1)
        {
            return Boolean.parseBoolean(nl.item(0).getTextContent());
        }
        else if (nl.getLength() > 1)
        {
            log.warn("incorrect number of isVisible element");
            return Boolean.parseBoolean(nl.item(0).getTextContent());
        }
        else
        {
            log.warn("incorrect number of isVisible element");
            return true;
        }
        // return visible;
    }

    /**
     * @param visible the visible to set
     */
    public void setVisible(boolean visible)
    {
        NodeList nl = fragmentOfProject.getElementsByTagNameNS("http://www.h-gis.org/hgis", "isVisible");
        if (nl.getLength() == 1)
        {
            nl.item(0).setTextContent(Boolean.toString(visible));
        }
        else if (nl.getLength() > 1)
        {
            log.warn("incorrect number of isVisible element");
            nl.item(0).setTextContent(Boolean.toString(visible));
        }
        else
        {
            log.error("incorrect number of isVisible element");
        }
        // this.visible = visible;
    }
    
    public String getRecordPath()
    {
    	String path = "";
    	try
        {
            XPathFactory factory = XPathFactory.newInstance();
            XPath xpath = factory.newXPath();
            xpath.setNamespaceContext(jp.nihu.gttime.utility.XMLUtil.getGtsNsContext());
            XPathExpression expr = xpath.compile("/gt:gtm/gt:resource/gt:filename/text()");
            path = (String) expr.evaluate(getDomDataset(), XPathConstants.STRING);
        }
        catch (XPathExpressionException e)
        {
            // never catch
            log.fatal("not valid xpath.", e);
        }
        return path;
    }
    
    public String getGtmVersion() {
//    	String v = doc.getDocumentElement().getAttribute("version");
    	String v = "";
    	try
        {
        	XPathFactory factory = XPathFactory.newInstance();
            XPath xpath = factory.newXPath();
            xpath.setNamespaceContext(jp.nihu.gttime.utility.XMLUtil.getGtsNsContext());
            XPathExpression expr = xpath.compile("/gt:gtm/gt:metadata/gt:dataVersion/text()");
            v = (String) expr.evaluate(getDomDataset(), XPathConstants.STRING);
        }
        catch (XPathExpressionException e)
        {
            // never catch
            log.fatal("not valid xpath.", e);
        }
    	return v;
    }
    
    public Element getMetadataNode()
    {
        return getElement("/gt:gtm/gt:metadata");
    }
}
