/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.data;

import java.math.BigDecimal;

/**
 * 時間の内容を設定、取得するクラスです。
 */
public class JulianDate
{
    /**
     * 最小値オブジェクト
     */
    private BigDecimal min;

    /**
     * 最大値オブジェクト
     */
    private BigDecimal max;

    /**
     * 日付オブジェクト
     */
    private BigDecimal datetime;

    /**
     * タイムゾーンオブジェクト
     */
    private String timezone = "Z";

    /**
     * 端点が開区間（open）または閉区間（close）か
     */
    private String edge = null;

    // public boolean isOpen()
    // {
    // if (edge.equals("open"))
    // {
    // return true;
    // }
    // return false;
    // }

    public String getEdge()
    {
        return edge;
    }

    /**
     * @param edge the edge to set
     */
    public void setEdge(String edge)
    {
        this.edge = edge;
    }

    /**
     * 日付を取得します。
     * 
     * @return 日付
     */
    public BigDecimal getDatetime()
    {
        return datetime;
    }

    /**
     * 日付を設定します。
     * 
     * @param datetime 日付
     */
    public void setDatetime(BigDecimal datetime)
    {
        this.datetime = datetime;
    }

    /**
     * 最大値を取得します。
     * 
     * @return 最大値
     */
    public BigDecimal getMax()
    {
        return max;
    }

    /**
     * 最大値を設定します。
     * 
     * @param max 最大値
     */
    public void setMax(BigDecimal max)
    {
        this.max = max;
    }

    /**
     * 最小値を取得します。
     * 
     * @return 最小値
     */
    public BigDecimal getMin()
    {
        return min;
    }

    /**
     * 最小値を設定します。
     * 
     * @param min 最小値
     */
    public void setMin(BigDecimal min)
    {
        this.min = min;
    }

    /**
     * タイムゾーンを取得します。
     * 
     * @return タイムゾーン
     */
    public String getTimezone()
    {
        return timezone;
    }

    /**
     * タイムゾーンを設定します。
     * 
     * @param timezone タイムゾーン
     */
    public void setTimezone(String timezone)
    {
        this.timezone = timezone;
    }
}
