/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.data.property;

import java.awt.Color;
import java.awt.Font;
import java.util.HashMap;
import java.util.Map;

public class PropertiesGts implements PropertiesIF
{
    private Map<String, LineIF> line = new HashMap<String, LineIF>();

    private Map<String, PolygonIF> polygon = new HashMap<String, PolygonIF>();

    private Map<String, SymbolIF> symbol = new HashMap<String, SymbolIF>();

    private Map<String, Font> font = new HashMap<String, Font>();

    private Map<String, Color> fontColor = new HashMap<String, Color>();

    private Color selectedColor = null;

    private Color backgroundColor = null;

    private Integer layerHeight = null;

    private PolygonIF scaleMain = null;

    private PolygonIF scaleSub = null;

    private Boolean dataReduction = false;

    private Integer dataReductionNum = 100000;

    private  boolean changed = false;

    private String detailFontName = null;
    
    private Boolean showGroupName = true;
    
    public Boolean isShowGroupName() {
        return showGroupName;
    }
    
    public void setShowGroupName(boolean show) {
        showGroupName = show;
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#getDataReductionNum()
     */
    public Integer getDataReductionNum()
    {
        return dataReductionNum;
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#setDataReductionNum(int)
     */
    public void setDataReductionNum(int dataReductionNum)
    {
        this.dataReductionNum = dataReductionNum;
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#isChanged()
     */
    public boolean isChanged()
    {
        return changed;
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#setChanged(boolean)
     */
    public void setChanged(boolean changed)
    {
        this.changed = changed;
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#getLine(java.lang.String)
     */
    public LineIF getLine(String group)
    {
    	if (!line.containsKey(group)) {
            group = "";
        }
        return line.get(group);
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#setLine(jp.nihu.gttime.data.property.Line, java.lang.String)
     */
    public void setLine(LineIF line, String group)
    {
        if (this.line.containsKey(group)) {
            this.line.remove(group);
        }
        this.line.put(group, line);
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#getPolygon(java.lang.String)
     */
    public PolygonIF getPolygon(String group)
    {
    	if (!polygon.containsKey(group)) {
            group = "";
        }
        return polygon.get(group);
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#setPolygon(jp.nihu.gttime.data.property.Polygon, java.lang.String)
     */
    public void setPolygon(PolygonIF polygon, String group)
    {
        if (this.polygon.containsKey(group)) {
            this.polygon.remove(group);
        }
        this.polygon.put(group, polygon);
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#getSymbol(java.lang.String)
     */
    public SymbolIF getSymbol(String group)
    {
    	if (!symbol.containsKey(group)) {
            group = "";
        }
        return symbol.get(group);
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#setSymbol(jp.nihu.gttime.data.property.Symbol, java.lang.String)
     */
    public void setSymbol(SymbolIF symbol, String group)
    {
        if (this.symbol.containsKey(group)) {
            this.symbol.remove(group);
        }
        this.symbol.put(group, symbol);
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#getFont(java.lang.String)
     */
    public Font getFont(String group)
    {
    	if (!font.containsKey(group)) {
            group = "";
        }
        return font.get(group);
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#setFont(java.awt.Font, java.lang.String)
     */
    public void setFont(Font font, String group)
    {
        if (this.font.containsKey(group)) {
            this.font.remove(group);
        }
        this.font.put(group, font);
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#getFontColor(java.lang.String)
     */
    public Color getFontColor(String group)
    {
    	if (!fontColor.containsKey(group)) {
            group = "";
        }
        return fontColor.get(group);
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#setFontColor(java.awt.Color, java.lang.String)
     */
    public void setFontColor(Color fontColor, String group)
    {
        if (this.fontColor.containsKey(group)) {
            this.fontColor.remove(group);
        }
        this.fontColor.put(group, fontColor);
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#getSelectedColor()
     */
    public Color getSelectedColor()
    {
        return selectedColor;
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#setSelectedColor(java.awt.Color)
     */
    public void setSelectedColor(Color selectedColor)
    {
        this.selectedColor = selectedColor;
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#getLayerHeight()
     */
    public Integer getLayerHeight()
    {
        return layerHeight;
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#setLayerHeight(int)
     */
    public void setLayerHeight(int layerHeight)
    {
        this.layerHeight = layerHeight;
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#getScaleMain()
     */
    public PolygonIF getScaleMain()
    {
        return scaleMain;
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#setScaleMain(jp.nihu.gttime.data.property.Polygon)
     */
    public void setScaleMain(PolygonIF scaleMain)
    {
        this.scaleMain = scaleMain;
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#getScaleSub()
     */
    public PolygonIF getScaleSub()
    {
        return scaleSub;
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#setScaleSub(jp.nihu.gttime.data.property.Polygon)
     */
    public void setScaleSub(PolygonIF scaleSub)
    {
        this.scaleSub = scaleSub;
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#isDataReduction()
     */
    public Boolean isDataReduction()
    {
        return dataReduction;
    }

    /*
     * (non-Javadoc)
     * 
     * @see jp.nihu.gttime.data.property.PropertiesIF#setDataReduction(boolean)
     */
    public void setDataReduction(boolean dataReduction)
    {
        this.dataReduction = dataReduction;
    }

    public Color getBackgroundColor()
    {
        return backgroundColor;
    }

    public void setBackgroundColor(Color backgroundColor)
    {
        this.backgroundColor = backgroundColor;
    }

    public String getDetailFontName()
    {
        return detailFontName;
    }

    public void setDetailFontName(String font)
    {
        this.detailFontName = font;
    }

    public Map<String, Color> getFontColors()
    {
        return fontColor;
    }

    public Map<String, Font> getFonts()
    {
        return font;
    }

    public Map<String, LineIF> getLines()
    {
        return line;
    }

    public Map<String, PolygonIF> getPolygons()
    {
        return polygon;
    }

    public Map<String, SymbolIF> getSymbols()
    {
        return symbol;
    }
}
