/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.image;

import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import javax.imageio.ImageIO;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.SwingUtilities;

import jp.nihu.gttime.utility.ExtensionFileFilter;
import jp.nihu.gttime.utility.ExtensionFileSaveChooser;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * 画像出力を実装しているクラスです。
 */
public class ImageExporter extends ExtensionFileSaveChooser
{
    /**
     * シリアルバーションID
     */
    private static final long serialVersionUID = 1L;

    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(ImageExporter.class);

    /**
     * JComponentオブジェクト
     */
    private JComponent parent;

    /**
     * FileFilterを設定します。
     * 
     * @param parent JComponentオブジェクト
     * @param defaultFileRootName 初期表示するファイル名
     */
    public ImageExporter(JComponent parent, String defaultFileRootName)
    {
        super(defaultFileRootName);
        this.parent = parent;
        String[] formats = ImageIO.getWriterFormatNames();
        ArrayList<String> added = new ArrayList<String>();
        for (int i = 0; i < formats.length; i++)
        {
            String lf = formats[i].toLowerCase();
            if (lf.length() == 3 && !added.contains(lf))
            {
                ExtensionFileFilter filter = new ExtensionFileFilter(lf);
                addChoosableFileFilter(filter);
                added.add(lf);
            }
        }
    }

    /**
     * 画像を保存します。
     */
    public void imageSave()
    {
        int returnVal = showSaveDialog(SwingUtilities.getRoot(parent));
        if (returnVal == JFileChooser.APPROVE_OPTION)
        {
            File file = getSelectedFile();
            String type = getFileFilter().getDescription();
            BufferedImage image = new BufferedImage(parent.getWidth(), parent.getHeight(), BufferedImage.TYPE_INT_BGR);
            Graphics graphics = image.createGraphics();
            parent.paint(graphics);
            graphics.dispose();
            try
            {
                ImageIO.write(image, type, file);
                log.debug("Image save to : " + file.getAbsolutePath());
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
        }
    }
}
