/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.layer;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JComponent;
import javax.swing.JLayeredPane;
import javax.swing.SwingUtilities;

import jp.nihu.gttime.TimePanel;
import jp.nihu.gttime.time.TimeAxisIF;
import jp.nihu.gttime.time.utility.JulianGregorianDate;

public class DateLabel extends JComponent implements MouseMotionListener, MouseListener
{
    /**
     * シリアルバージョンID
     */
    private static final long serialVersionUID = 1L;

    private String label = "";

    /**
     * コンストラクタ
     */
    public DateLabel()
    {
        super();
        setOpaque(false);
    }

    private void setLabel(String label)
    {
        this.label = label;
    }

    /**
     * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
     */
    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        g.setColor(Color.BLUE);
        g.drawString(label, 0, getFont().getSize());
        g.dispose();
    }

    /**
     * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
     */
    public void mouseMoved(MouseEvent e)
    {
        setPosition(e);
    }

    private void setPosition(MouseEvent e)
    {
        Component comp = (Component) e.getSource();
        int x = e.getX();
        int y = getY(e, comp);
        StringBuffer sb = new StringBuffer();
        if (comp instanceof TimePanel)
        {
            TimePanel t = (TimePanel) comp;
            TimeAxisIF data = t.getData();
            sb.append(JulianGregorianDate.toGregorian(t.pixelToJulian(x, data.getCurrentMin(), data.getCurrentMax()), "Z"));
        }
        else
        {
            sb.append(x).append(" & ").append(y);
        }
        setLabel(sb.toString());
        int sw = SwingUtilities.computeStringWidth(getFontMetrics(getFont()), sb.toString());
        if (comp.getWidth() < x + 1 + sw)
        {
            x = x - 1 - sw;
        }
        else
        {
            x = x + 1;
        }
        setBounds(x, y, sw, getFont().getSize());
    }

    private int getY(MouseEvent e, Component comp)
    {
        // int y = 0;
        int y = e.getY() - getFont().getSize();
        if (!(comp instanceof JLayeredPane))
        {
            y = y + comp.getY();
        }
        return y;
    }

    /**
     * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
     */
    public void mouseDragged(MouseEvent e)
    {
        // setBounds(0, 0, 0, 0);
        setPosition(e);
    }

    /**
     * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
     */
    public void mouseClicked(MouseEvent e)
    {
    }

    /**
     * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
     */
    public void mouseEntered(MouseEvent e)
    {
    }

    /**
     * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
     */
    public void mouseExited(MouseEvent e)
    {
        setBounds(0, 0, 0, 0);
    }

    /**
     * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
     */
    public void mousePressed(MouseEvent e)
    {
    }

    /**
     * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
     */
    public void mouseReleased(MouseEvent e)
    {
    }
}
