package jp.nihu.gttime.layer;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.html.HTMLEditorKit;

import jp.nihu.gttime.T2MapDialog;
import jp.nihu.gttime.data.DatasetGtm;
import jp.nihu.gttime.plugin.LayerPlugin;
import jp.nihu.gttime.utility.XMLUtil;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class GtmMetadataDialog extends T2MapDialog {
	private static final long serialVersionUID = 1L;

	private static final Log log = LogFactory.getLog(HighLighter.class);

	private LayerPlugin layer = null;

	public GtmMetadataDialog(Frame owner, boolean modal, LayerPlugin layer) {
		super(owner, modal);
		this.layer = layer;
		setBackground(Color.white);
		setSize(new java.awt.Dimension(350, 250));
		
		// ダイアログの表示位置を設定。
        Toolkit tk = getToolkit();
        Dimension screenSize = tk.getScreenSize();
        int posX = (screenSize.width / 2) - (getWidth() / 2);
        int posY = (screenSize.height / 2) - (getHeight() / 2);

        // BaseLayerPluginPreferences base = new BaseLayerPluginPreferences();
        // tabbedPane.add(base);

        setLocation(posX, posY);
	}

	public void showDialog() {
		Font f = new Font(layer.getProperties().getDetailFontName(), Font.PLAIN, 12);
		setFont(f);

		JTextPane textPane = new JTextPane();
		textPane.putClientProperty(JEditorPane.HONOR_DISPLAY_PROPERTIES, Boolean.TRUE);
		textPane.setFont(f);

		textPane.setEditorKit(new HTMLEditorKit());

		DatasetGtm dataset = (DatasetGtm) layer.getDataset();
		
		setTitle(dataset.getTitle());
		
		Element metadataRoot = dataset.getMetadataNode();
		
		StringBuilder html = new StringBuilder();
		html.append("<html><body>");
		html.append("<h2>").append(dataset.getTitle()).append("</h2>");
		NodeList nl = metadataRoot.getChildNodes();
		for (int i = 0; i < nl.getLength(); i++) {
			Node n = nl.item(i);
			if (n.getNodeType() == Node.ELEMENT_NODE) {
				Element e = (Element)n;
				String c = XMLUtil.getElementContentsAsString(e).trim();
				if (c != null && 0 < c.length()) {
					html.append("<li>");
					html.append(e.getNodeName());
					html.append("）");
					html.append(c);
					html.append("</li>");
				}
			} else if (n.getNodeType() == Node.TEXT_NODE && 0 < n.getNodeValue().trim().length()) {
//				html.append("<div style=\"margin-top:2px;margin-bottom:2px;\">").append(n.getNodeValue().trim()).append("</div>");
			}
		}
		html.append("</body></html>");
		
		log.debug(html.toString());

		textPane.setText(html.toString());
		textPane.setCaretPosition(0);// スクロールを先頭にするためにカーソルの位置を先頭に設定する。kind of a stupid code.
		textPane.setEditable(false);
		textPane.addHyperlinkListener(new HyperlinkListener() {
			public void hyperlinkUpdate(HyperlinkEvent e) {
				if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
					String os = System.getProperty("os.name");
					try {
						{
							if (layer.getBrowser() != null) {
								Runtime.getRuntime().exec(new String[] { layer.getBrowser(), e.getURL().toString() });
							} else {
								log.error("your os is not supported now : " + os + " : " + layer.getBrowser());
							}
						}
					} catch (Exception ex) {
						try {
							log.info(layer.getBrowser() + "の起動に失敗しました。");
							if (os.startsWith("Windows 98")) {
								Runtime.getRuntime().exec("start " + e.getURL());
							} else if (os.startsWith("Windows")) {
								Runtime.getRuntime().exec("cmd /c \"start " + e.getURL() + "\"");
							} else {
								log.fatal("your os is not supported now : " + os + " : " + layer.getBrowser() + "\n" + ex.getMessage(), ex);
							}
						} catch (Exception exx) {
							log.fatal("your os is not supported now : " + os + " : " + layer.getBrowser() + "\n" + exx.getMessage(), exx);
						}
					}
				}
			}
		});

		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
			public void windowClosed(WindowEvent e) {
				layer.setForcePaint();
				layer.repaint();
			}
		});

		JScrollPane scrPane = new JScrollPane();
		add(scrPane, BorderLayout.CENTER);
		scrPane.getViewport().setView(textPane);

		// setLocation(x, y);
		setVisible(true);
	}
}
