/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.layer;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.SwingUtilities;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.html.HTMLEditorKit;

import jp.nihu.gttime.T2MapDialog;
import jp.nihu.gttime.plugin.LayerPlugin;
import jp.nihu.gttime.plugin.LayerPlugin.RecordPlot;
import jp.nihu.gttime.plugin.layer.DefaultLayerPlugin.DefaultRecordPlot;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class HighLighter extends JComponent implements MouseMotionListener, MouseListener
{
    /**
     * シリアルバージョンID
     */
    private static final long serialVersionUID = 1L;

    /**
     * ログ用オブジェクト
     */
    private static final Log log = LogFactory.getLog(HighLighter.class);

    private LayerPlugin layer = null;

    private ArrayList<RecordPlot> shapes = null;

    private Image highLightImage = null;

    protected Color highlightColor = null;

    private int ajustY = 0;

    /**
     * コンストラクタ
     */
    public HighLighter()
    {
        super();
        setOpaque(false);
    }

    /**
     * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
     */
    @Override
    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        if (shapes != null)
        {
            Iterator<RecordPlot> it = shapes.iterator();
            while (it.hasNext())
            {
                RecordPlot shape = it.next();
                Rectangle rect = shape.getBounds();
                if (highLightImage != null)
                {
                    g.drawImage(highLightImage, rect.x, rect.y + ajustY, this);
                }
                else
                {
                    g.setColor(highlightColor);
                    
                    if (shape instanceof DefaultRecordPlot && ((DefaultRecordPlot)shape).isCircle()) {
                    	int plotSize = 6;
                        g.fillOval(rect.x, rect.y + ajustY, plotSize, plotSize);
                        g.drawOval(rect.x, rect.y + ajustY, plotSize, plotSize);
                    } else {
                        if (shape.jdStart == null && shape.jdEnd == null)
                        {
                            ((Graphics2D) g).setPaint(new GradientPaint(0, 0, getBackground(), getWidth() / 2, 0, g.getColor(), true));
                        }
                        else if (shape.jdStart == null)
                        {
                            ((Graphics2D) g).setPaint(new GradientPaint(0, 0, getBackground(), getWidth() / 4, 0, g.getColor()));
                        }
                        else if (shape.jdEnd == null)
                        {
                            ((Graphics2D) g).setPaint(new GradientPaint(getWidth() / 4 * 3, 0, g.getColor(), getWidth(), 0, getBackground()));
                        }
                        g.fillRect(rect.x, rect.y + ajustY, rect.width, rect.height);
                        if (shape.jdStart != null && shape.jdEnd != null)
                        {
                            g.drawRect(rect.x, rect.y + ajustY, rect.width, rect.height);
                        }
                    }
                }
            }
        }
    }

    /**
     * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
     */
    public void mouseDragged(MouseEvent e)
    {
        shapes = null;
        highLightImage = null;
    }

    /**
     * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
     */
    public void mouseMoved(MouseEvent e)
    {
        shapes = null;
        highLightImage = null;
        Component comp = (Component) e.getSource();
        // if (comp instanceof DataPanel)
        // {
        // DataPanel dp = (DataPanel) comp;
        // ajustY = comp.getY();
        // shapes = dp.getMouseOveredRecords();
        // highLightImage = dp.getHighLightImage();
        // }
        if (comp instanceof LayerPlugin)
        {
            layer = (LayerPlugin) comp;
            ajustY = comp.getY();
            shapes = layer.getMouseOveredRecords();
            highLightImage = layer.getHighLightImage();
            highlightColor = layer.getHighLightColor();
        }
        repaint();
    }

    /**
     * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
     */
    public void mouseClicked(MouseEvent e)
    {
    }

    /**
     * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
     */
    public void mouseEntered(MouseEvent e)
    {
    }

    /**
     * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
     */
    public void mouseExited(MouseEvent e)
    {
        shapes = null;
        highLightImage = null;
        repaint();
    }

    /**
     * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
     */
    public void mousePressed(MouseEvent e)
    {
    }

    /**
     * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
     */
    public void mouseReleased(MouseEvent e)
    {
        if (SwingUtilities.isLeftMouseButton(e))
        {
            if (shapes != null)
            {
                Iterator<RecordPlot> it = shapes.iterator();
                while (it.hasNext())
                {
                    final RecordPlot shape = it.next();
                    if (!shape.isClicked())
                    {
                        DetailDialog dialog = new DetailDialog(shape.getT2Map(), false, layer);

                        // ダイアログの表示位置を設定。
                        int curPanelX = (int) getLocationOnScreen().getX();
                        int curPanelY = (int) getLocationOnScreen().getY();

                        Dimension screenSize = getToolkit().getScreenSize();
                        int xForDialog = curPanelX + layer.getX() + e.getX() - dialog.getWidth();
                        if (xForDialog < 0)
                        {
                            xForDialog = 0;
                        }
                        else if (xForDialog > screenSize.getWidth())
                        {
                            xForDialog = (int) screenSize.getWidth() - dialog.getWidth();
                        }
                        int yForDialog = curPanelY + layer.getY() + e.getY();
                        if (yForDialog < 0)
                        {
                            yForDialog = 0;
                        }
                        else if (yForDialog + dialog.getHeight() > screenSize.getHeight())
                        {
                            yForDialog = (int) screenSize.getHeight() - dialog.getHeight();
                        }
                        dialog.showDialog(xForDialog, yForDialog, shape);
                    }
                }
                // layer.repaint();
            }
        }
        e.consume();
    }

    private class DetailDialog extends T2MapDialog
    {
        private static final long serialVersionUID = 1L;

        private LayerPlugin layerx = null;

        public DetailDialog(Frame owner, boolean modal, LayerPlugin layer)
        {
            super(owner, modal);
            layerx = layer;
            setBackground(Color.white);
            setSize(new java.awt.Dimension(350, 250));
        }

        public void showDialog(int x, int y, final RecordPlot shape)
        {
            Font f = new Font(layer.getProperties().getDetailFontName(), Font.PLAIN, 12);
//            if (f == null) {
//                f = new Font(Font.DIALOG, Font.PLAIN, 12);
//            }
            setFont(f);
            JTextPane textPane = new JTextPane();
            textPane.putClientProperty(JEditorPane.HONOR_DISPLAY_PROPERTIES, Boolean.TRUE);
            textPane.setFont(f);

            textPane.setEditorKit(new HTMLEditorKit());
            textPane.setText(shape.getHtml());
            textPane.setCaretPosition(0);// スクロールを先頭にするためにカーソルの位置を先頭に設定する。kind
            // of a stupid code.
            textPane.setEditable(false);
            textPane.addHyperlinkListener(new HyperlinkListener()
            {
                public void hyperlinkUpdate(HyperlinkEvent e)
                {
                    if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
                    {
                        String os = System.getProperty("os.name");
                        try
                        {
                            {
                                if (shape.getBrowser() != null)
                                {
                                    Runtime.getRuntime().exec(new String[]{shape.getBrowser(), e.getURL().toString()});
                                }
                                else
                                {
                                    log.error("your os is not supported now : " + os + " : " + shape.getBrowser());
                                }
                            }
                        }
                        catch (Exception ex)
                        {
                            try
                            {
                            	log.info(shape.getBrowser() + "の起動に失敗しました。");
                                if (os.startsWith("Windows 98"))
                                {
                                    Runtime.getRuntime().exec("start " + e.getURL());
                                }
                                else if (os.startsWith("Windows"))
                                {
                                    Runtime.getRuntime().exec("cmd /c \"start " + e.getURL() + "\"");
                                }
                                else
                                {
                                    log.fatal("your os is not supported now : " + os + " : " + shape.getBrowser() + "\n" + ex.getMessage(), ex);
                                }
                            }
                            catch (Exception exx)
                            {
                                log.fatal("your os is not supported now : " + os + " : " + shape.getBrowser() + "\n" + exx.getMessage(), exx);
                            }
                        }
                    }
                }
            });

            setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
            addWindowListener(new WindowAdapter()
            {
                public void windowClosed(WindowEvent e)
                {
                    shape.setClicked(false);
                    layerx.setForcePaint();
                    layerx.repaint();
                }
            });

            if (shape.title != null)
            {
                setTitle(shape.title.replaceAll("\n", ""));
            }
            layer.setForcePaint();
            shape.setClicked(true);
            JPanel c = new JPanel();
            c.setBackground(shape.highlightColorx);
            add(c, BorderLayout.NORTH);

            JScrollPane scrPane = new JScrollPane();
            add(scrPane, BorderLayout.CENTER);
            scrPane.getViewport().setView(textPane);

            setLocation(x, y);
            setVisible(true);
        }
    }
}
