/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.layer;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Hashtable;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.T2MapMenuBar;
import jp.nihu.gttime.data.DatasetException;
import jp.nihu.gttime.data.DatasetGtm;
import jp.nihu.gttime.data.DatasetIF;
import jp.nihu.gttime.image.ImageExporter;
import jp.nihu.gttime.plugin.ExportPlugin;
import jp.nihu.gttime.plugin.LayerPlugin;
import jp.nihu.gttime.plugin.PluginManager;
import jp.nihu.gttime.plugin.PluginManagerException;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.plugin.SavePlugin;
import jp.nihu.gttime.plugin.TimeMeasureDialog;
import jp.nihu.gttime.preferences.LayerPreferencesDialog;
import jp.nihu.gttime.rdf.RDFSave;
import jp.nihu.gttime.time.TimeAxisIF;
import jp.nihu.gttime.time.TimeLineData;
import jp.nihu.gttime.tree.TreeCheckNode;
import jp.nihu.gttime.tree.TreeDragDrop;
import jp.nihu.gttime.utility.ExtensionFileFilter;
import jp.nihu.gttime.utility.ExtensionFileSaveChooser;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.semarglproject.rdf.ParseException;

public class LayerPopupMenu extends JPopupMenu
{
    private static final long serialVersionUID = 1L;

    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(TreeDragDrop.class);
    
    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(LayerPopupMenu.class, T2Map.LOCALE);
    private static PluginResourceManager rmgrmb = PluginResourceManager.getManager(T2MapMenuBar.class, T2Map.LOCALE);

    private LayerPlugin layer = null;
    
    /**
     * PluginManagerオブジェクト
     */
    private PluginManager pluginManager = null;
    
    /**
     * T2Mapオブジェクト
     */
//    private T2Map t2map = null;

    public LayerPopupMenu(LayerPlugin layer)
    {
        this.layer = layer;
        this.pluginManager = T2Map.getPluginManager();
//        this.t2map = layer.getT2Map();
//        log.debug("t2map " + t2map);
    }
    
//    public void addDefaultMenus() {
//    	createDrawTimeAxisMenu();
//        createExportMenu();
//        createMeasureMenu();
//    }
    
    public void createZoomLayerMenu() {
    	JMenuItem item = new JMenuItem(rmgr.getString("menu.zoom.layer"));
        item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				try {
					//log.debug("レイヤ全体を表示");
					DatasetIF d = layer.getDataset();
					BigDecimal sd = d.getStartDatetime();
					BigDecimal ed = d.getEndDatetime();
					TimeAxisIF data = layer.getT2Map().getTimeAxisData();
					data.setCurrentMin(sd);
	                data.setCurrentMax(ed);
	                data.fireValueChanged();
				} catch (DatasetException e) {
					log.error(e.getMessage(), e);
				}
			}
		});
        add(item);//"【将来拡張】レイヤ全体を表示"
    }
    
    private JMenuItem getExportMenuItem(String title, String type) {
    	ArrayList<Hashtable<String, String>> plugins = pluginManager.getPlugins(type);
        
        final ArrayList<ExportPlugin> ps = new ArrayList<ExportPlugin>();
        for (int i = 0; i < plugins.size(); i++) {
        	try {
        		ExportPlugin plugin = (ExportPlugin) pluginManager.newInstance(plugins.get(i).get("class"));
        		plugin.init(layer.getT2Map(), pluginManager);
        		ps.add(plugin);
			} catch (PluginManagerException e1) {
				layer.getT2Map().procError(e1);
			}
        }
    	
        JMenuItem item0 = new JMenuItem(title);
        item0.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent e)
            {
            	DatasetIF dataset = layer.getDataset();
                String currentDirPath = T2Map.properties.getProperty("jp.nihu.gttime.T2Map.CURRENT_DIR");
                //String path = currentDirPath + File.separator + dataset.getPath().substring(0, dataset.getPath().length() - 4);
                String path = currentDirPath + File.separator + dataset.getTitle();
                final ExtensionFileSaveChooser chooser = new ExtensionFileSaveChooser(path);
                for (int i = 0; i < ps.size(); i++)
                {
                	ExportPlugin plugin = ps.get(i);
					chooser.setFileFilter(plugin.getExtensionFileFilter());
					JPanel panel = plugin.getJPanel();
					if (panel != null) {
						chooser.setAccessory(panel);
					} else {
						chooser.setAccessory(new JPanel());
					}
                }
                chooser.addPropertyChangeListener(new PropertyChangeListener() {
					
					public void propertyChange(PropertyChangeEvent evt) {
						String prop = evt.getPropertyName();
						if (JFileChooser.FILE_FILTER_CHANGED_PROPERTY.equals(prop)) {
							Object o = evt.getNewValue();
							if (o != null) {
								if (o instanceof ExtensionFileFilter) {
									ExtensionFileFilter f = (ExtensionFileFilter)evt.getNewValue();
									for (int i = 0; i < ps.size(); i++)
					                {
										ExportPlugin plugin = ps.get(i);
										if (plugin.getExtensionFileFilter().getDescription().equals(f.getDescription())) {
											JPanel panel = plugin.getJPanel();
											if (panel != null) {
												chooser.setAccessory(panel);
											} else {
												chooser.setAccessory(new JPanel());
											}
											chooser.getAccessory().updateUI();
										}
					                }
//								} else  {
//									chooser.setAccessory(new JPanel());
//									chooser.updateUI();
								}
							}
						}
					}
				});
                
                int returnVal = chooser.showSaveDialog(layer.getT2Map());
                if (returnVal == JFileChooser.APPROVE_OPTION)
                {
                	String ext = chooser.getFileFilter().getDescription();
                	for (int i = 0; i < ps.size(); i++)
                    {
                		if (ps.get(i).getExtensionFileFilter().getDescription().equals(ext)) {
            				ExportPlugin plugin = ps.get(i);
                            plugin.init(layer.getT2Map(), pluginManager);
                            plugin.save(dataset, chooser.getSelectedFile());
                			break;
                		}
                    }
                }
                // キャンセルボタンが押された時
                else
                {
                    T2Map.properties.setProperty("jp.nihu.gttime.T2Map.CURRENT_DIR", chooser.getCurrentDirectory().toString());
                }
                UIManager.put("FileChooser.readOnly", Boolean.FALSE);
            }
        });
    	return item0;
    }
    
    /**
     * ポップアップメニューについて初期化します。<br/>
     * <ul>
     * <li>「GTM,XML出力」をポップアップメニューとして生成します</li>
     * </ul>
     */
    public void createSaveExportMenu()
    {
        add(getExportMenuItem(rmgr.getString("data.save.export"), "ExportSavePlugin"));
    }
    
    public void createSaveMenu()
    {
    	JMenuItem item = new JMenuItem(rmgrmb.getString("menu.file.save"));
    	ArrayList<Hashtable<String, String>> plugins = pluginManager.getPlugins("SavePlugin");
    	final String clazz = plugins.get(0).get("class");
    	item.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent e)
            {
            	log.debug(clazz);
                try
                {
                    SavePlugin plugin = (SavePlugin) pluginManager.newInstance(clazz);
                    plugin.save(layer.getT2Map(), layer.getDataset());
                }
                catch (PluginManagerException ex)
                {
                	log.error(ex.getMessage(),ex);
                }
            }
        });
    	add(item);
    }
    
    
    private boolean printFullTimerange = true;
    
    private LayerPlugin getPrintLayer() {
    	LayerPlugin player = null;
    	
    	if (!printFullTimerange) {
    		player = layer;
    	} else {
        	String type = layer.getPluginName();
            PluginManager pm = T2Map.getPluginManager();
            Hashtable<?, ?> h = pm.getPlugins("LayerPlugin", type);
            if (h != null)
            {
                try
                {
                    player = (LayerPlugin) pm.newInstance(h.get("class").toString());
                }
                catch (PluginManagerException ex)
                {
                	log.error(ex.getMessage(), ex);
                }
            }
            
            if (player != null)
            {
                // Create new instance.
                // parameters :dataset, allow child = false, enable = true
                TreeCheckNode node = new TreeCheckNode(layer.getDataset(), false);
                // node.setSelected(true);
                // Set draw scale.
                node.setDrawTimeAxis(layer.getTreeCheckNode().isDrawTimeAxis());
                // Set image size.
                // node.setHeight(d.getSize( ).height);
                layer.getDataset().getProperties().setLayerHeight(layer.getSize().height);
                // Setting for layer.
                player.setSize(layer.getSize());
                
                TimeAxisIF axis = new TimeLineData();
                try {
    				BigDecimal from = layer.getDataset().getStartDatetime();
    				BigDecimal to = layer.getDataset().getEndDatetime();
    				axis.setApplicationMin(from);
                    axis.setApplicationMax(to);
                    axis.setMin(from);
                    axis.setMax(to);
                    axis.setCurrentMin(from);
                    axis.setCurrentMax(to);
    			} catch (DatasetException e1) {
    				log.error(e1.getMessage(), e1);
    			}
                player.initForPrint(axis, node);
            }
            else
            {
                log.error("Can't get LayerPlugin object.");
            }
    	}
    	return player;
    }
    
    /**
     * ポップアップメニューについて初期化します。<br/>
     * <ul>
     * <li>「画像出力」・「CSV出力」・「KML出力」・「クリップボードにコピー」をポップアップメニューとして生成します</li>
     * </ul>
     */
    public void createExportMenu()
    {
    	JMenu outMenu = new JMenu(rmgr.getString("data.export"));
    	add(outMenu);
    	
    	outMenu.add(getExportMenuItem(rmgr.getString("data.output"), "ExportPlugin"));
    	
        // final DatasetIF dataset = layer.getDataset();
        // 画像出力
        JMenuItem item2 = new JMenuItem(rmgr.getString("image.output"));
        item2.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                // layer.paintComponent(layer.getGraphics());
                String title = layer.getDataset().getTitle();
                if (title == null)
                {
                    title = "layer";
                }
                LayerPlugin player = getPrintLayer();
                ImageExporter exp = new ImageExporter(player, title);
                exp.imageSave();
            }
        });
        outMenu.add(item2);

//        // CSV出力
//        JMenuItem csvSave = new JMenuItem(rmgr.getString("csv.output"));
//        csvSave.addActionListener(new ActionListener()
//        {
//            public void actionPerformed(ActionEvent e)
//            {
//                new CSVSave(layer.getDataset(), (T2Map) SwingUtilities.getRoot(getParent()));
//            }
//        });
//        add(csvSave);
//        
//        // GTM,XML出力
//        JMenuItem gtmSave = new JMenuItem(rmgr.getString("gtmxml.output"));
//        gtmSave.addActionListener(new ActionListener()
//        {
//            public void actionPerformed(ActionEvent e)
//            {
//                new GTMXMLSave(layer.getDataset(), (T2Map) SwingUtilities.getRoot(getParent()));
//            }
//        });
//        add(gtmSave);
//        
//        // KML出力
//        JMenuItem kmlSave = new JMenuItem(rmgr.getString("kml.output"));
//        kmlSave.addActionListener(new ActionListener()
//        {
//            public void actionPerformed(ActionEvent e)
//            {
//                new KMLSave(layer.getDataset(), (T2Map) SwingUtilities.getRoot(getParent()));
//            }
//        });
//        add(kmlSave);

        // クリップボードにコピー
        JMenuItem item3 = new JMenuItem(rmgr.getString("clipboard.copy"));
        item3.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
            	LayerPlugin player = getPrintLayer();
//                Graphics g = player.getGraphics();
//                player.paintComponent(g);
                // JComponent parent = layer;
                BufferedImage image = new BufferedImage(player.getWidth(), player.getHeight(), BufferedImage.TYPE_INT_BGR);
                Graphics graphics = image.createGraphics();
                player.paint(graphics);
                ImageSelection ims = new ImageSelection(image);
                Toolkit.getDefaultToolkit().getSystemClipboard().setContents(ims, (T2Map) SwingUtilities.getRoot(getParent()));
            }
        });
        outMenu.add(item3);
        

        // RDF出力
        JMenuItem item4 = new JMenuItem(rmgr.getString("rdf.output"));
        item4.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
            	try {
//					RDFDataMgr.write(System.out, layer.getDataset().getModel(), Lang.TURTLE);
            		RDFSave rdf = new RDFSave(layer.getT2Map());
            		rdf.save(layer.getDataset().getModel(), layer.getDataset().getTitle());
				} catch (ParseException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				};
            }
        });
        outMenu.add(item4);
    }

    public Icon getCheckIcon()
    {
        return new ImageIcon(getClass().getClassLoader().getResource("CheckIcon.jpg"));
    }

    /**
     * 「暦表示」、「暦非表示」をポップアップメニューとして生成します
     */
    public void createDrawTimeAxisMenu()
    {
        final JMenuItem item = new JMenuItem();
        final TreeCheckNode node = layer.getTreeCheckNode();
        if (node.isDrawTimeAxis())
        {
            item.setText(rmgr.getString("calendar.display"));
            item.setIcon(getCheckIcon());
        }
        else
        {
            item.setText(rmgr.getString("calendar.display"));
            item.setIcon(null);
        }
        item.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                if (node.isDrawTimeAxis())
                {
                    item.setText(rmgr.getString("calendar.display"));
                    item.setIcon(null);
                    node.setDrawTimeAxis(false);
                }
                else
                {
                    item.setText(rmgr.getString("calendar.display"));
                    item.setIcon(getCheckIcon());
                    node.setDrawTimeAxis(true);
                }
                layer.updateUI();// .paintComponent(layer.getGraphics());
            }
        });
        add(item);
    }

    /**
     * 表示スタイル設定のメニュー生成します
     */
    public void createPropertiesMenu()
    {
        final JMenuItem item = new JMenuItem();
        item.setText(rmgr.getString("layer.preferences"));
        item.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                LayerPreferencesDialog pref = new LayerPreferencesDialog(layer);
                pref.setVisible(true);
                layer.updateUI();
            }
        });
        add(item);
    }
    
    public void createShowMetadaMenu()
    {
    	final JMenuItem item = new JMenuItem();
        item.setText(rmgr.getString("layer.metadata"));
        item.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
//                LayerPreferencesDialog pref = new LayerPreferencesDialog(layer);
//                pref.setVisible(true);
//                layer.updateUI();
            	DatasetIF d = layer.getDataset();
            	if (d instanceof DatasetGtm) {
        	        DatasetGtm gtm = (DatasetGtm)d;
        	        log.debug("GTM Version : " + gtm.getGtmVersion());
        	        GtmMetadataDialog dialog = new GtmMetadataDialog(layer.getT2Map(), false, layer);
//        	        dialog.setVisible(true);
        	        dialog.showDialog();
        	        layer.updateUI();
                }
            }
        });
        add(item);
    }
    
    public boolean fromTree = false;

    /**
     * 計測メニューを管理するメッソドです。
     * 
     * @param menu JPopupMenuオブジェクト
     * @return 計測メニューのオブジェクト
     */
    public void createMeasureMenu()
    {
        measureMenuItem = new JMenuItem(rmgr.getString("time.axis.measure"));
        add(measureMenuItem);
        measureMenuItem.setText(rmgr.getString("time.axis.measure"));
        measureMenuItem.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                TimeDurationSelector timeDurationSelector = layer.getTimeDurationSelector();
                BigDecimal[][] duls = timeDurationSelector.getDurations();
                
                // 時間幅測定機能の表示
                if (duls != null && duls.length > 0)
                {
//                	if (se != null)
                	if (!fromTree)
                    {
                		BigDecimal[] se = timeDurationSelector.setFocus(layer.popupPosX);
                    	timeDurationSelector.repaint();
                        BigDecimal all = null;
//                        BigDecimal[][] duls = timeDurationSelector.getDurations();
                        if (duls.length > 1)
                        {
                            for (int i = 0; i < duls.length; i++)
                            {
                                if (all == null)
                                {
                                    all = new BigDecimal(0);
                                }
                                all = all.add(duls[i][1].subtract(duls[i][0]));
                            }
                        }
                        new TimeMeasureDialog((T2Map) SwingUtilities.getRoot(getParent()), true, se[1].subtract(se[0]), all);
                        timeDurationSelector.clearFocus();
                    } else {
                    	timeDurationSelector.repaint();
                    	BigDecimal all = null;
//                        BigDecimal[][] duls = timeDurationSelector.getDurations();
                        if (duls.length > 0)
                        {
                            for (int i = 0; i < duls.length; i++)
                            {
                                if (all == null)
                                {
                                    all = new BigDecimal(0);
                                }
                                all = all.add(duls[i][1].subtract(duls[i][0]));
                            }
                        }
                        new TimeMeasureDialog((T2Map) SwingUtilities.getRoot(getParent()), true, all);
                        timeDurationSelector.clearFocus();
                    }
                }
            }
        });
    }

    private JMenuItem measureMenuItem = null;

    public void setVisibleMeasureMenuItem(boolean visible)
    {
        measureMenuItem.setVisible(visible);
    }

    /**
     * 画像出力を管理するメソッドです。
     */
    static class ImageSelection implements Transferable
    {
        /**
         * グラフィカルイメージオブジェクト
         */
        private Image image;

        /**
         * グラフィカルイメージを設定します。
         * 
         * @param image グラフィカルイメージ
         */
        public ImageSelection(Image image)
        {
            this.image = image;
        }

        /**
         * DataFlavorオブジェクトを作成します。
         * 
         * @return DataFlavor オブジェクト
         */
        public DataFlavor[] getTransferDataFlavors()
        {
            return new DataFlavor[] { DataFlavor.imageFlavor };
        }

        /**
         * Java Image クラスを表現することが出来るか。
         * 
         * @return 可能、不可能
         */
        public boolean isDataFlavorSupported(DataFlavor flavor)
        {
            return DataFlavor.imageFlavor.equals(flavor);
        }

        /**
         * グラフィカルイメージを取得します。
         * 
         * @throws UnsupportedFlavorException
         * @throws IOException
         */
        public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException, IOException
        {
            if (!DataFlavor.imageFlavor.equals(flavor))
            {
                throw new UnsupportedFlavorException(flavor);
            }
            return image;
        }
    }
}
