/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.layer;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.ParseException;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.JFormattedTextField.AbstractFormatter;
import javax.swing.text.DefaultFormatterFactory;
import javax.swing.text.InternationalFormatter;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.T2MapDialog;
import jp.nihu.gttime.plugin.PluginResourceManager;

public class ResizeHeightDialog extends T2MapDialog
{
    private static final long serialVersionUID = 1L;

    private static PluginResourceManager rmgr = PluginResourceManager.getManager(ResizeHeightDialog.class, T2Map.LOCALE);

    private JSpinner sp1;

    private int currentValue = -1;

    public ResizeHeightDialog(Frame owner, boolean modal, int currentHeight, int minHeight)
    {
        super(owner, modal);
        currentValue = currentHeight;
        setResizable(false);
        setSize(new Dimension(200, 100));
        setTitle(rmgr.getString("dialog.title"));
        setLocationRelativeTo(null);
        init(currentHeight, minHeight);
    }

    private void init(int current, int min)
    {
        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

        JPanel spanel = new JPanel();
        spanel.setLayout(new GridBagLayout());
        GridBagConstraints gridBagConstraints0 = new GridBagConstraints();
        gridBagConstraints0.gridx = 0;
        gridBagConstraints0.gridy = 0;
        gridBagConstraints0.anchor = GridBagConstraints.WEST;
        GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.anchor = GridBagConstraints.WEST;

        sp1 = new JSpinner(new SpinnerNumberModel(current, min, 1000, 1));
        sp1.setEditor(new SpinnerEditor2(sp1, "0"));

        spanel.add(sp1, gridBagConstraints0);

        JLabel unit = new JLabel(" px");
        spanel.add(unit, gridBagConstraints1);

        panel.add(spanel);

        JPanel bpanel = new JPanel();
        bpanel.setLayout(new BoxLayout(bpanel, BoxLayout.X_AXIS));

        JButton okButton = new JButton(rmgr.getString("ok.button"));
        okButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent ae)
            {
                dispose();
            }
        });
        okButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        bpanel.add(okButton);

        JButton closeButton = new JButton(rmgr.getString("cancel.button"));
        closeButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent ae)
            {
                sp1.setValue(Integer.valueOf(currentValue));
                dispose();
            }
        });
        closeButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        bpanel.add(closeButton);

        panel.add(bpanel);

        add(panel);

        addWindowFocusListener(new WindowFocusListener()
        {
            public void windowLostFocus(WindowEvent arg0)
            {
                sp1.setValue(Integer.valueOf(currentValue));
                dispose();
            }

            public void windowGainedFocus(WindowEvent e)
            {
            }
        });
    }

    public int getChangedHeight()
    {
        return ((Integer) sp1.getValue()).intValue();
    }

    /** 自作NumberEditorFormatterクラス */
    private static class MyNumberEditorFormatter extends InternationalFormatter
    {
        /**
         * 
         */
        private static final long serialVersionUID = 1L;

        private final SpinnerNumberModel model;

        MyNumberEditorFormatter(SpinnerNumberModel model, NumberFormat format)
        {
            super(format);
            this.model = model;
            setValueClass(model.getValue().getClass());
        }

        public Comparable getMaximum()
        {
            return model.getMaximum();
        }

        public Comparable getMinimum()
        {
            return model.getMinimum();
        }

        public void setMaximum(Comparable max)
        {
            model.setMaximum(max);
        }

        public void setMinimum(Comparable min)
        {
            model.setMinimum(min);
        }

        /**
         * 文字列→値変換メソッドをオーバーライドする。 範囲外の時は、値を最大値、または最小値に修正する。
         * 
         * @see javax.swing.text.InternationalFormatter#stringToValue(java.lang.String)
         */
        public Object stringToValue(String text) throws ParseException
        {
            try
            {
                return super.stringToValue(text);
            }
            catch (ParseException e)
            {
                // 範囲外の場合、ParseExceptionが投げられるのでキャッチする。

                Object object = getFormat().parseObject(text);

                Integer d;
                if (object instanceof Integer)
                {
                    d = (Integer) object;
                }
                else if (object instanceof Number)
                {
                    d = new Integer(((Number) object).intValue());
                }
                else
                {
                    throw e;
                }

                if (getMaximum().compareTo(d) < 0)
                {
                    // 最大を超えている場合は、最大値を返す。
                    return getMaximum();
                }
                else if (getMinimum().compareTo(d) > 0)
                {
                    // 最小に満たない場合は、最小値を返す。
                    return getMinimum();
                }
                else
                {
                    throw e;
                }
            }
        }
    }

    private static class SpinnerEditor2 extends JSpinner.NumberEditor
    {
        /**
         * 
         */
        private static final long serialVersionUID = 1L;

        /**
         * コンストラクタ
         * 
         * @param spinner
         * @param decimalFormatPattern
         */
        public SpinnerEditor2(JSpinner spinner, String decimalFormatPattern)
        {
            super(spinner, decimalFormatPattern);

            // Formatterを自作のものに変更する。
            MyNumberEditorFormatter formatter = new MyNumberEditorFormatter((SpinnerNumberModel) spinner.getModel(), new DecimalFormat(decimalFormatPattern));

            DefaultFormatterFactory factory = new DefaultFormatterFactory(formatter);

            getTextField().setFormatterFactory(factory);
        }

        /*
         * (non-Javadoc) override
         * 
         * @see javax.swing.JSpinner.DefaultEditor#commitEdit()
         */
        public void commitEdit() throws ParseException
        {
            Object old = getModel().getValue();
            String newStr = getTextField().getText();

            try
            {
                AbstractFormatter formatter = getTextField().getFormatter();
                String oldStr = formatter.valueToString(old);

                // 頭の部分が変わってなかったら、変更しない
                // if (newStr.startsWith(oldStr))
                // {
                //                    
                // return;
                // }

                if (!newStr.matches("^[0-9]+"))
                {
                    getTextField().setText(oldStr);
                    return;
                }
            }
            catch (ParseException e)
            {
                // 起こらないハズ
                e.printStackTrace();
            }
            // 変更を確定させる。
            super.commitEdit();
        }
    }

}
