/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.layer;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.SwingUtilities;

import jp.nihu.gttime.TimePanel;
import jp.nihu.gttime.plugin.LayerPlugin;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class TimeDurationSelector extends TimePanel implements MouseMotionListener, MouseListener
{
    /**
     * シリアルバージョンID
     */
    private static final long serialVersionUID = 1L;

    private static final Log log = LogFactory.getLog(TimeDurationSelector.class);

    private int startDraggingPosition = 0;

    private List<TimeDuration> durations = new ArrayList<TimeDuration>();

    private BigDecimal start = null;

    private BigDecimal end = null;

    /**
     * コンストラクタ
     */
    public TimeDurationSelector()
    {
        super();
        setOpaque(false);
    }

    public BigDecimal[][] getDurations()
    {
        BigDecimal[][] duls = new BigDecimal[durations.size()][2];
        Iterator<TimeDuration> it = durations.iterator();
        int pos = 0;
        while (it.hasNext())
        {
            TimeDuration td = it.next();
            duls[pos][0] = td.getStart();
            duls[pos][1] = td.getEnd();
            pos++;
        }
        return duls;
    }

    public BigDecimal[] setFocus(int x)
    {
        return setFocus(pixelToJulian(x, data.getCurrentMin(), data.getCurrentMax()));
    }

    public BigDecimal[] setFocus(BigDecimal x)
    {
        BigDecimal s = null;
        BigDecimal e = null;
        Iterator<TimeDuration> it = durations.iterator();
        while (it.hasNext())
        {
            TimeDuration dur = it.next();
            if (dur.isContain(x))
            {
                dur.setFocus(true);
                BigDecimal ns = dur.getStart();
                if (s == null || s.compareTo(ns) > 0)
                {
                    s = ns;
                }
                BigDecimal ne = dur.getEnd();
                if (e == null || e.compareTo(ne) < 0)
                {
                    e = ne;
                }
            }
            else
            {
                dur.setFocus(false);
            }
        }
        if (s != null && e != null)
        {
            BigDecimal[] se = { s, e };
            return se;
        }
        else
        {
            return null;
        }
    }

    public boolean isOnSelected(BigDecimal x)
    {
        Iterator<TimeDuration> it = durations.iterator();
        while (it.hasNext())
        {
            TimeDuration dur = it.next();
            if (dur.isContain(x))
            {
                return true;
            }
        }
        return false;
    }

    public void clearFocus()
    {
        Iterator<TimeDuration> it = durations.iterator();
        while (it.hasNext())
        {
            TimeDuration dur = it.next();
            dur.setFocus(false);
        }
    }

    /**
     * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
     */
    @Override
    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        Color nc = Color.GREEN;
        Color hc = Color.RED;
        Iterator<TimeDuration> it = durations.iterator();
        while (it.hasNext())
        {
            TimeDuration dur = it.next();
            int s = julianToPixel(dur.getStart(), data.getCurrentMin(), data.getCurrentMax());
            int e = julianToPixel(dur.getEnd(), data.getCurrentMin(), data.getCurrentMax());
            if (s < e)
            {
                Color c = nc;
                if (dur.isFocus())
                {
                    c = hc;
                }
                g.setColor(new Color(c.getRed(), c.getGreen(), c.getBlue(), 100));
                g.fillRect(s, 0, e - s, getParent().getHeight());
                // g.setColor(c);
                // int y = new Double(getVisibleRect().getY()).intValue() + 12;
                // g.drawString(Integer.toString(e - s), e, y);
            }
        }
        if (start != null && end != null)
        {
            int s = julianToPixel(start, data.getCurrentMin(), data.getCurrentMax());
            int e = julianToPixel(end, data.getCurrentMin(), data.getCurrentMax());
            g.setColor(new Color(nc.getRed(), nc.getGreen(), nc.getBlue(), 100));
            g.fillRect(s, 0, e - s, getParent().getHeight());
            // g.setColor(nc);
            // int y = new Double(getVisibleRect().getY()).intValue() + 12;
            // g.drawString(Integer.toString(e - s), e, y);
        }
        g.dispose();
    }

    private boolean doit = false;

    /**
     * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
     */
    public void mouseDragged(MouseEvent e)
    {
        if (isEnabled() && SwingUtilities.isLeftMouseButton(e))
        {
            if (e.isControlDown())
            {
                log.debug("ControlDown");
            }
            else
            {
                durations.clear();
            }

            log.debug(e.getX());
            doit = true;
            int x = e.getX();
            if (startDraggingPosition > x)
            {
                start = pixelToJulian(x, data.getCurrentMin(), data.getCurrentMax());
                end = pixelToJulian(startDraggingPosition, data.getCurrentMin(), data.getCurrentMax());
            }
            else
            {
                start = pixelToJulian(startDraggingPosition, data.getCurrentMin(), data.getCurrentMax());
                end = pixelToJulian(x, data.getCurrentMin(), data.getCurrentMax());
            }
            repaint();
        }
    }

    /**
     * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
     */
    public void mouseMoved(MouseEvent e)
    {
    }

    /**
     * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
     */
    public void mouseClicked(MouseEvent e)
    {
        int x = e.getX();
        if (e.getClickCount() == 2)
        {
            List<TimeDuration> rem = new ArrayList<TimeDuration>();
            Iterator<TimeDuration> it = durations.iterator();
            while (it.hasNext())
            {
                TimeDuration dur = it.next();
                if (julianToPixel(dur.getStart(), data.getCurrentMin(), data.getCurrentMax()) < x && x < julianToPixel(dur.getEnd(), data.getCurrentMin(), data.getCurrentMax()))
                {
                    rem.add(dur);
                }
            }
            durations.removeAll(rem);
        }
        else if (e.getClickCount() == 1)
        {
            boolean clearAll = true;

            Component comp = (Component) e.getSource();
            if (comp instanceof LayerPlugin)
            {
                LayerPlugin layer = (LayerPlugin) comp;
                if (layer.getMouseOveredRecords().size() > 0)
                {
                    clearAll = false;
                }
            }

            // List<TimeDuration> rem = new ArrayList<TimeDuration>();
            if (clearAll)
            {
                Iterator<TimeDuration> it = durations.iterator();
                while (it.hasNext())
                {
                    TimeDuration dur = it.next();
                    if (julianToPixel(dur.getStart(), data.getCurrentMin(), data.getCurrentMax()) < x && x < julianToPixel(dur.getEnd(), data.getCurrentMin(), data.getCurrentMax()))
                    {
                        clearAll = false;
                    }
                }
            }

            if (clearAll)
            {
                durations.clear();
            }
        }
        repaint();
    }

    /**
     * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
     */
    public void mouseEntered(MouseEvent e)
    {
    }

    /**
     * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
     */
    public void mouseExited(MouseEvent e)
    {
    }

    /**
     * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
     */
    public void mousePressed(MouseEvent e)
    {
        startDraggingPosition = e.getX();
    }

    /**
     * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
     */
    public void mouseReleased(MouseEvent e)
    {
        if (doit)
        {
            TimeDuration dur = new TimeDuration(start, end);
            Iterator<TimeDuration> it = durations.iterator();
            boolean addIt = true;
            List<TimeDuration> remList = new ArrayList<TimeDuration>();
            while (it.hasNext())
            {
                TimeDuration exist = it.next();
                int result = exist.compare(dur);
                if (result == TimeDuration.INTERSECT_IN)
                {
                    addIt = false;
                    break;
                }
                else
                {
                    if (result == TimeDuration.INTERSECT_OUT)
                    {
                        remList.add(exist);
                    }
                    else if (result == TimeDuration.INTERSECT_START)
                    {
                        dur.setStart(exist.getStart());
                        remList.add(exist);
                    }
                    else if (result == TimeDuration.INTERSECT_END)
                    {
                        dur.setEnd(exist.getEnd());
                        remList.add(exist);
                    }
                }
            }
            if (addIt)
            {
                durations.add(dur);
            }
            if (remList.size() > 0)
            {
                durations.removeAll(remList);
            }
            start = null;
            end = null;
            doit = false;
        }
        repaint();
    }

    private class TimeDuration
    {
        private BigDecimal start;

        private BigDecimal end;

        private boolean focus = false;

        public TimeDuration(BigDecimal start, BigDecimal end)
        {
            this.start = start;
            this.end = end;
        }

        public void setStart(BigDecimal start)
        {
            this.start = start;
        }

        public void setEnd(BigDecimal end)
        {
            this.end = end;
        }

        public BigDecimal getStart()
        {
            return start;
        }

        public BigDecimal getEnd()
        {
            return end;
        }

        public void setFocus(boolean focus)
        {
            this.focus = focus;
        }

        public boolean isContain(BigDecimal x)
        {
            if (start.compareTo(x) < 0 && end.compareTo(x) > 0)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public boolean isFocus()
        {
            return focus;
        }

        public static final int INTERSECT_NO = 0;

        public static final int INTERSECT_START = 1;

        public static final int INTERSECT_END = 2;

        public static final int INTERSECT_IN = 3;

        public static final int INTERSECT_OUT = 4;

        public int compare(TimeDuration duration)
        {
            BigDecimal cstart = duration.getStart();
            BigDecimal cend = duration.getEnd();
            int ss = start.compareTo(cstart);
            int ee = end.compareTo(cend);
            if (ss <= 0 && ee >= 0)
            {
                return INTERSECT_IN;
            }
            if (ss >= 0 && ee <= 0)
            {
                return INTERSECT_OUT;
            }
            int se = end.compareTo(cstart);
            int es = start.compareTo(cend);
            if (ss <= 0 && se >= 0)
            {
                return INTERSECT_START;
            }
            if (ee >= 0 && es <= 0)
            {
                return INTERSECT_END;
            }
            return INTERSECT_NO;
        }
    }
}
