/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;

import javax.swing.JFileChooser;

import jp.nihu.gttime.utility.ExtensionFileFilter;
import jp.nihu.gttime.utility.ExtensionFileSaveChooser;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * 警告及び致命的なエラーの内容を格納するオブジェクト
 */
public class IOError extends ArrayList<String>
{
    /**
     * シリアル・バージョンID
     */
    private static final long serialVersionUID = 1L;

    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(IOError.class);

    /**
     * 致命的なエラーの数
     */
    private int fatalNum = 0;

    /**
     * 警告の数
     */
    private int warnNum = 0;

    public void addFatalNum()
    {
        fatalNum++;
    }

    public void addWarnNum()
    {
        warnNum++;
    }

    public int getFatalNum()
    {
        return fatalNum;
    }

    public int getWarnNum()
    {
        return warnNum;
    }

    /**
     * データ読み込み中のエラーを処理します。
     */
    public void processError()
    {
        if (fatalNum != 0 || warnNum != 0)
        {
            ImportErrorDialog errorDialog = new ImportErrorDialog(null, true, fatalNum + warnNum, fatalNum, warnNum);
            errorDialog.setVisible(true);
            int next = errorDialog.getNextProc();
            if (next == ImportErrorDialog.SAVE_ERROR)
            {
                saveError();
            }
            else if (next == ImportErrorDialog.SHOW_ERROR)
            {
                ImportErrorDetailDialog errorDetailDialog = new ImportErrorDetailDialog(null, true, this);
                errorDetailDialog.setVisible(true);
                next = errorDetailDialog.getNextProc();
                if (next == ImportErrorDetailDialog.SAVE_ERROR)
                {
                    saveError();
                }
                errorDetailDialog.dispose();
            }
            errorDialog.dispose();
        }
    }

    private void saveError()
    {
        ExtensionFileSaveChooser chooser = new ExtensionFileSaveChooser("error_log");
        ExtensionFileFilter filter = new ExtensionFileFilter("txt");
        chooser.addChoosableFileFilter(filter);
        int returnVal = chooser.showSaveDialog(null);
        if (returnVal == JFileChooser.APPROVE_OPTION)
        {
            File file = chooser.getSelectedFile();
            Writer out = null;
            try
            {
                out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file), "UTF-8"));
                int el = this.size();
                for (int i = 0; i < el; i++)
                {
                    out.write(this.get(i));
                    out.write("\n");
                }
                log.debug("save file to : " + file.getAbsolutePath());
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
            if (out != null)
            {
                try
                {
                    out.flush();
                    out.close();
                }
                catch (IOException e)
                {
                    e.printStackTrace();
                }
            }
        }
    }
}
