/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin;

import java.io.File;
import java.io.IOException;

/**
 * 画像出力用プラグインインターフェース
 */
public interface ImageWriterPlugin extends Plugin
{

    /**
     * {@code File} に指定された形式で画像を出力する。
     * 
     * @param p 画像描画用オブジェクト
     * @param formatName 出力する形式（ファイル拡張子）
     * @param output 出力する {@code File}
     * @return 画像出力に成功した場合 {@code true}
     * @exception IOException 出力中にエラーが発生した場合
     */
    public boolean write(LayerPainter p, String formatName, File output) throws IOException;

    /**
     * 現在登録されている出力可能な形式（ファイル拡張子）を取得する。
     * 
     * @return 出力可能な形式（ファイル拡張子）の {@code String} 配列
     */
    public String[] getWriterFileSuffixes();

    /**
     * 指定した形式（ファイル拡張子）の別名を取得する。
     * 
     * @param formatName 形式（ファイル拡張子）
     * @return 形式（ファイル拡張子）の別名
     */
    public String[] getWriterFileSuffixAliases(String formatName);

    /**
     * 指定した形式（ファイル拡張子）の説明を取得する。
     * 
     * @param formatName 形式（ファイル拡張子）
     * @return 形式（ファイル拡張子）の説明
     */
    public String getWriterFileDescription(String formatName);

    /**
     * 指定した形式が出力可能かを判定する。
     * 
     * @param formatName 出力する形式（ファイル拡張子）
     * @return 出力可能な場合 {@code true}
     */
    public boolean isWritableFileFormat(String formatName);
}
