/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.util.ArrayList;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JList;
import javax.swing.JTextArea;
import javax.swing.ListCellRenderer;
import javax.swing.border.Border;
import javax.swing.border.LineBorder;
import javax.swing.plaf.basic.BasicGraphicsUtils;

import jp.nihu.gttime.T2Map;

public class ImportErrorDetailDialog extends javax.swing.JDialog
{
    /**
     * シリアルバーションID
     */
    private static final long serialVersionUID = 1L;

    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(ImportErrorDetailDialog.class, T2Map.LOCALE);

    ArrayList<String> strings = null;

    /** Creates new form ImportErrorDetailDialog */
    public ImportErrorDetailDialog(java.awt.Frame parent, boolean modal, ArrayList<String> strings)
    {
        super(parent, modal);
        this.strings = strings;
        initComponents();
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated
    // Code">//GEN-BEGIN:initComponents
    private void initComponents()
    {
        this.setTitle(rmgr.getString("title"));

        jScrollPane1 = new javax.swing.JScrollPane();
        jList1 = new javax.swing.JList();
        jList1.setCellRenderer(new TextAreaRenderer());
        jPanel1 = new javax.swing.JPanel();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setName("Form"); // NOI18N

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        jList1.setModel(new javax.swing.AbstractListModel()
        {
            private static final long serialVersionUID = 1L;

            public int getSize()
            {
                return strings.size();
            }

            public Object getElementAt(int i)
            {
                return strings.get(i);
            }
        });
        jList1.setName("jList1"); // NOI18N
        jScrollPane1.setViewportView(jList1);

        getContentPane().add(jScrollPane1, java.awt.BorderLayout.CENTER);

        jPanel1.setName("buttonPanel"); // NOI18N
        jPanel1.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        jButton1.setText(rmgr.getString("button.save.error")); // NOI18N
        jButton1.setName("button.save.error"); // NOI18N
        jButton1.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                nextProc = SAVE_ERROR;
                setVisible(false);
            }
        });
        jPanel1.add(jButton1);

        jButton2.setText(rmgr.getString("button.close")); // NOI18N
        jButton2.setName("button.close"); // NOI18N
        jButton2.addActionListener(new AbstractAction()
        {
            private static final long serialVersionUID = 1L;

            public void actionPerformed(ActionEvent e)
            {
                setVisible(false);
            }
        });
        jPanel1.add(jButton2);

        getContentPane().add(jPanel1, java.awt.BorderLayout.PAGE_END);

        pack();

        // ダイアログの表示位置を設定。
        Toolkit tk = getToolkit();
        Dimension screenSize = tk.getScreenSize();
        int posX = (screenSize.width / 2) - (getWidth() / 2);
        int posY = (screenSize.height / 2) - (getHeight() / 2);
        setLocation(posX, posY);
    }

    private int nextProc = 0;

    public static final int SAVE_ERROR = 1;

    public int getNextProc()
    {
        return nextProc;
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String args[])
    {
        java.awt.EventQueue.invokeLater(new Runnable()
        {
            public void run()
            {
                ArrayList<String> dummy = new ArrayList<String>();
                dummy.add("5行目のnumber属性(7カラム目)に数値以外が入力されています。");
                dummy.add("6行目のカラム数が合ってません。");
                dummy.add("11行目の終了日時(3カラム目)のフォーマットが違います。");
                dummy.add("13行目のカラム数が合ってません。");
                dummy.add("14行目のカラム数が合ってません。");
                dummy.add("16行目のカラム数が合ってません。");
                dummy.add("20行目の終了日時(3カラム目)のフォーマットが違います。");
                ImportErrorDetailDialog dialog = new ImportErrorDetailDialog(new javax.swing.JFrame(), true, dummy);
                dialog.addWindowListener(new java.awt.event.WindowAdapter()
                {
                    public void windowClosing(java.awt.event.WindowEvent e)
                    {
                        System.exit(0);
                    }
                });
                dialog.setVisible(true);
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;

    private javax.swing.JButton jButton2;

    private javax.swing.JList jList1;

    private javax.swing.JPanel jPanel1;

    private javax.swing.JScrollPane jScrollPane1;

    // End of variables declaration//GEN-END:variables

    class TextAreaRenderer extends JTextArea implements ListCellRenderer
    {
        /**
         * シリアルバーションID
         */
        private static final long serialVersionUID = 1L;

        private final Color evenColor = new Color(230, 255, 230);

        private final Border border = new DotBorder(Color.WHITE, 2);

        public Component getListCellRendererComponent(JList list, Object object, int index, boolean isSelected, boolean cellHasFocus)
        {
            setText((object == null) ? "" : object.toString());
            setBorder(cellHasFocus ? border : BorderFactory.createEmptyBorder(2, 2, 2, 2));
            if (isSelected)
            {
                setBackground(list.getSelectionBackground());
                setForeground(list.getSelectionForeground());
            }
            else
            {
                setBackground(index % 2 != 0 ? evenColor : list.getBackground());
                setForeground(list.getForeground());
            }
            return this;
        }
    }

    class DotBorder extends LineBorder
    {
        /**
         * シリアルバーションID
         */
        private static final long serialVersionUID = 1L;

        public boolean isBorderOpaque()
        {
            return true;
        }

        public DotBorder(Color color, int thickness)
        {
            super(color, thickness);
        }

        @Override
        public void paintBorder(Component c, Graphics g, int x, int y, int w, int h)
        {
            Graphics2D g2 = (Graphics2D) g;
            g2.translate(x, y);
            g2.setPaint(getLineColor());
            BasicGraphicsUtils.drawDashedRect(g2, 0, 0, w, h);
            g2.translate(-x, -y);
        }
    }

}
