/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;

import jp.nihu.gttime.T2Map;

public class ImportErrorDialog extends javax.swing.JDialog
{
    /**
     * シリアルバーションID
     */
    private static final long serialVersionUID = 1L;

    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(ImportErrorDialog.class, T2Map.LOCALE);

    /** Creates new form ImportErrorDialog */
    public ImportErrorDialog(java.awt.Frame parent, boolean modal, int totalNum, int fatalNum, int warnNum)
    {
        super(parent, modal);
        initComponents(totalNum, fatalNum, warnNum);
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated
    // Code">//GEN-BEGIN:initComponents
    private void initComponents(int totalNum, int fatalNum, int warnNum)
    {
        this.setTitle(rmgr.getString("title"));
        jPanel2 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jPanel1 = new javax.swing.JPanel();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();
        jButton3 = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setName("Form"); // NOI18N

        jPanel2.setName("jPanel2"); // NOI18N
        jPanel2.setLayout(new javax.swing.BoxLayout(jPanel2, javax.swing.BoxLayout.PAGE_AXIS));

        jLabel1.setText(rmgr.format("message.text.total", totalNum)); // NOI18N
        jLabel1.setName("jLabel1"); // NOI18N
        jPanel2.add(jLabel1);

        jLabel2.setText(rmgr.format("message.text.fatal", fatalNum)); // NOI18N
        jLabel2.setName("jLabel2"); // NOI18N
        jPanel2.add(jLabel2);

        jLabel3.setText(rmgr.format("message.text.warn", warnNum)); // NOI18N
        jLabel3.setName("jLabel3"); // NOI18N
        jPanel2.add(jLabel3);

        getContentPane().add(jPanel2, java.awt.BorderLayout.PAGE_START);

        jPanel1.setName("jPanel1"); // NOI18N
        jPanel1.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        jButton1.setText(rmgr.getString("button.save.error")); // NOI18N
        jButton1.setName("button.save.error"); // NOI18N
        jButton1.addActionListener(new AbstractAction()
        {
            private static final long serialVersionUID = 1L;

            public void actionPerformed(ActionEvent e)
            {
                nextProc = SAVE_ERROR;
                setVisible(false);
            }
        });
        jPanel1.add(jButton1);

        jButton2.setText(rmgr.getString("button.show.error")); // NOI18N
        jButton2.setName("button.show.error"); // NOI18N
        jButton2.addActionListener(new AbstractAction()
        {
            private static final long serialVersionUID = 1L;

            public void actionPerformed(ActionEvent e)
            {
                nextProc = SHOW_ERROR;
                setVisible(false);
            }
        });
        jPanel1.add(jButton2);

        jButton3.setText(rmgr.getString("button.close")); // NOI18N
        jButton3.setName("button.close"); // NOI18N
        jButton3.addActionListener(new AbstractAction()
        {
            private static final long serialVersionUID = 1L;

            public void actionPerformed(ActionEvent e)
            {
                setVisible(false);
            }
        });
        jPanel1.add(jButton3);

        getContentPane().add(jPanel1, java.awt.BorderLayout.CENTER);

        pack();

        // ダイアログの表示位置を設定。
        Toolkit tk = getToolkit();
        Dimension screenSize = tk.getScreenSize();
        int posX = (screenSize.width / 2) - (getWidth() / 2);
        int posY = (screenSize.height / 2) - (getHeight() / 2);
        setLocation(posX, posY);
    }

    private int nextProc = 0;

    public static final int SAVE_ERROR = 1;

    public static final int SHOW_ERROR = 2;

    public int getNextProc()
    {
        return nextProc;
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String args[])
    {
        java.awt.EventQueue.invokeLater(new Runnable()
        {
            public void run()
            {
                ImportErrorDialog dialog = new ImportErrorDialog(new javax.swing.JFrame(), true, 1, 2, 3);
                dialog.addWindowListener(new java.awt.event.WindowAdapter()
                {
                    public void windowClosing(java.awt.event.WindowEvent e)
                    {
                        System.exit(0);
                    }
                });
                dialog.setVisible(true);
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;

    private javax.swing.JButton jButton2;

    private javax.swing.JButton jButton3;

    private javax.swing.JLabel jLabel1;

    private javax.swing.JLabel jLabel2;

    private javax.swing.JLabel jLabel3;

    private javax.swing.JPanel jPanel1;

    private javax.swing.JPanel jPanel2;
    // End of variables declaration//GEN-END:variables

}
