/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin;

import java.awt.Dimension;

import jp.nihu.gttime.data.DatasetIF;

/**
 * {@code Dataset} オブジェクトの拡張用ラッパーインターフェース
 */
public interface LayerData
{

    /**
     * {@code Dataset} 名称を取得する。
     */
    public String getName();
    
    /**
     * {@code Dataset} グループ名を取得する。
     */
    public String getGroup();

    /**
     * {@code Dataset} 型を取得する。
     */
    public String getType();

    /**
     * {@code Dataset} オブジェクトを取得する。
     */
    public DatasetIF getDataset();

    /**
     * 対応する {@code LayerPlugin} オブジェクトの表示サイズを取得する。
     */
    public Dimension getSize();

    /**
     * 対応する {@code LayerPlugin} オブジェクトの表示サイズを設定する。
     */
    public void setSize(Dimension d);

    /**
     * 対応する {@code LayerPlugin} オブジェクトの表示幅を取得する。
     */
    public int getWidth();

    /**
     * 対応する {@code LayerPlugin} オブジェクトの表示幅を指定する。
     */
    public void setWidth(int width);

    /**
     * 対応する {@code LayerPlugin} オブジェクトの表示高を取得する。
     */
    public int getHeight();

    /**
     * 対応する {@code LayerPlugin} オブジェクトの表示高を指定する。
     */
    public void setHeight(int height);

    /**
     * 対応する {@code LayerPlugin} オブジェクトが可視状態かを判定する。
     * 
     * @return 可視の場合は {@code true}
     */
    public boolean isVisible();

    /**
     * 対応する {@code LayerPlugin} オブジェクトの可視状態を指定する。
     * 
     * @param b 可視の場合は {@code true}
     */
    public void setVisible(boolean b);

    /**
     * 時間目盛を表示するかを判定する。
     * 
     * @return 時間目盛を表示する場合 {@code true}
     */
    public boolean isDrawTimeAxis();

    /**
     * 時間目盛を表示するかを指定する。
     * 
     * @param b 時間目盛を表示する場合 {@code true}
     */
    public void setDrawTimeAxis(boolean b);
}
