/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferedImage;

/**
 * {@code LayerPlugin} オブジェクトの描画用インターフェース
 */
public interface LayerPainter
{

    /**
     * {@code LayerPlugin} オブジェクトを描画する。
     * 
     * @param g 描画に使用するグラフィックコンテキスト
     */
    public void paint(Graphics g, boolean repaintLayer);

    /**
     * {@code LayerPlugin} オブジェクトを描画する。
     * 
     * @param g 描画に使用するグラフィックコンテキスト
     * @param layer 描画対象となる {@code LayerPlugin} オブジェクト
     */
    public void paint(Graphics g, LayerPlugin layer, boolean repaintLayer);

    /**
     * {@code LayerPlugin} オブジェクトのイメージデータを取得する。
     */
    public BufferedImage getImage();

    /**
     * {@code LayerPlugin} オブジェクトのイメージデータを取得する。 指定したイメージ形式で構築した
     * {@code BufferedImage} に対して、 {@code LayerPlugin} オブジェクトの描画を行う。
     * 
     * @param imageType 取得するイメージの形式
     */
    public BufferedImage getImage(int imageType);

    /**
     * 描画対象となる {@code LayerPlugin} オブジェクトを取得する。
     * 
     * @return 描画対象の {@code LayerPlugin} オブジェクト配列
     */
    public LayerPlugin[] getLayerPlugins();

    /**
     * 描画サイズを取得する。
     * 
     * @return 描画サイズを示す {@code Dimension} オブジェクト
     */
    public Dimension getSize();

    /**
     * 各 {@code LayerPlugin} の間に罫線を描画するかを判定する。
     * 
     * @return 罫線を描画する場合 {@code true}
     */
    public boolean isDrawBorder();

    /**
     * 各 {@code LayerPlugin} の間に罫線を描画するかを指定する。
     * 
     * @param b 罫線を描画する場合 {@code true}
     */
    public void setDrawBorder(boolean b);
}
