/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin;

import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;
import java.math.BigDecimal;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.T2MapDialog;
import jp.nihu.gttime.data.Record;
import jp.nihu.gttime.layer.TimeDurationSelector;

/**
 * 数値計測のプラグインを管理するクラスです。
 */
public abstract class NumericMeasurePlugin implements Plugin
{
    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(NumericMeasurePlugin.class, T2Map.LOCALE);

    /**
     * T2MapDialogオブジェクト
     */
    protected T2MapDialog dialog = null;

    /**
     * JPanelオブジェクト
     */
    protected JPanel resultPanel = null;

    /**
     * 数値計測の処理を記述します。
     * 
     * @param records レコード
     * @param startTime 開始日時
     * @param endTime 終了日時
     */
    protected abstract void measure(Record[] records, BigDecimal startTime, BigDecimal endTime, BigDecimal[][] durations);
    
    protected abstract void measure(Record[] records, BigDecimal[][] durations);
    
    protected boolean onlyAll = false;

    /**
     * ダイアログを表示します。
     * 
     * @param records レコード
     * @param itemName 計測するプラグイン名
     * @param startPoint 開始位置
     * @param endPoint 終了位置
     * @param parent コンポーネントオブジェクト
     * @param node TreeCheckNodeオブジェクト
     */
    public void showDialog(Record[] records, final JMenuItem itemName, BigDecimal startPoint, BigDecimal endPoint, Component parent, final TimeDurationSelector timeDurationSelector)
    {
    	onlyAll = false;
        dialog = new T2MapDialog((T2Map) SwingUtilities.getRoot(parent), false);
        dialog.setModal(true);
        dialog.getContentPane().setLayout(new FlowLayout());

        dialog.setLocationRelativeTo(null);

        resultPanel = new JPanel();
        resultPanel.setLayout(new BoxLayout(resultPanel, BoxLayout.Y_AXIS));

        measure(records, startPoint, endPoint, timeDurationSelector.getDurations());

        JButton closeButton = new JButton(rmgr.getString("dialog.close"));
        closeButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent ae)
            {
                dialog.dispose();
                timeDurationSelector.clearFocus();
            }
        });
        closeButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        resultPanel.add(closeButton);

        dialog.add(resultPanel);
        dialog.setVisible(true);

        dialog.addWindowFocusListener(new WindowFocusListener()
        {
            public void windowLostFocus(WindowEvent arg0)
            {
                dialog.dispose();
                timeDurationSelector.clearFocus();
            }

            public void windowGainedFocus(WindowEvent e)
            {
            }

        });
    }
    
    public void showDialog(Record[] records, final JMenuItem itemName, Component parent, final TimeDurationSelector timeDurationSelector)
    {
    	onlyAll = true;
        dialog = new T2MapDialog((T2Map) SwingUtilities.getRoot(parent), false);
        dialog.setModal(true);
        dialog.getContentPane().setLayout(new FlowLayout());

        dialog.setLocationRelativeTo(null);

        resultPanel = new JPanel();
        resultPanel.setLayout(new BoxLayout(resultPanel, BoxLayout.Y_AXIS));

//        measure(records, startPoint, endPoint, timeDurationSelector.getDurations());
        measure(records, timeDurationSelector.getDurations());

        JButton closeButton = new JButton(rmgr.getString("dialog.close"));
        closeButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent ae)
            {
                dialog.dispose();
                timeDurationSelector.clearFocus();
            }
        });
        closeButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        resultPanel.add(closeButton);

        dialog.add(resultPanel);
        dialog.setVisible(true);

        dialog.addWindowFocusListener(new WindowFocusListener()
        {
            public void windowLostFocus(WindowEvent arg0)
            {
                dialog.dispose();
                timeDurationSelector.clearFocus();
            }

            public void windowGainedFocus(WindowEvent e)
            {
            }

        });
    }

}