/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import jp.nihu.gttime.data.Record;

/**
 * レコードの入出力を行うクラス
 */
public abstract class RecordIOFilter implements Plugin
{
    /**
     * エラーを管理するオブジェクト
     */
    protected IOError ioError = null;
    
    private List<String> groups = new ArrayList<String>();
    
    public List<String> getGroups() {
        return groups;
    }
    
    public void setGroup(String group) {
        if (!groups.contains(group)) {
            groups.add(group);
        }
    }

    /**
     * tmmデータの絶対位置を取得します。
     * 
     * @param tmmPath tmmデータの位置
     * @return tmmデータの絶対位置
     * @throws RecordIOFilterException
     */
    public abstract String getPath(String tmmPath) throws RecordIOFilterException;

    /**
     * レコードを読込みます。
     * 
     * @param is 入力ストリーム
     * @return Recordオブジェクトの配列
     * @throws RecordIOFilterException 読み込み時にエラーが発生した場合
     */
    public abstract Record[] read(InputStream is, IOError ioError) throws RecordIOFilterException;

    /**
     * レコードを読込みます。
     * 
     * @param is 入力ストリーム
     * @param code サポートする charset の名前
     * @return Recordオブジェクトの配列
     * @throws RecordIOFilterException 読み込み時にエラーが発生した場合
     */
    public abstract Record[] read(InputStream is, String code, IOError ioError) throws RecordIOFilterException;

    /**
     * レコードをバイト列に書き出します。
     * 
     * @param records 書き出すレコード
     * @return レコードのバイト列
     * @throws RecordIOFilterException 書き出し時にエラーが発生した場合
     */
    public abstract byte[] write(Record[] records) throws RecordIOFilterException;

    /**
     * 致命的なエラーをあらわす定数
     */
    protected static final int FATAL = 0;

    /**
     * 警告をあらわす定数
     */
    protected static final int WARN = 1;
}
