/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.data.DatasetIF;
import jp.nihu.gttime.data.ProjectException;
import jp.nihu.gttime.data.ProjectGts;
import jp.nihu.gttime.data.ProjectIF;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * 検索する為の抽象クラスです。
 */
public abstract class SearchPlugin implements Plugin
{
    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(SearchPlugin.class);

    /**
     * 検索を実行します。
     * 
     * @return データセット
     */
    public abstract DatasetIF[] search();

    /**
     * T2Mapオブジェクト
     */
    private T2Map t2map;

    /**
     * T2Mapを設定します。
     * 
     * @param t2map T2Mapオブジェクト
     */
    public void init(T2Map t2map)
    {
        this.t2map = t2map;
    }

    /**
     * T2Mapを取得します。
     * 
     * @return T2Mapオブジェクト
     */
    protected T2Map getT2Map()
    {
        return t2map;
    }

    /**
     * データセットをプロジェクトに追加します。
     * 
     * @param datasets データセット
     */
    public void setDatasets2Project(DatasetIF[] datasets)
    {
        if (datasets != null && datasets.length > 0)
        {
            ProjectIF project = T2Map.getProject();
            if (project == null)
            {
                // int type = T2Map.getProjectType();
                // if (type == 0)
                // {
                // project = new ProjectTms(datasets);
                // }
                // else if (type == 1)
                // {
                project = new ProjectGts(datasets);
                // }
            }
            else
            {
                for (int i = 0; i < datasets.length; i++)
                {
                    try
                    {
                        project.addDataset(datasets[i]);
                    }
                    catch (ProjectException e)
                    {
                        log.error(e.getMessage(), e);
                        e.printStackTrace();
                    }
                }
            }
            t2map.setProject(project);
        }
    }
}
