/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.math.BigDecimal;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.TitledBorder;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.T2MapDialog;

/**
 * 時間軸計測のダイアログを表示するクラスです。
 */
public class TimeMeasureDialog extends T2MapDialog
{
    /**
     * ログ用オブジェクト
     */
    private static final long serialVersionUID = 1L;

    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(TimeMeasureDialog.class, T2Map.LOCALE);

    /**
     * 大枠のパネル
     */
    private JPanel mainPanel = null;

    /**
     * 閉じるボタン
     */
    private JButton closeButton = null;

    /**
     * 空白
     */
    private JLabel explanation = null;

    /**
     * 時間軸計測ダイアログを作成します。
     * 
     * @param owner Frameオブジェクト
     * @param modal 表示するかしないか
     */
    public TimeMeasureDialog(Frame owner, boolean modal)
    {
        super(owner, modal);
    }

    /**
     * 時間軸計測ダイアログを作成します。
     * 
     * @param owner Frameオブジェクト
     * @param modal 表示するかしないか
     * @param node TreeCheckNodeオブジェクト
     * @param itemName 計測名
     */
    public TimeMeasureDialog(Frame owner, boolean modal, BigDecimal mainRangeBD, BigDecimal allRangeBD)
    {
        this(owner, modal);

        boolean showAllSelected = false;
        if (allRangeBD != null)
        {
            showAllSelected = true;
        }

        getContentPane().setLayout(new FlowLayout());
        setTitle(rmgr.getString("time.measure"));
        setLocationRelativeTo(null);

        if (showAllSelected)
        {
            setSize(300, 280);
        }
        else
        {
            setSize(300, 180);
        }

        this.setResizable(false);

        int ypos = 0;
        GridBagConstraints gridBagConstraints0 = new GridBagConstraints();
        gridBagConstraints0.gridx = 0;
        gridBagConstraints0.gridy = ypos++;
        GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
        if (showAllSelected)
        {
            gridBagConstraints1.gridx = 0;
            gridBagConstraints1.gridy = ypos++;
        }
        GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridwidth = 5;
        gridBagConstraints2.gridy = ypos++;
        GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
        gridBagConstraints3.gridx = 0;
        gridBagConstraints3.gridwidth = 5;
        gridBagConstraints3.anchor = GridBagConstraints.CENTER;
        gridBagConstraints3.gridy = ypos++;

        mainPanel = new JPanel();
        mainPanel.setLayout(new GridBagLayout());
        mainPanel.setSize(new Dimension(366, 82));

        int mainRange = mainRangeBD.setScale(100, BigDecimal.ROUND_HALF_UP).intValue();
        mainPanel.add(new ResultPanel(mainRange, rmgr.getString("title.selected.focus.range")), gridBagConstraints0);

        if (showAllSelected)
        {
            int mainRangex = allRangeBD.setScale(100, BigDecimal.ROUND_HALF_UP).intValue();
            mainPanel.add(new ResultPanel(mainRangex, rmgr.getString("title.selected.all.range")), gridBagConstraints1);
        }

        explanation = new JLabel();
        mainPanel.add(explanation, gridBagConstraints2);

        initCloseButton();
        mainPanel.add(closeButton, gridBagConstraints3);

        add(mainPanel);
        setVisible(true);

        // addWindowFocusListener(new WindowFocusListener()
        // {
        //
        // public void windowLostFocus(WindowEvent e)
        // {
        // dispose();
        // }
        //
        // public void windowGainedFocus(WindowEvent e)
        // {
        // }
        //
        // });
    }
    
    public TimeMeasureDialog(Frame owner, boolean modal, BigDecimal allRangeBD)
    {
        this(owner, modal);

//        boolean showAllSelected = false;
//        if (allRangeBD != null)
//        {
//            showAllSelected = true;
//        }

        getContentPane().setLayout(new FlowLayout());
        setTitle(rmgr.getString("time.measure"));
        setLocationRelativeTo(null);

//        if (showAllSelected)
//        {
//            setSize(300, 280);
//        }
//        else
//        {
            setSize(300, 180);
//        }

        this.setResizable(false);

        int ypos = 0;
        GridBagConstraints gridBagConstraints0 = new GridBagConstraints();
        gridBagConstraints0.gridx = 0;
        gridBagConstraints0.gridy = ypos++;
//        GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
//        if (showAllSelected)
//        {
//            gridBagConstraints1.gridx = 0;
//            gridBagConstraints1.gridy = ypos++;
//        }
        GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridwidth = 5;
        gridBagConstraints2.gridy = ypos++;
        GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
        gridBagConstraints3.gridx = 0;
        gridBagConstraints3.gridwidth = 5;
        gridBagConstraints3.anchor = GridBagConstraints.CENTER;
        gridBagConstraints3.gridy = ypos++;

        mainPanel = new JPanel();
        mainPanel.setLayout(new GridBagLayout());
//        mainPanel.setSize(new Dimension(366, 82));

//        int mainRange = mainRangeBD.setScale(100, BigDecimal.ROUND_HALF_UP).intValue();
//        mainPanel.add(new ResultPanel(mainRange, rmgr.getString("title.selected.focus.range")), gridBagConstraints0);

//        if (showAllSelected)
//        {
            int mainRangex = allRangeBD.setScale(100, BigDecimal.ROUND_HALF_UP).intValue();
            mainPanel.add(new ResultPanel(mainRangex, rmgr.getString("title.selected.all.range")), gridBagConstraints0);
//        }

        explanation = new JLabel();
        mainPanel.add(explanation, gridBagConstraints2);

        initCloseButton();
        mainPanel.add(closeButton, gridBagConstraints3);

        add(mainPanel);
        setVisible(true);

        // addWindowFocusListener(new WindowFocusListener()
        // {
        //
        // public void windowLostFocus(WindowEvent e)
        // {
        // dispose();
        // }
        //
        // public void windowGainedFocus(WindowEvent e)
        // {
        // }
        //
        // });
    }

    private void initCloseButton()
    {
        closeButton = new JButton();
        closeButton.setText(rmgr.getString("measure.exit"));
        closeButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                dispose();
            }
        });
    }

    private class ResultPanel extends JPanel
    {
        /**
         * 
         */
        private static final long serialVersionUID = 1L;

        /**
         * 計測結果を表示するラベル
         */
        private JLabel resultLabel = null;

        /**
         * 年のラジオボタン
         */
        private JRadioButton rbYear = null;

        /**
         * 月のラジオボタン
         */
        private JRadioButton rbMonth = null;

        /**
         * 週のラジオボタン
         */
        private JRadioButton rbWeek = null;

        /**
         * 日のラジオボタン
         */
        private JRadioButton rbDay = null;

        /**
         * 時間のラジオボタン
         */
        private JRadioButton rbHour = null;

        private int mainRange;

        public ResultPanel(int mainRange, String title)
        {
            this.mainRange = mainRange;
            setBorder(new TitledBorder(title));
            initRadioButtonPanel();
        }

        private void initRadioButtonPanel()
        {
            initResultLabel();
            initCloseButton();
            initYearRadioButton();
            initMonthRadioButton();
            initWeekRadioButton();
            initDayRadioButton();
            initHourRadioButton();

            GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
            gridBagConstraints4.gridx = 0;
            gridBagConstraints4.gridwidth = 5;
            gridBagConstraints4.gridy = 0;
            GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
            gridBagConstraints12.gridx = 4;
            gridBagConstraints12.gridy = 1;
            GridBagConstraints gridBagConstraints10 = new GridBagConstraints();
            gridBagConstraints10.gridx = 3;
            gridBagConstraints10.gridy = 1;
            GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
            gridBagConstraints9.gridx = 2;
            gridBagConstraints9.gridy = 1;
            GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
            gridBagConstraints8.gridx = 1;
            gridBagConstraints8.gridy = 1;
            GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
            gridBagConstraints5.gridx = 0;
            gridBagConstraints5.gridy = 1;
            setLayout(new GridBagLayout());
            add(resultLabel, gridBagConstraints4);
            add(rbYear, gridBagConstraints5);
            add(rbMonth, gridBagConstraints8);
            add(rbWeek, gridBagConstraints9);
            add(rbDay, gridBagConstraints10);
            add(rbHour, gridBagConstraints12);

            ButtonGroup bg = new ButtonGroup();
            bg.add(rbYear);
            bg.add(rbMonth);
            bg.add(rbWeek);
            bg.add(rbDay);
            bg.add(rbHour);
        }

        private void initResultLabel()
        {
            resultLabel = new JLabel();
            resultLabel.setText(rmgr.format("day", Integer.toString(mainRange)));
            resultLabel.setFont(new Font("Serif", Font.PLAIN, 26));
        }

        private void initYearRadioButton()
        {
            rbYear = new JRadioButton();
            rbYear.setText(rmgr.getString("title.year"));
            rbYear.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    float measureSelectInt = (float) (mainRange / 365.25);
                    BigDecimal bd = BigDecimal.valueOf(measureSelectInt);
                    resultLabel.setText(rmgr.format("year", bd.setScale(3, BigDecimal.ROUND_HALF_UP).toString()));
                    repaint();
                }
            });
        }

        private void initMonthRadioButton()
        {
            rbMonth = new JRadioButton();
            rbMonth.setText(rmgr.getString("title.month"));
            rbMonth.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    float measureMonthSelectInt = (float) (mainRange / 30.4375);
                    BigDecimal bd = BigDecimal.valueOf(measureMonthSelectInt);
                    float measureSelectInt = bd.setScale(3, BigDecimal.ROUND_HALF_UP).floatValue();

                    resultLabel.setText(rmgr.format("month", Float.toString(measureSelectInt)));
                    repaint();
                }
            });
        }

        private void initWeekRadioButton()
        {
            rbWeek = new JRadioButton();
            rbWeek.setText(rmgr.getString("title.week"));
            rbWeek.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    float measureSelectInt = (float) mainRange / 7;
                    BigDecimal bd = BigDecimal.valueOf(measureSelectInt);
                    resultLabel.setText(rmgr.format("week", bd.setScale(3, BigDecimal.ROUND_HALF_UP).toString()));
                    repaint();
                }
            });
        }

        private void initDayRadioButton()
        {
            rbDay = new JRadioButton(rmgr.getString("title.day"), true);
            rbDay.setText(rmgr.getString("title.day"));
            rbDay.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    resultLabel.setText(rmgr.format("day", Integer.toString(mainRange)));
                    repaint();
                }
            });
        }

        private void initHourRadioButton()
        {
            rbHour = new JRadioButton();
            rbHour.setText(rmgr.getString("title.hour"));
            rbHour.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    int measureSelectInt = mainRange * 24;
                    resultLabel.setText(rmgr.format("hour", Integer.toString(measureSelectInt)));
                    repaint();
                }
            });
        }
    }
}
