/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin;

import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class XMLResourceBundle extends ResourceBundle
{
    /**
     * ログ用オブジェクト
     */
    private static final Log log = LogFactory.getLog(XMLResourceBundle.class);

    private Properties properties = null;

    // public XMLResourceBundle(InputStream stream)
    // {
    // properties = new Properties();
    // properties.loadFromXML(stream);
    // }

    public XMLResourceBundle(String baseName, Locale locale)
    {
        InputStream in = getInputStream(baseName, locale);
        properties = new Properties();
        try
        {
            properties.loadFromXML(in);
        }
        catch (IOException e)
        {
            log.fatal(baseName + "\n" + e.getMessage(), e);
        }
    }

    private InputStream getInputStream(String baseName, Locale locale)
    {
    	String resourceName = null;
    	InputStream in = null;
    	if (locale != null) {
    		resourceName = getResourceName(baseName, locale.getLanguage(), locale.getCountry());
            in = getClass().getClassLoader().getResourceAsStream(resourceName);
            if (in == null)
            {
            	resourceName = getResourceName(baseName, locale.getLanguage(), null);
            	in = getClass().getClassLoader().getResourceAsStream(resourceName);
            }
    	} else {
    		resourceName = getResourceName(baseName, null, null);
            in = getClass().getClassLoader().getResourceAsStream(resourceName);
    	}
        if (in == null)
        {
            if (!locale.equals(Locale.getDefault()))
            {
                in = getInputStream(baseName, Locale.getDefault());
            }
            else
            {
                resourceName = getResourceName(baseName, null, null);
                in = getClass().getClassLoader().getResourceAsStream(resourceName);
            }
        }
        return in;
    }

    private String getResourceName(String baseName, String language, String country)
    {
        String resourceName = baseName.replaceAll("\\.", "/");
        if (language != null && country != null)
        {
            resourceName = language + "_" + country + "/" + resourceName;
        }
        else if (language != null)
        {
            resourceName = language + "/" + resourceName;
        }
        // else
        // {
        // resourceName = "default/" +resourceName;
        // }
        return resourceName + ".xml";
    }

    @Override
    @SuppressWarnings("unchecked")
    public Enumeration<String> getKeys()
    {
        return (Enumeration<String>) properties.propertyNames();
    }

    @Override
    protected Object handleGetObject(String key)
    {
        if (key == null)
        {
            throw new NullPointerException();
        }
        return properties.get(key);
    }

    public static final ResourceBundle getXMLBundle(String baseName)
    {
        return getXMLBundle(baseName, Locale.getDefault());
    }

    public static final ResourceBundle getXMLBundle(String baseName, Locale locale)
    {
        XMLResourceBundle bandle = null;
        bandle = new XMLResourceBundle(baseName, locale);
        return bandle;
    }
}
