/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin.analyzer;

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.T2MapDialog;
import jp.nihu.gttime.data.DatasetException;
import jp.nihu.gttime.data.DatasetGtm;
import jp.nihu.gttime.data.DatasetIF;
import jp.nihu.gttime.data.Event;
import jp.nihu.gttime.data.ProjectIF;
import jp.nihu.gttime.data.Record;
import jp.nihu.gttime.plugin.AnalizePlugin;
import jp.nihu.gttime.plugin.LayerPlugin;
import jp.nihu.gttime.plugin.PluginManager;
import jp.nihu.gttime.plugin.PluginManagerException;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.time.TimeAxisIF;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * レイヤについての検索機能（プラグイン）を実装するクラスです。
 */
public class ProjectSearchPlugin extends AnalizePlugin implements ActionListener
{
    /**
     * シリアルバーションID
     */
    private static Log log = LogFactory.getLog(ProjectSearchPlugin.class);

    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(ProjectSearchPlugin.class, T2Map.LOCALE);

    /**
     * 検索文字列の格納用
     */
    private String searchKey = null;

    /**
     * 新規レイヤを生成する際の形式を格納する
     */
    private String layerType = "";

    /**
     * データセット格納用
     */
    private DatasetIF[] ds = null;

    /**
     * 選択レイヤ種別を格納する
     */
    JComboBox layerSelect = new JComboBox();

    /**
     * レイヤ名格納用
     */
    Map<String, String> vecLayerName = new TreeMap<String, String>();

    /**
     * 検索ダイアログの作成
     * 
     * @param project Projectオブジェクトを初期化します。
     * @return Dataset[]
     */
    protected DatasetIF[] analize(final ProjectIF project)
    {
        final JDialog dialog = new T2MapDialog(getT2Map(), true);

        JLabel searchLabel = new JLabel(rmgr.getString("search.title"));
        final TextField searchText = new TextField(25);

        JRadioButton rb1 = new JRadioButton(rmgr.getString("highlight.title"), false);
        rb1.addActionListener(this);
        rb1.setFont(new Font("Dialog", Font.PLAIN, 12));
        rb1.setActionCommand(String.valueOf(HIGHLIGHT));
        rb1.doClick();

        JRadioButton rb2 = new JRadioButton(rmgr.getString("new.layer.title"), false);
        rb2.addActionListener(this);
        rb2.setFont(new Font("Dialog", Font.PLAIN, 12));
        rb2.setActionCommand(String.valueOf(NEWLAYER));

        PluginManager pm = getPluins();

        ArrayList<Hashtable<String, String>> plugins = pm.getPlugins("LayerPlugin");
        for (int j = 0; j < plugins.size(); j++)
        {
            LayerPlugin p;
            try
            {
                p = (LayerPlugin) pm.newInstance(plugins.get(j).get("class"));
                vecLayerName.put(p.getName(), plugins.get(j).get("name"));
                p = null;
            }
            catch (PluginManagerException e)
            {
                log.fatal(e.getMessage(), e);
            }
        }

        // layerSelect.setFont(new Font("Dialog", Font.PLAIN, 12));
        Iterator<String> vecI = vecLayerName.keySet().iterator();
        while (vecI.hasNext())
        {
            String name = vecI.next();
            layerSelect.addItem(name);
            if (vecLayerName.get(name).equals("DefaultLayer"))
            {
                layerSelect.setSelectedItem(name);
            }
        }
        layerSelect.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                JComboBox cb = (JComboBox) e.getSource();
                String layerName = (String) cb.getSelectedItem();
                log.debug("選択レイヤ： " + layerType);
                setLayerType(vecLayerName.get(layerName));
                vecLayerName.clear();
            }
        });

        JButton btn_ok = new JButton(rmgr.getString("find.button"));
        btn_ok.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent ae)
            {
                searchKey = searchText.getText();
                if (searchKey.length() == 0)
                {
                    log.info("検索文字列は未入力です");
                }
                else
                {
                    if (searchKey == null)
                    {
                        dialog.dispose();
                    }
                    else
                    {
                        log.info("検索文字列：" + searchKey);
                        if (project != null)
                        {
                            // int ptype = T2Map.getProjectType();
                            // if (ptype == 0)
                            // {
                            // ds = new DatasetTmm[1];
                            // }
                            // else if (ptype == 1)
                            // {
                            ds = new DatasetGtm[1];
                            // }
                            ds[0] = searchNode(project, searchKey);
                            // ハイライトを選択した場合
                            if (getHowProcess() == HIGHLIGHT)
                            {
                                ds[0] = null;
                            }
                            // 新規レイヤを選択した場合
                            else if (getHowProcess() == NEWLAYER)
                            {
                            }
                        }
                    }
                    dialog.dispose();
                }
            }
        });

        JButton btn_no = new JButton(rmgr.getString("cancel.button"));
        btn_no.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent ae)
            {
                // int ptype = T2Map.getProjectType();
                // if (ptype == 0)
                // {
                // ds = new DatasetTmm[1];
                // }
                // else if (ptype == 1)
                // {
                ds = new DatasetGtm[1];
                // }
                ds[0] = null;
                dialog.dispose();
            }
        });

        ButtonGroup bg = new ButtonGroup();
        bg.add(rb1);
        bg.add(rb2);

        // top
        JPanel headPanel = new JPanel();
        headPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
        headPanel.add(searchLabel);
        headPanel.add(searchText);

        // center
        JPanel centerPanel = new JPanel();
        centerPanel.setLayout(new BoxLayout(centerPanel, BoxLayout.X_AXIS));
        TitledBorder optionBoder = new TitledBorder(new LineBorder(Color.black), rmgr.getString("display.option"));
        optionBoder.setTitleFont(new Font("Dialog", Font.PLAIN, 12));
        optionBoder.setTitleColor(Color.black);
        centerPanel.setBorder(optionBoder);
        JPanel innerPanel = new JPanel();
        innerPanel.setLayout(new BoxLayout(innerPanel, BoxLayout.X_AXIS));
        innerPanel.add(rb1);
        innerPanel.add(rb2);
        innerPanel.add(layerSelect);
        centerPanel.add(innerPanel);

        // under
        JPanel underPanel = new JPanel();
        underPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
        underPanel.add(btn_ok);
        underPanel.add(btn_no);

        // main
        JPanel mainPanel = new JPanel();
        mainPanel.setLayout(new GridLayout(3, 0));
        mainPanel.add(headPanel);
        mainPanel.add(centerPanel);
        mainPanel.add(underPanel);
        dialog.add(mainPanel);

        dialog.setSize(new java.awt.Dimension(320, 170));
        dialog.setLocation(getT2Map().getX(), getT2Map().getY());
        dialog.setTitle(rmgr.getString("search.condition.input.title"));
        dialog.setVisible(true);
        return ds;
    }

    /**
     * コンボボックスを選択可能にさせるか、させないか
     * 
     * @param e ActionEventオブジェクト
     */
    public void actionPerformed(ActionEvent e)
    {
        log.debug("検索機能選択： " + e.getActionCommand() + " です");
        setHowProcess(Integer.parseInt(e.getActionCommand()));

        if (getHowProcess() == NEWLAYER)
        {
            layerSelect.setEnabled(true);
        }
        else if (getHowProcess() == HIGHLIGHT)
        {
            layerSelect.setEnabled(false);
        }
    }

    /**
     * 新規レイヤを生成する際の形式を設定する
     * 
     * @param layerType
     */
    private void setLayerType(String layerType)
    {
        this.layerType = layerType;
    }

    /**
     * Recordオブジェクト格納用
     */
    private ArrayList<Record> vecRecord = new ArrayList<Record>();

    /**
     * 検索処理したDatasetオブジェクトを取得します。
     * 
     * @param project 検索対象のProjectオブジェクト
     * @param searchKey 検索キーとなる文字列
     * @return 検索処理後のDatasetオブジェクト
     */
    private DatasetIF searchNode(ProjectIF project, String searchKey)
    {
        int matchNum = 0;
        log.debug("Search Key : " + searchKey);
        DatasetIF ds = null;
        TimeAxisIF data = getT2Map().getTimeAxisData();
        BigDecimal sd = data.getMin();
        BigDecimal ed = data.getMax();
        DatasetIF[] dss = project.getDatasets();
        Pattern p = Pattern.compile(searchKey);
        for (int i = 0; i < dss.length; i++)
        {
            log.debug("Layer Title : " + dss[i].getTitle());
            Record[] records = dss[i].getRecords(sd, ed);
            for (int j = 0; j < records.length; j++)
            {
                Event[] event = records[j].getEvents();
                for (int k = 0; k < event.length; k++)
                {
                    Matcher m = p.matcher(event[k].getData());
                    if (m.find())
                    {
                        matchNum++;
                        if (getHowProcess() == HIGHLIGHT)
                        {
                            records[j].setHit(true);
                            break;
                        }

                        if (getHowProcess() == NEWLAYER)
                        {
                            vecRecord.add(records[j]);
                            break;
                        }
                    }
                }
            }
            log.debug("Number of Hit Records : " + matchNum);
        }

        ProjectIF[] projects = project.getProjects();
        if (projects != null)
        {
            for (int a = 0; a < projects.length; a++)
            {
                searchNode(projects[a], searchKey);
            }
        }

        if (getHowProcess() == NEWLAYER)
        {
            try
            {
                ds = createDataset();
            }
            catch (DatasetException e)
            {
                log.fatal(e.getMessage(), e);
            }
        }
        return ds;
    }

    /**
     * Datasetオブジェクトと新規作成
     * 
     * @return Datasetオブジェクト
     * @throws DatasetException
     */
    private DatasetIF createDataset() throws DatasetException
    {
        Record[] arrRecord = new Record[vecRecord.size()];
        arrRecord = vecRecord.toArray(arrRecord);
        DatasetIF ds = null;
        // int ptype = T2Map.getProjectType();
        // if (ptype == 0)
        // {
        // ds = new DatasetTmm(searchKey, layerType, arrRecord);
        // }
        // else if (ptype == 1)
        // {
        ds = new DatasetGtm(searchKey, layerType, arrRecord);
        // }

        return ds;
    }

    /**
     * 検索結果の（ハイライト・新規レイヤなど）表現方法を設定する。
     * 
     * @param howProcess 検索結果の（ハイライト・新規レイヤなど）表現方法
     */
    private void setHowProcess(int howProcess)
    {
        this.howProcess = howProcess;
    }

    /**
     * 検索結果の（ハイライト・新規レイヤなど）表現方法を取得する。
     * 
     * @return 検索結果の（ハイライト・新規レイヤなど）表現方法
     */
    private int getHowProcess()
    {
        return howProcess;
    }

    // /**
    // * 検索結果を（ハイライト・新規レイヤなど）選択した表現方法で実施する。
    // *
    // * @param project 検索結果として表示したいProjectオブジェクト
    // * @param ds Projectオブジェクトに追加したいDatasetオブジェクト
    // * @throws ProjectException
    // */
    // private void procResult(Project project, Dataset ds) throws
    // ProjectException
    // {
    // switch (howProcess)
    // {
    // case HIGHLIGHT:
    // getT2Map().setProject(project);
    // break;
    // case NEWLAYER:
    // project.addDataset(ds);
    // getT2Map().setProject(project);
    // break;
    // default:
    // log.error("Invalid select search button: " + howProcess);
    // break;
    // }
    // }

    /**
     * ハイライトのコード値を設定しています。
     */
    private static final int HIGHLIGHT = 0;

    /**
     * 新規レイヤのコード値を設定しています。
     */
    private static final int NEWLAYER = 1;

    /**
     * 検索結果の表現方法についてのコード値を設定しています。
     */
    private int howProcess = HIGHLIGHT;
}
