/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin.analyzer.searchrecord;

import java.io.Serializable;

import jp.nihu.gttime.plugin.analyzer.AnalyzeConstants.Operator;

/**
 * イベント検索に関するクエリを表現するクラスです。
 */
public class EventQuery implements Serializable
{
    // シリアルID
    private static final long serialVersionUID = 1L;

    // イベント検索の属性情報
    private String type;

    // 演算子
    private Operator assign;

    // イベント検索のキー
    private String key;

    /**
     * コンストラクタ
     */
    public EventQuery()
    {
    }

    /**
     * コンストラクタ
     * 
     * @param type
     * @param assign
     * @param key
     */
    public EventQuery(String type, Operator assign, String key)
    {
        setType(type);
        setAssign(assign);
        setKey(key);
    }

    /**
     * イベント検索のキーを取得します。
     * 
     * @return イベント検索のキー
     */
    public String getType()
    {
        return type;
    }

    /**
     * イベント検索のキーを設定します。
     * 
     * @param type
     */
    public void setType(String type)
    {
        this.type = type;
    }

    /**
     * 演算子を取得します
     * 
     * @return 演算子
     */
    public Operator getAssign()
    {
        return assign;
    }

    /**
     * 演算子を設定します。
     * 
     * @param assign
     */
    public void setAssign(Operator assign)
    {
        this.assign = assign;
    }

    /**
     * イベント検索の属性情報を取得します。
     * 
     * @return イベント検索の属性情報
     */
    public String getKey()
    {
        return key;
    }

    /**
     * イベント検索の属性情報を設定します。
     * 
     * @param key
     */
    public void setKey(String key)
    {
        this.key = key;
    }
}
