/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin.analyzer.searchrecord;

import java.awt.geom.Point2D;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JTextField;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.plugin.LayerData;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.plugin.analyzer.AnalyzeConstants.Operator;
import jp.nihu.gttime.time.utility.ISO8601Exception;
import jp.nihu.gttime.time.utility.JulianGregorianDate;

/**
 * 検索条件クラスです。
 */
public class Query implements Serializable
{
    // シリアルID
    private static final long serialVersionUID = 1L;

    public static final BigDecimal DATE_MAX = new BigDecimal(Double.MAX_VALUE);

    public static final BigDecimal DATE_MIN = new BigDecimal(Double.MIN_VALUE);

    // リソースバンドルの管理
    protected static PluginResourceManager rmgr = PluginResourceManager.getManager(SearchRecordPlugin.class, T2Map.LOCALE);

    // 選択レイヤ
    private List<LayerData> layerList = new ArrayList<LayerData>();

    // 空間範囲
    private Point2D p1 = null;

    private Point2D p2 = null;

    // 時間範囲
    // private BigDecimal start; // 開始時間
    // private BigDecimal end; // 終了時間
    protected List<BigDecimal[]> times = new ArrayList<BigDecimal[]>();

    // 文字列（イベント）検索
    private EventQuery eq1 = new EventQuery(); // イベント情報

    private EventQuery eq2 = new EventQuery();

    private EventQuery eq3 = new EventQuery();

    // 表示オプション
    private boolean isHighlight = true; // ハイライトか（falseなら新規レイヤ）

    private String createLayer = null; // 作成するレイヤ

    private boolean createMask = false;

    /**
     * 選択されたレイヤを取得します。
     * 
     * @return 選択されたレイヤ
     */
    public LayerData[] getLayers()
    {
        return layerList.toArray(new LayerData[0]);
    }

    /**
     * レイヤを設定します。
     * 
     * @param layers
     */
    public void setLayers(Object[] layers)
    {
        // Initialized.
        layerList.clear();
        // 汚いコードだが、仕方がない？？？
        // 本当はObject[]でもらいたくないが、getSelectedValuesの戻り値がObject[]なので
        if (layers != null)
        {
            for (Object o : layers)
            {
                if (o instanceof LayerData)
                {
                    layerList.add((LayerData) o);
                }
            }
        }
    }

    public void setPoint1(double x, double y)
    {
        p1 = new Point2D.Double(x, y);
    }

    public Point2D getPoint1()
    {
        return p1;
    }

    public void setPoint2(double x, double y)
    {
        p2 = new Point2D.Double(x, y);
    }

    public Point2D getPoint2()
    {
        return p2;
    }

    /**
     * 時間範囲の開始時を設定します。
     * 
     * @param start
     */
    // public void setStart(BigDecimal start) {
    // this.start = start;
    // }
    /**
     * 時間範囲の開始時を設定します。
     * 
     * @param start
     * @throws ISO8601Exception
     */
    // public void setStart(String start) throws ISO8601Exception {
    // if (start!=null && !start.equals(""))
    // this.start = JulianGregorianDate.toJulian(start);
    // }
    /**
     * 時間範囲の開始時を取得します。
     * 
     * @return 開始時
     */
    // public BigDecimal getStart() {
    // return start;
    // }
    /**
     * 時間範囲の終了時を設定します。
     * 
     * @param end
     */
    // public void setEnd(BigDecimal end) {
    // this.end = end;
    // }
    /**
     * 時間範囲の終了時を設定します。
     * 
     * @param end
     * @throws ISO8601Exception
     */
    // public void setEnd(String end) throws ISO8601Exception {
    // if (end!=null && !end.equals("")) {
    // this.end = JulianGregorianDate.toJulian(end);
    // }
    // }
    /**
     * 時間範囲の終了時を返します。
     * 
     * @return 終了時
     */
    // public BigDecimal getEnd() {
    // return end;
    // }
    /**
     * イベント検索条件1を設定します。
     * 
     * @param eq
     */
    public void setEventQuery1(EventQuery eq)
    {
        eq1 = eq;
    }

    /**
     * イベント検索条件１を取得します。
     * 
     * @return イベント検索条件１
     */
    public EventQuery getEventQuery1()
    {
        return eq1;
    }

    /**
     * イベント検索条件２を設定します。
     * 
     * @param eq
     */
    public void setEventQuery2(EventQuery eq)
    {
        eq2 = eq;
    }

    /**
     * イベント検索条件２を取得します。
     * 
     * @return イベント検索条件２
     */
    public EventQuery getEventQuery2()
    {
        return eq2;
    }

    /**
     * イベント検索条件３を設定します。
     * 
     * @param eq
     */
    public void setEventQuery3(EventQuery eq)
    {
        eq3 = eq;
    }

    /**
     * イベント検索条件３を取得します。
     * 
     * @return イベント検索条件
     */
    public EventQuery getEventQuery3()
    {
        return eq3;
    }

    /**
     * ハイライト選択か否かを設定します。
     * 
     * @param b
     */
    public void setHighlight(boolean b)
    {
        isHighlight = b;
    }

    /**
     * ハイライト選択か否かを取得します。
     * 
     * @return ハイライト選択か否か
     */
    public boolean isHighlight()
    {
        return isHighlight;
    }

    /**
     * 新規レイヤ選択の場合に作成するレイヤ名を設定します。
     * 
     * @param layer
     */
    public void setCreateLayer(String layer)
    {
        this.createLayer = layer;
    }

    /**
     * 新規レイヤ選択の場合に作成するレイヤ名を取得します。
     * 
     * @return レイヤ名称
     */
    public String getCreateLayer()
    {
        return createLayer;
    }

    /**
     * マスクを作成するかどうかを取得します。
     * 
     * @return マスクを作成するか
     */
    public boolean isCreateMask()
    {
        return createMask;
    }

    /**
     * マスクを作成します。
     * 
     * @param b
     */
    public void setCreateMask(boolean b)
    {
        this.createMask = b;
    }

    /**
     * クエリの内容をカンマ区切りで戻します。
     * 
     * @return クエリ内容
     */
    public String getQuery()
    {
        // Create new instance.
        StringBuffer buf = new StringBuffer();
        buf.append(getQuery(getLayers()));
        buf.append(getQuery(p1, p2));
        Iterator<BigDecimal[]> it = times.iterator();
        while (it.hasNext())
        {
            BigDecimal[] n = it.next();
            buf.append(getQuery(n[0], n[1]));
        }
        buf.append(getQuery(eq1, eq2, eq3));
        return buf.toString();
    }

    private String getQuery(LayerData[] layers)
    {
        // Create new instance.
        StringBuffer buf = new StringBuffer();
        String delm = rmgr.getString("query.format.delm");
        for (LayerData l : getLayers())
        {
            if (buf.length() > 0)
            {
                buf.append(delm);
            }
            buf.append(l.getName());
        }
        return buf.toString();
    }

    private String getQuery(Point2D p1, Point2D p2)
    {
        // Create new instance.
        StringBuffer buf = new StringBuffer();
        String delm = rmgr.getString("query.format.delm");
        String s1 = (p1 != null) ? p1.getY() + delm + p1.getX() : "";
        String s2 = (p2 != null) ? p2.getY() + delm + p2.getX() : "";
        if (s1.length() > 0 || s2.length() > 0)
        {
            buf.append(delm);
            buf.append(rmgr.format("query.format.point", s1, s2));
        }
        return buf.toString();
    }

    private String getQuery(BigDecimal n1, BigDecimal n2)
    {
        // Create new instance.
        StringBuffer buf = new StringBuffer();
        String s1 = "";
        String s2 = "";
        if (n1 != null)
        {
            s1 = JulianGregorianDate.toGregorian(n1.setScale(0, BigDecimal.ROUND_HALF_UP).toBigInteger());
        }
        if (n2 != null)
        {
            s2 = JulianGregorianDate.toGregorian(n2.setScale(0, BigDecimal.ROUND_HALF_UP).toBigInteger());
        }
        if (s1.length() > 0 || s2.length() > 0)
        {
            buf.append(rmgr.getString("query.format.delm"));
            buf.append(rmgr.format("query.format.date", s1, s2));
        }
        return buf.toString();
    }

    private String getQuery(EventQuery... events)
    {
        // Create new instance.
        StringBuffer buf = new StringBuffer();
        for (EventQuery evt : events)
        {
            if (evt != null)
            {
                String type = evt.getType();
                String key = evt.getKey();
                Operator ope = evt.getAssign();
                if (key != null && key.trim().length() > 0)
                {
                    if (buf.length() > 0)
                    {
                        buf.append(rmgr.getString("query.format.delm"));
                    }
                    if (ope != null)
                    {
                        buf.append(rmgr.format("query.format.item.number", type, ope, key));
                    }
                    else
                    {
                        buf.append(rmgr.format("query.format.item.other", type, key));
                    }
                }
            }
        }
        if (buf.length() > 0)
        {
            String s = buf.toString();
            buf = new StringBuffer(rmgr.getString("query.format.delm"));
            buf.append(rmgr.format("query.format.item", s));
        }
        return buf.toString();
    }

    public void setTime(List<JTextField[]> timeTF) throws ISO8601Exception
    {
        Iterator<JTextField[]> it = timeTF.iterator();
        while (it.hasNext())
        {
            JTextField[] n = it.next();
            BigDecimal[] x = { JulianGregorianDate.toJulian(n[0].getText()), JulianGregorianDate.toJulian(n[1].getText()) };
            times.add(x);
        }
    }

    public List<BigDecimal[]> getTime()
    {
        return times;
    }
}
