/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin.analyzer.searchrecord;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.plugin.LayerData;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.plugin.analyzer.AnalyzeConstants.Operator;
import jp.nihu.gttime.time.utility.ISO8601Exception;
import jp.nihu.gttime.time.utility.JulianGregorianDate;
import jp.nihu.gttime.utility.TextValidator;
import jp.nihu.gttime.utility.TextValidator.DATE;
import jp.nihu.gttime.utility.TextValidator.NUMBER;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * レイヤについての検索機能（プラグイン）を実装するクラスです。
 */
public class SearchRecordDialog extends JDialog
{
    // シリアル番号
    private static final long serialVersionUID = 1L;

    // ロギング
    private static Log logger = LogFactory.getLog(SearchRecordDialog.class);

    // 呼び出し元のプラグイン
    protected SearchRecordPlugin plugin = null;

    // リソースバンドルの管理
    protected static PluginResourceManager rmgr = PluginResourceManager.getManager(SearchRecordPlugin.class, T2Map.LOCALE);

    // 共通コンポーネント群
    /**
     * リスト
     */
    protected JList layerList = null;

    /**
     * コンボボックス
     */
    // レイヤ選択
    protected JComboBox layerCmb = null;

    // 属性選択
    protected JComboBox attrCmb1 = null;

    protected JComboBox attrCmb2 = null;

    protected JComboBox attrCmb3 = null;

    // 演算子選択
    protected JComboBox assignCmb1 = null;

    protected JComboBox assignCmb2 = null;

    protected JComboBox assignCmb3 = null;

    /**
     * テキストフィールド
     */
    protected final JTextField strTF1 = new JTextField(25);

    protected final JTextField strTF2 = new JTextField(25);

    protected final JTextField strTF3 = new JTextField(25);

    protected final JTextField numTF1 = new JTextField(15);

    protected final JTextField numTF2 = new JTextField(15);

    protected final JTextField numTF3 = new JTextField(15);

    // protected final JTextField timeStartTF = new JTextField(10);
    protected List<JTextField[]> timeTF = new ArrayList<JTextField[]>();

    // protected final JTextField timeEndTF = new JTextField(10);

    protected final JTextField latTF1 = new JTextField(5);

    protected final JTextField longTF1 = new JTextField(5);

    protected final JTextField latTF2 = new JTextField(5);

    protected final JTextField longTF2 = new JTextField(5);

    /**
     * ラジオボタン
     */
    protected ButtonGroup optBGrp = new ButtonGroup(); // ボタングループ

    protected JRadioButton hlRB = null; // ハイライト

    protected JRadioButton nlRB = null; // 新規レイヤ

    /**
     * チェックボックス
     */
    protected JCheckBox maskCB = new JCheckBox();

    /**
     * ボタン
     */
    protected JButton searchBtn = null; // 検索ボタン

    protected JButton cancelBtn = null; // キャンセルボタン

    Map<String, String> layers = null;

    BigDecimal[][] durations = null;

    /**
     * コンストラクタ
     * 
     * @param frame 親フレーム
     * @param isModal ダイアログをモーダルにするか
     * @param plugin データソースとなるプラグイン
     */
    public SearchRecordDialog(Frame frame, boolean isModal, SearchRecordPlugin plugin, BigDecimal[][] durations)
    {
        super(frame, isModal); // モーダルにしない
        this.plugin = plugin;
        this.durations = durations;
        initComponent();
        initDialog();
    }

    /**
     * 共通コンポーネントの初期化
     */
    protected void initComponent()
    {
        // 検索領域
        attrCmb1 = createComboBox(SearchService.getEventNames(T2Map.getProject()), "attrCmb1", new EventNameSelectActionListener());
        attrCmb2 = createComboBox(SearchService.getEventNames(T2Map.getProject()), "attrCmb2", new EventNameSelectActionListener());
        attrCmb3 = createComboBox(SearchService.getEventNames(T2Map.getProject()), "attrCmb3", new EventNameSelectActionListener());
        assignCmb1 = createComboBox(SearchService.getNumberAssigns(), "assignCombo1", null);
        assignCmb2 = createComboBox(SearchService.getNumberAssigns(), "assignCombo2", null);
        assignCmb3 = createComboBox(SearchService.getNumberAssigns(), "assignCombo3", null);
        assignCmb1.setVisible(false);
        assignCmb2.setVisible(false);
        assignCmb3.setVisible(false);

        // 表示オプション領域
        hlRB = createRadioButton(rmgr.getString("label.hl"), new OptionSelectActionListener());
        nlRB = createRadioButton(rmgr.getString("label.nl"), new OptionSelectActionListener());
        layers = SearchService.getLayerNames(plugin);

        layerCmb = createComboBox(layers.keySet(), "layerCombo", null);
        layerCmb.setEnabled(false);
        for (int i = 0; i < layerCmb.getItemCount(); i++)
        {
            String s = layerCmb.getItemAt(i).toString();
            if (rmgr.getString("default.value.layer").equalsIgnoreCase(s))
            {
                layerCmb.setSelectedIndex(i);
                break;
            }
        }

        // hlRB.addChangeListener(new ChangeListener() {
        // public void stateChanged(ChangeEvent e)
        // {
        // JRadioButton r = (JRadioButton)e.getSource();
        // maskCB.setEnabled(!r.isSelected());
        // }
        // });
        //        
        // nlRB.addChangeListener(new ChangeListener() {
        // public void stateChanged(ChangeEvent e)
        // {
        // JRadioButton r = (JRadioButton)e.getSource();
        // maskCB.setEnabled(r.isSelected());
        // }
        // });

        optBGrp.add(hlRB);
        optBGrp.add(nlRB);
        optBGrp.setSelected(hlRB.getModel(), true); // デフォルトはハイライト
        maskCB.setEnabled(false); // デフォルトは使用不可

        // ボタン領域
        // 検索ボタン
        searchBtn = createButton(rmgr.getString("label.button.search"), new SearchActionListener());
        searchBtn.setEnabled(false);
        // キャンセルボタン
        cancelBtn = createButton(rmgr.getString("label.button.cancel"), new CancelActionListener());

        // 初期値設定
        // BigDecimal ds = plugin.getT2Map().getTimeAxisData().getCurrentMin();
        // BigDecimal de = plugin.getT2Map().getTimeAxisData().getCurrentMax();

        for (int i = 0; i < durations.length; i++)
        {
            JTextField a = new JTextField(10);
            JTextField b = new JTextField(10);
            a.setText(JulianGregorianDate.toGregorian(durations[i][0].setScale(0, BigDecimal.ROUND_HALF_UP).toBigInteger()));
            b.setText(JulianGregorianDate.toGregorian(durations[i][1].setScale(0, BigDecimal.ROUND_HALF_UP).toBigInteger()));
            JTextField[] t = { a, b };
            timeTF.add(t);
        }

        if (!(timeTF.size() > 0))
        {
            // 2009-10-27に変更
            // BigDecimal ds =
            // plugin.getT2Map().getTimeAxisData().getCurrentMin();
            // BigDecimal de =
            // plugin.getT2Map().getTimeAxisData().getCurrentMax();
            BigDecimal ds = plugin.getT2Map().getTimeAxisData().getInitialMin();
            BigDecimal de = plugin.getT2Map().getTimeAxisData().getInitialMax();

            JTextField a = new JTextField(10);
            JTextField b = new JTextField(10);
            a.setText(JulianGregorianDate.toGregorian(ds.setScale(0, BigDecimal.ROUND_HALF_UP).toBigInteger()));
            b.setText(JulianGregorianDate.toGregorian(de.setScale(0, BigDecimal.ROUND_HALF_UP).toBigInteger()));
            JTextField[] t = { a, b };
            timeTF.add(t);
        }

        latTF1.setText(rmgr.getString("default.value.lat1"));
        latTF2.setText(rmgr.getString("default.value.lat2"));
        longTF1.setText(rmgr.getString("default.value.long1"));
        longTF2.setText(rmgr.getString("default.value.long2"));
    }

    /**
     * 検索ダイアログを作成作成します。
     */
    protected void initDialog()
    {
        logger.debug("start");

        JScrollPane spane = new JScrollPane();
        JPanel top = new JPanel();
        
        top.add(spane);
        spane.getViewport().add(createTopPanel());
        JPanel middle = createMiddlePanel();
        JPanel bottom = createBottomPanel();
        JPanel main = createMainPanel(top, middle, bottom);
        add(main);

        setSize(new java.awt.Dimension(rmgr.getInt("dialog.dimension.x"), rmgr.getInt("dialog.dimension.y")));
        setLocation(plugin.getT2Map().getX(), plugin.getT2Map().getY());
        setTitle(rmgr.getString("label.title"));

        // ダイアログサイズの決定
        pack();
        // リサイズ不可
        setResizable(false);
        // ダイアログを親ウインドウの中央に配備
        setLocationRelativeTo(getOwner());

        setVisible(true);
    }

    /**
     * 検索ダイアログのトップを作成します。
     * 
     * @return トップ部分のパネル
     */
    protected JPanel createTopPanel()
    {
        // レイヤ選択
        JLabel layerLbl = new JLabel(rmgr.getString("label.layerselect"));

        // 空間範囲
        JLabel spaceLbl = new JLabel(rmgr.getString("label.spacerange"));
        JLabel latLbl1 = new JLabel(rmgr.getString("label.lat1"));
        JLabel latLbl2 = new JLabel(rmgr.getString("label.lat2"));
        JLabel longLbl1 = new JLabel(rmgr.getString("label.long1"));
        JLabel longLbl2 = new JLabel(rmgr.getString("label.long2"));

        // 時間範囲
        JLabel timeLbl = new JLabel(rmgr.getString("label.timerange"));
        // JLabel hiphenLbl = new JLabel(rmgr.getString("label.hyphen"));

        // 検索フィールドラベル
        JLabel strLbl1 = new JLabel(rmgr.getString("label.string"));
        JLabel strLbl2 = new JLabel(rmgr.getString("label.string"));
        JLabel strLbl3 = new JLabel(rmgr.getString("label.string"));
        JLabel andLbl1 = new JLabel(rmgr.getString("label.and"));
        JLabel andLbl2 = new JLabel(rmgr.getString("label.and"));

        // トップパネル
        JPanel top = new JPanel();
        top.setLayout(new GridBagLayout());
        // GridBagLayout 配置用オブジェクト
        GridBagConstraints c = new GridBagConstraints();
        // 初期化
        c.gridx = c.gridy = 0;
        c.ipadx = c.ipady = 2; // パディング

        top.setBorder((BorderFactory.createTitledBorder(rmgr.getString("label.searchquery"))));

        // レイヤ選択
        // ラベル
        c.anchor = GridBagConstraints.EAST;
        top.add(layerLbl, c);
        c.gridx++;

        // 本体
        LayerData[] datas = SearchService.getLayerDatas(T2Map.getProject());

        JPanel layerPanel = new JPanel();
        JScrollPane layerScroll = new JScrollPane();
        layerList = createJList(datas, new ListChangedActionListener());
        layerScroll.getViewport().add(layerList);
        layerScroll.setPreferredSize(new Dimension(rmgr.getInt("scroll.dimension.x"), rmgr.getInt("scroll.dimension.y")));
        layerPanel.add(layerScroll);
        c.anchor = GridBagConstraints.WEST;
        top.add(layerPanel, c);
        // グリッド Y 座標移動
        c.gridy++;
        // グリッド X 初期化
        c.gridx = 0;

        // 空間範囲
        // ラベル
        c.anchor = GridBagConstraints.EAST;
        top.add(spaceLbl, c);
        c.gridx++;

        // 本体部分
        JPanel spacePanel1 = new JPanel();
        spacePanel1.setLayout(new FlowLayout(FlowLayout.LEFT));
        JPanel spacePanel2 = new JPanel();
        spacePanel2.setLayout(new BoxLayout(spacePanel2, BoxLayout.Y_AXIS));
        // 緯度（１）..の部分
        JPanel spacePanel3 = new JPanel();
        spacePanel3.setLayout(new FlowLayout(FlowLayout.LEFT));
        spacePanel3.add(latLbl1);
        spacePanel3.add(longLbl1);
        spacePanel3.add(latLbl2);
        spacePanel3.add(longLbl2);
        spacePanel2.add(spacePanel3);
        // テキストフィールド部分
        JPanel spacePanel4 = new JPanel();
        spacePanel4.setLayout(new FlowLayout(FlowLayout.LEFT));
        spacePanel4.add(latTF1);
        spacePanel4.add(longTF1);
        spacePanel4.add(latTF2);
        spacePanel4.add(longTF2);
        spacePanel2.add(spacePanel4);

        spacePanel1.add(spacePanel2);

        // 四角形
        JPanel figure = new JPanel();
        JLabel lbl1 = new JLabel("1");
        lbl1.setPreferredSize(new Dimension(10, 50));
        lbl1.setVerticalAlignment(JLabel.TOP);
        JLabel lbl2 = new JLabel("2");
        lbl2.setPreferredSize(new Dimension(10, 50));
        lbl2.setVerticalAlignment(JLabel.BOTTOM);
        figure.add(lbl1);
        JPanel square = new JPanel();
        square.setPreferredSize(new Dimension(50, 50));
        square.setLayout(new BoxLayout(square, BoxLayout.X_AXIS));

        TitledBorder border = new TitledBorder(new LineBorder(Color.black), "");
        border.setTitleFont(new Font("Dialog", Font.PLAIN, 12));
        border.setTitleColor(Color.black);
        square.setBorder(border);

        figure.add(square);
        figure.add(lbl2);
        spacePanel1.add(figure);

        c.anchor = GridBagConstraints.WEST;
        top.add(spacePanel1, c);
        // グリッド Y 座標移動
        c.gridy++;
        // グリッド X 初期化
        c.gridx = 0;

        // 時間範囲
        // ラベル
        c.anchor = GridBagConstraints.EAST;
        top.add(timeLbl, c);
        c.gridx++;

        // 本体1
        timePanelTop = new JPanel();
        timePanelTop.setLayout(new BoxLayout(timePanelTop, BoxLayout.Y_AXIS));

        Iterator<JTextField[]> it = timeTF.iterator();
        while (it.hasNext())
        {
            JTextField[] x = it.next();
            JLabel hiphenLblxx = new JLabel(rmgr.getString("label.hyphen"));
            // JTextField timeStartTFxx = new JTextField(10);
            // JTextField timeEndTFxx = new JTextField(10);
            JPanel timePanelxx = new JPanel();
            timePanelxx.setLayout(new FlowLayout(FlowLayout.LEFT));
            timePanelxx.add(createDeleteTimeFormButton(timePanelTop, timePanelxx));
            timePanelxx.add(x[0]);
            timePanelxx.add(hiphenLblxx);
            timePanelxx.add(x[1]);
            timePanelTop.add(timePanelxx);
        }

        // JPanel timePanel = new JPanel();
        // timePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
        // timePanel.add(createDeleteTimeFormButton(timePanelTop, timePanel));
        // timePanel.add(timeStartTF);
        // timePanel.add(hiphenLbl);
        // timePanel.add(timeEndTF);
        // timePanelTop.add(timePanel);

        JPanel timePanel3 = new JPanel();
        timePanel3.setLayout(new FlowLayout(FlowLayout.LEFT));

        JButton addButton = new JButton("+");
        addButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                JLabel hiphenLblx = new JLabel(rmgr.getString("label.hyphen"));
                JTextField timeStartTFx = new JTextField(10);
                JTextField timeEndTFx = new JTextField(10);

                JPanel timePanelx = new JPanel();
                timePanelx.setLayout(new FlowLayout(FlowLayout.LEFT));
                timePanelx.add(createDeleteTimeFormButton(timePanelTop, timePanelx));
                timePanelx.add(timeStartTFx);
                timePanelx.add(hiphenLblx);
                timePanelx.add(timeEndTFx);
                timePanelTop.add(timePanelx, timePanelTop.getComponentCount() - 1);
                timePanelTop.updateUI();
            }
        });
        timePanel3.add(addButton);
        timePanelTop.add(timePanel3);

        c.anchor = GridBagConstraints.WEST;
        top.add(timePanelTop, c);
        // グリッド Y 座標移動
        c.gridy++;
        // グリッド X 初期化
        c.gridx = 0;

        // 文字列検索
        numTF1.setVisible(false);
        numTF2.setVisible(false);
        numTF3.setVisible(false);

        // 文字列１
        JPanel strPanel1 = new JPanel();
        strPanel1.setLayout(new FlowLayout(FlowLayout.LEFT));
        strPanel1.add(attrCmb1);
        strPanel1.add(assignCmb1);
        strPanel1.add(strTF1);
        strPanel1.add(numTF1);
        strPanel1.add(andLbl1);

        c.anchor = GridBagConstraints.EAST;
        top.add(strLbl1, c);
        c.gridx++;
        c.anchor = GridBagConstraints.WEST;
        top.add(strPanel1, c);
        // グリッド Y 座標移動
        c.gridy++;
        // グリッド X 初期化
        c.gridx = 0;

        // 文字列２
        JPanel strPanel2 = new JPanel();
        strPanel2.setLayout(new FlowLayout(FlowLayout.LEFT));
        strPanel2.add(attrCmb2);
        strPanel2.add(assignCmb2);
        strPanel2.add(strTF2);
        strPanel2.add(numTF2);
        strPanel2.add(andLbl2);

        c.anchor = GridBagConstraints.EAST;
        top.add(strLbl2, c);
        c.gridx++;
        c.anchor = GridBagConstraints.WEST;
        top.add(strPanel2, c);
        // グリッド Y 座標移動
        c.gridy++;
        // グリッド X 初期化
        c.gridx = 0;

        // 文字列３
        JPanel strPanel3 = new JPanel();
        strPanel3.setLayout(new FlowLayout(FlowLayout.LEFT));
        strPanel3.add(attrCmb3);
        strPanel3.add(assignCmb3);
        strPanel3.add(strTF3);
        strPanel3.add(numTF3);

        c.anchor = GridBagConstraints.EAST;
        top.add(strLbl3, c);
        c.gridx++;
        c.anchor = GridBagConstraints.WEST;
        top.add(strPanel3, c);

        return top;
    }

    private JPanel timePanelTop = null;

    private JButton createDeleteTimeFormButton(final JPanel parent, final JPanel panel)
    {
        JButton button = new JButton("-");
        button.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                parent.remove(panel);
                parent.updateUI();
            }
        });
        return button;
    }

    /**
     * 検索ダイアログのミドルを作成します。
     * 
     * @return ミドル部分のパネル
     */
    protected JPanel createMiddlePanel()
    {
        // マスク
        JLabel maskLbl = new JLabel(rmgr.getString("label.mask"));

        JPanel middle = new JPanel();
        middle.setLayout(new BoxLayout(middle, BoxLayout.X_AXIS));
        /*
         * TitledBorder border = new TitledBorder( new LineBorder(Color.black),
         * rmgr.getString("label.opt")); border.setTitleFont(new Font("Dialog",
         * Font.PLAIN, 12)); border.setTitleColor(Color.black);
         * middle.setBorder(border);
         */
        middle.setBorder((BorderFactory.createTitledBorder(rmgr.getString("label.opt"))));
        middle.add(hlRB);
        middle.add(nlRB);
        middle.add(layerCmb);
        middle.add(maskCB);
        middle.add(maskLbl);

        return middle;
    }

    /**
     * 検索ダイアログのボトムを作成します。
     * 
     * @return ボトム部分のパネル
     */
    protected JPanel createBottomPanel()
    {

        JPanel bottom = new JPanel();
        bottom.setLayout(new FlowLayout(FlowLayout.CENTER));
        bottom.add(searchBtn);
        bottom.add(cancelBtn);
        return bottom;
    }

    /**
     * 検索ダイアログのメインパネルを作成します。
     * 
     * @param top
     * @param middle
     * @param bottom
     * @return メインパネル
     */
    private JPanel createMainPanel(JPanel top, JPanel middle, JPanel bottom)
    {
        // メインパネル
        JPanel main = new JPanel();
        // main.setLayout(new FlowLayout(FlowLayout.LEFT, 15, 5));
        main.setLayout(new BoxLayout(main, BoxLayout.Y_AXIS));
        main.add(top);
        main.add(middle);
        main.add(bottom);
        return main;
    }

    /**
     * ボタンを作成します。
     * 
     * @param name ボタン名
     * @param listener
     * @return ボタン
     */
    protected JButton createButton(String name, ActionListener listener)
    {
        /* 検索ボタン */
        JButton btn = new JButton(name);
        btn.addActionListener(listener);

        return btn;
    }

    /**
     * ラジオボタンを作成します。
     * 
     * @param name ボタン名
     * @param listener
     * @return ラジオボタン
     */
    protected JRadioButton createRadioButton(String name, ActionListener listener)
    {
        JRadioButton radio = new JRadioButton(name, false);
        radio.addActionListener(listener);
        radio.setFont(new Font("dialog", Font.PLAIN, 12));
        return radio;
    }

    /**
     * コンボボックスを作成します。
     * 
     * @param items 選択できるアイテム
     * @param listener
     * @return コンボボックス
     */
    protected JComboBox createComboBox(Collection<?> items, String cmd, ActionListener listener)
    {
        JComboBox combo = new JComboBox();
        if (cmd != null)
        {
            combo.setActionCommand(cmd);
        }
        combo.setFont(new Font("dialog", Font.PLAIN, 12));
        for (Object item : items)
        {
            combo.addItem(item);
        }
        combo.addActionListener(listener);
        Dimension d = combo.getPreferredSize();
        d.width += rmgr.getInt("combo.width.inc");
        combo.setPreferredSize(d);
        return combo;
    }

    /**
     * セレクト用のリストを作成します。
     * 
     * @param datas
     * @param listener
     * @return リスト
     */
    protected JList createJList(LayerData[] datas, ListSelectionListener listener)
    {
        JList list = new JList();
        list.addListSelectionListener(listener);
        list.setListData(datas);

        return list;
    }

    /**
     * ActionListeners
     */

    /**
     * 検索ボタンが押された時に実行されるリスナー
     */
    class SearchActionListener implements ActionListener
    {

        /*
         * (非 Javadoc)
         * 
         * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
         */
        public void actionPerformed(ActionEvent ae)
        {
            // 入力チェック
            if (!validateText())
                return;

            // クエリの設定
            Query query = new Query();
            // 選択レイヤー設定
            query.setLayers(layerList.getSelectedValues());
            // 空間範囲クエリ設定
            String lat1 = latTF1.getText();
            String long1 = longTF1.getText();
            String lat2 = latTF2.getText();
            String long2 = longTF2.getText();
            if (lat1 != null && !lat1.equals("") && long1 != null && !long1.equals(""))
                query.setPoint1(Double.parseDouble(long1), Double.parseDouble(lat1));
            if (lat2 != null && !lat2.equals("") && long2 != null && !long2.equals(""))
                query.setPoint2(Double.parseDouble(long2), Double.parseDouble(lat2));

            // 時間範囲クエリ設定
            try
            {
                // query.setStart(timeStartTF.getText());
                // query.setEnd(timeEndTF.getText());
                query.setTime(timeTF);
            }
            catch (ISO8601Exception e)
            {
                // // ありえないけど。。。
                // timeStartTF.setBackground(Color.RED);
                // timeEndTF.setBackground(Color.RED);
                return;
            }

            // 文字列クエリ設定
            if (assignCmb1.isVisible())
            {
                query.setEventQuery1(new EventQuery((String) attrCmb1.getSelectedItem(), (Operator) assignCmb1.getSelectedItem(), numTF1.getText()));
            }
            else
            {
                query.setEventQuery1(new EventQuery((String) attrCmb1.getSelectedItem(), null, strTF1.getText()));
            }
            if (assignCmb2.isVisible())
            {
                query.setEventQuery2(new EventQuery((String) attrCmb2.getSelectedItem(), (Operator) assignCmb2.getSelectedItem(), numTF2.getText()));
            }
            else
            {
                query.setEventQuery2(new EventQuery((String) attrCmb2.getSelectedItem(), null, strTF2.getText()));
            }
            if (assignCmb3.isVisible())
            {
                query.setEventQuery3(new EventQuery((String) attrCmb3.getSelectedItem(), (Operator) assignCmb3.getSelectedItem(), numTF3.getText()));
            }
            else
            {
                query.setEventQuery3(new EventQuery((String) attrCmb3.getSelectedItem(), null, strTF3.getText()));
            }

            // ハイライト選択 or 新規レイヤ
            query.setHighlight(optBGrp.isSelected(hlRB.getModel()));
            // レイヤ型
            query.setCreateLayer(layers.get((String) layerCmb.getSelectedItem()));

            // マスク生成
            query.setCreateMask(maskCB.isSelected());

            // 検索処理の実行
            if (T2Map.getProject() != null)
            {
                SearchService service = new SearchService();
                int count = service.search(query);
                plugin.setResult(service.getResult());
                // Log [debug]
                logger.debug("count=" + count);

                if (count > 0)
                {
                    dispose();
                }
                else
                {
                    JOptionPane.showMessageDialog(plugin.getT2Map(), rmgr.getString("warn.norecord.message"), rmgr.getString("function.title"), JOptionPane.WARNING_MESSAGE);
                }
            }
        }

        /**
         * 検索フォームのエラーチェックをします。
         * 
         * @return 入力が正しいか
         */
        /*
         * protected boolean validateForm() { // エラーがあるか boolean flag = true; //
         * デフォルト背景色 Color c1 = Color.WHITE; // エラー時の背景色 Color c2 = Color.RED; //
         * レイヤー選択 if (layerList.getSelectedValues()==null ||
         * layerList.getSelectedValues().length==0) { flag = false;
         * layerList.setBackground(c2); } else { layerList.setBackground(c1); } //
         * 日付入力がある場合は日付前後のチェック if (timeStartTF.getText()!=null &&
         * !timeStartTF.getText().equals("") && timeEndTF.getText()!=null &&
         * !timeEndTF.getText().equals("")) { try { SimpleDateFormat sdf = new
         * SimpleDateFormat("yyyy-MM-dd"); Date d1 =
         * sdf.parse(timeStartTF.getText()); Date d2 =
         * sdf.parse(timeEndTF.getText()); if(d1.compareTo(d2)<0) {
         * timeStartTF.setBackground(c1); timeEndTF.setBackground(c1); } else {
         * flag=false; timeStartTF.setBackground(c2);
         * timeEndTF.setBackground(c2); } } catch (ParseException e) { //
         * 処理しなくてもいいけど一応。。。 flag=false; timeStartTF.setBackground(c2);
         * timeEndTF.setBackground(c2); } } return flag; }
         */

        private boolean validateText()
        {
            List<TextValidator> vs = new ArrayList<TextValidator>();
            vs.add(new NUMBER(latTF1, "##0.####", -90, 90));
            vs.add(new NUMBER(longTF1, "##0.####", -180, 180));
            vs.add(new NUMBER(latTF2, "##0.####", -90, 90));
            vs.add(new NUMBER(longTF2, "##0.####", -180, 180));
            Iterator<JTextField[]> it = timeTF.iterator();
            while (it.hasNext())
            {
                JTextField[] x = it.next();
                vs.add(new DATE(x[0], "y-MM-dd"));
                vs.add(new DATE(x[1], "y-MM-dd"));
            }
            if (numTF1.isVisible())
                vs.add(new NUMBER(numTF1, "#####0.####"));
            if (numTF2.isVisible())
                vs.add(new NUMBER(numTF2, "#####0.####"));
            if (numTF3.isVisible())
                vs.add(new NUMBER(numTF3, "#####0.####"));
            return TextValidator.validate(vs);
        }
    }

    /**
     * キャンセルボタンが押されたときに実行するリスナー
     */
    class CancelActionListener implements ActionListener
    {
        public void actionPerformed(ActionEvent ae)
        {
            dispose();
        }
    }

    /**
     * 表示オプションのハイライトと新規レイヤが選択されたときに実行するリスナー
     */
    class OptionSelectActionListener implements ActionListener
    {

        public void actionPerformed(ActionEvent e)
        {
            boolean on = !optBGrp.isSelected(hlRB.getModel());
            layerCmb.setEnabled(on);
            maskCB.setEnabled(on);
        }
    }

    /**
     * 文字列検索の属性であるイベント名が選択されたときに実行するリスナー
     */
    class EventNameSelectActionListener implements ActionListener
    {

        public void actionPerformed(ActionEvent e)
        {
            Object o = e.getSource();
            String cmd = e.getActionCommand();

            if (!(o instanceof JComboBox) || cmd == null || cmd.equals(""))
            {
                return;
            }
            else
            {
                JComboBox cb = (JComboBox) e.getSource();
                String name = (String) cb.getSelectedItem();

                if (name != null && name.equals("number"))
                {
                    if (cmd.equals("attrCmb1"))
                    {
                        strTF1.setVisible(false);
                        assignCmb1.setVisible(true);
                        numTF1.setVisible(true);
                        numTF1.setBackground(Color.WHITE);
                    }
                    else if (cmd.equals("attrCmb2"))
                    {
                        strTF2.setVisible(false);
                        assignCmb2.setVisible(true);
                        numTF2.setVisible(true);
                        numTF2.setBackground(Color.WHITE);
                    }
                    else
                    {
                        strTF3.setVisible(false);
                        assignCmb3.setVisible(true);
                        numTF3.setVisible(true);
                        numTF3.setBackground(Color.WHITE);
                    }
                }
                else
                {
                    if (cmd.equals("attrCmb1"))
                    {
                        strTF1.setVisible(true);
                        assignCmb1.setVisible(false);
                        numTF1.setVisible(false);
                    }
                    else if (cmd.equals("attrCmb2"))
                    {
                        strTF2.setVisible(true);
                        assignCmb2.setVisible(false);
                        numTF2.setVisible(false);
                    }
                    else
                    {
                        strTF3.setVisible(true);
                        assignCmb3.setVisible(false);
                        numTF3.setVisible(false);
                    }
                }
            }
        }
    }

    /**
     * リストが選択されていない場合には検索ボタンを使用できないようにするリスナー
     */
    class ListChangedActionListener implements ListSelectionListener
    {
        /*
         * (非 Javadoc)
         * 
         * @see javax.swing.event.ListSelectionListener#valueChanged(javax.swing.event.ListSelectionEvent)
         */
        public void valueChanged(ListSelectionEvent e)
        {
            if (e.getValueIsAdjusting())
            {
                Object o = e.getSource();
                if (o instanceof JList)
                {
                    searchBtn.setEnabled(!((JList) o).isSelectionEmpty());
                }
            }
        }
    }
}
