/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin.analyzer.searchrecord;

import javax.swing.JOptionPane;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.data.DatasetIF;
import jp.nihu.gttime.data.ProjectIF;
import jp.nihu.gttime.plugin.AnalizePlugin;
import jp.nihu.gttime.plugin.PluginManager;
import jp.nihu.gttime.plugin.PluginResourceManager;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * レイヤについての検索機能（プラグイン）を実装するクラスです。
 */
public class SearchRecordPlugin extends AnalizePlugin
{
    /**
     * シリアルバーションID
     */
    private static Log logger = LogFactory.getLog(SearchRecordPlugin.class);

    // リソースバンドルの管理
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(SearchRecordPlugin.class, T2Map.LOCALE);

    // 本体に戻る結果
    private DatasetIF[] result = null;

    /**
     * 検索結果として本体に戻る結果を設定します。
     * 
     * @param result
     */
    public void setResult(DatasetIF[] result)
    {
        this.result = result;
    }

    /*
     * (非 Javadoc)
     * 
     * @see jp.nihu.gttime.plugin.AnalizePlugin#getT2Map()
     */
    public T2Map getT2Map()
    {
        return super.getT2Map();
    }

    /**
     * プラグインマネージャーを取得します。
     * 
     * @return プラグインマネージャー
     */
    public PluginManager getPluginManager()
    {
        return super.getPluins();
    }

    /**
     * 検索ダイアログの作成
     * 
     * @param project Projectオブジェクトを初期化します。
     */
    public DatasetIF[] analize(final ProjectIF project)
    {
        // Log [debug]
        logger.debug("start");
        // 検索結果初期化
        setResult(null);

        if (T2Map.getProject() != null)
        {
            new SearchRecordDialog(getT2Map(), true, this, getT2Map().getDurations());
        }
        else
        {
            JOptionPane.showMessageDialog(getT2Map(), rmgr.getString("warn.output.message"), rmgr.getString("function.title"), JOptionPane.WARNING_MESSAGE);
        }

        return result;
    }
}
