/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin.io;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Locale;
import java.util.Map;
import java.util.TreeMap;
import java.util.concurrent.Callable;
import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.FutureTask;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.T2MapFileChooser;
import jp.nihu.gttime.T2MapProgressBarDialog;
import jp.nihu.gttime.data.DatasetException;
import jp.nihu.gttime.data.DatasetGtm;
import jp.nihu.gttime.data.DatasetIF;
import jp.nihu.gttime.data.DatasetTmm;
import jp.nihu.gttime.data.ProjectException;
import jp.nihu.gttime.data.ProjectGts;
import jp.nihu.gttime.data.ProjectIF;
import jp.nihu.gttime.data.ProjectTms;
import jp.nihu.gttime.data.Record;
import jp.nihu.gttime.plugin.ImportPlugin;
import jp.nihu.gttime.plugin.LayerPlugin;
import jp.nihu.gttime.plugin.OpenPluginException;
import jp.nihu.gttime.plugin.PluginManager;
import jp.nihu.gttime.plugin.PluginManagerException;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.utility.ExtensionFileFilter;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * csvファイルをインポートするクラスです。
 */
public class CSVImportPlugin extends ImportPlugin
{
    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(CSVImportPlugin.class);

    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(CSVImportPlugin.class, T2Map.LOCALE);

    /**
     * アクセサリコンポーネントのパネルオブジェクト
     */
    private JPanel jPanel = null;

    /**
     * 「データセット名」のラベルオブジェクト
     */
    private JLabel jLabel = null;

    /**
     * 「データセット名」のTextFieldオブジェクト
     */
    private JTextField jTextField = null;

    /**
     * 「表示レイヤタイプ」のラベルオブジェクト
     */
    private JLabel jLabel1 = null;

    /**
     * 「文字コード」のラベルオブジェクト
     */
    private JLabel jLabel2 = null;

    /**
     * 「表示レイヤタイプ」のセレクトボックスオブジェクト
     */
    private JComboBox jComboBox = null;

    /**
     * 「文字コード」のセレクトボックスオブジェクト
     */
    private JComboBox jComboBox1 = null;

    /**
     * ファイル選択画面を表示します。
     * 
     * @return project
     * @throws OpenPluginException
     */
    public ProjectIF open() throws OpenPluginException
    {
        log.debug("Import Records from CSV file.");
        ProjectIF project = null;
        UIManager.put("FileChooser.readOnly", Boolean.TRUE);
        JFileChooser chooser = new T2MapFileChooser();
        ExtensionFileFilter filter = new ExtensionFileFilter(rmgr.getString("file.filter"));
        chooser.setFileFilter(filter);
        JPanel jpanel = getJPanel();
        chooser.setAccessory(jpanel);
        int returnVal = chooser.showOpenDialog(getParentComponent());
        if (returnVal == JFileChooser.APPROVE_OPTION)
        {
            project = open(chooser.getSelectedFile(), null);
        }
        UIManager.put("FileChooser.readOnly", Boolean.FALSE);
        return project;
    }

    @Override
    public ExtensionFileFilter getExtensionFileFilter() {
    	return new ExtensionFileFilter(rmgr.getString("file.filter"));
    }

    @Override
    public ProjectIF open(File file, ProjectIF insertToProject) throws OpenPluginException
    {
    	String title = jTextField.getText();
    	Object layertype = jComboBox.getSelectedItem();
    	String code = jComboBox1.getSelectedItem().toString();
    	
        int ptype = T2Map.getProjectType();
        if (ptype == -1)
        {
            T2Map.setProjectType(1);
            ptype = 1;
        }
        Frame frame = (Frame) SwingUtilities.getRoot(getParentComponent());
        String dtitle = rmgr.getString("progress.title");
        String dmessage = rmgr.format("progress.message", file.getAbsolutePath());
        T2MapProgressBarDialog<Object> progress = new T2MapProgressBarDialog<Object>(frame, true, dtitle, dmessage);
        Callable<Object> callable = (Callable<Object>) new LoadProject(file, title, layertype, code);
        FutureTask<Object> f = (FutureTask<Object>) progress.submit(callable);
        progress.setVisible(true);
        T2Map tmap = CSVImportPlugin.this.getParentComponent();
        DatasetIF ds = null;
        ProjectIF project = null;
        try
        {
            ds = (DatasetIF) f.get();
            if (insertToProject != null)
            {// 親のプロジェクトが存在する場合、そのプロジェクトの最後尾にデータセットを追加する。
                project = insertToProject;
                try
                {
                    project.addDataset(ds);
                }
                catch (ProjectException e)
                {
                    e.printStackTrace();
                }
                tmap.updateView();
            }
            else
            {// 親のプロジェクトが存在しない場合、新しいプロジェクトを生成し、データセットを追加する。
                DatasetIF[] datasets = null;
                if (ptype == 0)
                {
                    datasets = new DatasetTmm[1];
                    datasets[0] = ds;
                    project = new ProjectTms(datasets);
                }
                else if (ptype == 1)
                {
                    datasets = new DatasetGtm[1];
                    datasets[0] = ds;
                    project = new ProjectGts(datasets);
                }
                tmap.setProject(project);
            }
        }
        catch (InterruptedException e)
        {
            throw new OpenPluginException(e.getMessage(), e);
        }
        catch (ExecutionException e)
        {
            throw new OpenPluginException(e.getMessage(), e);
        }
        catch (CancellationException e)
        {
        }
        return project;
    }

    private class LoadProject implements Callable<Object>
    {
        private File file;

        private String title;

        private Object layertype;

        private String code;

        public LoadProject(File file, String title, Object layertype, String code)
        {
            this.file = file;
            this.title = title;
            this.layertype = layertype;
            this.code = code;
        }

        public Object call() throws Exception
        {
            Record[] records = getRecords(file.getPath(), "NIHU TIMESPACE CSV", code);
            DatasetIF ds = null;
            try
            {
                // int ptype = T2Map.getProjectType();
                // if (ptype == 0)
                // {
                // ds = new DatasetTmm(title, layers.get(layertype.toString()),
                // "NIHU TIMESPACE CSV", records);
                // }
                // else if (ptype == 1)
                // {
                ds = new DatasetGtm(title, layers.get(layertype.toString()), "NIHU TIMESPACE CSV", records);
                ds.setGroups(getGroups());
                // }
            }
            catch (DatasetException e)
            {
                log.fatal("File Import Error", e);
                throw new OpenPluginException("File Import Error", e);
            }
            return ds;
        }
    }

    /**
     * ファイルを読みます。
     * 
     * @param path ファイルパス
     * @return InputStreamオブジェクト
     * @throws OpenPluginException
     */
    public InputStream getRecordsInputStream(String path) throws OpenPluginException
    {
        return getFileInputStream(path);
    }

    /**
     * ファイルを読みます。
     * 
     * @param path ファイルパス
     * @return InputStreamオブジェクト
     * @throws OpenPluginException
     */
    private InputStream getFileInputStream(String path) throws OpenPluginException
    {
        InputStream is = null;
        try
        {
            is = new FileInputStream(path);
        }
        catch (FileNotFoundException e)
        {

            throw new OpenPluginException(e.getMessage(), e);
        }
        return is;
    }

    @Override
    public JPanel getJPanel()
    {
        if (jPanel == null)
        {
            GridBagConstraints gridBagConstraints21 = new GridBagConstraints();
            gridBagConstraints21.fill = GridBagConstraints.VERTICAL;
            gridBagConstraints21.gridy = 5;
            gridBagConstraints21.weightx = 1.0;
            gridBagConstraints21.insets = new Insets(0, 5, 0, 0);
            gridBagConstraints21.anchor = GridBagConstraints.WEST;
            gridBagConstraints21.ipadx = 30;
            gridBagConstraints21.gridx = 0;
            GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
            gridBagConstraints11.fill = GridBagConstraints.VERTICAL;
            gridBagConstraints11.gridy = 3;
            gridBagConstraints11.weightx = 1.0;
            gridBagConstraints11.insets = new Insets(0, 5, 0, 0);
            gridBagConstraints11.anchor = GridBagConstraints.WEST;
            gridBagConstraints11.ipadx = 30;
            gridBagConstraints11.gridx = 0;
            GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
            gridBagConstraints3.gridx = 0;
            gridBagConstraints3.anchor = GridBagConstraints.WEST;
            gridBagConstraints3.insets = new Insets(20, 5, 0, 0);
            gridBagConstraints3.gridy = 4;
            jLabel2 = new JLabel();
            jLabel2.setText(rmgr.getString("char.encode.title"));
            GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
            gridBagConstraints2.gridx = 0;
            gridBagConstraints2.anchor = GridBagConstraints.SOUTHWEST;
            gridBagConstraints2.ipadx = 0;
            gridBagConstraints2.ipady = 0;
            gridBagConstraints2.weighty = 0.0;
            gridBagConstraints2.insets = new Insets(20, 5, 0, 0);
            gridBagConstraints2.gridy = 2;
            jLabel1 = new JLabel();
            jLabel1.setText(rmgr.getString("layer.type.title"));
            GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
            gridBagConstraints1.fill = GridBagConstraints.VERTICAL;
            gridBagConstraints1.gridy = 1;
            gridBagConstraints1.weightx = 1.0;
            gridBagConstraints1.gridwidth = 2;
            gridBagConstraints1.ipadx = 30;
            gridBagConstraints1.insets = new Insets(0, 5, 0, 0);
            gridBagConstraints1.anchor = GridBagConstraints.WEST;
            gridBagConstraints1.gridx = 0;
            GridBagConstraints gridBagConstraints = new GridBagConstraints();
            gridBagConstraints.gridx = 0;
            gridBagConstraints.insets = new Insets(0, 5, 0, 0);
            gridBagConstraints.anchor = GridBagConstraints.WEST;
            gridBagConstraints.gridy = 0;
            jLabel = new JLabel();
            jLabel.setText(rmgr.getString("dataset.title"));
            jPanel = new JPanel();
            jPanel.setLayout(new GridBagLayout());
            jPanel.setSize(new Dimension(190, 187));
            jPanel.add(jLabel, gridBagConstraints);
            jPanel.add(getJTextField(), gridBagConstraints1);
            jPanel.add(jLabel1, gridBagConstraints2);
            jPanel.add(jLabel2, gridBagConstraints3);
            jPanel.add(getJComboBox(), gridBagConstraints11);
            jPanel.add(getJComboBox1(), gridBagConstraints21);
        }
        return jPanel;
    }

    /**
     * 「データセット名」のTextFieldオブジェクト
     * 
     * @return 「データセット名」のTextFieldオブジェクト
     */
    private JTextField getJTextField()
    {
        if (jTextField == null)
        {
            jTextField = new JTextField();
            jTextField.setText(rmgr.getString("dataset.name"));
        }
        return jTextField;
    }

    /**
     * 「表示レイヤタイプ」のセレクトボックスを作成します。
     * 
     * @return 「表示レイヤタイプ」のセレクトボックスオブジェクト
     */
    private JComboBox getJComboBox()
    {
        ArrayList<String> combodata = new ArrayList<String>();
        PluginManager pm = getPlugins();
        ArrayList<Hashtable<String, String>> plugins = pm.getPlugins("LayerPlugin");
        for (int j = 0; j < plugins.size(); j++)
        {
            try
            {
                LayerPlugin layer = (LayerPlugin) pm.newInstance(plugins.get(j).get("class"));
                layers.put(layer.getName(), plugins.get(j).get("name"));
                combodata.add(layer.getName());
            }
            catch (PluginManagerException e)
            {
                e.printStackTrace();
            }
        }
        DefaultComboBoxModel model = new DefaultComboBoxModel(combodata.toArray());
        if (jComboBox == null)
        {
            jComboBox = new JComboBox(model);
        }
        return jComboBox;
    }

    private Map<String, String> layers = new TreeMap<String, String>();

    /**
     * 「文字コード」のセレクトボックスを作成します。
     * 
     * @return 「文字コード」のセレクトボックスオブジェクト
     */
    @SuppressWarnings("unchecked")
    private JComboBox getJComboBox1()
    {
        ArrayList combodata = new ArrayList();
        String[] encodings = rmgr.getString("char.encodings").split(",", -2);
        int index = 0;
        for (int i = 0; i < encodings.length; i++)
        {
            combodata.add(encodings[i].trim());
            if (!T2Map.LOCALE.getLanguage().equals(Locale.JAPAN.getLanguage()) && encodings[i].trim().toUpperCase().startsWith("UTF")) {
            	index = i;
            }
        }
        DefaultComboBoxModel model = new DefaultComboBoxModel(combodata.toArray());
        if (jComboBox1 == null)
        {
            jComboBox1 = new JComboBox(model);
            jComboBox1.setSelectedIndex(index);
        }
        return jComboBox1;
    }
}
