/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin.io;

import java.awt.Color;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Locale;
import java.util.concurrent.Callable;
import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.FutureTask;

import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.SwingUtilities;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.T2MapProgressBarDialog;
import jp.nihu.gttime.data.DatasetIF;
import jp.nihu.gttime.data.Record;
import jp.nihu.gttime.plugin.ExportPlugin;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.plugin.RecordIOFilterException;
import jp.nihu.gttime.utility.ExtensionFileFilter;
import jp.nihu.gttime.utility.ExtensionFileSaveChooser;
import jp.nihu.gttime.utility.GTM;
import jp.nihu.gttime.utility.TMM;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xerces.dom.DOMImplementationImpl;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.ls.DOMImplementationLS;
import org.w3c.dom.ls.LSOutput;
import org.w3c.dom.ls.LSSerializer;

/**
 * csv出力時のダイアログ及びgtmファイルを作成します。
 */
public class CSVSave extends ExportPlugin
{
    /**
     * シリアルバーションID
     */
    private static final long serialVersionUID = 1L;

    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(CSVSave.class);

    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(CSVSave.class, T2Map.LOCALE);

    /**
     * パネルオブジェクト
     */
    private JPanel jPanel = null;

    /**
     * TextPaneオブジェクト
     */
    private JLabel jTextPane = null;

    /**
     * ラジオボタン(する)オブジェクト
     */
    private JRadioButton jRadioButton = null;

    /**
     * ラジオボタン(しない)オブジェクト
     */
    private JRadioButton jRadioButton1 = null;

    /**
     * csvファイル保存ダイアログの表示。
     * 
     * @param dataset Datasetオブジェクト
     * @param t2map T2Mapオブジェクト
     */
    public void save(Object data, File file)
    {
//        String currentDirPath = T2Map.properties.getProperty("jp.nihu.gttime.T2Map.CURRENT_DIR");
//        String path = currentDirPath + File.separator + dataset.getPath().substring(0, dataset.getPath().length() - 4);
//        ExtensionFileSaveChooser chooser = new ExtensionFileSaveChooser(path);
//
//        ExtensionFileFilter filter = new ExtensionFileFilter(rmgr.getString("file.filter"));
//        chooser.addChoosableFileFilter(filter);
//        JPanel jpanel = getJPanel();
//        chooser.setAccessory(jpanel);
//
//        int returnVal = chooser.showSaveDialog(SwingUtilities.getRoot(getParentComponent()));
//        // 保存ボタンが押された時
//        if (returnVal == JFileChooser.APPROVE_OPTION)
//        {
//            File file = chooser.getSelectedFile();
            // saveDataset(dataset, file.getParentFile(), file.getName(),
            // jRadioButton.isSelected());
    	
    	if (data instanceof DatasetIF) {
    		DatasetIF dataset = (DatasetIF)data;
            Frame frame = (Frame) SwingUtilities.getRoot(getParentComponent());
            String title = rmgr.getString("progress.title");
            String message = rmgr.format("progress.message", file.getAbsolutePath());
            T2MapProgressBarDialog<Object> progress = new T2MapProgressBarDialog<Object>(frame, true, title, message);
            Callable<Object> callable = (Callable<Object>) new SaveProject(dataset, file.getParentFile(), file.getName(), jRadioButton.isSelected());
            FutureTask<Object> f = (FutureTask<Object>) progress.submit(callable);
            progress.setVisible(true);
            try
            {
                f.get();
            }
            catch (CancellationException e)
            {
            }
            catch (InterruptedException e)
            {
                log.fatal(e.getMessage(), e);
            }
            catch (ExecutionException e)
            {
                log.fatal(e.getMessage(), e);
            }
//        }
        // キャンセルボタンが押された時
//        else
//        {
//            T2Map.properties.setProperty("jp.nihu.gttime.T2Map.CURRENT_DIR", chooser.getCurrentDirectory().toString());
//        }
        try
        {
            T2Map.properties.store();
        }
        catch (FileNotFoundException e)
        {
            log.fatal(e.getMessage(), e);
        }
        catch (IOException e)
        {
            log.fatal(e.getMessage(), e);
        }
    	} else {
    		log.fatal("エクスポートするオブジェクトがDatasetではありません。");
    	}
    }

    private class SaveProject implements Callable<Object>
    {
        private DatasetIF dataset;

        private File base;

        private String relative;

        private boolean saveTmm;

        public SaveProject(DatasetIF dataset, File base, String relative, boolean saveTmm)
        {
            this.dataset = dataset;
            this.base = base;
            this.relative = relative;
            this.saveTmm = saveTmm;
        }

        public Object call() throws Exception
        {
            saveDataset(dataset, base, relative, saveTmm);
            return dataset;
        }
    }

    /**
     * tmm、csvファイルを作ります。
     * 
     * @param dataset Datasetオブジェクト
     * @param base 親の抽象パス名
     * @param relative ファイル名
     * @param tmmFile tmmファイルを出力するか
     */
    private void saveDataset(DatasetIF dataset, File base, String relative, boolean saveTmm) throws RecordIOFilterException
    {
        boolean doit = true;

        if (doit)
        {
            String tmmPath = relative.replace(".csv", ".tmm");
            Document doc = TMM.create(dataset.getTitle(), dataset.getLayerType(), dataset, "NIHU TIMESPACE CSV");
            ;
            // tmmファイルの中身を作成します。
            // if (T2Map.getProjectType() == 1)
            // {
            tmmPath = relative.replace(".csv", ".gtm");
            doc = GTM.create(dataset.getTitle(), dataset.getLayerType(), dataset, "NIHU TIMESPACE CSV", tmmPath);
            // }
            File file = new File(base, tmmPath);
            log.info("SAVE : " + file.getAbsolutePath());
            try
            {
                if (saveTmm)
                {
                    saveTmm(doc, file);
                }
                Record[] records = dataset.getRecords();
                NIHUTimeSpaceCSVIOFilter filter = new NIHUTimeSpaceCSVIOFilter();
                file = new File(filter.getPath(file.getAbsolutePath()));
                if (doit)
                {
                    log.info("SAVE : " + file.getAbsolutePath());
                    filter.setEncoding((String) comboBox.getSelectedItem());
                    // csvファイルの中身を作成します。
                    byte[] csv = filter.write(records);
                    saveBytedata(csv, file);
                }
            }
            catch (IOException e)
            {
                log.fatal(e.getMessage(), e);
                throw new RecordIOFilterException(e.getMessage(), e);
            }
            catch (RecordIOFilterException e)
            {
                log.fatal(e.getMessage(), e);
                throw e;
            }
        }
    }

    /**
     * csvファイルに書き込みます。
     * 
     * @param data byteデータ
     * @param file Fileオブジェクト
     * @throws IOException
     */
    private void saveBytedata(byte[] data, File file) throws IOException
    {
        OutputStream out = new BufferedOutputStream(new FileOutputStream(file));
        out.write(data);
        out.close();
    }

    /**
     * tmmファイルに書き込みます。
     * 
     * @param doc Documentオブジェクト
     * @param file Fileオブジェクト
     * @throws IOException
     */
    private void saveTmm(Document doc, File file) throws IOException
    {
        File parent = file.getParentFile();
        if (parent.exists() || parent.mkdirs())
        {
            DOMImplementation domImpl = new DOMImplementationImpl();
            DOMImplementationLS domImplLS = (DOMImplementationLS) domImpl.getFeature("LS", "3.0");
            LSOutput lsOutput = domImplLS.createLSOutput();
            LSSerializer lsSerializer = domImplLS.createLSSerializer();
            FileOutputStream fos = new FileOutputStream(file);
            lsOutput.setByteStream(fos);
            lsSerializer.write(doc, lsOutput);
            fos.close();
        }
    }

    /** コンボボックスモデル */
    private DefaultComboBoxModel cboModel = null;

    /** コンボボックス */
    private JComboBox comboBox = null;

    /**
     * パネルを作成します。
     * 
     * @return Panelオブジェクト
     */
    public JPanel getJPanel()
    {
        if (jPanel == null)
        {
            GridBagConstraints gridBagConstraints0 = new GridBagConstraints();
            gridBagConstraints0.gridx = 0;
            gridBagConstraints0.gridy = 0;
            gridBagConstraints0.gridwidth = 2;
            gridBagConstraints0.fill = GridBagConstraints.BOTH;
            GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
            gridBagConstraints1.gridx = 0;
            gridBagConstraints1.gridy = 1;
            gridBagConstraints1.ipadx = 15;
            gridBagConstraints1.fill = GridBagConstraints.BOTH;
            GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
            gridBagConstraints2.gridx = 1;
            gridBagConstraints2.gridy = 1;
            GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
            gridBagConstraints3.gridx = 0;
            gridBagConstraints3.gridy = 2;
            gridBagConstraints3.gridwidth = 2;
            gridBagConstraints3.fill = GridBagConstraints.BOTH;
            GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
            gridBagConstraints4.gridx = 0;
            gridBagConstraints4.gridy = 3;
            gridBagConstraints4.gridwidth = 2;
            gridBagConstraints4.fill = GridBagConstraints.BOTH;

            jPanel = new JPanel();
            jPanel.setLayout(new GridBagLayout());
            jPanel.add(getJLabel(), gridBagConstraints0);

            JRadioButton b = getJRadioButton();
            JRadioButton b1 = getJRadioButton1();
            ButtonGroup bg = new ButtonGroup();
            b1.setSelected(true);
            bg.add(b);
            bg.add(b1);
            jPanel.add(b, gridBagConstraints1);
            jPanel.add(b1, gridBagConstraints2);

            jPanel.add(new JLabel(rmgr.getString("encode.label")), gridBagConstraints3);

            cboModel = new DefaultComboBoxModel();
            String[] encs = rmgr.getString("encode.avairable").split(",");
            int index = 0;
            for (int i = 0; i < encs.length; i++)
            {
                cboModel.addElement(encs[i].trim());
                if (!T2Map.LOCALE.getLanguage().equals(Locale.JAPAN.getLanguage()) && encs[i].trim().toUpperCase().startsWith("UTF")) {
                	index = i;
                }
            }
            comboBox = new JComboBox(cboModel);
            comboBox.setSelectedIndex(index);
            jPanel.add(comboBox, gridBagConstraints4);
        }
        return jPanel;
    }

    /**
     * TextPaneを作成します。
     * 
     * @return TextPaneオブジェクト
     */
    private JLabel getJLabel()
    {
        if (jTextPane == null)
        {
            jTextPane = new JLabel();
            jTextPane.setText(rmgr.format("tmmfile.output.title", getDatasetType()));
            jTextPane.setBackground(new Color(0, 0, 0, 0));
        }
        return jTextPane;
    }

    private String getDatasetType()
    {
        String datasetType = "tmm";
        // if (T2Map.getProjectType() == 1)
        // {
        datasetType = "gtm";
        // }
        return datasetType;
    }

    /**
     * 「する」のラジオボタンを作成します。
     * 
     * @return ラジオボタン(する)オブジェクト
     */
    private JRadioButton getJRadioButton()
    {
        if (jRadioButton == null)
        {
            jRadioButton = new JRadioButton();
            jRadioButton.setText(rmgr.getString("tmmfile.output.yes"));
        }
        return jRadioButton;
    }

    /**
     * 「しない」のラジオボタンを作成します。
     * 
     * @return ラジオボタン(しない)オブジェクト
     */
    private JRadioButton getJRadioButton1()
    {
        if (jRadioButton1 == null)
        {
            jRadioButton1 = new JRadioButton();
            jRadioButton1.setText(rmgr.getString("tmmfile.output.no"));
        }
        return jRadioButton1;
    }

	@Override
	public ExtensionFileFilter getExtensionFileFilter() {
		return new ExtensionFileFilter(rmgr.getString("file.filter"));
	}
}
