/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin.io;

import java.awt.Frame;
import java.io.File;
import java.io.InputStream;
import java.util.concurrent.Callable;
import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.FutureTask;

import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.T2MapFileChooser;
import jp.nihu.gttime.T2MapProgressBarDialog;
import jp.nihu.gttime.data.DatasetException;
import jp.nihu.gttime.data.DatasetGtm;
import jp.nihu.gttime.data.DatasetIF;
import jp.nihu.gttime.data.ProjectException;
import jp.nihu.gttime.data.ProjectGts;
import jp.nihu.gttime.data.ProjectIF;
import jp.nihu.gttime.plugin.ImportPlugin;
import jp.nihu.gttime.plugin.OpenPlugin;
import jp.nihu.gttime.plugin.OpenPluginException;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.utility.ExtensionFileFilter;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class GTMImportPlugin extends ImportPlugin
{
    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(GTMImportPlugin.class);

    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(GTMImportPlugin.class, T2Map.LOCALE);

    @Override
    public ProjectIF open() throws OpenPluginException
    {
        log.debug("Import Records from GTM file.");
        ProjectIF project = null;
        UIManager.put("FileChooser.readOnly", Boolean.TRUE);
        JFileChooser chooser = new T2MapFileChooser();
        ExtensionFileFilter filter = new ExtensionFileFilter(rmgr.getString("file.filter"));
        chooser.setFileFilter(filter);
        int returnVal = chooser.showOpenDialog(getParentComponent());
        if (returnVal == JFileChooser.APPROVE_OPTION)
        {
            project = open(chooser.getSelectedFile(), null);
        }
        UIManager.put("FileChooser.readOnly", Boolean.FALSE);
        return project;
    }
    
    @Override
    public ExtensionFileFilter getExtensionFileFilter() {
    	return new ExtensionFileFilter(rmgr.getString("file.filter"));
    }

    @Override
    public ProjectIF open(File file, ProjectIF insertToProject) throws OpenPluginException
    {
        Frame frame = (Frame) SwingUtilities.getRoot(getParentComponent());
        String dtitle = rmgr.getString("progress.title");
        String dmessage = rmgr.format("progress.message", file.getAbsolutePath());
        T2MapProgressBarDialog<Object> progress = new T2MapProgressBarDialog<Object>(frame, true, dtitle, dmessage);
        Callable<Object> callable = (Callable<Object>) new LoadProject(file);
        FutureTask<Object> f = (FutureTask<Object>) progress.submit(callable);
        progress.setVisible(true);
        DatasetIF ds = null;
        ProjectIF project = null;
        T2Map tmap = GTMImportPlugin.this.getParentComponent();
        try
        {
            ds = (DatasetIF) f.get();
            
            if (insertToProject.isRoot()) {
            	DatasetIF[] d = insertToProject.getDatasets();
            	if (d == null || d.length == 0) {
            		insertToProject = null;
            	}
            }
            
            if (insertToProject != null)
            {// 親のプロジェクトが存在する場合、そのプロジェクトの最後尾にデータセットを追加する。
            	
//            	log.debug("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA");
            	
                project = insertToProject;
                try
                {
                    project.addDataset(ds);
                }
                catch (ProjectException e)
                {
                    e.printStackTrace();
                }
                tmap.updateView();
            }
            else
            {// 親のプロジェクトが存在しない場合、新しいプロジェクトを生成し、データセットを追加する。
            	
//            	log.debug("BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB");
            	
                DatasetIF[] datasets = null;
                // int ptype = T2Map.getProjectType();
                // if (ptype == 0)
                // {
                // datasets = new DatasetTmm[1];
                // }
                // else if (ptype == 1)
                // {
                datasets = new DatasetGtm[1];
                // }
                datasets[0] = ds;
                // int type = T2Map.getProjectType();
                // if (type == 0)
                // {
                // project = new ProjectTms(datasets);
                // }
                // else if (type == 1)
                // {
                project = new ProjectGts(datasets);
                // }

                tmap.setProject(project);
            }
        }
        catch (InterruptedException e)
        {
            throw new OpenPluginException(e.getMessage(), e);
        }
        catch (ExecutionException e)
        {
            throw new OpenPluginException(e.getMessage(), e);
        }
        catch (CancellationException e)
        {
        }
        return project;
    }

    private class LoadProject implements Callable<Object>
    {
        private File file;

        public LoadProject(File file)
        {
            this.file = file;
        }

        public Object call() throws Exception
        {
            // TMMを読込む
            DatasetIF ds = null;
            // int ptype = T2Map.getProjectType();
            // if (ptype == 0)
            // {
            // ds = new DatasetTmm();
            // }
            // else if (ptype == 1)
            // {
            ds = new DatasetGtm();
            // }
            try
            {
                OpenPlugin fileOpenPLiugin = new FileOpenPlugin();
                fileOpenPLiugin.init(getParentComponent(), getPlugins());
                //ds.loadDataset(fileOpenPLiugin, file.getAbsolutePath(), null, "TG" + System.currentTimeMillis() + ".gtm");
                ds.loadDataset(fileOpenPLiugin, file.getAbsolutePath(), null);
                fileOpenPLiugin.processError();
            }
            catch (DatasetException e)
            {
                log.fatal("File Import Error", e);
                throw new OpenPluginException("File Import Error", e);
            }
            return ds;
        }
    }

    /**
     * ファイルを読みます。
     * 
     * @param path ファイルパス
     * @return InputStreamオブジェクト
     * @throws OpenPluginException
     */
    @Override
    public InputStream getRecordsInputStream(String path) throws OpenPluginException
    {
        return null;
    }
    
    @Override
    public JPanel getJPanel()
    {
    	return null;
    }
}
