/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin.io;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.regex.Pattern;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.TimeMapGeograhicCoordinates;
import jp.nihu.gttime.data.Event;
import jp.nihu.gttime.data.JulianDate;
import jp.nihu.gttime.data.Location;
import jp.nihu.gttime.data.Record;
import jp.nihu.gttime.data.RecordException;
import jp.nihu.gttime.plugin.IOError;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.plugin.RecordIOFilter;
import jp.nihu.gttime.plugin.RecordIOFilterException;
import jp.nihu.gttime.time.utility.ISO8601Exception;
import jp.nihu.gttime.time.utility.JulianGregorianDate;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.SAXException;

/**
 * CSVファイルの内容を読み書きします。
 */
public class NIHUTimeSpaceCSVIOFilter extends RecordIOFilter
{
    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(NIHUTimeSpaceCSVIOFilter.class);

    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(NIHUTimeSpaceCSVIOFilter.class, T2Map.LOCALE);

    /**
     * 出力先パスを取得します。
     * 
     * @param tmmPath パス
     * @return パス
     * @throws RecordIOFilterException
     */
    public String getPath(String tmmPath) throws RecordIOFilterException
    {
        int x = tmmPath.lastIndexOf(".");
        if (x != -1)
        {
            return tmmPath.substring(0, x) + ".csv";
        }
        else
        {
            return tmmPath + ".csv";
        }
    }

    // public Record[] read(InputStream is, String code, IOError ioErrorx)
    // throws RecordIOFilterException
    // {
    // ioError = ioErrorx;
    // log.info("##########CSV READ START##########");
    // InputStreamReader isr = null;
    // try
    // {
    // // 文字コードに「SHIFT_JIS」が指定されていた時、機種依存文字が化けるので、「MS932」になおす。
    // if (code.equals("SHIFT_JIS"))
    // {
    // code = "MS932";
    // }
    // isr = new InputStreamReader(is, code.toString());
    // }
    // catch (UnsupportedEncodingException e)
    // {
    // log.fatal(e.getMessage(), e);
    // }
    // CSVReader reader = new CSVReader(isr);
    // NIHUTimeSpaceCSVConfigParser handler = new
    // NIHUTimeSpaceCSVConfigParser();
    // try
    // {
    // int readCount = 2;
    // String[] koumoku = reader.readNext();
    // String[] data;
    // while ((data = reader.readNext()) != null)
    // {
    // // nextLine[] is an array of values from the line
    // StringBuffer sb = new StringBuffer();
    // for (int n = 0; n < data.length; n++)
    // {
    // sb.append(data[n]).append(',');
    // }
    // log.info(sb.toString());
    //
    // // レコードのエラーチェック
    // boolean columnCheck = errorCheck(koumoku, data, readCount);
    // if (columnCheck)
    // {
    // try
    // {
    // handler.setRecord(koumoku, data);
    // }
    // catch (ISO8601Exception e)
    // {
    // // never catch
    // e.printStackTrace();
    // }
    // }
    // readCount++;
    // }
    //
    // isr.close();
    // is.close();
    // log.info("##########CSV READ END##########");
    // }
    // catch (IOException e)
    // {
    // log.fatal("CSV File read error.");
    // throw new RecordIOFilterException("CSV File read error.", e);
    // }
    //
    // return handler.getRecords();
    // }

    /**
     * csvファイルを読み込みます。
     * 
     * @param is InputStreamオブジェクト
     * @param code サポートする charset の名前
     * @return records Record[]オブジェクト
     * @throws RecordIOFilterException
     */
    public Record[] read(InputStream is, String code, IOError ioErrorx) throws RecordIOFilterException
    {
        ioError = ioErrorx;
        log.info("##########CSV READ START##########");
        InputStreamReader isr = null;
        try
        {
            // 文字コードに「SHIFT_JIS」が指定されていた時、機種依存文字が化けるので、「MS932」になおす。
            if (code.equals("SHIFT_JIS"))
            {
                code = "MS932";
            }
            isr = new InputStreamReader(is, code.toString());
        }
        catch (UnsupportedEncodingException e)
        {
            log.fatal(e.getMessage(), e);
            throw new RecordIOFilterException(e.getMessage(), e);
        }
        BufferedReader br = new BufferedReader(isr);
        NIHUTimeSpaceCSVConfigParser handler = new NIHUTimeSpaceCSVConfigParser();
        try
        {
            String[] koumoku = itemDivision(br.readLine());
            for (int i = 0; i < koumoku.length; i++)
            {
                if (koumoku[i] != null && koumoku[i].indexOf("\"") == 0 && koumoku[i].lastIndexOf("\"") == (koumoku[i].length() - 1))
                {
                    koumoku[i] = koumoku[i].replaceFirst("\"", "");
                    koumoku[i] = koumoku[i].substring(0, (koumoku[i].length() - 1));
                    koumoku[i] = koumoku[i].replace("\"\"", "\"");
                }
            }
            int readCount = 2;
            while (br.ready())
            {
                String csv = br.readLine();

                String curam = csv.substring(0, 1);
                String[] data2 = new String[1000];
                int demiCount = 2;
                if (csv.substring(0, 1).equals("\""))
                {
                    demiCount++;
                }
                int d = 0;
                for (int i = 1; i < csv.length(); i++)
                {
                    if (!csv.substring(i, i + 1).equals(""))
                    {
                        // ,(カンマ)が出現
                        if (String.valueOf(csv.charAt(i)).equals(",") && ((demiCount % 2) == 0))
                        {
                            data2[d] = curam;
                            d++;
                            if ((i + 1) < csv.length())
                            {
                                if (String.valueOf(csv.charAt(i + 1)).equals("\""))
                                {
                                    curam = csv.substring(i + 1, i + 2);
                                    demiCount++;
                                    i++;
                                }
                                else if (!csv.substring(i + 1, i + 2).equals(","))
                                {
                                    curam = csv.substring(i + 1, i + 2);
                                    i++;
                                }
                                else
                                {
                                    curam = null;
                                }
                            }
                            else if ((i + 1) == csv.length())
                            {
                                curam = null;
                            }
                        }
                        else
                        {
                            // "(ダブルコート)の数をカウント
                            if (String.valueOf(csv.charAt(i)).equals("\""))
                            {
                                demiCount++;
                            }
                            // 文字を追加
                            if (curam != null)
                            {
                                curam = curam + csv.substring(i, i + 1);
                            }
                        }
                    }
                }

                // 文字列に改行文字が入っていた場合
                while ((demiCount % 2) != 0)
                {
                    csv = br.readLine();

                    curam = curam + "\n" + csv.substring(0, 1);
                    if (csv.substring(0, 1).equals("\""))
                    {
                        demiCount++;
                    }
                    for (int i = 1; i < csv.length(); i++)
                    {
                        if (!csv.substring(i, i + 1).equals(""))
                        {
                            // ,(カンマ)が出現
                            if (String.valueOf(csv.charAt(i)).equals(",") && ((demiCount % 2) == 0))
                            {
                                data2[d] = curam;
                                d++;
                                if ((i + 1) < csv.length())
                                {
                                    if (String.valueOf(csv.charAt(i + 1)).equals("\""))
                                    {
                                        curam = csv.substring(i + 1, i + 2);
                                        demiCount++;
                                        i++;
                                    }
                                    else if (!csv.substring(i + 1, i + 2).equals(","))
                                    {
                                        curam = csv.substring(i + 1, i + 2);
                                        i++;
                                    }
                                    else
                                    {
                                        curam = null;
                                    }
                                }
                                else if ((i + 1) == csv.length())
                                {
                                    curam = null;
                                }
                            }
                            else
                            {
                                // "(ダブルコート)の数をカウント
                                if (String.valueOf(csv.charAt(i)).equals("\""))
                                {
                                    demiCount++;
                                }
                                // 文字を追加
                                if (curam != null)
                                {
                                    curam = curam + csv.substring(i, i + 1);
                                }
                            }
                        }
                    }
                    continue;
                }

                data2[d] = curam;
                String[] data = new String[d + 1];
                for (int i = 0; i < (d + 1); i++)
                {
                    data[i] = data2[i];
                }

                // String[] data = csv.split(",",-2);
                // レコードのエラーチェック
                boolean columnCheck = errorCheck(koumoku, data, readCount);
                if (columnCheck)
                {
                    try
                    {
                        handler.setRecord(koumoku, data);
                    }
                    catch (ISO8601Exception e)
                    {
                        // never catch
                        e.printStackTrace();
                        throw new RecordIOFilterException(e.getMessage(), e);
                    }
                    catch (RecordException e)
                    {
                        // never catch
                        e.printStackTrace();
                        throw new RecordIOFilterException(e.getMessage(), e);
                    }
                }
                readCount++;
            }
            br.close();
            isr.close();
            is.close();
            log.info("##########CSV READ END##########");
        }
        catch (IOException e)
        {
            log.fatal("CSV File read error.");
            throw new RecordIOFilterException("CSV File read error.", e);
        }

        return handler.getRecords();
    }

    private String encoding = "UTF-8";

    public void setEncoding(String encoding)
    {
        this.encoding = encoding;
    }

    /**
     * csvファイルに内容を書き込みます。
     * 
     * @param records Record[]
     * @return byte[]
     * @throws RecordIOFilterException
     */
    public byte[] write(Record[] records) throws RecordIOFilterException
    {
        String items[] = new String[100];

        // 1行目の項目を作成
        int count = 0;
        StringBuffer sb = new StringBuffer("VALUE_ID,FROM_DATE,TO_DATE,LATI,LONG");
        
        //boolean writeGroup = false;
        for (int i = 0; i < records.length; i++)
        {
            String group = records[i].getGroup();
            if (group != null && group.length() > 0) {
                sb.append(",group");
                break;
            }
        }
        
        for (int i = 0; i < records.length; i++)
        {
            Event[] events = records[i].getEvents();
            for (int j = 0; j < events.length; j++)
            {
                int sameNameCount = 0;
                if (!sb.toString().matches(".*,\"" + events[j].getName() + "\",?.*"))
                {
                    sb.append(",\"" + escapeCSVChar(events[j].getName()) + "\"");
                    items[count] = events[j].getName();
                    count++;
                }
                else
                {
                    for (int k = 0; k < count; k++)
                    {
                        if (items[k].equals(events[j].getName()))
                        {
                            sameNameCount++;
                        }
                    }
                    int l = 1;
                    for (int k = j + 1; k < events.length; k++)
                    {
                        if (events[j].getName().equals(events[k].getName()))
                        {
                            l++;
                        }
                    }
                    if (sameNameCount < l)
                    {
                        for (int m = 0; m < (l - sameNameCount); m++)
                        {
                            sb.append(",\"" + escapeCSVChar(events[j].getName()) + "\"");
                            items[count] = events[j].getName();
                            count++;
                        }
                    }
                }
            }
        }
        sb.append("\n");

        // 2行目以降のデータを作成。
        int newId = 0;
        for (int i = 0; i < records.length; i++)
        {
            if (records[i].getId() != null)
            {
                sb.append(records[i].getId() + ",");
            }
            else
            {
                sb.append("id" + newId + ",");
                newId++;
            }
            String sdStr = "";
            JulianDate js = records[i].getStartDatetime();
            if (js != null)
            {
                BigDecimal sdt = js.getDatetime();
                if (sdt != null)
                {
                    sdStr = JulianGregorianDate.toGregorian(sdt, js.getTimezone());
                }
            }
            sb.append(sdStr + ",");
            String edStr = "";
            JulianDate je = records[i].getEndDatetime();
            if (je != null)
            {
                BigDecimal edt = je.getDatetime();
                if (edt != null)
                {
                    edStr = JulianGregorianDate.toGregorian(edt, je.getTimezone());
                }
            }
            sb.append(edStr + ",");
            Location location = records[i].getLocation();
            if (location != null)
            {
                sb.append(location.getPointsLongLaties()[0].getLatitude());
                sb.append(",");
                sb.append(location.getPointsLongLaties()[0].getLongitude());
            }
            else
            {
                sb.append(",");
            }
            
            String group = records[i].getGroup();
            if (group != null && group.length() > 0) {
            	sb.append(",");
                sb.append("\"");
                sb.append(escapeCSVChar(group));
                sb.append("\"");
            }
            
            int[] itemsCount = new int[100];
            int countItem = 0;

            for (int j = 0; j < count; j++)
            {
                boolean agreement = false;
                Event[] events = records[i].getEvents();
                for (int k = 0; k < events.length; k++)
                {
                    if (items[j].equals(events[k].getName()))
                    {
                        boolean sub = true;
                        for (int l = 0; l < countItem; l++)
                        {
                            if (itemsCount[l] == k)
                            {
                                sub = false;
                            }
                        }
                        if (sub)
                        {
                            itemsCount[countItem] = k;
                            countItem++;
                            sb.append(",\"" + escapeCSVChar(events[k].getData()) + "\"");
                            agreement = true;
                            break;
                        }
                    }
                }
                if (!agreement)
                {
                    sb.append(",");
                }
            }
            sb.append("\n");
        }

        byte[] bxml = null;
        try
        {
            bxml = sb.toString().getBytes(encoding);
        }
        catch (UnsupportedEncodingException e)
        {
            log.fatal(e.getMessage(), e);
            throw new RecordIOFilterException(e.getMessage(), e);
        }
        return bxml;
    }

    /**
     * 禁則文字を置き換えます。
     * 
     * @param str 文字
     * @return 置き換えられた文字
     */
    private String escapeCSVChar(String str)
    {
        str = str.trim();
        str = str.replaceAll("\"", "\"\"");
        return str;
    }

    /**
     * レコードのチェックや設定を行うクラスです。
     */
    class NIHUTimeSpaceCSVConfigParser
    {
        /**
         * StringBufferオブジェクト
         */
        private StringBuffer value = new StringBuffer();

        /**
         * Vectorオブジェクト
         */
        private ArrayList<Record> vrecord = new ArrayList<Record>();

        /**
         * Recordオブジェクト
         */
        private Record record = null;

        /**
         * Eventオブジェクト
         */
        private Event event = null;

        /**
         * JulianDateオブジェクト
         */
        private JulianDate date = null;

        /**
         * Locationオブジェクト
         */
        private Location location = null;

        /**
         * TimeMapGeograhicCoordinatesオブジェクト
         */
        private TimeMapGeograhicCoordinates point = null;

        /**
         * StringBufferオブジェクトに指定された文字列を追加します。
         * 
         * @param ch 文字列
         * @param start 開始位置
         * @param length 長さ
         * @throws SAXException
         */
        public void characters(char[] ch, int start, int length)
        {
            value.append(ch, start, length);
        }

        /**
         * レコードを取得します。
         * 
         * @return Record[]オブジェクト
         */
        public Record[] getRecords()
        {
            Record[] records = new Record[vrecord.size()];
            records = vrecord.toArray(records);
            vrecord = null;
            return records;
        }

        /**
         * ジュリアン日を取得します。
         * 
         * @param iso8601 日時
         * @return ジュリアン日
         * @throws ISO8601Exception
         */
        private BigDecimal getJulianDate(String iso8601) throws ISO8601Exception
        {
            return JulianGregorianDate.toJulian(iso8601);
        }

        /**
         * レコードの内容を設定します。
         * 
         * @param koumoku レコードの項目名
         * @param data レコードのデータ
         * @throws ISO8601Exception
         * @throws RecordException
         */
        private void setRecord(String koumoku[], String data[]) throws ISO8601Exception, RecordException
        {
            record = new Record();
            location = new Location();

            // 1カラム目はIDをセット
            record.setId(data[0]);

            // 2カラム目は開始日時をセット
            if (data[1] != null)
            {
                date = new JulianDate();
                date.setDatetime(getJulianDate(data[1]));
                date.setTimezone(JulianGregorianDate.getTimezone(data[1]));
                record.setStartDatetime(date);
            }

            // 3カラム目は終了日時をセット
            if (data[2] != null)
            {
                date = new JulianDate();
                date.setDatetime(getJulianDate(data[2]));
                date.setTimezone(JulianGregorianDate.getTimezone(data[2]));
                record.setEndDatetime(date);
            }

            // 4カラム目は緯度をセット
            // 5カラム目は経度をセット
            point = null;
            if (data[3] != null && data[4] != null)
            {
                if (Pattern.matches(LON_LAT_REGEX, data[3]) && Pattern.matches(LON_LAT_REGEX, data[4]))
                {
                    if ((-90 <= Double.parseDouble(data[3]) && Double.parseDouble(data[3]) <= 90) && (-180 <= Double.parseDouble(data[4]) && Double.parseDouble(data[4]) <= 180))
                    {
                        point = new TimeMapGeograhicCoordinates(Double.parseDouble(data[3]), Double.parseDouble(data[4]));
                        TimeMapGeograhicCoordinates[] points = new TimeMapGeograhicCoordinates[1];
                        points[0] = point;
                        location.setPointsLongLaties(points);
                        record.setLocation(location);
                    }
                }
            }

            // 6カラム目以降はイベントをセット
            ArrayList<Event> ev = new ArrayList<Event>();
            if (koumoku.length == data.length)
            {
                boolean hasGroup = false;
                for (int i = 5; i < koumoku.length; i++)
                {
                    if (data[i] != null)
                    {
                        data[i] = data[i].replace("\"\"", "\"");
                        if (!koumoku[i].equals("group")) {
                            event = new Event();
                            event.setName(koumoku[i]);
                            if (!koumoku[i].equals("number"))
                            {
                                event.setData(data[i]);
                                ev.add(event);
                            }
                            else if (data[i].length() == 0 || Pattern.matches(NUMBER_REGEX, data[i]))
                            {
                                event.setData(data[i]);
                                ev.add(event);
                            }
                        } else {
                            hasGroup = true;
                            record.setGroup(data[i]);
                            setGroup(data[i]);
                        }
                    }
                }
                if (!hasGroup) {
                    setGroup("");
                }
            }
            Event[] events = new Event[ev.size()];
            ev.toArray(events);
            record.setEvents(events);
            vrecord.add(record);
        }
    }

    private static String NUMBER_REGEX = "^-?[0-9]+([.][0-9]+)?$";

    private static String LON_LAT_REGEX = "^-?[0-9]+([.][0-9]+([E][-]?[0-9]+)?)?$";

    /**
     * レコード内容のエラーチェックを行います。
     * 
     * @param koumoku レコードの項目名
     * @param data レコードのデータ
     * @param readCount CSVファイルでの行数
     * @return エラー項目があるか、ないか
     * @throws RecordIOFilterException
     */
    private boolean errorCheck(String[] koumoku, String[] data, int readCount) throws RecordIOFilterException
    {
        for (int i = 0; i < data.length; i++)
        {
            if (data[i] != null)
            {
                // データの最初と最後の文字が"(ダブルコーテーション)だった場合
                if (data[i].indexOf("\"") == 0 && data[i].lastIndexOf("\"") == (data[i].length() - 1))
                {
                    data[i] = data[i].replaceFirst("\"", "");
                    data[i] = data[i].substring(0, (data[i].length() - 1));
                }
            }
        }

        boolean fatal = false;
        boolean warn = false;

        // カラム数のチェック
        if (koumoku.length != (data.length))
        {
            fatal = procError("error.curam.length.suitable", FATAL, readCount);
        }
        else
        {
            // IDのチェック(未実装)
            // 開始日時のフォーマットチェック
            // if (data[1] != null)
            // {
            if (data[1] != null && data[1].length() > 0 && !JulianGregorianDate.isValidISO8601(data[1]))
            {
                fatal = procError("error.from.date.format", data[1], FATAL, readCount);
            }
            // }
            // else
            // {
            // fatal = procError("error.from.date.null", FATAL, readCount);
            // }

            // 終了日時のフォーマットチェック
            // if (data[2] != null)
            // {
            if (data[2] != null && data[2].length() > 0 && !JulianGregorianDate.isValidISO8601(data[2]))
            {
                fatal = procError("error.to.date.format", data[2], FATAL, readCount);
            }
            // }
            // else
            // {
            // fatal = procError("error.to.date.null", FATAL, readCount);
            // }

            // 時間情報の開始、終了の比較（開始が終了より未来の場合はエラー）
            if (!fatal)
            {
                try
                {
                    if ((data[1] != null && data[1].length() > 0) && (data[2] != null && data[2].length() > 0))
                    {
                        BigDecimal s = JulianGregorianDate.toJulian(data[1]);
                        BigDecimal e = JulianGregorianDate.toJulian(data[2]);
                        if (s.compareTo(e) > 0)// 20100708 同じ日時をエラーとしないようにした。
                        {
                            fatal = procError("error.from.date.invalid", FATAL, readCount);
                        }
                    }
                }
                catch (ISO8601Exception e)
                {
                    // never catch
                    e.printStackTrace();
                    throw new RecordIOFilterException(e.getMessage(), e);
                }
            }

            // 緯度のフォーマットチェック
            if (data[3] != null)
            {
                if (!Pattern.matches(LON_LAT_REGEX, data[3]))
                {
                    warn = procError("warn.lati.number", data[3], WARN, readCount);
                }
                else
                {
                    if (Float.parseFloat(data[3]) < -90 || 90 < Float.parseFloat(data[3]))
                    {
                        warn = procError("warn.lati.max.min", data[3], WARN, readCount);
                    }
                }
            }
            else if (data[4] != null)
            {
                warn = procError("warn.lati.null", WARN, readCount);
            }

            // 経度のフォーマットチェック
            if (data[4] != null)
            {
                if (!Pattern.matches(LON_LAT_REGEX, data[4]))
                {
                    warn = procError("warn.long.number", data[4], WARN, readCount);
                }
                else
                {
                    if (Float.parseFloat(data[4]) < -180 || 180 < Float.parseFloat(data[4]))
                    {
                        warn = procError("warn.long.max.min", data[4], WARN, readCount);
                    }
                }
            }
            else if (data[3] != null)
            {
                warn = procError("warn.long.null", WARN, readCount);
            }

            // number属性がある場合は、number属性のフォーマットチェックチェック
            for (int i = 5; i < koumoku.length; i++)
            {
                if (koumoku[i].equals("number") && data[i] != null)
                {
                    if (data[i].length() != 0 && !data[i].matches(NUMBER_REGEX))
                    {
                        warn = procError("warn.number.format", data[i], WARN, readCount, i + 1);
                    }
                }
            }
        }

        if (fatal)
        {
            ioError.addFatalNum();
        }
        else
        {
            if (warn)
            {
                ioError.addWarnNum();
            }
        }

        return !fatal;
    }

    private boolean procError(String code, String value, int type, int lineNum, int colmNum)
    {
        String message = rmgr.format(code, String.valueOf(lineNum), String.valueOf(colmNum));
        message = message + " : " + value;
        return procError(message, type);
    }

    private boolean procError(String code, String value, int type, int lineNum)
    {
        String message = rmgr.format(code, String.valueOf(lineNum));
        message = message + " : " + value;
        return procError(message, type);
    }

    private boolean procError(String code, int type, int lineNum)
    {
        String message = rmgr.format(code, String.valueOf(lineNum));
        return procError(message, type);
    }

    private boolean procError(String message, int type)
    {
        if (type == FATAL)
        {
            ioError.add(message);
            log.error(message);
        }
        else if (type == WARN)
        {
            ioError.add(message);
            log.warn(message);
        }
        else
        {
            return false;
        }
        return true;
    }

    /**
     * 読込んだ1行をカンマごとに区切ります。
     * 
     * @param writeRow 読込んだ1行の文字列
     * @return カンマで分けた文字列の配列
     */
    private String[] itemDivision(String writeRow)
    {
        String curam = writeRow.substring(0, 1);
        String[] data2 = new String[1000];
        int demiCount = 2;
        if (writeRow.substring(0, 1).equals("\""))
        {
            demiCount++;
        }
        int d = 0;
        for (int i = 1; i < writeRow.length(); i++)
        {
            if (!writeRow.substring(i, i + 1).equals(""))
            {
                // ,(カンマ)が出現
                if (String.valueOf(writeRow.charAt(i)).equals(",") && ((demiCount % 2) == 0))
                {
                    data2[d] = curam;
                    d++;
                    if ((i + 1) < writeRow.length())
                    {
                        if (String.valueOf(writeRow.charAt(i + 1)).equals("\""))
                        {
                            curam = writeRow.substring(i + 1, i + 2);
                            demiCount++;
                            i++;
                        }
                        else if (!writeRow.substring(i + 1, i + 2).equals(","))
                        {
                            curam = writeRow.substring(i + 1, i + 2);
                            i++;
                        }
                        else
                        {
                            curam = null;
                        }
                    }
                    else if ((i + 1) == writeRow.length())
                    {
                        curam = null;
                    }
                }
                else
                {
                    // "(ダブルコート)の数をカウント
                    if (String.valueOf(writeRow.charAt(i)).equals("\""))
                    {
                        demiCount++;
                    }
                    // 文字を追加
                    if (curam != null)
                    {
                        curam = curam + writeRow.substring(i, i + 1);
                    }
                }
            }
        }

        data2[d] = curam;
        String[] data = new String[d + 1];
        for (int i = 0; i < (d + 1); i++)
        {
            data[i] = data2[i];
        }
        return data;
    }

    /**
     * @see jp.nihu.gttime.plugin.RecordIOFilter#read(java.io.InputStream,
     *      org.h_gis.launcher.gttime.plugin.io.IOError)
     */
    public Record[] read(InputStream is, IOError ioError) throws RecordIOFilterException
    {
        return read(is, "SHIFT_JIS", ioError);
    }
}