package jp.nihu.gttime.plugin.io;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;

import jp.nihu.gttime.data.DatasetIF;
import jp.nihu.gttime.data.ProjectIF;
import jp.nihu.gttime.data.Record;
import jp.nihu.gttime.plugin.RecordIOFilterException;
import jp.nihu.gttime.utility.GTM;
import jp.nihu.gttime.utility.TMM;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xerces.dom.DOMImplementationImpl;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.ls.DOMImplementationLS;
import org.w3c.dom.ls.LSOutput;
import org.w3c.dom.ls.LSSerializer;

public class ProjectSave
{
    private static Log log = LogFactory.getLog(ProjectSave.class);
    
    private ArrayList<String> done = new ArrayList<String>();
    
    private Document getTMSDocument(Document gts)
    {
        DOMResult res = null;
        try
        {
            TransformerFactory factory = TransformerFactory.newInstance();
            InputStream inXslt = getClass().getClassLoader().getResourceAsStream("gts2tms.xsl");
            Transformer transformer = factory.newTransformer(new StreamSource(inXslt));
            res = new DOMResult();
            transformer.transform(new DOMSource(gts), res);
        }
        catch (TransformerConfigurationException e)
        {
            log.fatal(e.getMessage(), e);
        }
        catch (TransformerException e)
        {
            log.fatal(e.getMessage(), e);
        }
        return (Document) res.getNode();
    }
    
    public void saveProject(ProjectIF project, File base, String relative) {
        File file = new File(base, relative);
        log.info("SAVE : " + file.getAbsolutePath());

        boolean isTms = false;

        if (file.getName().endsWith("tms"))
        {
            // log.debug("############## TMS
            // #################");
            isTms = true;
        }
        // else
        // {
        // log.debug("############## GTS #################");
        // }

        Document doc = project.getDomProject();
        try
        {

            if (isTms)
            {
                // docをTMSに変換する。
                saveXml(getTMSDocument(doc), file);
            }
            else
            {
                saveXml(doc, file);
            }

//            ProjectIF[] cp = project.getProjects();
//            if (cp != null)
//            {
//                for (int i = 0; i < cp.length; i++)
//                {
//                    String path = cp[i].getPath();
//                    File cpf = new File(path);
//                    if (cpf.isAbsolute())
//                    {
//                        base = cpf.getParentFile();
//                        relative = cpf.getName();
//                    }
//                    else
//                    {
//                        base = file.getParentFile();
//                        relative = path;
//                    }
//
//                    if (isTms)
//                    {
//                        relative = relative.replaceAll("\\..+$", ".tms");
//                        // log.debug(relative.replaceAll("\\..+$",
//                        // ".tms"));
//                    }
//
//                    saveProject(cp[i], base, relative);
//                }
//            }
//            DatasetIF[] cd = project.getDatasets();
//            if (cd != null)
//            {
//                for (int i = 0; i < cd.length; i++)
//                {
//                    String path = cd[i].getPath();
//                    File cdf = new File(path);
//                    if (cdf.isAbsolute())
//                    {
//                        base = cdf.getParentFile();
//                        relative = cdf.getName();
//                    }
//                    else
//                    {
//                        base = file.getParentFile();
//                        relative = path;
//                    }
//
//                    if (isTms)
//                    {
//                        relative = relative.replaceAll("\\..+$", ".tmm");
//                        // log.debug("############# TMM " +
//                        // relative.replaceAll("\\..+$", ".tmm"));
//                    }
//                    saveDataset(cd[i], base, relative);
//                }
//            }
            
            Object[] objs = project.getNodes();
            if (objs != null)
            {
            	for (int i = 0; i < objs.length; i++)
                {
            		if (objs[i] instanceof ProjectIF)
            		{
            			ProjectIF p = (ProjectIF)objs[i];
            			String path = p.getPath();
            			File cpf = new File(path);
                        if (cpf.isAbsolute())
                        {
                            base = cpf.getParentFile();
                            relative = cpf.getName();
                        }
                        else
                        {
                            base = file.getParentFile();
                            relative = path;
                        }

                        if (isTms)
                        {
                            relative = relative.replaceAll("\\..+$", ".tms");
                        }
                        saveProject(p, base, relative);
            		}
            		else if (objs[i] instanceof DatasetIF)
            		{
            			DatasetIF d = (DatasetIF)objs[i];
            			String path = d.getPath();
                        File cdf = new File(path);
                        if (cdf.isAbsolute())
                        {
                            base = cdf.getParentFile();
                            relative = cdf.getName();
                        }
                        else
                        {
                            base = file.getParentFile();
                            relative = path;
                        }

                        if (isTms)
                        {
                            relative = relative.replaceAll("\\..+$", ".tmm");
                        }
                        saveDataset(d, base, relative);
            		}
                }
            }

        }
        catch (IOException e)
        {
            log.fatal(e.getMessage(), e);
        }
    }
    
    private void saveDataset(DatasetIF dataset, File base, String relative)
    {
        try
        {
//            boolean doit = true;
            File file = new File(base, relative);
            if (!done.contains(file.getAbsolutePath()))
            {
                done.add(file.getAbsolutePath());
//                if (file.exists())
//                {
//                    doit = doOverWrite(file);
//                }
//                if (doit)
//                {
                    log.info("SAVE : " + file.getAbsolutePath());

                    Document doc = null;

                    NIHUTimeSpaceXmlIOFilter filter = new NIHUTimeSpaceXmlIOFilter();
                    File filer = new File(filter.getPath(file.getAbsolutePath()));

                    if (file.getName().endsWith("tmm"))
                    {
                        // log.debug("############## TMM
                        // #################");
                        doc = TMM.create(dataset.getTitle(), dataset.getLayerType(), dataset, "NIHU TIMESPACE XML");
                    }
                    else
                    {
                        // log.debug("############## GTM
                        // #################");
                        doc = GTM.create(dataset.getTitle(), dataset.getLayerType(), dataset, "XML for HuTime", filer.getName());
                    }

                    saveXml(doc, file);

                    Record[] records = dataset.getRecords();
                    if (!done.contains(filer.getAbsolutePath()))
                    {
                        done.add(filer.getAbsolutePath());
//                        if (filer.exists())
//                        {
//                            doit = doOverWrite(filer);
//                        }
//                        if (doit)
//                        {
                            log.info("SAVE : " + filer.getAbsolutePath());
                            byte[] xml = filter.write(records);
                            saveBytedata(xml, filer);
//                        }
                    }
//                }
            }
        }
        catch (IOException e)
        {
            log.fatal(e.getMessage(), e);
        }
        catch (RecordIOFilterException e)
        {
            log.fatal(e.getMessage(), e);
        }
    }
    
    /**
     * バイトデータをファイル出力ストリームに書き込みます。
     * 
     * @param data バイトデータ
     * @param file Fileオブジェクト
     * @throws IOException
     */
    private void saveBytedata(byte[] data, File file) throws IOException
    {
        FileOutputStream out = new FileOutputStream(file);
        out.write(data);
        out.close();
    }

    /**
     * XMLファイルに記入する。
     * 
     * @param doc Documentオブジェクト
     * @param file Fileオブジェクト
     * @throws IOException
     */
    private void saveXml(Document doc, File file) throws IOException
    {
        File parent = file.getParentFile();
        if (parent.exists() || parent.mkdirs())
        {
            DOMImplementation domImpl = new DOMImplementationImpl();
            DOMImplementationLS domImplLS = (DOMImplementationLS) domImpl.getFeature("LS", "3.0");
            LSOutput lsOutput = domImplLS.createLSOutput();
            LSSerializer lsSerializer = domImplLS.createLSSerializer();
            FileOutputStream fos = new FileOutputStream(file);
            lsOutput.setByteStream(fos);
            lsSerializer.write(doc, lsOutput);
            fos.close();
        }
    }
}
