/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin.layer;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.math.BigDecimal;
import java.util.List;

import javax.swing.ImageIcon;
import javax.xml.xpath.XPathExpressionException;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.data.Record;
import jp.nihu.gttime.data.property.PropertiesGts;
import jp.nihu.gttime.data.property.PropertiesIF;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.preferences.BaseLayerPluginPreferences;
import jp.nihu.gttime.time.TimeAxisIF;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * レイヤについてのプロット形式による描画機能（プラグイン）を実装するクラスである。
 */
public class BarChartLayerPlugin extends NumericLayerPlugin
{
    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(BarChartLayerPlugin.class, T2Map.LOCALE);

    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(BarChartLayerPlugin.class);

    protected ImageIcon icon = new ImageIcon(getClass().getClassLoader().getResource("BarChartLayer.gif"));

    protected String name = rmgr.getString("name");

    @Override
    public ImageIcon getIcon()
    {
        return icon;
    }

    @Override
    public String getName()
    {
        return name;
    }

    public String getPluginName()
    {
        return rmgr.getString("plugin.name");
    }

    public String getStyleName()
    {
        return getPluginName() + ".type";
    }

    /**
     * シリアルバーションID
     */
    private static final long serialVersionUID = 1L;

    /**
     * プロットを新規作成します。
     * 
     * @param record Recordオブジェクト
     * @param data TimeAxisIFオブジェクト
     * @return RecordPlot
     */
    protected RecordPlot createRecordPlot(Record record, TimeAxisIF data)
    {
        return new RecordPlot(record, data);
    }

    protected int getPlotSize(String group)
    {
        return 0;
    }

    /**
     * y軸の最大値、最小値を判断します。
     * 
     * @param numberPos 順番
     * @param value 値
     */
    protected void setMinHandMaxH(int numberPos, float value)
    {
        if (numberPos == 0)
        {
            minH = value;
            maxH = value;
        }
        else
        {
            if (value > maxH)
            {
                maxH = value;
            }
            else if (value < minH)
            {
                minH = value;
            }
            else if (value == maxH && value == minH)
            {
                minH = value;
                maxH = minH + 1.1f;
            }
        }

        if (maxH < 0)
        {
            maxH = 0;
        }

        if (minH > 0)
        {
            minH = 0;
        }
    }

    public BaseLayerPluginPreferences getLayerPluginPreferencesDialog()
    {
        BaseLayerPluginPreferences prep = new BarChartLayerPluginPreferences();
//        List<String> groups = dataset.getGroups();
//        for (int i = 0; i < groups.size(); i++) {
//            prep.initValueForLayer(dataset.getProperties(), groups.get(i));
//        }
        prep.initValueForLayer(dataset.getProperties());
        return prep;
    }

    protected void setLayerProperties()
    {
        PropertiesIF properties = dataset.getProperties();
        if (properties == null)
        {
            properties = new PropertiesGts();
            dataset.setProperties(properties);
        }
        try
        {
            List<String> groups = dataset.getGroups();
            for (int i = 0; i < groups.size(); i++) {
                if (properties.getPolygon(groups.get(i)) == null) {
                    setPolygonProperties(groups.get(i));
                }
            }
            if (properties.getSelectedColor() == null)
            {
                setSelectedColorProperties();
            }
            if (properties.getBackgroundColor() == null)
            {
                setBackgroundColorProperties();
            }
            if (properties.getDetailFontName() == null)
            {
                setDetailFontNameProperties();
            }
            if (properties.isShowGroupName() == null)
            {
                setShowGroupNameProperties();
            }
            if (properties.getLayerHeight() == null)
            {
                setLayerHeightProperties();
            }
            if (properties.isDataReduction() == null)
            {
                setDataReductionProperties();
            }
            if (properties.getDataReductionNum() == null)
            {
                setDataReductionNumProperties();
            }
        }
        catch (XPathExpressionException e)
        {
            log.fatal(e.getMessage(), e);
        }
    }

    protected Color getColor(Color c, Color d)
    {
        if (c != null)
        {
            return c;
        }
        else
        {
            return d;
        }
    }

    // /**
    // * number属性の値が存在していれば、レイヤに描画します。
    // *
    // * @param g
    // */
    // public void paintLayer(Graphics g)
    // {
    // super.paintLayer(g);
    // }

    protected boolean doReduce(int ps)
    {
        // return false;
        if (dataset.getProperties().getDataReductionNum() < ps && dataset.getProperties().isDataReduction())
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * プロットの描画やイベントオブジェクトを押下した際の動作などを実装している。
     */
    public class RecordPlot extends NumericLayerPlugin.NumericRecordPlot
    {
        /**
         * プロット描画に関するコンストラクタ
         * 
         * @param record Recordオブジェクト
         * @param data TimeAxisIFオブジェクト
         */
        public RecordPlot(Record record, TimeAxisIF data)
        {
            super(record, data);
        }
        
        @Override
        public void printLegend(Graphics g, float minH, float maxH) {
            // DO NOTHING
        }

        /**
         * 描画処理を実施します。
         * 
         * @param g
         * @param minH Y軸の最小値
         * @param maxH Y軸の最大値
         */
        public void paint(final Graphics g, float minH, float maxH)
        {
            // if (dataset.getProperties().getPolygon() == null)
            // {
            // setLayerProperties();
            // }

            int y = getPositionY(numberVal, minH, maxH, getHeightInsets(), getHeight() - getHeightInsets());
            BigDecimal sd = jdStart.getDatetime();
            BigDecimal ed = jdEnd.getDatetime();
            BigDecimal csd = data.getCurrentMin();
            BigDecimal ced = data.getCurrentMax();
            int xStart = julianToPixel(sd, csd, ced);
            if (xStart > 0)
            {
                int xEndX = julianToPixel(ed, csd, ced);

                int screenRange = (getHeight() - getHeightInsets()) - getHeightInsets();
                float valueRange = maxH - minH;
                float pixelsPerUnit = screenRange / valueRange;
                int mainasu = (int) ((maxH - 0) * pixelsPerUnit + .5) + getHeightInsets();
                int purasu = (int) ((maxH - minH) * pixelsPerUnit + .5) + getHeightInsets();
                // getHeightInsets();

                int ax = xStart;
                int ay = y;
                int aw = xEndX - xStart;
                int ah = getHeight() - (y + getHeightInsets());
                // データにマイナスの値がある場合
                if (mainasu < purasu)
                {
                    if (numberVal >= 0)
                    {
                        ah = mainasu - y;
                    }
                    else
                    {
                        ay = mainasu;
                        ah = y - mainasu;
                    }
                }
                if ((record.isHit() || isClicked()))
                {
                    g.setColor(highlightColorx);
                    g.fillRect(ax, ay, aw, ah);
                    g.drawRect(ax, ay, aw, ah);
                }
                else
                {
                    g.setColor(dataset.getProperties().getPolygon(getGroup()).getFillColor());
                    g.fillRect(ax, ay, aw, ah);
                    g.setColor(dataset.getProperties().getPolygon(getGroup()).getLineColor());
                    g.drawRect(ax, ay, aw, ah);
                }
                addShape(new Rectangle(ax, ay, aw, ah));
            }
        }
    }
}
