/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin.layer;

import javax.xml.xpath.XPathExpressionException;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.data.property.PropertiesIF;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.preferences.BaseLayerPluginPreferences;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * メニューバーの「編集」→「設定...」の棒グラフレイヤのオブジェクトを取得するクラスです。
 */
public class BarChartLayerPluginPreferences extends BaseLayerPluginPreferences
{
    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(BarChartLayerPluginPreferences.class, T2Map.LOCALE);

    /**
     * シリアルバーションID
     */
    private static final long serialVersionUID = 1L;

    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(BarChartLayerPluginPreferences.class);

    @Override
    protected void init()
    {
        super.init();
        setName(rmgr.getString("layer.title"));
    }

    @Override
    public void initValue()
    {
        super.initValue();
        try
        {
            String type = "BarChartLayer.type";
            initPolygon(type);
            initSelectedColor(type);
            initBackgroundColor(type);

            initDetailFontName(type);
            initShowGroupName(type);

            initLayerHeight(type);
            initDataReduction(type);
        }
        catch (XPathExpressionException e)
        {
            log.fatal(e.getMessage(), e);
        }
    }

    @Override
    public void save()
    {
        try
        {
            String type = "BarChartLayer.type";
            savePolygon(type);
            saveSelectedColor(type);
            saveBackgroundColor(type);

            saveDetailFontName(type);
            saveShowGroupName(type);

            saveLayerHeight(type);
            saveDataReduction(type);
        }
        catch (XPathExpressionException e)
        {
            log.fatal(e.getMessage(), e);
        }
    }

    @Override
    public void initValueForLayer(PropertiesIF properties)
    {
        initPolygonForLayer(properties);
        initSelectedColorForLayer(properties);
        initBackgroundColorForLayer(properties);

        initDetailFontNameForLayer(properties);
        initShowGroupNameForLayer(properties);

        initLayerHeightForLayer(properties);
        initDataReductionForLayer(properties);
    }

    @Override
    public void saveForLayer(PropertiesIF properties)
    {
        savePolygonForLayer(properties);
        saveSelectedColorForLayer(properties);
        saveBackgroundColorForLayer(properties);
        
        saveDetailFontNameForLayer(properties);
        saveShowGroupNameForLayer(properties);

        saveLayerHeightForLayer(properties);
        saveDataReductionForLayer(properties);
    }
}
