/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin.layer;

import java.awt.Dimension;
import java.util.List;

import javax.swing.ImageIcon;
import javax.xml.xpath.XPathExpressionException;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.data.property.PropertiesGts;
import jp.nihu.gttime.data.property.PropertiesIF;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.preferences.BaseLayerPluginPreferences;
import jp.nihu.gttime.time.TimeAxisIF;
import jp.nihu.gttime.tree.TreeCheckNode;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class BarTimeScaleLayerPlugin extends DefaultLayerPlugin
{
    /**
     * シリアルバーションID
     */
    private static final long serialVersionUID = 1L;

    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(BarTimeScaleLayerPlugin.class);

    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(BarTimeScaleLayerPlugin.class, T2Map.LOCALE);

    public String getName()
    {
        return rmgr.getString("name");
    }

    public String getStyleName()
    {
        return "BarTimeScaleLayer.type";
    }

    public String getPluginName()
    {
        return rmgr.getString("plugin.name");
    }

    public void init(TimeAxisIF data, final TreeCheckNode node)
    {
        log.debug("Initialize BarTimeScaleLayerPlugin.");
        super.init(data, node);
        // node.setHeight(70);
        dataset.getProperties().setLayerHeight(70);
        this.setMaximumSize(new Dimension(20000000, dataset.getProperties().getLayerHeight()));
        this.setPreferredSize(new Dimension(getWidth(), dataset.getProperties().getLayerHeight()));
    }

    public BaseLayerPluginPreferences getLayerPluginPreferencesDialog()
    {
        BaseLayerPluginPreferences prep = new BarTimeScaleLayerPluginPreferences();
//        List<String> groups = dataset.getGroups();
//        for (int i = 0; i < groups.size(); i++) {
//            prep.initValueForLayer(dataset.getProperties(), groups.get(i));
//        }
        prep.initValueForLayer(dataset.getProperties());
        return prep;
    }

    @Override
    protected void setLayerProperties()
    {
        PropertiesIF properties = dataset.getProperties();
        if (properties == null)
        {
            properties = new PropertiesGts();
            dataset.setProperties(properties);
        }
        try
        {
            List<String> groups = dataset.getGroups();
            for (int i = 0; i < groups.size(); i++) {
                String group = groups.get(i);
                if (properties.getPolygon(group) == null) {
                    setPolygonProperties(group);
                }
                if (properties.getFont(group) == null) {
                    setFontProperties(group);
                }
                if (properties.getFontColor(group) == null) {
                    setFontColorProperties(group);
                }
            }
            if (properties.getSelectedColor() == null)
            {
                setSelectedColorProperties();
            }
            if (properties.getBackgroundColor() == null)
            {
                setBackgroundColorProperties();
            }
            if (properties.getDetailFontName() == null)
            {
                setDetailFontNameProperties();
            }
            if (properties.isShowGroupName() == null)
            {
                setShowGroupNameProperties();
            }
            if (properties.isDataReduction() == null)
            {
                setDataReductionProperties();
            }
            if (properties.getDataReductionNum() == null)
            {
                setDataReductionNumProperties();
            }
        }
        catch (XPathExpressionException e)
        {
            log.fatal(e.getMessage(), e);
        }
    }

    private ImageIcon icon = new ImageIcon(getClass().getClassLoader().getResource("TimeChartLayer2.gif"));

    @Override
    public ImageIcon getIcon()
    {
        return icon;
    }
}
