/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin.layer;

import java.awt.Color;
import java.awt.Graphics;
import java.util.List;

import javax.swing.ImageIcon;
import javax.xml.xpath.XPathExpressionException;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.data.Record;
import jp.nihu.gttime.data.property.PropertiesGts;
import jp.nihu.gttime.data.property.PropertiesIF;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.preferences.BaseLayerPluginPreferences;
import jp.nihu.gttime.time.TimeAxisIF;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * 折れ線グラフのレイヤを描画する機能（プラグイン）を実装したクラスです。
 */
public class LineChartLayerPlugin extends PlotChartLayerPlugin
{
    /**
     * シリアルバーションID
     */
    private static final long serialVersionUID = 1L;

    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(LineChartLayerPlugin.class);

    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(LineChartLayerPlugin.class, T2Map.LOCALE);

    public String getName()
    {
        return rmgr.getString("name");
    }

    public String getPluginName()
    {
        return rmgr.getString("plugin.name");
    }

    public String getStyleName()
    {
        return getPluginName() + ".type";
    }

    /**
     * プロットを新規作成します。
     * 
     * @param record Recordオブジェクト
     * @param data TimeAxisIFオブジェクト
     * @return RecordPlot
     */
    protected RecordPlot createRecordPlot(Record record, TimeAxisIF data)
    {
        return new RecordPlot(record, data);
    }

    /**
     * レイヤの描画
     * 
     * @param g
     */
    public void paintLayer(Graphics g)
    {
        super.paintLayer(g);
        initXY();
    }

    public BaseLayerPluginPreferences getLayerPluginPreferencesDialog()
    {
        BaseLayerPluginPreferences prep = new LineChartLayerPluginPreferences();
//        List<String> groups = dataset.getGroups();
//        for (int i = 0; i < groups.size(); i++) {
//            prep.initValueForLayer(dataset.getProperties(), groups.get(i));
//        }
        prep.initValueForLayer(dataset.getProperties());
        return prep;
    }

    @Override
    protected void setLayerProperties()
    {
        PropertiesIF properties = dataset.getProperties();
        if (properties == null)
        {
            properties = new PropertiesGts();
            dataset.setProperties(properties);
        }
        try
        {
            List<String> groups = dataset.getGroups();
            for (int i = 0; i < groups.size(); i++) {
                String group = groups.get(i);
                if (properties.getLine(group) == null) {
                    setLineProperties(group);
                }
                if (properties.getSymbol(group) == null) {
                    setSymbolProperties(group);
                }
            }
            if (properties.getSelectedColor() == null)
            {
                setSelectedColorProperties();
            }
            if (properties.getBackgroundColor() == null)
            {
                setBackgroundColorProperties();
            }
            if (properties.getDetailFontName() == null)
            {
                setDetailFontNameProperties();
            }
            if (properties.isShowGroupName() == null)
            {
                setShowGroupNameProperties();
            }
            if (properties.getLayerHeight() == null)
            {
                setLayerHeightProperties();
            }
            if (properties.isDataReduction() == null)
            {
                setDataReductionProperties();
            }
            if (properties.getDataReductionNum() == null)
            {
                setDataReductionNumProperties();
            }
        }
        catch (XPathExpressionException e)
        {
            log.fatal(e.getMessage(), e);
        }
    }

    /**
     * 折れ線の描画やイベントオブジェクトを押下した際の動作などを実装している。
     */
    public class RecordPlot extends PlotChartLayerPlugin.RecordPlot
    {
        /**
         * 折れ線描画に関するコンストラクタ
         * 
         * @param record Recordオブジェクト
         * @param data TimeAxisIFオブジェクト
         */
        public RecordPlot(Record record, TimeAxisIF data)
        {
            super(record, data);
        }

        // /* (non-Javadoc)
        // * @see
        // jp.nihu.gttime.plugin.layer.PlotChartLayerPlugin.RecordPlot#isIntersect(int,
        // int)
        // */
        // @Override
        // protected boolean isIntersect(int x, int y)
        // {
        // if (previouseEnd != null &&
        // (this.record.getStartDatetime().getDatetime().subtract(previouseEnd)).compareTo(BigDecimal.valueOf(est_coninuance_num))
        // > 0)
        // {
        // return false;
        // }
        // return super.isIntersect(x, y);
        // }

        // protected boolean doPaint()
        // {
        // if (previouseEnd != null &&
        // (this.record.getStartDatetime().getDatetime().subtract(previouseEnd)).compareTo(BigDecimal.valueOf(est_coninuance_num))
        // > 0)
        // {
        // return true;
        // }
        // return super.doPaint();
        // }

        protected String getGroupName()
        {
            return getGroup();
        }

        private class LineChartGroupInfo
        {
            protected int prevx = 0;

            protected int prevy = 0;
        }

        @Override
        protected void paintPlot(Graphics g, int x, int y)
        {
            String group = getGroupName();
            LineChartGroupInfo info = (LineChartGroupInfo) groups.get(group);
            if (info == null)
            {
                info = new LineChartGroupInfo();
                previouseX = info.prevx;
                previouseY = info.prevy;
            }
            else
            {
                previouseX = info.prevx;
                previouseY = info.prevy;
            }

            // boolean drowLine = false;
            // if (previouseEnd != null)
            // {
            // drowLine =
            // (this.record.getStartDatetime().getDatetime().subtract(previouseEnd)).compareTo(BigDecimal.valueOf(est_coninuance_num))
            // < 0;
            // }
            // if (drowLine && previouseX != 0)

            if (previouseX != 0 && x > previouseX)
            {
                Color lineColor = null;
                try
                {
                    lineColor = dataset.getProperties().getLine(getGroup()).getLineColor();
                }
                catch (NullPointerException e)
                {
                    setLayerProperties();
                }
                int lineThick = dataset.getProperties().getLine(getGroup()).getThin();
                g.setColor(lineColor);
                for (int i = 0; i < lineThick; i++)
                {
                    g.drawLine(x, y - i, previouseX, previouseY - i);
                }
            }

            {
                if (isNextValueNull)
                {
                    info.prevx = 0;
                    info.prevy = 0;
                    previouseX = 0;
                    previouseY = 0;
                }
                else
                {
                    info.prevx = x;
                    info.prevy = y;
                    previouseX = x;
                    previouseY = y;
                }
                groups.put(group, info);
            }
            super.paintPlot(g, x, y);
        }

        /*
         * @Override protected void paintPlot(Graphics g, int x, int y) { // if
         * (dataset.getProperties().getLine() == null ||
         * dataset.getProperties().getSymbol() == null) // { //
         * setLayerProperties(); // } boolean drowLine = false; if (previouseEnd !=
         * null) {
         * //log.debug(this.record.getStartDatetime().getDatetime() + " - " +
         * previouseEnd + " = " +
         * this.record.getStartDatetime().getDatetime().subtract(previouseEnd));
         * drowLine =
         * (this.record.getStartDatetime().getDatetime().subtract(previouseEnd)).compareTo(BigDecimal.valueOf(est_coninuance_num)) <
         * 0; } //if (previouseX != 0 && previouseY != 0) if (drowLine &&
         * previouseX != 0) { Color lineColor =
         * dataset.getProperties().getLine().getLineColor(); try { lineColor =
         * dataset.getProperties().getLine().getLineColor(); }
         * catch(NullPointerException e) { setLayerProperties(); } int lineThick =
         * dataset.getProperties().getLine().getThin(); g.setColor(lineColor); //
         * int cal = lineThick / 2; int calx =
         * dataset.getProperties().getSymbol().getSize() / 2; for (int i = 0; i <
         * lineThick; i++) { if (x - calx > previouseX + calx) { // g.drawLine(x -
         * calx, y + cal - i, previouseX + calx, previouseY + cal - i);
         * g.drawLine(x, y - i, previouseX, previouseY - i); } else { if (y >
         * previouseY) { // g.drawLine(x, y + cal - i - calx, previouseX,
         * previouseY + cal - i + calx); g.drawLine(x, y - i, previouseX,
         * previouseY - i); } else { // g.drawLine(x, y + cal - i + calx,
         * previouseX, previouseY + cal - i - calx); g.drawLine(x, y - i,
         * previouseX, previouseY - i); } } } } // if (isNextValueNull ||
         * isValueNull) // { // initXY(); // } // else { previouseX = x;
         * previouseY = y; } super.paintPlot(g, x, y); }
         */
    }

    // private double est_coninuance_num = 1;

    /**
     * 線を結ぶときの2点のうち画面に向かって左側のプロットのX値
     */
    private int previouseX = 0;

    /**
     * 線を結ぶときの2点のうち画面に向かって左側のプロットのY値
     */
    private int previouseY = 0;

    /**
     * 線を結ぶときの2点のうち画面に向かって左側のプロットのX値とY値を初期化します。
     */
    private void initXY()
    {
        previouseX = 0;
        previouseY = 0;
    }

    private ImageIcon icon = new ImageIcon(getClass().getClassLoader().getResource("LineChartLayer.gif"));

    @Override
    public ImageIcon getIcon()
    {
        return icon;
    }
}
