/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin.layer;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.xml.xpath.XPathExpressionException;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.data.Record;
import jp.nihu.gttime.data.property.PropertiesGts;
import jp.nihu.gttime.data.property.PropertiesIF;
import jp.nihu.gttime.data.property.SymbolGts;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.preferences.BaseLayerPluginPreferences;
import jp.nihu.gttime.time.TimeAxisIF;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * レイヤについてのプロット形式による描画機能（プラグイン）を実装するクラスである。
 */
public class PlotChartLayerPlugin extends NumericLayerPlugin
{
    /**
     * シリアルバーションID
     */
    private static final long serialVersionUID = 1L;

    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(PlotChartLayerPlugin.class);

    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(PlotChartLayerPlugin.class, T2Map.LOCALE);

    public String getName()
    {
        return rmgr.getString("name");
    }

    public String getPluginName()
    {
        return rmgr.getString("plugin.name");
    }

    public String getStyleName()
    {
        return getPluginName() + ".type";
    }

    protected Map<String, Image> plotImages = new HashMap<String, Image>();

    /**
     * プロットを間引く処理を適用する表示レコード数
     */
    // private int plotReduceNum = REDUCE_PLOT_NUMBER;
    /**
     * レイヤの描画
     * 
     * @param g
     */
    public void paintLayer(Graphics g)
    {
        setPlotStyle();
        super.paintLayer(g);
    }
    
    private void setPlotStyle() {
        List<String> groups = dataset.getGroups();
        for (int i = 0; i < groups.size(); i++) {
            setPlotStyle(groups.get(i));
        }
    }
    
    private void setPlotStyle(String group) {
        int plotSize = dataset.getProperties().getSymbol(group).getSize();
        if (highLightImage == null || dataset.getProperties().isChanged())
        {
            highLightImage = createImage(plotSize + 2, plotSize + 2);
            Graphics2D gx1 = (Graphics2D) highLightImage.getGraphics();
            gx1.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            gx1.setColor(this.getHighLightColor());

            
            String symbol = dataset.getProperties().getSymbol(group).getSymbol();
            if (symbol.equals(SymbolGts.CIRCLE_STRING))
            {
                gx1.fillOval(0, 0, plotSize, plotSize);
                gx1.drawOval(0, 0, plotSize, plotSize);
            }
            else if (symbol.equals(SymbolGts.RECTANGLE_STRING))
            {
            	gx1.fillRect(0, 0, plotSize, plotSize);
            	gx1.drawRect(0, 0, plotSize, plotSize);
            }
            else if (symbol.equals(SymbolGts.TRIANGLE_STRING))
            {
                int pointNum = 3;
                int[] xpoints = new int[pointNum];
                int[] ypoints = new int[pointNum];
                xpoints[0] = 0;
                ypoints[0] = 0 + plotSize;
                xpoints[1] = 0 + plotSize;
                ypoints[1] = 0 + plotSize;
                xpoints[2] = 0 + (plotSize / 2);
                ypoints[2] = 0;
                Polygon polygon = new Polygon(xpoints, ypoints, pointNum);
                gx1.fillPolygon(polygon);
                gx1.drawPolygon(polygon);
            }
            else
            {
            	gx1.fillRect(0, 0, plotSize, plotSize);
            	gx1.drawRect(0, 0, plotSize, plotSize);
            }
            
        }
        Image plotImage = plotImages.get(group);
        if (plotImage == null || dataset.getProperties().isChanged())
        {
            plotImage = createImage(plotSize + 2, plotSize + 2);
            Graphics2D gx2 = (Graphics2D) plotImage.getGraphics();
            gx2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            drawPlot(gx2, 0, 0, plotSize, group);
            plotImages.put(group, plotImage);
        }
    }

//    private void drawPlot(Graphics2D g, int x, int y, int plotSize)
//    {
//        drawPlot(g, x, y, plotSize, null);
//    }
    
    private void drawPlot(Graphics2D g, int x, int y, int plotSize, String group)
    {
        Color plotColorIn = dataset.getProperties().getSymbol(group).getFillColor();
        
        Color plotColorOut = dataset.getProperties().getSymbol(group).getLineColor();
        String symbol = dataset.getProperties().getSymbol(group).getSymbol();
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        if (symbol.equals(SymbolGts.CIRCLE_STRING))
        {
            g.setColor(plotColorIn);
            g.fillOval(x, y, plotSize, plotSize);
            g.setColor(plotColorOut);
            g.drawOval(x, y, plotSize, plotSize);
        }
        else if (symbol.equals(SymbolGts.RECTANGLE_STRING))
        {
            g.setColor(plotColorIn);
            g.fillRect(x, y, plotSize, plotSize);
            g.setColor(plotColorOut);
            g.drawRect(x, y, plotSize, plotSize);
        }
        else if (symbol.equals(SymbolGts.TRIANGLE_STRING))
        {
            int pointNum = 3;
            int[] xpoints = new int[pointNum];
            int[] ypoints = new int[pointNum];
            xpoints[0] = x;
            ypoints[0] = y + plotSize;
            xpoints[1] = x + plotSize;
            ypoints[1] = y + plotSize;
            xpoints[2] = x + (plotSize / 2);
            ypoints[2] = y;

            Polygon polygon = new Polygon(xpoints, ypoints, pointNum);
            g.setColor(plotColorIn);
            g.fillPolygon(polygon);
            g.setColor(plotColorOut);
            g.drawPolygon(polygon);
        }
        else
        {
            g.setColor(plotColorIn);
            g.fillOval(x, y, plotSize, plotSize);
            g.setColor(plotColorOut);
            g.drawOval(x, y, plotSize, plotSize);
        }
    }
    

    @Override
    protected boolean doReduce(int ps)
    {
        if (dataset.getProperties().getDataReductionNum() < ps && dataset.getProperties().isDataReduction())
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    public BaseLayerPluginPreferences getLayerPluginPreferencesDialog()
    {
        BaseLayerPluginPreferences prep = new PlotChartLayerPluginPreferences();
//        List<String> groups = dataset.getGroups();
//        for (int i = 0; i < groups.size(); i++) {
//            prep.initValueForLayer(dataset.getProperties(), groups.get(i));
//        }
        prep.initValueForLayer(dataset.getProperties());
        return prep;
    }

    @Override
    protected void setLayerProperties()
    {
        PropertiesIF properties = dataset.getProperties();
        if (properties == null)
        {
            properties = new PropertiesGts();
            dataset.setProperties(properties);
        }
        try
        {
            List<String> groups = dataset.getGroups();
            for (int i = 0; i < groups.size(); i++) {
                if (properties.getSymbol(groups.get(i)) == null) {
                    setSymbolProperties(groups.get(i));
                }
            }
            if (properties.getSelectedColor() == null)
            {
                setSelectedColorProperties();
            }
            if (properties.getBackgroundColor() == null)
            {
                setBackgroundColorProperties();
            }
            if (properties.getDetailFontName() == null)
            {
                setDetailFontNameProperties();
            }
            if (properties.isShowGroupName() == null)
            {
                setShowGroupNameProperties();
            }
            if (properties.getLayerHeight() == null)
            {
                setLayerHeightProperties();
            }
            if (properties.isDataReduction() == null)
            {
                setDataReductionProperties();
            }
            if (properties.getDataReductionNum() == null)
            {
                setDataReductionNumProperties();
            }
        }
        catch (XPathExpressionException e)
        {
            log.fatal(e.getMessage(), e);
        }
    }

    protected Color getColor(Color c, Color d)
    {
        if (c != null)
        {
            return c;
        }
        else
        {
            return d;
        }
    }

    protected int getSize(String size, int defaultSize)
    {
        if (size != null && size.length() > 0)
        {
            return Integer.parseInt(size);
        }
        else
        {
            return defaultSize;
        }
    }

    protected int getPlotSize(String group)
    {
        int plotSize = dataset.getProperties().getSymbol(group).getSize();
        return plotSize;
    }

    /**
     * プロットを新規作成します。
     * 
     * @param record Recordオブジェクト
     * @param data TimeAxisIFオブジェクト
     * @return RecordPlot
     */
    protected RecordPlot createRecordPlot(Record record, TimeAxisIF data)
    {
        return new RecordPlot(record, data);
    }

    private Shape prevPlotShape = null;

    /**
     * プロット形式の描画やイベントオブジェクトを押下した際の動作などを実装している。
     */
    public class RecordPlot extends NumericLayerPlugin.NumericRecordPlot
    {
        /**
         * プロット形式の描画に関するコンストラクタ
         * 
         * @param record Recordオブジェクト
         * @param data TimeAxisIFオブジェクト
         */
        public RecordPlot(Record record, TimeAxisIF data)
        {
            super(record, data);
        }
        
        @Override
        public void printLegend(Graphics g, float minH, float maxH) {
            log.debug(dataset.getProperties());
            log.debug(dataset.getProperties().isShowGroupName());
            if (!getGroup().equals("") && dataset.getProperties().isShowGroupName()) {
                int y = getPositionY(numberVal, minH, maxH, getHeightInsets(), getHeight() - getHeightInsets());

                BigDecimal sd = jdStart.getDatetime();
                BigDecimal ed = jdEnd.getDatetime();
                BigDecimal csd = data.getCurrentMin();
                BigDecimal ced = data.getCurrentMax();
                BigDecimal plotPoint = (sd.add(ed)).divide(new BigDecimal(2.0));
                int x = julianToPixel(plotPoint, csd, ced);
                
                String fontName = null;
//                if (dataset.getProperties().getFont("") != null) {
//                    fontName = dataset.getProperties().getFont("").getFontName();
//                }
                g.setFont(new Font(fontName, Font.PLAIN, 12));
                Dimension strSize = stringSize(getGroup(), g);
                //g.setColor(Color.black);
                g.setColor(dataset.getProperties().getSymbol(getGroup()).getFillColor());
                g.drawString(getGroup(), x, y - strSize.height);
                //g.drawLine(1, h - 1, w - 1, h - 1);
            }
        }

        /**
         * 描画処理を実施します。
         * 
         * @param g グラフィックオブジェクト
         * @param minH Y軸の最小値
         * @param maxH Y軸の最大値
         */
        public void paint(Graphics g, float minH, float maxH)
        {
            int y = getPositionY(numberVal, minH, maxH, getHeightInsets(), getHeight() - getHeightInsets());

            BigDecimal sd = jdStart.getDatetime();
            BigDecimal ed = jdEnd.getDatetime();
            BigDecimal csd = data.getCurrentMin();
            BigDecimal ced = data.getCurrentMax();
            // int x = (julianToPixel(sd, csd, ced) + julianToPixel(ed, csd,
            // ced)) / 2;
            BigDecimal plotPoint = (sd.add(ed)).divide(new BigDecimal(2.0));
            int x = julianToPixel(plotPoint, csd, ced);

            // if (this.isClicked() || this.record.isHit() || doPaint() || (x >=
            // 0 && !isIntersect(x, y)))
            if (this.isClicked() || this.record.isHit() || (x >= 0 && !isIntersect(x, y)))
            // if (this.isClicked() || this.record.isHit() || !isIntersect(x,
            // y))
            {
                paintPlot(g, x, y);
            }
        }

        /**
         * プロットを描画します。
         * 
         * @param g グラフィックオブジェクト
         */
        protected void paintPlot(Graphics g, int x, int y)
        {
            String group = getGroup();
            int plotSize = dataset.getProperties().getSymbol(group).getSize();
            int cal = plotSize / 2;
            if (record.isHit() || isClicked())
            {
//                g.setColor(highlightColorx);
//                g.fillOval(x - cal, y - cal, plotSize, plotSize);
//                g.drawOval(x - cal, y - cal, plotSize, plotSize);
            	String symbol = dataset.getProperties().getSymbol(group).getSymbol();
                ((Graphics2D)g).setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
                if (symbol.equals(SymbolGts.CIRCLE_STRING))
                {
                    g.setColor(highlightColorx);
                    g.fillOval(x - cal, y - cal, plotSize, plotSize);
                    g.setColor(highlightColorx);
                    g.drawOval(x - cal, y - cal, plotSize, plotSize);
                }
                else if (symbol.equals(SymbolGts.RECTANGLE_STRING))
                {
                    g.setColor(highlightColorx);
                    g.fillRect(x - cal, y - cal, plotSize, plotSize);
                    g.setColor(highlightColorx);
                    g.drawRect(x - cal, y - cal, plotSize, plotSize);
                }
                else if (symbol.equals(SymbolGts.TRIANGLE_STRING))
                {
                    int pointNum = 3;
                    int[] xpoints = new int[pointNum];
                    int[] ypoints = new int[pointNum];
                    xpoints[0] = x - cal;
                    ypoints[0] = y - cal + plotSize;
                    xpoints[1] = x - cal + plotSize;
                    ypoints[1] = y - cal + plotSize;
                    xpoints[2] = x - cal + (plotSize / 2);
                    ypoints[2] = y - cal;

                    Polygon polygon = new Polygon(xpoints, ypoints, pointNum);
                    g.setColor(highlightColorx);
                    g.fillPolygon(polygon);
                    g.setColor(highlightColorx);
                    g.drawPolygon(polygon);
                }
                else
                {
                    g.setColor(highlightColorx);
                    g.fillOval(x - cal, y - cal, plotSize, plotSize);
                    g.setColor(highlightColorx);
                    g.drawOval(x - cal, y - cal, plotSize, plotSize);
                }
                
            }
            else
            {
                if (paintMode != 3)
                {
                    Image plotImage = plotImages.get(group);
                    if (plotImage == null) {
                        setPlotStyle(group);
                    }
                    plotImage = plotImages.get(group);
                    g.drawImage(plotImage, x - cal, y - cal, PlotChartLayerPlugin.this);
                }
                else
                {
                    drawPlot((Graphics2D) g, x - cal, y - cal, plotSize, group);
                }
            }
            addShape(new Rectangle(x - cal, y - cal, plotSize, plotSize));
        }

        protected boolean isIntersect(int x, int y)
        {
            if (!reduce)
            {
                return false;
            }
            int plotSize = dataset.getProperties().getSymbol(getGroup()).getSize();
            int cal = plotSize / 2;
            boolean intersect = false;
            // Rectangle a = new Rectangle(x - cal, y - cal, plotSize * 4,
            // plotSize * 4);
            Rectangle a = new Rectangle(x - cal, y - cal, plotSize, plotSize);
            if (reduce && prevPlotShape != null)
            {
                intersect = prevPlotShape.intersects(a);
            }
            if (!intersect)
            {
                prevPlotShape = a;
            }
            return intersect;
        }
    }

    private ImageIcon icon = new ImageIcon(getClass().getClassLoader().getResource("PlotChartLayer.gif"));

    @Override
    public ImageIcon getIcon()
    {
        return icon;
    }
}
