/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin.layer;

import javax.xml.xpath.XPathExpressionException;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.data.property.PropertiesIF;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.preferences.BaseLayerPluginPreferences;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * メニューバーの「編集」→「設定...」の年表表示レイヤのオブジェクトを取得するクラスです。
 */
public class TimeScaleLayerPluginPreferences extends BaseLayerPluginPreferences
{
    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(TimeScaleLayerPluginPreferences.class, T2Map.LOCALE);

    /**
     * シリアルバーションID
     */
    private static final long serialVersionUID = 1L;

    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(TimeScaleLayerPluginPreferences.class);

    @Override
    protected void init()
    {
        super.init();
        setName(rmgr.getString("layer.title"));
    }

    @Override
    public void initValue()
    {
        super.initValue();
        try
        {
            String type = "TimeScaleLayer.type";
            initFont(type);
            initScaleMain(type);
            initScaleSub(type);
            initBackgroundColor(type);
            initDataReduction(type);
        }
        catch (XPathExpressionException e)
        {
            log.fatal(e.getMessage(), e);
        }
    }

    @Override
    public void save()
    {
        try
        {
            String type = "TimeScaleLayer.type";
            saveFont(type);
            saveScaleMain(type);
            saveScaleSub(type);
            saveBackgroundColor(type);
            saveDataReduction(type);
        }
        catch (XPathExpressionException e)
        {
            log.fatal(e.getMessage(), e);
        }
    }

    @Override
    public void initValueForLayer(PropertiesIF properties)
    {
        initFontForLayer(properties);
        initScaleMainForLayer(properties);
        initScaleSubForLayer(properties);
        initBackgroundColorForLayer(properties);
        initDataReductionForLayer(properties);
    }

    @Override
    public void saveForLayer(PropertiesIF properties)
    {
        saveFontForLayer(properties);
        saveScaleMainForLayer(properties);
        saveScaleSubForLayer(properties);
        saveBackgroundColorForLayer(properties);
        saveDataReductionForLayer(properties);
    }
}
