/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin.measure;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.math.BigDecimal;
import java.math.BigInteger;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.data.Event;
import jp.nihu.gttime.data.JulianDate;
import jp.nihu.gttime.data.Record;
import jp.nihu.gttime.plugin.NumericMeasurePlugin;
import jp.nihu.gttime.plugin.PluginResourceManager;

/**
 * 指定時間範囲内の最大最小平均値を表示する機能
 */
public class MaxMinAveNum extends NumericMeasurePlugin
{
    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(MaxMinAveNum.class, T2Map.LOCALE);

    protected void measure(Record[] records, BigDecimal[][] durations)
    {
    	init();
        for (int i = 0; i < records.length; i++)
        {
            Event[] events = records[i].getEvents();
            for (int j = 0; j < events.length; j++)
            {
                if (events[j].getName().equals("number"))
                {
                    for (int k = 0; k < durations.length; k++)
                    {
                        if (isContain(records[i], durations[k][0], durations[k][1]))
                        {
                            float cnum = Float.valueOf(events[j].getData()).floatValue();
                            setASelectMaxMinH(anumberNum, cnum);
                            aaverage = aaverage + cnum;
                            aaverage = Math.round(aaverage * 1000.0);
                            aaverage = aaverage / 1000;
                            anumberNum++;
                        }
                    }

                    break;
                }
            }
        }
        setResultToPanel();
    }
    
    
    /**
     * 計測結果を表示するダイアログを作成します。
     * 
     * @param records Record[]オブジェクト
     * @param startPoint 開始日時
     * @param endPoint 終了日時
     */
    public void measure(Record[] records, BigDecimal startPoint, BigDecimal endPoint, BigDecimal[][] durations)
    {
        init();
        for (int i = 0; i < records.length; i++)
        {
            Event[] events = records[i].getEvents();
            for (int j = 0; j < events.length; j++)
            {
                if (events[j].getName().equals("number"))
                {
                    if (isContain(records[i], startPoint, endPoint))
                    {
                        float cnum = Float.valueOf(events[j].getData()).floatValue();
                        setSelectMaxMinH(numberNum, cnum);
                        average = average + cnum;
                        average = Math.round(average * 1000.0);
                        average = average / 1000;
                        numberNum++;
                    }

                    for (int k = 0; k < durations.length; k++)
                    {
                        if (isContain(records[i], durations[k][0], durations[k][1]))
                        {
                            float cnum = Float.valueOf(events[j].getData()).floatValue();
                            setASelectMaxMinH(anumberNum, cnum);
                            aaverage = aaverage + cnum;
                            aaverage = Math.round(aaverage * 1000.0);
                            aaverage = aaverage / 1000;
                            anumberNum++;
                        }
                    }

                    break;
                }
            }
        }
        setResultToPanel();
    }

    protected void init()
    {
        dialog.setResizable(false);
        dialog.setTitle(rmgr.getString("measure.title"));
    }

    protected void setResultToPanel()
    {
    	if (onlyAll)
    	{
    		dialog.setSize(200, 170);
    		setResultToPanel2(rmgr.getString("measure.title.part2"));
    		return;
    	}
        if (numberNum != anumberNum)
        {
            dialog.setSize(200, 270);
            setResultToPanel1(rmgr.getString("measure.title.part1"));
            setResultToPanel2(rmgr.getString("measure.title.part2"));
        }
        else
        {
            dialog.setSize(200, 170);
            setResultToPanel1(rmgr.getString("measure.title.part1"));
        }
    }

    private void setResultToPanel1(String title)
    {
        JPanel p = new JPanel();
        p.setLayout(new GridBagLayout());
        GridBagConstraints gridBagConstraints0 = new GridBagConstraints();
        gridBagConstraints0.gridx = 0;
        gridBagConstraints0.gridy = 0;
        gridBagConstraints0.anchor = GridBagConstraints.WEST;
        GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.anchor = GridBagConstraints.WEST;
        GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.anchor = GridBagConstraints.WEST;
        GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
        gridBagConstraints3.gridx = 0;
        gridBagConstraints3.gridy = 3;
        gridBagConstraints3.anchor = GridBagConstraints.WEST;

        p.setBorder(new TitledBorder(title));

        JLabel maxName = new JLabel(rmgr.getString("max.value") + selectMax);
        p.add(maxName, gridBagConstraints0);

        JLabel minName = new JLabel(rmgr.getString("min.value") + selectMin);
        p.add(minName, gridBagConstraints1);

        average = Math.round(average * 1000);
        numberNum = numberNum * 1000;
        JLabel aveName = null;
        if (numberNum != 0)
        {
            aveName = new JLabel(rmgr.getString("ave.value") + average / numberNum);
        }
        else
        {
            aveName = new JLabel(rmgr.getString("ave.value") + selectMin);
        }
        p.add(aveName, gridBagConstraints2);

        numberNum = numberNum / 1000;
        JLabel num = new JLabel(rmgr.getString("record.count") + numberNum);
        p.add(num, gridBagConstraints3);

        p.setPreferredSize(new Dimension(dialog.getWidth() - 20, 100));
        resultPanel.add(p, Component.CENTER_ALIGNMENT);
    }

    private void setResultToPanel2(String title)
    {
        JPanel p = new JPanel();
        p.setLayout(new GridBagLayout());
        GridBagConstraints gridBagConstraints0 = new GridBagConstraints();
        gridBagConstraints0.gridx = 0;
        gridBagConstraints0.gridy = 0;
        gridBagConstraints0.anchor = GridBagConstraints.WEST;
        GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 1;
        gridBagConstraints1.anchor = GridBagConstraints.WEST;
        GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.anchor = GridBagConstraints.WEST;
        GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
        gridBagConstraints3.gridx = 0;
        gridBagConstraints3.gridy = 3;
        gridBagConstraints3.anchor = GridBagConstraints.WEST;

        p.setBorder(new TitledBorder(title));

        JLabel maxName = new JLabel(rmgr.getString("max.value") + aselectMax);
        p.add(maxName, gridBagConstraints0);

        JLabel minName = new JLabel(rmgr.getString("min.value") + aselectMin);
        p.add(minName, gridBagConstraints1);

        aaverage = Math.round(aaverage * 1000);
        anumberNum = anumberNum * 1000;
        JLabel aveName = null;
        if (anumberNum != 0)
        {
            aveName = new JLabel(rmgr.getString("ave.value") + aaverage / anumberNum);
        }
        else
        {
            aveName = new JLabel(rmgr.getString("ave.value") + aselectMin);
        }
        p.add(aveName, gridBagConstraints2);

        anumberNum = anumberNum / 1000;
        JLabel num = new JLabel(rmgr.getString("record.count") + anumberNum);
        p.add(num, gridBagConstraints3);

        p.setPreferredSize(new Dimension(dialog.getWidth() - 20, 100));
        resultPanel.add(p, Component.CENTER_ALIGNMENT);
    }

    protected boolean isContain(Record record, BigDecimal startPoint, BigDecimal endPoint)
    {
        JulianDate jdStart = record.getStartDatetime();
        JulianDate jdEnd = record.getEndDatetime();
        BigDecimal sdpoint = jdStart.getDatetime();
        BigDecimal edpoint = jdEnd.getDatetime();

        BigInteger sPoint = startPoint.toBigInteger();
        BigInteger ePoint = endPoint.toBigInteger();
        BigInteger plotPoint = ((edpoint.add(sdpoint)).divide(new BigDecimal(2.0))).toBigInteger();

        if (sPoint.compareTo(plotPoint) < 0 && ePoint.compareTo(plotPoint) > 0)
        {
            return true;
        }

        return false;
    }

    /**
     * レコード数
     */
    protected int numberNum = 0;

    /**
     * 平均値
     */
    protected float average = 0;

    /**
     * 最大値
     */
    protected float selectMax = 0;

    /**
     * 最小値
     */
    protected float selectMin = 0;

    /**
     * 測定時Y軸の最小値と最大値を求める
     * 
     * @param numberPos カウント個数
     * @param value Y軸の値
     */
    public void setSelectMaxMinH(int numberPos, float value)
    {
        if (numberPos == 0)
        {
            selectMin = value;
            selectMax = value;
        }
        else
        {
            if (value > selectMax)
            {
                selectMax = value;
            }
            else if (value < selectMin)
            {
                selectMin = value;
            }
            else if (value == selectMax && value == selectMin)
            {
                selectMin = value;
                selectMax = selectMin;
            }
        }
    }

    /**
     * レコード数
     */
    protected int anumberNum = 0;

    /**
     * 平均値
     */
    protected float aaverage = 0;

    /**
     * 最大値
     */
    protected float aselectMax = 0;

    /**
     * 最小値
     */
    protected float aselectMin = 0;

    /**
     * 測定時Y軸の最小値と最大値を求める
     * 
     * @param numberPos カウント個数
     * @param value Y軸の値
     */
    public void setASelectMaxMinH(int numberPos, float value)
    {
        if (numberPos == 0)
        {
            aselectMin = value;
            aselectMax = value;
        }
        else
        {
            if (value > aselectMax)
            {
                aselectMax = value;
            }
            else if (value < aselectMin)
            {
                aselectMin = value;
            }
            else if (value == aselectMax && value == aselectMin)
            {
                aselectMin = value;
                aselectMax = aselectMin;
            }
        }
    }
}