/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin.printer;

import java.awt.Dimension;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.plugin.ImageOutputPlugin;
import jp.nihu.gttime.plugin.LayerData;
import jp.nihu.gttime.time.TimeAxisIF;

/**
 * {@code LayerPlugin} オブジェクトの印刷／画像出力を行うプラグイン抽象クラス
 */
public abstract class AbstractImageOutputPlugin implements ImageOutputPlugin
{

    private T2Map t2map;

    /**
     * オブジェクトの初期化を行う。
     * 
     * @param t2map {@code T2Map} オブジェクト
     */
    public void initPlugin(T2Map t2map)
    {
        this.t2map = t2map;
        initPlugin();
    }

    /**
     * オブジェクトの初期化を行う。
     */
    protected abstract void initPlugin();

    /**
     * {@code T2Map} オブジェクトを取得する。
     */
    protected T2Map getT2Map()
    {
        return t2map;
    }

    /**
     * 印刷／画像出力用ダイアログを表示する。
     */
    public abstract void output();

    /**
     * 現在アプリケーション側で表示している {@code LayerData} オブジェクト配列 を取得する。
     */
    public abstract LayerData[] getLayerDatas();

    /**
     * 現在アプリケーション側で表示している時間情報を {@code TimeAxisIF} オブジェクトとして取得する。
     */
    public abstract TimeAxisIF getTimeAxisData();

    /**
     * 保存可能なファイル拡張子を取得する。
     */
    public abstract String[] getWriterFileSuffixes();

    /**
     * 現在アプリケーション側で表示しているレイヤ用パネルコンポーネントの サイズを取得する。
     */
    public abstract Dimension getLayerViewSize();

    /**
     * 指定した条件で描画した画像の印刷を行う。
     * 
     * @param axis 出力する時間範囲を表す {@code TimeAxisIF} オブジェクト
     * @param datas 出力対象となる {@code LayerData} オブジェクト配列
     * @return 出力に成功した場合 {@code true}
     */
    public abstract boolean printImage(TimeAxisIF axis, LayerData[] datas);

    /**
     * 指定した条件で描画した画像をファイルへ出力する。
     * 
     * @param axis 出力する時間範囲を表す {@code TimeAxisIF} オブジェクト
     * @param datas 出力対象となる {@code LayerData} オブジェクト配列
     * @return 出力に成功した場合 {@code true}
     */
    public abstract boolean writeImageFile(TimeAxisIF axis, LayerData[] datas, String type);

    /**
     * 指定した条件で描画した画像をクリップボードへ出力する。
     * 
     * @param axis 出力する時間範囲を表す {@code TimeAxisIF} オブジェクト
     * @param datas 出力対象となる {@code LayerData} オブジェクト配列
     * @return 出力に成功した場合 {@code true}
     */
    public abstract boolean copyToClipboard(TimeAxisIF axis, LayerData[] datas);

    /**
     * 指定した条件で描画した画像のプレビューを行う。
     * 
     * @param axis 出力する時間範囲を表す {@code TimeAxisIF} オブジェクト
     * @param datas 出力対象となる {@code LayerData} オブジェクト配列
     */
    public abstract void previewImage(TimeAxisIF axis, LayerData[] datas);
}
