package jp.nihu.gttime.plugin.printer;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.util.ArrayList;
import java.util.List;

import jp.nihu.gttime.data.DatasetIF;
import jp.nihu.gttime.data.ProjectIF;
import jp.nihu.gttime.layer.LayerViewPart;
import jp.nihu.gttime.plugin.LayerData;
import jp.nihu.gttime.plugin.LayerPlugin;
import jp.nihu.gttime.plugin.analyzer.DatasetList;
import jp.nihu.gttime.tree.TreeCheckNode;

public class DatasetListForImagePrinter extends DatasetList
{
    // private static Log log = LogFactory.getLog(DatasetList.class);

    private static final String NAME_SEPARATOR = " / ";

    public class Item implements LayerData
    {
        private String name;
        
        private String group;

        private String type;

        private DatasetIF dataset;

        private Dimension size;

        private boolean visible;

        private boolean scale;

        private Item(DatasetIF dataset, String name, String type, String group)
        {
            this.dataset = dataset;
            this.name = name;
            this.type = type;
            this.group = group;
            setSize(new Dimension());
        }
        
        public String getGroup() {
            return group;
        }

        public String getName()
        {
            String n = name;
            String g = getGroup();
            if (g != null && g.length() > 0) {
                n = n + "（" + g + "）";
            }
            return n;
        }

        public String getType()
        {
            return type;
        }

        public DatasetIF getDataset()
        {
            return dataset;
        }

        public Dimension getSize()
        {
            return size;
        }

        public void setSize(Dimension d)
        {
            size = (d != null) ? d : (new Dimension());
        }

        public int getWidth()
        {
            return size.width;
        }

        public void setWidth(int width)
        {
            size.width = width;
        }

        public int getHeight()
        {
            return size.height;
        }

        public void setHeight(int height)
        {
            size.height = height;
        }

        public boolean isVisible()
        {
            return visible;
        }

        public void setVisible(boolean b)
        {
            visible = b;
        }

        public boolean isDrawTimeAxis()
        {
            return scale;
        }

        public void setDrawTimeAxis(boolean b)
        {
            scale = b;
        }

        @Override
        public String toString()
        {
            return getName();
        }
    }
    
    /**
     * {@code LayerData} を要素に持つ {@code List} を取得する。
     * 
     * @param p {@code Project} オブジェクト
     */
    public static List<LayerData> getList(ProjectIF p)
    {
        return getList(p, null);
    }

    /**
     * {@code LayerData} を要素に持つ {@code List} を取得する。
     * 
     * @param p {@code Project} オブジェクト
     * @param part {@code LayerViewPart} オブジェクト
     */
    public static List<LayerData> getList(ProjectIF p, LayerViewPart part)
    {
        List<LayerData> list = new ArrayList<LayerData>();
        DatasetListForImagePrinter dl = new DatasetListForImagePrinter();
        dl.setList(list, p, null);
        dl.setSize(list, part);
        return list;
    }

    /**
     * {@code LayerData} オブジェクトの生成し {@code List} に追加する。
     */
    private void setList(List<LayerData> list, ProjectIF p, String name)
    {
        if (p != null)
        {
            DatasetIF[] datas = p.getDatasets();
            ProjectIF[] childs = p.getProjects();
            StringBuffer buf = new StringBuffer();
            if (name != null)
            {
                buf.append(NAME_SEPARATOR).append(name);
            }
            if (datas != null)
            {
                for (DatasetIF d : datas)
                {
                    String s = d.getTitle() + buf.toString();
                    list.add(new Item(d, s, d.getLayerType(), ""));
                }
            }
            if (childs != null)
            {
                for (ProjectIF c : childs)
                {
                    setList(list, c, c.getTitle() + buf.toString());
                }
            }
        }
    }

    /**
     * {@code LayerData} オブジェクトの表示サイズを設定する。
     */
    private void setSize(List<LayerData> list, LayerViewPart part)
    {
        if (part != null)
        {
            // if (part.getViewport( ).getComponentCount( ) > 0) {
            // Component parent = part.getViewport( ).getComponent(0);
            // if (parent instanceof Container) {
            // for (Component c : ((Container)parent).getComponents( )) {
            // if (c instanceof LayerPlugin) {
            // setSize(list, (LayerPlugin)c);
            // }
            // }
            // }
            // }
            if (part.getLayerList().getComponentCount() > 0)
            {
                Component parent = part.getLayerList();
                for (Component c : ((Container) parent).getComponents())
                {
                    if (c instanceof LayerPlugin)
                    {
                        setSize(list, (LayerPlugin) c);
                    }
                }
            }
        }
    }

    /**
     * {@code LayerData} オブジェクトの表示サイズを設定する。
     */
    private void setSize(List<LayerData> list, LayerPlugin layer)
    {
        TreeCheckNode node = layer.getTreeCheckNode();
        if (node != null)
        {
            Object o = node.getUserObject();
            if (o instanceof DatasetIF)
            {
                for (LayerData d : list)
                {
                    if (d.getDataset().equals((DatasetIF) o))
                    {
                        d.setHeight
                        // (Math.max(layer.getHeight( ), node.getHeight( )));
                                        (Math.max(layer.getHeight(), d.getDataset().getProperties().getLayerHeight()));
                        d.setVisible(layer.isVisible());
                        break;
                    }
                }
            }
        }
    }
    
    /**
     * {@code LayerData} を配列を取得する。
     * 
     * @param p {@code Project} オブジェクト
     * @param part {@code LayerViewPart} オブジェクト
     */
    public static LayerData[] getArray(ProjectIF p, LayerViewPart part)
    {
        return getList(p, part).toArray(new Item[0]);
    }
}
