/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.plugin.printer;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.plugin.PluginResourceManager;

/**
 * 画像プレビュー用ダイアログクラス
 */
@SuppressWarnings("serial")
public class ImagePreviewDialog extends JDialog implements ActionListener
{

    private static PluginResourceManager rmgr = PluginResourceManager.getManager(ImagePreviewDialog.class, T2Map.LOCALE);

    /**
     * 画像プレビュー用ダイアログを生成する。
     * 
     * @param owner ダイアログを表示する {@code Frame}
     * @param image 表示する {@code Image} オブジェクト
     */
    public ImagePreviewDialog(Frame owner, Image image)
    {
        super(owner, true);
        initDialog(image);
    }

    /**
     * 画像プレビュー用ダイアログを生成する。
     * 
     * @param owner ダイアログを表示する {@code Dialog}
     * @param image 表示する {@code Image} オブジェクト
     */
    public ImagePreviewDialog(Dialog owner, Image image)
    {
        super(owner, true);
        initDialog(image);
    }

    /**
     * ダイアログの構築を行う
     */
    private void initDialog(Image image)
    {
        // Create new instance.
        JLabel lab = new JLabel(new ImageIcon(image));
        JButton btn = new JButton(rmgr.getString("button.close"));
        Dimension d = new Dimension(image.getWidth(this), image.getHeight(this));
        lab.setPreferredSize(d);
        lab.setVerticalAlignment(JLabel.TOP);
        lab.setHorizontalAlignment(JLabel.LEFT);
        btn.addActionListener(this);

        // Crate new instance.
        JScrollPane scrollPane = new JScrollPane();
        JPanel buttonPane = new JPanel();
        scrollPane.getViewport().setBackground(Color.DARK_GRAY);
        scrollPane.setPreferredSize(new Dimension(rmgr.getInt("scroll.width"), rmgr.getInt("scroll.height")));
        // Add Components.
        scrollPane.getViewport().add(lab);
        buttonPane.add(btn);
        getContentPane().add(scrollPane, BorderLayout.CENTER);
        getContentPane().add(buttonPane, BorderLayout.SOUTH);
        setTitle(rmgr.getString("dialog.title"));
        pack();
        setLocationRelativeTo(getOwner());
    }

    /**
     * アクションが発生すると呼び出される。
     */
    public void actionPerformed(ActionEvent evt)
    {
        dispose();
    }
}
