/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.preferences;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GraphicsEnvironment;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.xml.xpath.XPathExpressionException;

import jp.nihu.gttime.NColorChooser;
import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.T2MapProperties;
import jp.nihu.gttime.data.property.LineIF;
import jp.nihu.gttime.data.property.PolygonIF;
import jp.nihu.gttime.data.property.PropertiesIF;
import jp.nihu.gttime.data.property.SymbolGts;
import jp.nihu.gttime.data.property.SymbolIF;
import jp.nihu.gttime.plugin.PluginPreferences;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.utility.Utilities;
import jp.nihu.gttime.utility.XMLUtil;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public abstract class BaseLayerPluginPreferences extends PluginPreferences
{
    private static final long serialVersionUID = 1L;
    
    private static Log log = LogFactory.getLog(BaseLayerPluginPreferences.class);
    
    public static final int PARTS_WIDTH = 400;

    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(BaseLayerPluginPreferences.class, T2Map.LOCALE);

    public static final int SYMBOL_TYPE_CIRCLE = 0;

    public static final int SYMBOL_TYPE_TRIANGLE = 1;

    public static final int SYMBOL_TYPE_RECTANGLE = 2;

    protected Map<String,LinePreferencePanel> linePreferencePanel = new HashMap<String, LinePreferencePanel>();

    protected Map<String,PlotPreferencePanel> plotPreferencePanel = new HashMap<String, PlotPreferencePanel>();

    protected Map<String,SymbolPreferencePanel> symbolPreferencePanel = new HashMap<String, SymbolPreferencePanel>();

    protected Map<String,FontPreferencePanel> fontPreferencePanel = new HashMap<String, FontPreferencePanel>();

    protected SelectedColorPreferencePanel selectedColorPreferencePanel = null;

    protected BackgroundPreferencePanel backgroundPreferencePanel = null;

    protected HeightPreferencePanel heightPreferencePanel = null;

    protected ScaleMainPreferencePanel scaleMainPreferencePanel = null;

    protected ScaleSubPreferencePanel scaleSubPreferencePanel = null;

    protected RecordReducePreferencePanel recordReducePreferencePanel = null;
    
    protected DetailFontNamePreferencePanel detailFontNamePreferencePanel = null;
    
    protected ShowGroupNamePreferencePanel showGroupNamePreferencePanel = null;
    
    protected JTabbedPane tabbedpane = null;
    
    protected Map<String, JPanel> tabPanels = new HashMap<String, JPanel>();
    
    private void addItemToTabPanel(String group, JComponent comp) {
        log.debug("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA 0");
        if (tabbedpane == null) {
            tabbedpane = new JTabbedPane();
            this.add(tabbedpane, BorderLayout.CENTER);
            log.debug("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA 1");
        }
        JPanel panel = tabPanels.get(group);
        if (panel == null) {
            log.debug("AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA 2");
            panel = new JPanel();
            panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
            tabPanels.put(group, panel);
            if (group.length() == 0) {
                tabbedpane.addTab("--", panel);
            } else {
                tabbedpane.addTab(group, panel);
            }
        }
        panel.add(comp);
        
        Component[] comps = comp.getComponents();
        int h = 0;
        int w = 0;
        for (int i = 0; i < comps.length; i++)
        {
            Dimension d = comps[i].getPreferredSize();
            if (d.width > w)
            {
                w = d.width;
            }
//            h = h + d.height + 15;
            h = h + d.height + 5;
        }
//        comp.setPreferredSize(new Dimension(w, h));
        tabbedpane.setPreferredSize(new Dimension(comp.getWidth(), h));
//        tabbedpane.setMaximumSize(new Dimension(comp.getWidth(), h));
//        tabbedpane.setMaximumSize(new Dimension(comp.getWidth(), h+50));
//        tabbedpane.setMinimumSize(new Dimension(comp.getWidth(), h));
    }

    @Override
    protected void init()
    {
        setPreferredSize(new Dimension(200, 200));
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    }

    @Override
    public void initValue()
    {
        if (tabbedpane != null) {
            tabbedpane.removeAll();
            tabbedpane = null;
        }
        tabPanels.clear();
        this.removeAll();
    }

//    @Override
//    public void save()
//    {
//    }

    public abstract void initValueForLayer(PropertiesIF properties);

    public abstract void saveForLayer(PropertiesIF properties);

    private String getBasePath(String type)
    {
        return "/gt:gtm/hgis:hgisApplications/hgis:time[@xsi:type='" + type + "']";
    }

    private T2MapProperties getT2MapProperties()
    {
        return T2Map.properties;
    }

    protected void initLineForLayer(PropertiesIF properties)
    {
        Map<String,LineIF> lines = properties.getLines();
        Iterator<String> it = lines.keySet().iterator();
        while (it.hasNext()) {
            String key = it.next();
            LineIF line = lines.get(key);
            linePreferencePanel.put(key, new LinePreferencePanel(line.getLineColor(), line.getThin(), key, true));
        }
    }

    protected void initSymbolForLayer(PropertiesIF properties)
    {
        Map<String,SymbolIF> symbols = properties.getSymbols();
        Iterator<String> it = symbols.keySet().iterator();
        while (it.hasNext()) {
            String key = it.next();
            SymbolIF symbol = symbols.get(key);
            symbolPreferencePanel.put(key, new SymbolPreferencePanel(symbol.getFillColor(), symbol.getLineColor(), symbol.getThin(), symbol.getSize(), symbol.getSymbol(), key, true));
        }
    }

    protected void initPolygonForLayer(PropertiesIF properties)
    {
        Map<String,PolygonIF> polygons = properties.getPolygons();
        Iterator<String> it = polygons.keySet().iterator();
        while (it.hasNext()) {
            String key = it.next();
            PolygonIF polygon = polygons.get(key);
            plotPreferencePanel.put(key, new PlotPreferencePanel(polygon.getFillColor(), polygon.getLineColor(), polygon.getThin(), key, true));
        }
    }

    protected void initFontForLayer(PropertiesIF properties)
    {
        Map<String,Font> fonts = properties.getFonts();
        Map<String,Color> fontColors = properties.getFontColors();
        Iterator<String> it = fonts.keySet().iterator();
        while (it.hasNext()) {
            String key = it.next();
            Font font = fonts.get(key);
            fontPreferencePanel.put(key, new FontPreferencePanel(font.getFamily(), font.getStyle(), font.getSize(), fontColors.get(key), key, true));
        }
    }

    protected void initSelectedColorForLayer(PropertiesIF properties)
    {
        selectedColorPreferencePanel = new SelectedColorPreferencePanel(properties.getSelectedColor());
    }

    protected void initBackgroundColorForLayer(PropertiesIF properties)
    {
        backgroundPreferencePanel = new BackgroundPreferencePanel(properties.getBackgroundColor());
    }
    
    protected void initDetailFontNameForLayer(PropertiesIF properties)
    {
        detailFontNamePreferencePanel = new DetailFontNamePreferencePanel(properties.getDetailFontName());
    }
    
    protected void initShowGroupNameForLayer(PropertiesIF properties)
    {
        showGroupNamePreferencePanel = new ShowGroupNamePreferencePanel(properties.isShowGroupName());
    }

    protected void initLayerHeightForLayer(PropertiesIF properties)
    {
        heightPreferencePanel = new HeightPreferencePanel(properties.getLayerHeight());
    }

    protected void initDataReductionForLayer(PropertiesIF properties)
    {
        recordReducePreferencePanel = new RecordReducePreferencePanel(properties.isDataReduction(), properties.getDataReductionNum());
    }

    protected void initScaleMainForLayer(PropertiesIF properties)
    {
        PolygonIF scaleMain = properties.getScaleMain();
        scaleMainPreferencePanel = new ScaleMainPreferencePanel(scaleMain.getLineColor(), scaleMain.getFillColor(), scaleMain.getThin());
    }

    protected void initScaleSubForLayer(PropertiesIF properties)
    {
        PolygonIF scaleSub = properties.getScaleSub();
        scaleSubPreferencePanel = new ScaleSubPreferencePanel(scaleSub.getLineColor(), scaleSub.getFillColor(), scaleSub.getThin());
    }

    protected void saveLineForLayer(PropertiesIF properties)
    {
        Map<String,LineIF> lines = properties.getLines();
        Iterator<String> it = linePreferencePanel.keySet().iterator();
        while (it.hasNext()) {
            String key = it.next();
            LinePreferencePanel p = linePreferencePanel.get(key);
            LineIF line = lines.get(key);
            line.setLineColor(p.getColor());
            line.setThin(p.getThickness());
        }
    }

    protected void saveSymbolForLayer(PropertiesIF properties)
    {
        Map<String,SymbolIF> symbols = properties.getSymbols();
        Iterator<String> it = symbolPreferencePanel.keySet().iterator();
        while (it.hasNext()) {
            String key = it.next();
            SymbolPreferencePanel s = symbolPreferencePanel.get(key);
            SymbolIF symbol = symbols.get(key);
            symbol.setFillColor(s.getColor());
            symbol.setLineColor(s.getOutLineColor());
            symbol.setThin(s.getOutLineThickness());
            symbol.setSize(s.getSymbolSize());
            symbol.setSymbol(s.getType());
        }
    }

    protected void savePolygonForLayer(PropertiesIF properties)
    {
        Map<String,PolygonIF> polygons = properties.getPolygons();
        Iterator<String> it = plotPreferencePanel.keySet().iterator();
        while (it.hasNext()) {
            String key = it.next();
            PlotPreferencePanel p = plotPreferencePanel.get(key);
            PolygonIF polygon = polygons.get(key);
            polygon.setFillColor(p.getColor());
            polygon.setLineColor(p.getOutLineColor());
            polygon.setThin(p.getOutLineThickness());
        }
    }

    protected void saveFontForLayer(PropertiesIF properties)
    {
        Map<String,Font> fonts = properties.getFonts();
        Map<String,Color> colors = properties.getFontColors();
        Iterator<String> it = fontPreferencePanel.keySet().iterator();
        while (it.hasNext()) {
            String key = it.next();
            FontPreferencePanel f = fontPreferencePanel.get(key);

            if (fonts.keySet().contains(key)) {
            	fonts.remove(key);
            }
            fonts.put(key, new Font(f.getName(), f.getStyle(), f.getFontSize()));
            
            if (colors.keySet().contains(key)) {
            	colors.remove(key);
            }
            colors.put(key, f.getColor());
        }
    }

    protected void saveSelectedColorForLayer(PropertiesIF properties)
    {
        properties.setSelectedColor(selectedColorPreferencePanel.getColor());
    }

    protected void saveBackgroundColorForLayer(PropertiesIF properties)
    {
        properties.setBackgroundColor(backgroundPreferencePanel.getColor());
    }

    protected void saveLayerHeightForLayer(PropertiesIF properties)
    {
        properties.setLayerHeight(heightPreferencePanel.getLayerHeight());
    }

    protected void saveDataReductionForLayer(PropertiesIF properties)
    {
        properties.setDataReduction(recordReducePreferencePanel.isRecordReduceEnable());
        properties.setDataReductionNum(recordReducePreferencePanel.getNumberOfRecord());
    }
    
    protected void saveShowGroupNameForLayer(PropertiesIF properties)
    {
        properties.setShowGroupName(showGroupNamePreferencePanel.isShow());
    }
    
    protected void saveDetailFontNameForLayer(PropertiesIF properties)
    {
        properties.setDetailFontName(detailFontNamePreferencePanel.getName());
    }

    protected void saveScaleMainForLayer(PropertiesIF properties)
    {
        PolygonIF scaleMain = properties.getScaleMain();
        scaleMain.setFillColor(scaleMainPreferencePanel.getColor());
        scaleMain.setLineColor(scaleMainPreferencePanel.getOutLineColor());
        scaleMain.setThin(scaleMainPreferencePanel.getOutLineThickness());
    }

    protected void saveScaleSubForLayer(PropertiesIF properties)
    {
        PolygonIF scaleSub = properties.getScaleSub();
        scaleSub.setFillColor(scaleSubPreferencePanel.getColor());
        scaleSub.setLineColor(scaleSubPreferencePanel.getOutLineColor());
        scaleSub.setThin(scaleSubPreferencePanel.getOutLineThickness());
    }

    protected void initLine(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        NodeList nl = p.getStyleValues(prefix + "/hgis:line");
        if (nl != null) {
            for (int i = 0; i < nl.getLength(); i++) {
                Element elem = (Element)nl.item(i);
                String group = elem.getAttribute("group");
                if (group == null) {
                    group = "";
                }
                String _thin = XMLUtil.getValue(elem, "hgis:thin/text()", XMLUtil.getGtsNsContext());
                String _lineColor = XMLUtil.getValue(elem, "hgis:lineColor/text()", XMLUtil.getGtsNsContext());
                LinePreferencePanel lpp = new LinePreferencePanel(Utilities.getColor(_lineColor), Integer.parseInt(_thin.trim()), group, true);
                if (linePreferencePanel.containsKey(group)) {
                    linePreferencePanel.remove(group);
                }
                linePreferencePanel.put(group, lpp);
            }
        }
    }

    protected void initSymbol(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        NodeList nl = p.getStyleValues(prefix + "/hgis:symbol");
        if (nl != null) {
            for (int i = 0; i < nl.getLength(); i++) {
                Element elem = (Element)nl.item(i);
                String group = elem.getAttribute("group");
                if (group == null) {
                    group = "";
                }
                String _symbol = XMLUtil.getValue(elem, "hgis:symbol/text()", XMLUtil.getGtsNsContext());
                String _size = XMLUtil.getValue(elem, "hgis:size/text()", XMLUtil.getGtsNsContext());
                String _fillColor = XMLUtil.getValue(elem, "hgis:fillColor/text()", XMLUtil.getGtsNsContext());
                String _thin = XMLUtil.getValue(elem, "hgis:thin/text()", XMLUtil.getGtsNsContext());
                String _lineColor = XMLUtil.getValue(elem, "hgis:lineColor/text()", XMLUtil.getGtsNsContext());
                SymbolPreferencePanel spp = new SymbolPreferencePanel(Utilities.getColor(_fillColor), Utilities.getColor(_lineColor), Integer.parseInt(_thin.trim()), Integer
                                .parseInt(_size.trim()), _symbol, group, true);
                if (symbolPreferencePanel.containsKey(group)) {
                    symbolPreferencePanel.remove(group);
                }
                symbolPreferencePanel.put(group, spp);
            }
        }
    }

    protected void initPolygon(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        NodeList nl = p.getStyleValues(prefix + "/hgis:polygon");
        if (nl != null) {
            for (int i = 0; i < nl.getLength(); i++) {
                Element elem = (Element)nl.item(i);
                String group = elem.getAttribute("group");
                if (group == null) {
                    group = "";
                }
                String _polygonColor = XMLUtil.getValue(elem, "hgis:fillColor/text()", XMLUtil.getGtsNsContext());
                String _polygonThin = XMLUtil.getValue(elem, "hgis:thin/text()", XMLUtil.getGtsNsContext());
                String _polygonLineColor = XMLUtil.getValue(elem, "hgis:lineColor/text()", XMLUtil.getGtsNsContext());
                PlotPreferencePanel ppp = new PlotPreferencePanel(Utilities.getColor(_polygonColor), Utilities.getColor(_polygonLineColor), Integer.parseInt(_polygonThin.trim()), group, true);
                if (plotPreferencePanel.containsKey(group)) {
                    plotPreferencePanel.remove(group);
                }
                plotPreferencePanel.put(group, ppp);
            }
        }
    }

    protected void initFont(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        NodeList nl = p.getStyleValues(prefix + "/hgis:font");
        if (nl != null) {
            for (int i = 0; i < nl.getLength(); i++) {
                Element elem = (Element)nl.item(i);
                String group = elem.getAttribute("group");
                if (group == null) {
                    group = "";
                }
                String _family = XMLUtil.getValue(elem, "hgis:font-family/text()", XMLUtil.getGtsNsContext());
                String _name = XMLUtil.getValue(elem, "hgis:font-name/text()", XMLUtil.getGtsNsContext());
                String _size = XMLUtil.getValue(elem, "hgis:font-size/text()", XMLUtil.getGtsNsContext());
                String _color = XMLUtil.getValue(elem, "hgis:font-color/text()", XMLUtil.getGtsNsContext());
                String _style = XMLUtil.getValue(elem, "hgis:font-style/text()", XMLUtil.getGtsNsContext());
                FontPreferencePanel fpp = new FontPreferencePanel(_family, Utilities.getFontStyle(_style), Integer.parseInt(_size.trim()), Utilities.getColor(_color), group, true);
                if (fontPreferencePanel.containsKey(group)) {
                    fontPreferencePanel.remove(group);
                }
                fontPreferencePanel.put(group, fpp);
            }
        }
    }

    protected void initSelectedColor(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        String selectedColor = p.getStyleValue(prefix + "/hgis:selectedColor/text()");
        selectedColorPreferencePanel = new SelectedColorPreferencePanel(Utilities.getColor(selectedColor));
    }

    protected void initBackgroundColor(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        String backgroundColor = p.getStyleValue(prefix + "/hgis:backgroundColor/text()");
        backgroundPreferencePanel = new BackgroundPreferencePanel(Utilities.getColor(backgroundColor));
    }
    
    protected void initDetailFontName(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        String detailFontName = p.getStyleValue(prefix + "/hgis:detailFontName/text()");
        detailFontNamePreferencePanel = new DetailFontNamePreferencePanel(detailFontName);
    }
    
    protected void initShowGroupName(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        String showGroupName = p.getStyleValue(prefix + "/hgis:showGroupName/text()");
        showGroupNamePreferencePanel = new ShowGroupNamePreferencePanel(Boolean.parseBoolean(showGroupName));
    }

    protected void initLayerHeight(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        String layerHight = p.getStyleValue(prefix + "/hgis:layerHight/text()");
        heightPreferencePanel = new HeightPreferencePanel(Integer.parseInt(layerHight.trim()));
    }

    protected void initDataReduction(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        String dataReduction = p.getStyleValue(prefix + "/hgis:dataReduction/text()");
        String dataReductionEnableRecordNum = p.getStyleValue(prefix + "/hgis:dataReductionEnableRecordNum/text()");
        recordReducePreferencePanel = new RecordReducePreferencePanel(Boolean.parseBoolean(dataReduction), Integer.parseInt(dataReductionEnableRecordNum.trim()));
    }

    protected void initScaleMain(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        String scaleMainColor = p.getStyleValue(prefix + "/hgis:scaleMain/hgis:fillColor/text()");
        String scaleMainThin = p.getStyleValue(prefix + "/hgis:scaleMain/hgis:thin/text()");
        String scaleMainLineColor = p.getStyleValue(prefix + "/hgis:scaleMain/hgis:lineColor/text()");
        scaleMainPreferencePanel = new ScaleMainPreferencePanel(Utilities.getColor(scaleMainLineColor), Utilities.getColor(scaleMainColor), Integer.parseInt(scaleMainThin.trim()));
    }

    protected void initScaleSub(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        String scaleSubColor = p.getStyleValue(prefix + "/hgis:scaleSub/hgis:fillColor/text()");
        String scaleSubThin = p.getStyleValue(prefix + "/hgis:scaleSub/hgis:thin/text()");
        String scaleSubLineColor = p.getStyleValue(prefix + "/hgis:scaleSub/hgis:lineColor/text()");
        scaleSubPreferencePanel = new ScaleSubPreferencePanel(Utilities.getColor(scaleSubLineColor), Utilities.getColor(scaleSubColor), Integer.parseInt(scaleSubThin.trim()));
    }

    protected void saveLine(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        Iterator<String> it = linePreferencePanel.keySet().iterator();
        while (it.hasNext()) {
            String group = it.next();
            String groupAttr = "";
            if (group.length() > 0) {
                groupAttr = "[@group='" + group + "']";
            } else {
                groupAttr = "[not(@group)]";
            }
            LinePreferencePanel lpp = linePreferencePanel.get(group);
            p.setStyleValue(prefix + "/hgis:line"+groupAttr+"/hgis:lineColor", Utilities.getColor(lpp.getColor()));
            p.setStyleValue(prefix + "/hgis:line"+groupAttr+"/hgis:thin", Integer.toString(lpp.getThickness()));
        }
        
    }

    protected void saveSymbol(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        Iterator<String> it = symbolPreferencePanel.keySet().iterator();
        while (it.hasNext()) {
            String group = it.next();
            String groupAttr = "";
            if (group.length() > 0) {
                groupAttr = "[@group='" + group + "']";
            } else {
                groupAttr = "[not(@group)]";
            }
            SymbolPreferencePanel spp = symbolPreferencePanel.get(group);
            p.setStyleValue(prefix + "/hgis:symbol"+groupAttr+"/hgis:symbol", spp.getType());
            p.setStyleValue(prefix + "/hgis:symbol"+groupAttr+"/hgis:size", Integer.toString(spp.getSymbolSize()));
            p.setStyleValue(prefix + "/hgis:symbol"+groupAttr+"/hgis:fillColor", Utilities.getColor(spp.getColor()));
            p.setStyleValue(prefix + "/hgis:symbol"+groupAttr+"/hgis:thin", Integer.toString(spp.getOutLineThickness()));
            p.setStyleValue(prefix + "/hgis:symbol"+groupAttr+"/hgis:lineColor", Utilities.getColor(spp.getOutLineColor()));
        }
    }

    protected void savePolygon(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        Iterator<String> it = plotPreferencePanel.keySet().iterator();
        while (it.hasNext()) {
            String group = it.next();
            String groupAttr = "";
            if (group.length() > 0) {
                groupAttr = "[@group='" + group + "']";
            } else {
                groupAttr = "[not(@group)]";
            }
            PlotPreferencePanel ppp = plotPreferencePanel.get(group);
            p.setStyleValue(prefix + "/hgis:polygon"+groupAttr+"/hgis:fillColor", Utilities.getColor(ppp.getColor()));
            p.setStyleValue(prefix + "/hgis:polygon"+groupAttr+"/hgis:thin", Integer.toString(ppp.getOutLineThickness()));
            p.setStyleValue(prefix + "/hgis:polygon"+groupAttr+"/hgis:lineColor", Utilities.getColor(ppp.getOutLineColor()));
        }
    }

    protected void saveFont(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        Iterator<String> it = fontPreferencePanel.keySet().iterator();
        while (it.hasNext()) {
            String group = it.next();
            String groupAttr = "";
            if (group.length() > 0) {
                groupAttr = "[@group='" + group + "']";
            } else {
                groupAttr = "[not(@group)]";
            }
            FontPreferencePanel fpp = fontPreferencePanel.get(group);
            p.setStyleValue(prefix + "/hgis:font"+groupAttr+"/hgis:font-family", fpp.getName());
            p.setStyleValue(prefix + "/hgis:font"+groupAttr+"/hgis:font-name", fpp.getName());
            p.setStyleValue(prefix + "/hgis:font"+groupAttr+"/hgis:font-size", Integer.toString(fpp.getFontSize()));
            p.setStyleValue(prefix + "/hgis:font"+groupAttr+"/hgis:font-color", Utilities.getColor(fpp.getColor()));
            p.setStyleValue(prefix + "/hgis:font"+groupAttr+"/hgis:font-style", Integer.toString(fpp.getStyle()));
        }
    }

    protected void saveSelectedColor(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        p.setStyleValue(prefix + "/hgis:selectedColor", Utilities.getColor(selectedColorPreferencePanel.getColor()));
        ;
    }

    protected void saveBackgroundColor(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        p.setStyleValue(prefix + "/hgis:backgroundColor", Utilities.getColor(backgroundPreferencePanel.getColor()));
    }
    
    protected void saveDetailFontName(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        p.setStyleValue(prefix + "/hgis:detailFontName", detailFontNamePreferencePanel.getName());
    }
    
    protected void saveShowGroupName(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        p.setStyleValue(prefix + "/hgis:showGroupName", Boolean.toString(showGroupNamePreferencePanel.isShow()));
    }

    protected void saveLayerHeight(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        p.setStyleValue(prefix + "/hgis:layerHight", Integer.toString(heightPreferencePanel.getLayerHeight()));
    }

    protected void saveDataReduction(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();
        p.setStyleValue(prefix + "/hgis:dataReduction", Boolean.toString(recordReducePreferencePanel.isRecordReduceEnable()));
        p.setStyleValue(prefix + "/hgis:dataReductionEnableRecordNum", Integer.toString(recordReducePreferencePanel.getNumberOfRecord()));
    }

    protected void saveScaleMain(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();

        p.setStyleValue(prefix + "/hgis:scaleMain/hgis:fillColor", Utilities.getColor(scaleMainPreferencePanel.getColor()));
        p.setStyleValue(prefix + "/hgis:scaleMain/hgis:thin", Integer.toString(scaleMainPreferencePanel.getOutLineThickness()));
        p.setStyleValue(prefix + "/hgis:scaleMain/hgis:lineColor", Utilities.getColor(scaleMainPreferencePanel.getOutLineColor()));
    }

    protected void saveScaleSub(String type) throws XPathExpressionException
    {
        String prefix = getBasePath(type);
        T2MapProperties p = getT2MapProperties();

        p.setStyleValue(prefix + "/hgis:scaleSub/hgis:fillColor", Utilities.getColor(scaleSubPreferencePanel.getColor()));
        p.setStyleValue(prefix + "/hgis:scaleSub/hgis:thin", Integer.toString(scaleSubPreferencePanel.getOutLineThickness()));
        p.setStyleValue(prefix + "/hgis:scaleSub/hgis:lineColor", Utilities.getColor(scaleSubPreferencePanel.getOutLineColor()));
    }

    public void setDefault(String type)
    {
        try
        {
            String prefix = getBasePath(type);
            T2MapProperties p = getT2MapProperties();
            
            if (linePreferencePanel.size() >= 0)
            {
                NodeList nl = p.getStyleValues(prefix + "/hgis:line");
                if (nl != null) {
                    Map<String, LinePreferencePanel> map = new HashMap<String, LinePreferencePanel>();
                    for (int i = 0; i < nl.getLength(); i++) {
                        Element elem = (Element)nl.item(i);
                        String group = elem.getAttribute("group");
                        if (group == null) {
                            group = "";
                        }
                        String _thin = XMLUtil.getValue(elem, "hgis:thin/text()", XMLUtil.getGtsNsContext());
                        String _lineColor = XMLUtil.getValue(elem, "hgis:lineColor/text()", XMLUtil.getGtsNsContext());
                        LinePreferencePanel lpp = new LinePreferencePanel(Utilities.getColor(_lineColor), Integer.parseInt(_thin.trim()), group, false);
//                        if (linePreferencePanel.containsKey(group)) {
//                            linePreferencePanel.remove(group);
//                        }
//                        linePreferencePanel.put(group, lpp);
                        
                        map.put(group, lpp);
                    }
                    
                    for (String group : linePreferencePanel.keySet()) {
                        LinePreferencePanel def = map.get(group);
                        if (def == null) {
                            def = map.get("");
                        }
                        if (def != null) {
                            LinePreferencePanel c = linePreferencePanel.get(group);
                            c.setColor(def.getColor());
                            c.setThickness(def.getThickness());
                        }
                    }
                }
            }

            if (plotPreferencePanel.size() >= 0)
            {
                NodeList nl = p.getStyleValues(prefix + "/hgis:polygon");
                if (nl != null) {
                    Map<String, PlotPreferencePanel> map = new HashMap<String, PlotPreferencePanel>();
                    for (int i = 0; i < nl.getLength(); i++) {
                        Element elem = (Element)nl.item(i);
                        String group = elem.getAttribute("group");
                        if (group == null) {
                            group = "";
                        }
                        String _polygonColor = XMLUtil.getValue(elem, "hgis:fillColor/text()", XMLUtil.getGtsNsContext());
                        String _polygonThin = XMLUtil.getValue(elem, "hgis:thin/text()", XMLUtil.getGtsNsContext());
                        String _polygonLineColor = XMLUtil.getValue(elem, "hgis:lineColor/text()", XMLUtil.getGtsNsContext());
                        PlotPreferencePanel ppp = new PlotPreferencePanel(Utilities.getColor(_polygonColor), Utilities.getColor(_polygonLineColor), Integer.parseInt(_polygonThin.trim()), group, false);
//                        if (plotPreferencePanel.containsKey(group)) {
//                            plotPreferencePanel.remove(group);
//                        }
//                        plotPreferencePanel.put(group, ppp);
                        
                        map.put(group, ppp);
                    }
                    
                    for (String group : plotPreferencePanel.keySet()) {
                        PlotPreferencePanel def = map.get(group);
                        if (def == null) {
                            def = map.get("");
                        }
                        if (def != null) {
                            PlotPreferencePanel c = plotPreferencePanel.get(group);
                            c.setColor(def.getColor());
                            c.setOutLineColor(def.getOutLineColor());
                            c.setOutLineThickness(def.getOutLineThickness());
                        }
                    }
                }
            }

            if (symbolPreferencePanel.size() >= 0)
            {
                NodeList nl = p.getStyleValues(prefix + "/hgis:symbol");
                if (nl != null) {
                    Map<String, SymbolPreferencePanel> map = new HashMap<String, SymbolPreferencePanel>();
                    for (int i = 0; i < nl.getLength(); i++) {
                        Element elem = (Element)nl.item(i);
                        String group = elem.getAttribute("group");
                        if (group == null) {
                            group = "";
                        }
                        String _symbol = XMLUtil.getValue(elem, "hgis:symbol/text()", XMLUtil.getGtsNsContext());
                        String _size = XMLUtil.getValue(elem, "hgis:size/text()", XMLUtil.getGtsNsContext());
                        String _fillColor = XMLUtil.getValue(elem, "hgis:fillColor/text()", XMLUtil.getGtsNsContext());
                        String _thin = XMLUtil.getValue(elem, "hgis:thin/text()", XMLUtil.getGtsNsContext());
                        String _lineColor = XMLUtil.getValue(elem, "hgis:lineColor/text()", XMLUtil.getGtsNsContext());
                        SymbolPreferencePanel spp = new SymbolPreferencePanel(Utilities.getColor(_fillColor), Utilities.getColor(_lineColor), Integer.parseInt(_thin.trim()), Integer
                                        .parseInt(_size.trim()), _symbol, group, false);
//                        if (symbolPreferencePanel.containsKey(group)) {
//                            symbolPreferencePanel.remove(group);
//                        }
//                        symbolPreferencePanel.put(group, spp);
                        
                        map.put(group, spp);
                    }
                    
                    for (String group : symbolPreferencePanel.keySet()) {
                        SymbolPreferencePanel def = map.get(group);
                        if (def == null) {
                            def = map.get("");
                        }
                        if (def != null) {
                            SymbolPreferencePanel c = symbolPreferencePanel.get(group);
                            c.setSymbolSize(def.getSymbolSize());
                            c.setType(def.getType());
                            c.setColor(def.getColor());
                            c.setOutLineColor(def.getOutLineColor());
                            c.setOutLineThickness(def.getOutLineThickness());
                        }
                    }
                }
            }

            if (fontPreferencePanel.size() >= 0)
            {
                NodeList nl = p.getStyleValues(prefix + "/hgis:font");
                if (nl != null) {
                    Map<String, FontPreferencePanel> map = new HashMap<String, FontPreferencePanel>();
                    for (int i = 0; i < nl.getLength(); i++) {
                        Element elem = (Element)nl.item(i);
                        String group = elem.getAttribute("group");
                        if (group == null) {
                            group = "";
                        }
                        String _family = XMLUtil.getValue(elem, "hgis:font-family/text()", XMLUtil.getGtsNsContext());
                        String _name = XMLUtil.getValue(elem, "hgis:font-name/text()", XMLUtil.getGtsNsContext());
                        String _size = XMLUtil.getValue(elem, "hgis:font-size/text()", XMLUtil.getGtsNsContext());
                        String _color = XMLUtil.getValue(elem, "hgis:font-color/text()", XMLUtil.getGtsNsContext());
                        String _style = XMLUtil.getValue(elem, "hgis:font-style/text()", XMLUtil.getGtsNsContext());
                        FontPreferencePanel fpp = new FontPreferencePanel(_family, Utilities.getFontStyle(_style), Integer.parseInt(_size.trim()), Utilities.getColor(_color), group, false);
//                        if (fontPreferencePanel.containsKey(group)) {
//                            fontPreferencePanel.remove(group);
//                        }
//                        fontPreferencePanel.put(group, fpp);
                        
                        map.put(group, fpp);
                    }
                    
                    for (String group : fontPreferencePanel.keySet()) {
                        FontPreferencePanel def = map.get(group);
                        if (def == null) {
                            def = map.get("");
                        }
                        if (def != null) {
                            FontPreferencePanel c = fontPreferencePanel.get(group);
                            c.setName(def.getName());
                            c.setFontSize(def.getFontSize());
                            c.setColor(def.getColor());
                            c.setStyle(def.getStyle());
                        }
                    }
                }
            }

            if (selectedColorPreferencePanel != null)
            {
                String selectedColor = p.getStyleValue(prefix + "/hgis:selectedColor/text()");
                selectedColorPreferencePanel.setColor(Utilities.getColor(selectedColor));
            }

            if (backgroundPreferencePanel != null)
            {
                String backgroundColor = p.getStyleValue(prefix + "/hgis:backgroundColor/text()");
                backgroundPreferencePanel.setColor(Utilities.getColor(backgroundColor));
            }
            
            if (detailFontNamePreferencePanel != null)
            {
                String detailFontName = p.getStyleValue(prefix + "/hgis:detailFontNamer/text()");
                if (detailFontName == null || detailFontName.length() == 0) {
                    detailFontName = Font.DIALOG;
                }
                detailFontNamePreferencePanel.setName(detailFontName);
            }
            
            if (showGroupNamePreferencePanel != null)
            {
                String showGroupName = p.getStyleValue(prefix + "/hgis:showGroupName/text()");
                showGroupNamePreferencePanel.setShow(Boolean.parseBoolean(showGroupName));
            }

            if (heightPreferencePanel != null)
            {
                String layerHight = p.getStyleValue(prefix + "/hgis:layerHight/text()");
                heightPreferencePanel.setLayerHeight(Integer.parseInt(layerHight.trim()));
            }

            if (scaleMainPreferencePanel != null)
            {
                String scaleMainColor = p.getStyleValue(prefix + "/hgis:scaleMain/hgis:fillColor/text()");
                String scaleMainThin = p.getStyleValue(prefix + "/hgis:scaleMain/hgis:thin/text()");
                String scaleMainLineColor = p.getStyleValue(prefix + "/hgis:scaleMain/hgis:lineColor/text()");
                scaleMainPreferencePanel.setColor(Utilities.getColor(scaleMainColor));
                scaleMainPreferencePanel.setOutLineThickness(Integer.parseInt(scaleMainThin.trim()));
                scaleMainPreferencePanel.setOutLineColor(Utilities.getColor(scaleMainLineColor));
            }

            if (scaleSubPreferencePanel != null)
            {
                String scaleSubColor = p.getStyleValue(prefix + "/hgis:scaleSub/hgis:fillColor/text()");
                String scaleSubThin = p.getStyleValue(prefix + "/hgis:scaleSub/hgis:thin/text()");
                String scaleSubLineColor = p.getStyleValue(prefix + "/hgis:scaleSub/hgis:lineColor/text()");
                scaleSubPreferencePanel.setColor(Utilities.getColor(scaleSubColor));
                scaleSubPreferencePanel.setOutLineThickness(Integer.parseInt(scaleSubThin.trim()));
                scaleSubPreferencePanel.setOutLineColor(Utilities.getColor(scaleSubLineColor));
            }

            if (recordReducePreferencePanel != null)
            {
                String dataReduction = p.getStyleValue(prefix + "/hgis:dataReduction/text()");
                String dataReductionEnableRecordNum = p.getStyleValue(prefix + "/hgis:dataReductionEnableRecordNum/text()");
                recordReducePreferencePanel.setRecordReduceEnable(Boolean.parseBoolean(dataReduction));
                recordReducePreferencePanel.setNumberOfRecord(Integer.parseInt(dataReductionEnableRecordNum.trim()));
            }
        }
        catch (XPathExpressionException e)
        {
            // never catch.
            e.printStackTrace();
        }
    }

    private class BasePreferencePanel extends JPanel
    {
        private static final long serialVersionUID = 1L;

        protected JButton getColorButton(Color color)
        {
            JButton button = new JButton();
            Insets insets = button.getMargin();
            button.setMargin(new Insets(insets.top, insets.left - 10, insets.bottom, insets.right - 10));
            setColorIcon(button, color);
            return button;
        }

        protected void setColorIcon(JButton button, Color color)
        {
            int w = 30;
            int h = button.getFont().getSize();
            BufferedImage image = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
            Graphics g = image.getGraphics();
            g.setColor(color);
            g.fillRect(0, 0, w, h);
            g.drawRect(0, 0, w, h);
            button.setIcon(new ImageIcon(image));
        }

        protected JPanel getItemPanel(String title)
        {
            JPanel panel = getItemPanel();
            JLabel label = new JLabel(title);
            panel.add(label);
            return panel;
        }

        protected JPanel getItemPanel()
        {
            JPanel panel = new JPanel();
            // panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
            // add(panel);
            return panel;
        }

        protected JSpinner getSpinner(int val, int min, int max, int step, boolean editable)
        {
            JSpinner spinner = new JSpinner(new SpinnerNumberModel(val, min, max, step));
            spinner.setMaximumSize(new Dimension(50, spinner.getPreferredSize().height));
            if (!editable)
            {
                ((JSpinner.NumberEditor) spinner.getEditor()).getTextField().setEditable(false);
            }
            return spinner;
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.swing.JComponent#setPreferredSize(java.awt.Dimension)
         */
        @Override
        public void setPreferredSize(Dimension preferredSize)
        {
            super.setPreferredSize(preferredSize);
            setMaximumSize(preferredSize);
            setMinimumSize(preferredSize);
        }
    }

    protected class LinePreferencePanel extends BasePreferencePanel
    {
        private static final long serialVersionUID = 1L;

        private int thickness = 1;

        private Color color = Color.BLACK;

        /**
         * @return the thickness
         */
        public int getThickness()
        {
            return thickness;
        }

        public void setThickness(int thickness)
        {
            this.thickness = thickness;
            spinner.setValue(thickness);
        }

        public void setColor(Color color)
        {
            this.color = color;
            setColorIcon(button, color);
        }

        /**
         * @return the color
         */
        public Color getColor()
        {
            return color;
        }

        public LinePreferencePanel(Color color, int thickness, String group, boolean addToTab)
        {
            this.color = color;
            this.thickness = thickness;
            setBorder(new TitledBorder(rmgr.getString("line.label")));
            setLayout(new GridBagLayout());
            setPreferredSize(new Dimension(PARTS_WIDTH, 60));
            addColorPreference(rmgr.getString("line.color.label"));
            addLineThicknessPreference(rmgr.getString("line.thickness.label"));
//            BaseLayerPluginPreferences.this.add(this);
            if (addToTab) {
                addItemToTabPanel(group, this);
            }
        }

        JButton button;

        protected void addColorPreference(String title)
        {
            JPanel panel = getItemPanel(title);
            button = getColorButton(color);
            button.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    NColorChooser ncc = new NColorChooser(color);
                    ncc.setVisible(true);
                    setColor(ncc.getColor());
                }
            });
            panel.add(button);
            GridBagConstraints constraints = new GridBagConstraints();
            constraints.gridx = 0;
            constraints.gridy = 0;
            add(panel, constraints);
        }

        JSpinner spinner;

        protected void addLineThicknessPreference(String title)
        {
            JPanel panel = getItemPanel(title);
            spinner = getSpinner(thickness, 1, 5, 1, false);
            spinner.addChangeListener(new ChangeListener()
            {
                public void stateChanged(ChangeEvent e)
                {
                    thickness = (Integer) ((JSpinner) e.getSource()).getValue();
                }
            });
            panel.add(spinner);
            GridBagConstraints constraints = new GridBagConstraints();
            constraints.gridx = 1;
            constraints.gridy = 0;
            add(panel, constraints);
        }
    }

    protected class PlotPreferencePanel extends BasePreferencePanel
    {
        private static final long serialVersionUID = 1L;

        private int outLineThickness = 1;

        private Color outLineColor = Color.RED;

        private Color color = Color.BLACK;

        /**
         * @param outLineThickness the outLineThickness to set
         */
        public void setOutLineThickness(int outLineThickness)
        {
            this.outLineThickness = outLineThickness;
            spinner.setValue(outLineThickness);
        }

        /**
         * @param outLineColor the outLineColor to set
         */
        public void setOutLineColor(Color outLineColor)
        {
            this.outLineColor = outLineColor;
            setColorIcon(button2, outLineColor);
        }

        /**
         * @param color the color to set
         */
        public void setColor(Color color)
        {
            this.color = color;
            setColorIcon(button1, color);
        }

        /**
         * @return the outLineThickness
         */
        public int getOutLineThickness()
        {
            return outLineThickness;
        }

        /**
         * @return the outLineColor
         */
        public Color getOutLineColor()
        {
            return outLineColor;
        }

        /**
         * @return the color
         */
        public Color getColor()
        {
            return color;
        }

        public PlotPreferencePanel(Color color, Color outLineColor, int outLineThickness, String group, boolean addToTab)
        {
            this(rmgr.getString("plot.label"), color, outLineColor, outLineThickness, group, addToTab);
        }

        public PlotPreferencePanel(String title, Color color, Color outLineColor, int outLineThickness, String group, boolean addToTab)
        {
            this.color = color;
            this.outLineColor = outLineColor;
            this.outLineThickness = outLineThickness;
            setBorder(new TitledBorder(title));
            setLayout(new GridBagLayout());
            setPreferredSize(new Dimension(PARTS_WIDTH, 60));
            addColorPreference(rmgr.getString("plot.color.label"));
            addOutLineColorPreference(rmgr.getString("plot.outline.color.label"));
            addOutLineThicknessPreference(rmgr.getString("plot.outline.thickness.label"));
//            BaseLayerPluginPreferences.this.add(this);
            if (addToTab) {
                addItemToTabPanel(group, this);
            }
        }

        JButton button1;

        protected void addColorPreference(String title)
        {
            JPanel panel = getItemPanel(title);
            button1 = getColorButton(color);
            button1.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    NColorChooser ncc = new NColorChooser(color);
                    ncc.setVisible(true);
                    setColor(ncc.getColor());
                }
            });
            panel.add(button1);
            GridBagConstraints constraints = new GridBagConstraints();
            constraints.gridx = 0;
            constraints.gridy = 0;
            add(panel, constraints);
        }

        JButton button2;

        protected void addOutLineColorPreference(String title)
        {
            JPanel panel = getItemPanel(title);
            button2 = getColorButton(outLineColor);
            button2.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    NColorChooser ncc = new NColorChooser(outLineColor);
                    ncc.setVisible(true);
                    setOutLineColor(ncc.getColor());
                }
            });
            panel.add(button2);
            GridBagConstraints constraints = new GridBagConstraints();
            constraints.gridx = 1;
            constraints.gridy = 0;
            add(panel, constraints);
        }

        JSpinner spinner;

        protected void addOutLineThicknessPreference(String title)
        {
            JPanel panel = getItemPanel(title);
            spinner = getSpinner(outLineThickness, 1, 5, 1, false);
            spinner.addChangeListener(new ChangeListener()
            {
                public void stateChanged(ChangeEvent e)
                {
                    outLineThickness = (Integer) ((JSpinner) e.getSource()).getValue();
                }
            });
            panel.add(spinner);
            // GridBagConstraints constraints = new GridBagConstraints();
            // constraints.gridx = 2;
            // constraints.gridy = 0;
            // add(panel, constraints);
        }
    }

    protected class SymbolPreferencePanel extends PlotPreferencePanel
    {
        private static final long serialVersionUID = 1L;

        private int size = 1;

        private int type = SYMBOL_TYPE_CIRCLE;

        /**
         * @param size the size to set
         */
        public void setSymbolSize(int size)
        {
            this.size = size;
            spinner.setValue(size);
        }

        /**
         * @param type the type to set
         */
        public void setType(String type)
        {
            this.type = getTypeIndex(type);
//            log.debug(this.type + " :: " + type);
            combobox.setSelectedIndex(this.type);
        }

        /**
         * @return the size
         */
        public int getSymbolSize()
        {
            return size;
        }

        /**
         * @return the type
         */
        public String getType()
        {
            return getType(type);
        }

        public SymbolPreferencePanel(Color color, Color outLineColor, int outLineThickness, int size, String type, String group, boolean addToTab)
        {
            super(rmgr.getString("symbol.label"), color, outLineColor, outLineThickness, group, addToTab);
            setPreferredSize(new Dimension(PARTS_WIDTH, 98));
            this.size = size;
            this.type = getTypeIndex(type);
            addSizePreference(rmgr.getString("symbol.size.label"));
            addTypePreference(rmgr.getString("symbol.shape.label"));
//            BaseLayerPluginPreferences.this.add(this);
            if (addToTab) {
                addItemToTabPanel(group, this);
            }
        }

        JSpinner spinner;

        protected void addSizePreference(String title)
        {
            JPanel panel = getItemPanel(title);
            spinner = getSpinner(size, 1, 10, 1, false);
            spinner.addChangeListener(new ChangeListener()
            {
                public void stateChanged(ChangeEvent e)
                {
                    size = (Integer) ((JSpinner) e.getSource()).getValue();
                }
            });
            panel.add(spinner);

            JLabel unit = new JLabel(" px");
            panel.add(unit);

            GridBagConstraints constraints = new GridBagConstraints();
            constraints.gridx = 0;
            constraints.gridy = 1;
            constraints.gridwidth = 2;
            add(panel, constraints);
        }

        JComboBox combobox;

        protected void addTypePreference(String title)
        {
            JPanel panel = getItemPanel(title);
            String[] types = new String[3];
            types[SYMBOL_TYPE_CIRCLE] = rmgr.getString("symbol.shape.circle");
            types[SYMBOL_TYPE_TRIANGLE] = rmgr.getString("symbol.shape.triangle");
            types[SYMBOL_TYPE_RECTANGLE] = rmgr.getString("symbol.shape.rectangle");
            combobox = new JComboBox(types);
            combobox.setMaximumSize(new Dimension(50, combobox.getPreferredSize().height));
            combobox.setSelectedIndex(type);
            combobox.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    type = ((JComboBox) e.getSource()).getSelectedIndex();
                }
            });
            panel.add(combobox);
            GridBagConstraints constraints = new GridBagConstraints();
            constraints.gridx = 2;
            constraints.gridy = 1;
            add(panel, constraints);
        }

        private int getTypeIndex(String type)
        {
            if (type.equals(SymbolGts.CIRCLE_STRING))
            {
                return 0;
            }
            else if (type.equals(SymbolGts.TRIANGLE_STRING))
            {
                return 1;
            }
            else if (type.equals(SymbolGts.RECTANGLE_STRING))
            {
                return 2;
            }
            else
            {
                return 0;
            }
        }

        private String getType(int index)
        {
            switch (index)
            {
                case 0:
                    return SymbolGts.CIRCLE_STRING;
                case 1:
                    return SymbolGts.TRIANGLE_STRING;
                case 2:
                    return SymbolGts.RECTANGLE_STRING;
                default:
                    return SymbolGts.CIRCLE_STRING;
            }
        }
    }

    protected class FontPreferencePanel extends BasePreferencePanel
    {
        private static final long serialVersionUID = 1L;

        private String name = Font.DIALOG;// family name

        private int size = 10;

        private Color color = Color.BLACK;

        private int style = Font.PLAIN;

        private List<String> fs = null;

        /**
         * @param size the size to set
         */
        public void setFontSize(int size)
        {
            this.size = size;
            spinner.setValue(size);
        }

        /**
         * @param name the name to set
         */
        public void setName(String name)
        {
            this.name = name;
            combobox1.setSelectedIndex(fs.indexOf(name));
        }

        /**
         * @param color the color to set
         */
        public void setColor(Color color)
        {
            this.color = color;
            setColorIcon(button, color);
        }

        /**
         * @param style the style to set
         */
        public void setStyle(int style)
        {
            this.style = style;
            combobox2.setSelectedIndex(getStyleIndex(style));
        }

        /**
         * @return the name
         */
        public String getName()
        {
            return name;
        }

        /**
         * @return the size
         */
        public int getFontSize()
        {
            return size;
        }

        /**
         * @return the color
         */
        public Color getColor()
        {
            return color;
        }

        /**
         * @return the style
         */
        public int getStyle()
        {
            return style;
        }

        public FontPreferencePanel(String name, int style, int size, Color color, String group, boolean addToTab)
        {
            this.name = name;
            this.size = size;
            this.style = style;
            this.color = color;
            setBorder(new TitledBorder(rmgr.getString("font.label")));
            setLayout(new GridBagLayout());
            setPreferredSize(new Dimension(PARTS_WIDTH, 98));
            addNamePreference(rmgr.getString("font.family.name.label"));
            addSizePreference(rmgr.getString("font.size.label"));
            addColorPreference(rmgr.getString("font.color.label"));
            addStylePreference(rmgr.getString("font.style.label"));
//            BaseLayerPluginPreferences.this.add(this);
            if (addToTab) {
                addItemToTabPanel(group, this);
            }
        }

        JComboBox combobox1;

        protected void addNamePreference(String title)
        {
            JPanel panel = getItemPanel(title);
            if (fs == null)
            {
                GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
                Font[] fonts = ge.getAllFonts();
                fs = new ArrayList<String>();
                for (int i = 0; i < fonts.length; i++)
                {
                    if (!fs.contains(fonts[i].getFamily()))
                    {
                        fs.add(fonts[i].getFamily());
                    }
                }
                Collections.sort(fs);
            }
            combobox1 = new JComboBox(fs.toArray());
            combobox1.setMaximumSize(new Dimension(200, combobox1.getPreferredSize().height));
            int index = fs.indexOf(name);
            if (index >= 0)
            {
                combobox1.setSelectedIndex(index);
            }
            else
            {
                combobox1.setSelectedIndex(fs.indexOf(Font.DIALOG));
            }
            combobox1.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    name = fs.get(((JComboBox) e.getSource()).getSelectedIndex());
                }
            });
            panel.add(combobox1);
            GridBagConstraints constraints = new GridBagConstraints();
            constraints.gridx = 0;
            constraints.gridy = 0;
            constraints.gridwidth = 2;
            add(panel, constraints);
        }

        JButton button;

        protected void addColorPreference(String title)
        {
            JPanel panel = getItemPanel(title);
            button = getColorButton(color);
            button.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    NColorChooser ncc = new NColorChooser(color);
                    ncc.setVisible(true);
                    setColor(ncc.getColor());
                }
            });
            panel.add(button);
            GridBagConstraints constraints = new GridBagConstraints();
            constraints.gridx = 2;
            constraints.gridy = 0;
            add(panel, constraints);
        }

        JSpinner spinner;

        protected void addSizePreference(String title)
        {
            JPanel panel = getItemPanel(title);
            spinner = getSpinner(size, 6, 18, 1, false);
            spinner.addChangeListener(new ChangeListener()
            {
                public void stateChanged(ChangeEvent e)
                {
                    size = (Integer) ((JSpinner) e.getSource()).getValue();
                }
            });
            panel.add(spinner);

            JLabel unit = new JLabel(" pt");
            panel.add(unit);

            GridBagConstraints constraints = new GridBagConstraints();
            constraints.gridx = 0;
            constraints.gridy = 1;
            add(panel, constraints);
        }

        JComboBox combobox2;

        protected void addStylePreference(String title)
        {
            JPanel panel = getItemPanel(title);
            String[] styles = { rmgr.getString("font.plain.label"), rmgr.getString("font.bold.label"), rmgr.getString("font.bold.italic.label"), rmgr.getString("font.italic.label") };
            combobox2 = new JComboBox(styles);
            combobox2.setMaximumSize(new Dimension(100, combobox2.getPreferredSize().height));
            combobox2.setSelectedIndex(getStyleIndex(style));
            combobox2.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    style = getStyle(((JComboBox) e.getSource()).getSelectedIndex());
                }
            });
            panel.add(combobox2);
            GridBagConstraints constraints = new GridBagConstraints();
            constraints.gridx = 1;
            constraints.gridy = 1;
            add(panel, constraints);
        }

        private int getStyleIndex(int style)
        {
            switch (style)
            {
                case Font.PLAIN:
                    return 0;
                case Font.BOLD:
                    return 1;
                case Font.BOLD | Font.ITALIC:
                    return 2;
                case Font.ITALIC:
                    return 3;
                default:
                    return 0;
            }
        }

        private int getStyle(int index)
        {
            switch (index)
            {
                case 0:
                    return Font.PLAIN;
                case 1:
                    return Font.BOLD;
                case 2:
                    return Font.BOLD | Font.ITALIC;
                case 3:
                    return Font.ITALIC;
                default:
                    return Font.PLAIN;
            }
        }
    }

    protected class BackgroundPreferencePanel extends BasePreferencePanel
    {
        private static final long serialVersionUID = 1L;

        private Color color = Color.BLACK;

        /**
         * @param color the color to set
         */
        public void setColor(Color color)
        {
            this.color = color;
            setColorIcon(button, color);
        }

        /**
         * @return the color
         */
        public Color getColor()
        {
            return color;
        }

        public BackgroundPreferencePanel(Color color)
        {
            this.color = color;
            setBorder(new TitledBorder(rmgr.getString("background.label")));
            setLayout(new GridBagLayout());
            setPreferredSize(new Dimension(PARTS_WIDTH, 60));
            addColorPreference(rmgr.getString("background.color.label"));
            BaseLayerPluginPreferences.this.add(this);
        }

        JButton button;

        protected void addColorPreference(String title)
        {
            JPanel panel = getItemPanel(title);
            button = getColorButton(color);
            button.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    NColorChooser ncc = new NColorChooser(color);
                    ncc.setVisible(true);
                    setColor(ncc.getColor());
                }
            });
            panel.add(button);
            GridBagConstraints constraints = new GridBagConstraints();
            constraints.gridx = 0;
            constraints.gridy = 0;
            add(panel, constraints);
        }
    }

    protected class SelectedColorPreferencePanel extends BasePreferencePanel
    {
        private static final long serialVersionUID = 1L;

        private Color color = Color.BLACK;

        /**
         * @param color the color to set
         */
        public void setColor(Color color)
        {
            this.color = color;
            setColorIcon(button, color);
        }

        /**
         * @return the color
         */
        public Color getColor()
        {
            return color;
        }

        public SelectedColorPreferencePanel(Color color)
        {
            this.color = color;
            setBorder(new TitledBorder(rmgr.getString("highlight.label")));
            setLayout(new GridBagLayout());
            setPreferredSize(new Dimension(PARTS_WIDTH, 60));
            addColorPreference(rmgr.getString("highlight.color.label"));
            BaseLayerPluginPreferences.this.add(this);
        }

        JButton button;

        protected void addColorPreference(String title)
        {
            JPanel panel = getItemPanel(title);
            button = getColorButton(color);
            button.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    NColorChooser ncc = new NColorChooser(color);
                    ncc.setVisible(true);
                    setColor(ncc.getColor());
                }
            });
            panel.add(button);
            GridBagConstraints constraints = new GridBagConstraints();
            constraints.gridx = 0;
            constraints.gridy = 0;
            add(panel, constraints);
        }
    }
    
    protected class DetailFontNamePreferencePanel extends BasePreferencePanel
    {
        private static final long serialVersionUID = 1L;
        
        private List<String> fs = null;
        
        private String name = Font.DIALOG;// family name
        
        public void setName(String name)
        {
            this.name = name;
            combobox1.setSelectedIndex(fs.indexOf(name));
        }
        
        public String getName()
        {
            return name;
        }
        
        public DetailFontNamePreferencePanel(String name)
        {
            this.name = name;
            setBorder(new TitledBorder(rmgr.getString("detail.font.label")));
            setLayout(new GridBagLayout());
            setPreferredSize(new Dimension(PARTS_WIDTH, 90));
            addNamePreference(rmgr.getString("font.family.name.label"));
            BaseLayerPluginPreferences.this.add(this);
        }

        JComboBox combobox1;

        protected void addNamePreference(String title)
        {
            JPanel panel = getItemPanel(title);
            if (fs == null)
            {
                GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
                Font[] fonts = ge.getAllFonts();
                fs = new ArrayList<String>();
                for (int i = 0; i < fonts.length; i++)
                {
                    if (!fs.contains(fonts[i].getFamily()))
                    {
                        fs.add(fonts[i].getFamily());
                    }
                }
                Collections.sort(fs);
            }
            combobox1 = new JComboBox(fs.toArray());
            combobox1.setMaximumSize(new Dimension(200, combobox1.getPreferredSize().height));
            int index = fs.indexOf(name);
            if (index >= 0)
            {
                combobox1.setSelectedIndex(index);
            }
            else
            {
                combobox1.setSelectedIndex(fs.indexOf(Font.DIALOG));
            }
            combobox1.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    name = fs.get(((JComboBox) e.getSource()).getSelectedIndex());
                }
            });
            panel.add(combobox1);
            GridBagConstraints constraints = new GridBagConstraints();
            constraints.gridx = 0;
            constraints.gridy = 0;
            constraints.gridwidth = 2;
            add(panel, constraints);
        }
    }
    
    protected class ShowGroupNamePreferencePanel extends BasePreferencePanel
    {
        private static final long serialVersionUID = 1L;

        private boolean show = false;

        /**
         * @param enable the enable to set
         */
        public void setShow(boolean show)
        {
            this.show = show;
            on.setSelected(show);
            off.setSelected(!show);
        }

        /**
         * @return the enable
         */
        public boolean isShow()
        {
            return show;
        }

        public ShowGroupNamePreferencePanel(boolean show)
        {
            this.show = show;
            setBorder(new TitledBorder(rmgr.getString("show.group.name.label")));
            setLayout(new GridBagLayout());
            setPreferredSize(new Dimension(PARTS_WIDTH, 98));
            addShowPreference();
            BaseLayerPluginPreferences.this.add(this);
        }

        JRadioButton on;

        JRadioButton off;

        protected void addShowPreference()
        {
            JPanel panel = getItemPanel();
            ButtonGroup group = new ButtonGroup();
            log.debug(show);
            on = new JRadioButton(rmgr.getString("show.group.name.on.label"), show);
            on.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    if (((JRadioButton) e.getSource()).isSelected())
                    {
                        show = true;
                    }
                }
            });
            group.add(on);
            panel.add(on);

            off = new JRadioButton(rmgr.getString("show.group.name.off.label"), !show);
            off.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    if (((JRadioButton) e.getSource()).isSelected())
                    {
                        show = false;
                    }
                }
            });
            group.add(off);
            panel.add(off);
            GridBagConstraints constraints = new GridBagConstraints();
            constraints.gridx = 0;
            constraints.gridy = 0;
            add(panel, constraints);
        }
    }

    protected class HeightPreferencePanel extends BasePreferencePanel
    {
        private static final long serialVersionUID = 1L;

        private int height = 100;

        /**
         * @param height the height to set
         */
        public void setLayerHeight(int height)
        {
            this.height = height;
            spinner.setValue(height);
        }

        /**
         * @return the height
         */
        public int getLayerHeight()
        {
            return height;
        }

        public HeightPreferencePanel(int height)
        {
            this.height = height;
            setBorder(new TitledBorder(rmgr.getString("layer.height.label")));
            setLayout(new GridBagLayout());
            setPreferredSize(new Dimension(PARTS_WIDTH, 60));
            addHeightPreference(rmgr.getString("height.label"));
            BaseLayerPluginPreferences.this.add(this);
        }

        JSpinner spinner;

        protected void addHeightPreference(String title)
        {
            JPanel panel = getItemPanel(title);
            spinner = getSpinner(height, 70, 1000, 1, true);
            spinner.setMaximumSize(new Dimension(100, spinner.getPreferredSize().height));
            spinner.addChangeListener(new ChangeListener()
            {
                public void stateChanged(ChangeEvent e)
                {
                    height = (Integer) ((JSpinner) e.getSource()).getValue();
                }
            });
            panel.add(spinner);

            JLabel unit = new JLabel(" px");
            panel.add(unit);

            GridBagConstraints constraints = new GridBagConstraints();
            constraints.gridx = 0;
            constraints.gridy = 0;
            add(panel, constraints);
        }
    }

    protected class ScaleMainPreferencePanel extends BasePreferencePanel
    {
        private static final long serialVersionUID = 1L;

        private int outLineThickness = 1;

        private Color outLineColor = Color.RED;

        private Color color = Color.BLACK;

        /**
         * @param outLineThickness the outLineThickness to set
         */
        public void setOutLineThickness(int outLineThickness)
        {
            this.outLineThickness = outLineThickness;
            spinner.setValue(outLineThickness);
        }

        /**
         * @param outLineColor the outLineColor to set
         */
        public void setOutLineColor(Color outLineColor)
        {
            this.outLineColor = outLineColor;
            setColorIcon(button2, outLineColor);
        }

        /**
         * @param color the color to set
         */
        public void setColor(Color color)
        {
            this.color = color;
            setColorIcon(button1, color);
        }

        /**
         * @return the outLineThickness
         */
        public int getOutLineThickness()
        {
            return outLineThickness;
        }

        /**
         * @return the outLineColor
         */
        public Color getOutLineColor()
        {
            return outLineColor;
        }

        /**
         * @return the color
         */
        public Color getColor()
        {
            return color;
        }

        public ScaleMainPreferencePanel(Color outLineColor, Color color, int outLineThickness)
        {
            this(rmgr.getString("scale.main.label"), outLineColor, color, outLineThickness);
        }

        public ScaleMainPreferencePanel(String title, Color outLineColor, Color color, int outLineThickness)
        {
            this.outLineThickness = outLineThickness;
            this.outLineColor = outLineColor;
            this.color = color;
            setBorder(new TitledBorder(title));
            setLayout(new GridBagLayout());
            setPreferredSize(new Dimension(PARTS_WIDTH, 60));
            addColorPreference(rmgr.getString("scale.color.label"));
            addOutLineThicknessPreference(rmgr.getString("scale.thickness.label"));
            addOutLineColorPreference(rmgr.getString("scale.outline.color.label"));
            BaseLayerPluginPreferences.this.add(this);
        }

        JButton button1;

        protected void addColorPreference(String title)
        {
            JPanel panel = getItemPanel(title);
            button1 = getColorButton(color);
            button1.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    NColorChooser ncc = new NColorChooser(color);
                    ncc.setVisible(true);
                    setColor(ncc.getColor());
                }
            });
            panel.add(button1);
            GridBagConstraints constraints = new GridBagConstraints();
            constraints.gridx = 0;
            constraints.gridy = 0;
            add(panel, constraints);
        }

        JButton button2;

        protected void addOutLineColorPreference(String title)
        {
            JPanel panel = getItemPanel(title);
            button2 = getColorButton(outLineColor);
            button2.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    NColorChooser ncc = new NColorChooser(outLineColor);
                    ncc.setVisible(true);
                    setOutLineColor(ncc.getColor());
                }
            });
            panel.add(button2);
            // GridBagConstraints constraints = new GridBagConstraints();
            // constraints.gridx = 1;
            // constraints.gridy = 0;
            // add(panel, constraints);
        }

        JSpinner spinner;

        protected void addOutLineThicknessPreference(String title)
        {
            JPanel panel = getItemPanel(title);
            spinner = getSpinner(outLineThickness, 1, 5, 1, false);
            spinner.setMaximumSize(new Dimension(50, spinner.getPreferredSize().height));
            spinner.addChangeListener(new ChangeListener()
            {
                public void stateChanged(ChangeEvent e)
                {
                    outLineThickness = (Integer) ((JSpinner) e.getSource()).getValue();
                }
            });
            panel.add(spinner);
            // GridBagConstraints constraints = new GridBagConstraints();
            // constraints.gridx = 2;
            // constraints.gridy = 0;
            // add(panel, constraints);
        }
    }

    protected class ScaleSubPreferencePanel extends ScaleMainPreferencePanel
    {
        private static final long serialVersionUID = 1L;

        public ScaleSubPreferencePanel(Color outLineColor, Color color, int outLineThickness)
        {
            super(rmgr.getString("scale.sub.label"), outLineColor, color, outLineThickness);
        }
    }

    protected class RecordReducePreferencePanel extends BasePreferencePanel
    {
        private static final long serialVersionUID = 1L;

        private boolean enable = true;

        private int numberOfRecord = 100;

        private JSpinner spinner;

        /**
         * @param enable the enable to set
         */
        public void setRecordReduceEnable(boolean enable)
        {
            this.enable = enable;
            on.setSelected(enable);
            off.setSelected(!enable);
            spinner.setEnabled(enable);
        }

        /**
         * @param numberOfRecord the numberOfRecord to set
         */
        public void setNumberOfRecord(int numberOfRecord)
        {
            this.numberOfRecord = numberOfRecord;
            spinner.setValue(numberOfRecord);
        }

        /**
         * @return the enable
         */
        public boolean isRecordReduceEnable()
        {
            return enable;
        }

        /**
         * @return the numberOfRecord
         */
        public int getNumberOfRecord()
        {
            return numberOfRecord;
        }

        public RecordReducePreferencePanel(boolean enable, int numberOfRecord)
        {
            this.enable = enable;
            this.numberOfRecord = numberOfRecord;
            setBorder(new TitledBorder(rmgr.getString("data.reduction.label")));
            setLayout(new GridBagLayout());
            setPreferredSize(new Dimension(PARTS_WIDTH, 98));
            addEnablePreference();
            addNumberOfRecordPreference(rmgr.getString("data.reduction.apply.number.label"));
            BaseLayerPluginPreferences.this.add(this);
        }

        JRadioButton on;

        JRadioButton off;

        protected void addEnablePreference()
        {
            JPanel panel = getItemPanel();
            ButtonGroup group = new ButtonGroup();
            log.debug(enable);
            on = new JRadioButton(rmgr.getString("data.reduction.on.label"), enable);
            on.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    if (((JRadioButton) e.getSource()).isSelected())
                    {
                        enable = true;
                        spinner.setEnabled(enable);
                    }
                }
            });
            group.add(on);
            panel.add(on);

            off = new JRadioButton(rmgr.getString("data.reduction.off.label"), !enable);
            off.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    if (((JRadioButton) e.getSource()).isSelected())
                    {
                        enable = false;
                        spinner.setEnabled(enable);
                    }
                }
            });
            group.add(off);
            panel.add(off);
            GridBagConstraints constraints = new GridBagConstraints();
            constraints.gridx = 0;
            constraints.gridy = 0;
            add(panel, constraints);
        }

        protected void addNumberOfRecordPreference(String title)
        {
            JPanel panel = getItemPanel(title);
            spinner = getSpinner(numberOfRecord, 0, 1000000, 100, true);
            spinner.setMaximumSize(new Dimension(150, spinner.getPreferredSize().height));
            spinner.setEnabled(enable);
            spinner.addChangeListener(new ChangeListener()
            {
                public void stateChanged(ChangeEvent e)
                {
                    numberOfRecord = (Integer) ((JSpinner) e.getSource()).getValue();
                }
            });
            panel.add(spinner);
            GridBagConstraints constraints = new GridBagConstraints();
            constraints.gridx = 0;
            constraints.gridy = 1;
            add(panel, constraints);
        }
    }
}
