/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.preferences;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.TreeMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.AbstractAction;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.T2MapMenuBar;
import jp.nihu.gttime.plugin.PluginPreferences;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.utility.ExtensionFileFilter;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * メニューバーの「編集」→「設定...」の「一般」のオブジェクトを取得するクラスです。
 */
public class GeneralPreferences extends PluginPreferences
{
    /**
     * シリアルバーションID
     */
    private static final long serialVersionUID = 1L;

    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(GeneralPreferences.class);

    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(GeneralPreferences.class, T2Map.LOCALE);
    private static PluginResourceManager rmgrmb = PluginResourceManager.getManager(T2MapMenuBar.class, T2Map.LOCALE);

    /**
     * プラグインのディレクトリ
     */
    private static final String PLUGIN_DIR = "jp.nihu.gttime.T2Map.PLUGIN_DIR";

    private static final String WEBBROWSER_PATH = "jp.nihu.gttime.plugin.LayerPlugin.WEBBROWSER_PATH";

    private static final String LOCALE = "jp.nihu.gttime.T2Map.LOCALE";

    private static final String SHOW_OPEN_CLOSE = "jp.nihu.gttime.T2Map.SHOW_OPEN_CLOSE";
    
    private static final String SHOW_ALERT_ON_SAVE = "jp.nihu.gttime.T2Map.SHOW_ALERT_ON_SAVE";
    
    private static final String SAVE_REMOTE_FILE_ALWAYS = "jp.nihu.gttime.T2Map.SAVE_REMOTE_FILE_ALWAYS";

    /**
     * プラグインディレクトリを表示する為のJPanelオブジェクト
     */
    private JPanel panel1;

    private JPanel browserpanel;

    private JPanel localepanel;
    
    private JPanel memorypanel;

    private JSpinner minSpinner;

    private JSpinner maxSpinner;

    private JCheckBox showOpenCloseEdgeCheckBox;
    
    private JCheckBox saveCheckBox;
    
    private JCheckBox saveRemoteFileCheckBox;

    /**
     * プラグインディレクトリのレイアウトを表示します。
     */
    protected void init()
    {
        this.removeAll();
        log.debug("Initialize GeneralPreferences. : " + T2Map.LOCALE);
        setName(rmgr.getString("title"));
        add(getPluginDirPropPanel());
        add(getBrowserPropPanel());
        add(getLocalePropPanel());
        panel1.setPreferredSize(new Dimension(PARTS_WIDTH, 88));
        browserpanel.setPreferredSize(new Dimension(PARTS_WIDTH, 60));
        localepanel.setPreferredSize(new Dimension(PARTS_WIDTH, 60));
        String[] vals = getMemorySetting();
        if (vals != null)
        {
            add(getMemoryPropPanel(Integer.parseInt(vals[0]), Integer.parseInt(vals[1])));
            memorypanel.setPreferredSize(new Dimension(PARTS_WIDTH, 60));
        }
        add(getShowOpenCloseEdgePanel());
        add(getSavePanel());
    }
    
    private JPanel getSavePanel()
    {
        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
        panel.setBorder(new TitledBorder(rmgrmb.getString("menu.file.save")));
        saveCheckBox = new JCheckBox(rmgr.getString("not.show.alert.for.save"), T2Map.properties.getBoolean(SHOW_ALERT_ON_SAVE));
        panel.add(saveCheckBox);
        saveRemoteFileCheckBox = new JCheckBox(rmgr.getString("save.remote.faile.always"), T2Map.properties.getBoolean(SAVE_REMOTE_FILE_ALWAYS));
        panel.add(saveRemoteFileCheckBox);
        panel.setPreferredSize(new Dimension(PARTS_WIDTH, 100));
        return panel;
    }

    private JPanel getShowOpenCloseEdgePanel()
    {
        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
        panel.setBorder(new TitledBorder(rmgr.getString("display.setting.name")));
        showOpenCloseEdgeCheckBox = new JCheckBox(rmgr.getString("show.open.close.edge"), T2Map.properties.getBoolean(SHOW_OPEN_CLOSE));
        panel.add(showOpenCloseEdgeCheckBox);
        panel.setPreferredSize(new Dimension(PARTS_WIDTH, 60));
        return panel;
    }

    private JPanel getMemoryPropPanel(int min, int max)
    {
        memorypanel = new JPanel();
        memorypanel.setLayout(new BoxLayout(memorypanel, BoxLayout.X_AXIS));
        memorypanel.setBorder(new TitledBorder(rmgr.getString("memory.name")));
        JLabel minLabel = new JLabel(rmgr.getString("memory.min.name"));
        minSpinner = new JSpinner(new SpinnerNumberModel(min, 256, max, 32));
        ((JSpinner.NumberEditor) minSpinner.getEditor()).getTextField().setEditable(false);
        JLabel unitLabel1 = new JLabel(rmgr.getString("memory.unit.name") + ", ");
        JLabel maxLabel = new JLabel(rmgr.getString("memory.max.name"));
        maxSpinner = new JSpinner(new SpinnerNumberModel(max, min, 2048, 32));
        ((JSpinner.NumberEditor) maxSpinner.getEditor()).getTextField().setEditable(false);
        JLabel unitLabel2 = new JLabel(rmgr.getString("memory.unit.name"));

        minSpinner.addChangeListener(new ChangeListener()
        {
            public void stateChanged(ChangeEvent e)
            {
                ((SpinnerNumberModel) maxSpinner.getModel()).setMinimum((Integer) ((JSpinner) e.getSource()).getValue());
            }
        });
        maxSpinner.addChangeListener(new ChangeListener()
        {
            public void stateChanged(ChangeEvent e)
            {
                ((SpinnerNumberModel) minSpinner.getModel()).setMaximum((Integer) ((JSpinner) e.getSource()).getValue());
            }
        });

        memorypanel.add(minLabel);
        memorypanel.add(minSpinner);
        memorypanel.add(unitLabel1);
        memorypanel.add(maxLabel);
        memorypanel.add(maxSpinner);
        memorypanel.add(unitLabel2);

        return memorypanel;
    }

    private String[] getMemorySetting()
    {
        String[] vals = null;
        File[] execFiles = new File(".").getAbsoluteFile().getParentFile().listFiles(new FilenameFilter()
        {
            public boolean accept(File dir, String name)
            {
                if (name.equals("gt-time.cmd") || name.equals("gt-time.sh") || name.equals("hutime.cmd") || name.equals("hutime.sh"))
                {
                    return true;
                }
                return false;
            }
        });
        if (execFiles.length > 0)
        {
            vals = new String[2];
            String regex1 = "-Xms[0-9]+m";
            Pattern p1 = Pattern.compile(regex1);
            String regex2 = "-Xmx[0-9]+m";
            Pattern p2 = Pattern.compile(regex2);
            try
            {
                BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(execFiles[0])));
                String line;
                while ((line = in.readLine()) != null)
                {
                    Matcher m1 = p1.matcher(line);
                    if (m1.find())
                    {
                        String matchstr = m1.group();
                        vals[0] = matchstr.substring(4, matchstr.length() - 1);
                    }
                    Matcher m2 = p2.matcher(line);
                    if (m2.find())
                    {
                        String matchstr = m2.group();
                        vals[1] = matchstr.substring(4, matchstr.length() - 1);
                    }
                }
                in.close();
            }
            catch (FileNotFoundException e)
            {
                e.printStackTrace();
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
        }
        return vals;
    }

    private void saveMemorySetting(String min, String max)
    {
        File[] execFiles = new File(".").getAbsoluteFile().getParentFile().listFiles(new FilenameFilter()
        {
            public boolean accept(File dir, String name)
            {
                if (name.equals("gt-time.cmd") || name.equals("gt-time.sh"))
                {
                    return true;
                }
                return false;
            }
        });
        for (int i = 0; i < execFiles.length; i++)
        {
            try
            {
                BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(execFiles[i])));
                String line;
                StringBuffer result = new StringBuffer();
                while ((line = in.readLine()) != null)
                {
                    line = line.replaceAll("-Xms[0-9]+m", "-Xms" + min + "m");
                    line = line.replaceAll("-Xmx[0-9]+m", "-Xmx" + max + "m");
                    result.append(line).append("\n");
                }
                in.close();
                OutputStream out = new FileOutputStream(execFiles[i]);
                out.write(result.toString().getBytes("UTF-8"));
                out.flush();
                out.close();
            }
            catch (FileNotFoundException e)
            {
                e.printStackTrace();
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
        }
    }

    private JPanel getBrowserPropPanel()
    {
        browserpanel = new JPanel();
        browserpanel.setLayout(new BoxLayout(browserpanel, BoxLayout.Y_AXIS));
        browserpanel.setBorder(new TitledBorder(rmgr.getString("browser.file.name")));
        String file = T2Map.properties.getProperty(WEBBROWSER_PATH);
        browserpanel.add(new BrowserFileForm(file));
        return browserpanel;
    }

    private JPanel getLocalePropPanel()
    {
        localepanel = new JPanel();
        localepanel.setLayout(new BoxLayout(localepanel, BoxLayout.Y_AXIS));
        localepanel.setBorder(new TitledBorder(rmgr.getString("locale.name")));
        ArrayList<LocaleItem> al = new ArrayList<LocaleItem>();
        Map<String, LocaleItem> lv = getLocaleItem();
        al.addAll(lv.values());
        JComboBox localeList = new JComboBox(al.toArray());
        localeList.setSelectedItem(lv.get(T2Map.LOCALE.getDisplayLanguage(T2Map.LOCALE)));
        localepanel.add(localeList);
        return localepanel;
    }

    private Map<String, LocaleItem> getLocaleItem()
    {
        Map<String, LocaleItem> lv = new TreeMap<String, LocaleItem>();
        
        log.info("classpath : " + System.getProperty("java.class.path"));
        String[] pss = System.getProperty("java.class.path").split(File.pathSeparator);
        String pp = "";
        for (int i = 0; i < pss.length; i++) {
        	if (pss[i].contains("resource")) {
        		pp = pss[i];
        	}
        }
        log.info("classpath : " + pp);
        
        if (pp.equals("")) {
        	for (int i = 0; i < pss.length; i++) {
            	if (pss[i].endsWith(".exe") || pss[i].endsWith(".jar")) {
            		log.info(pss[i] + " --> " + File.separator);
            		pp = pss[i].substring(0, pss[i].lastIndexOf(File.separator));
            		pp += File.separator + "lib" + File.separator + "resource";
            	}
            }
        }
        
        File lib = new File(pp);// new File("./lib/resource");
        if (lib.exists())
        {
            Locale[] al = Locale.getAvailableLocales();
            List<Locale> all = Arrays.asList(al);
//            for (int i = 0; i < all.size(); i++)
//            {
//            	System.out.println(all.get(i).getLanguage() + ":" + all.get(i).getCountry() + ":" + all.get(i).getDisplayCountry() + ":" + all.get(i).getDisplayName());
//            }
            File[] c = lib.listFiles();
            for (int i = 0; i < c.length; i++)
            {
                String p = c[i].getAbsolutePath();
                p = p.substring(p.lastIndexOf(File.separator) + 1);
                Locale tl = null;
                if (p.contains("_")) {
                	String[] ps = p.split("_");
                	tl = new Locale(ps[0], ps[1]);
                } else {
                	tl = new Locale(p);
                }
                if (all.contains(tl))
                {
                    lv.put(tl.getDisplayLanguage(T2Map.LOCALE), new LocaleItem(tl));
                }
            }
        }
        return lv;
    }

    private JPanel getPluginDirPropPanel()
    {
        panel1 = new JPanel();
        panel1.setLayout(new BoxLayout(panel1, BoxLayout.Y_AXIS));
        panel1.setBorder(new TitledBorder(rmgr.getString("plugin.dir.name")));
        String[] dirs = T2Map.properties.getProperties(PLUGIN_DIR);
        if (dirs != null)
        {
            for (int i = 0; i < dirs.length; i++)
            {
                panel1.add(new PluginDirForm(dirs[i]));
            }
        }
        JPanel panel2 = new JPanel();
        panel2.setLayout(new BoxLayout(panel2, BoxLayout.X_AXIS));
        JButton jButton0 = new JButton(rmgr.getString("plus.button"));
        jButton0.addActionListener(new AbstractAction()
        {
            private static final long serialVersionUID = 1L;

            public void actionPerformed(ActionEvent e)
            {
                int num = panel1.getComponentCount();
                panel1.add(new PluginDirForm(""), num - 1);
                panel1.setPreferredSize(new Dimension(PARTS_WIDTH, panel1.getHeight() + 30));
                updateUI();
            }
        });
        panel2.add(jButton0);
        panel2.add(Box.createHorizontalGlue());
        panel1.add(panel2);
        return panel1;
    }

    /**
     * 指定されたプラグインディレクトリを削除します。
     * 
     * @param panel JPanelオブジェクト
     */
    private void removePluginDir(JPanel panel)
    {
        panel1.remove(panel);
        panel1.setPreferredSize(new Dimension(PARTS_WIDTH, panel1.getHeight() - 30));
        updateUI();
    }

    /**
     * 現在の設定を保存します。
     */
    public void save()
    {
        savePluginDir();
        saveBrowserFile();
        saveLocale();
        if (minSpinner != null && maxSpinner != null)
        {
            saveMemorySetting(((Integer) minSpinner.getValue()).toString(), ((Integer) maxSpinner.getValue()).toString());
        }
        saveShowOpenCloseEdge();
        saveSaveAlert();
        saveSaveRemoteFileAlert();
    }

    private void saveShowOpenCloseEdge()
    {
        T2Map.properties.put(SHOW_OPEN_CLOSE, Boolean.toString(showOpenCloseEdgeCheckBox.isSelected()));
    }

    private void saveSaveAlert()
    {
        T2Map.properties.put(SHOW_ALERT_ON_SAVE, Boolean.toString(saveCheckBox.isSelected()));
    }
    
    private void saveSaveRemoteFileAlert()
    {
        T2Map.properties.put(SAVE_REMOTE_FILE_ALWAYS, Boolean.toString(saveRemoteFileCheckBox.isSelected()));
    }

    private void saveBrowserFile()
    {
        Component[] comps = browserpanel.getComponents();
        String browserFile = "";
        for (int i = 0; i < comps.length; i++)
        {
            if (comps[i] instanceof BrowserFileForm)
            {
                browserFile = ((BrowserFileForm) comps[i]).getText();
                break;
            }
        }
        T2Map.properties.put(WEBBROWSER_PATH, browserFile);
    }

    private void saveLocale()
    {
        Component[] comps = localepanel.getComponents();
        String loc = "";
        for (int i = 0; i < comps.length; i++)
        {
            if (comps[i] instanceof JComboBox)
            {
            	Locale l = ((LocaleItem) ((JComboBox) comps[i]).getSelectedItem()).getLocale();
            	String c = l.getCountry();
            	if (c != null && !c.equals("")) {
            		loc = l.getLanguage() + "_" + c;
//            		System.out.println(l.getScript());
//            		System.out.println(l.toString());
//            		System.out.println(l.getISO3Country());
//            		System.out.println(l.getISO3Language());
//            		String s = l.getScript();
//            		if (s != null && !s.equals("")) {
//            			loc += "_" + s;
//            		}
            	} else {
            		loc = l.getLanguage();
            	}
                break;
            }
        }

        if (!T2Map.LOCALE.getLanguage().equals(loc))
        {
            JOptionPane.showMessageDialog(this, rmgr.getString("change.locale.message"));
        }

        T2Map.properties.put(LOCALE, loc);
    }

    private class LocaleItem
    {
        private Locale locale;

        public LocaleItem(Locale locale)
        {
            this.locale = locale;
        }

        public Locale getLocale()
        {
            return locale;
        }

        public String toString()
        {
            // return locale.getDisplayLanguage(T2Map.LOCALE);
            // return locale.getDisplayLanguage(locale) + " (" + locale.getDisplayLanguage(T2Map.LOCALE) + ")";
            return locale.getDisplayLanguage(locale) + " (" + locale.getDisplayName(T2Map.LOCALE) + ")";
        }
    }

    private void savePluginDir()
    {
        Component[] comps = panel1.getComponents();
        String pluginDirs = "";
        int n = 0;
        for (int i = 0; i < comps.length; i++)
        {
            if (comps[i] instanceof PluginDirForm)
            {
                if (n > 0)
                {
                    pluginDirs = pluginDirs + ";";
                }
                pluginDirs = pluginDirs + ((PluginDirForm) comps[i]).getText();
                n++;
            }
        }
        T2Map.properties.put(PLUGIN_DIR, pluginDirs);
    }

    /**
     * ブラウザ実行ファイルのフォームを指定するクラスです。
     */
    private class BrowserFileForm extends JPanel
    {
        /**
         * シリアルバーションID
         */
        private static final long serialVersionUID = 1L;

        /**
         * ディレクトリを表示するJTextFieldオブジェクト
         */
        private JTextField pluginDir;

        /**
         * ディレクトリパスを取得します。
         * 
         * @return ディレクトリの絶対パス
         */
        public String getText()
        {
            return pluginDir.getText();
        }

        /**
         * プラグインディレクトリ変更用の画面を表示します。
         * 
         * @param dir ディレクトリパス
         */
        public BrowserFileForm(String dir)
        {
            setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

            pluginDir = new JTextField();
            pluginDir.setPreferredSize(new Dimension(200, pluginDir.getHeight()));
            add(pluginDir);

            if (dir != null && dir.length() > 0)
            {
                pluginDir.setText(dir);
            }

            JButton jButton1 = new JButton(rmgr.getString("reference.button"));
            jButton1.addActionListener(new AbstractAction()
            {
                private static final long serialVersionUID = 1L;

                public void actionPerformed(ActionEvent e)
                {
                    JFileChooser chooser = new JFileChooser(new File("."));
                    chooser.setFileFilter(new ExtensionFileFilter("exe"));
                    chooser.setAcceptAllFileFilterUsed(false);
                    chooser.setLocale(T2Map.LOCALE);
                    chooser.updateUI();
                    chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
                    int returnVal = chooser.showOpenDialog(GeneralPreferences.this);
                    if (returnVal == JFileChooser.APPROVE_OPTION)
                    {
                        pluginDir.setText(chooser.getSelectedFile().getAbsolutePath());
                    }
                }
            });
            add(jButton1);
        }
    }

    /**
     * プラグインディレクトリのフォームを指定するクラスです。
     */
    class PluginDirForm extends JPanel
    {
        /**
         * シリアルバーションID
         */
        private static final long serialVersionUID = 1L;

        /**
         * ディレクトリを表示するJTextFieldオブジェクト
         */
        private JTextField pluginDir;

        /**
         * ディレクトリパスを取得します。
         * 
         * @return ディレクトリの絶対パス
         */
        public String getText()
        {
            return pluginDir.getText();
        }

        /**
         * プラグインディレクトリ変更用の画面を表示します。
         * 
         * @param dir ディレクトリパス
         */
        public PluginDirForm(String dir)
        {
            setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
            JButton jButton0 = new JButton(rmgr.getString("minus.button"));
            jButton0.addActionListener(new AbstractAction()
            {
                private static final long serialVersionUID = 1L;

                public void actionPerformed(ActionEvent e)
                {
                    removePluginDir(PluginDirForm.this);
                }
            });
            add(jButton0);

            pluginDir = new JTextField();
            pluginDir.setPreferredSize(new Dimension(PARTS_WIDTH, pluginDir.getHeight()));
            add(pluginDir);

            if (dir != null && dir.length() > 0)
            {
                pluginDir.setText(dir);
            }

            JButton jButton1 = new JButton(rmgr.getString("reference.button"));
            jButton1.addActionListener(new AbstractAction()
            {
                private static final long serialVersionUID = 1L;

                public void actionPerformed(ActionEvent e)
                {
                    JFileChooser chooser = new JFileChooser(new File("."));
                    chooser.setLocale(T2Map.LOCALE);
                    chooser.updateUI();
                    chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                    int returnVal = chooser.showOpenDialog(GeneralPreferences.this);
                    if (returnVal == JFileChooser.APPROVE_OPTION)
                    {
                        pluginDir.setText(chooser.getSelectedFile().getAbsolutePath());
                    }
                }
            });
            add(jButton1);
        }
    }

    @Override
    public void initValue()
    {
        init();
    }
}
