/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.preferences;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.data.property.PropertiesIF;
import jp.nihu.gttime.plugin.LayerPlugin;
import jp.nihu.gttime.plugin.PluginResourceManager;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * メニューバーの「編集」→「設定...」の「一般」の参照ボタン押下時のダイアログを表示するクラスです。
 */
public class LayerPreferencesDialog extends JDialog
{
    /**
     * シリアルバーションID
     */
    private static final long serialVersionUID = 1L;

    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(LayerPreferencesDialog.class);

    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(LayerPreferencesDialog.class, T2Map.LOCALE);

    private BaseLayerPluginPreferences prep = null;

    private LayerPlugin layer = null;

    /**
     * ダイアログを表示します。
     */
    public LayerPreferencesDialog(LayerPlugin layer)
    {
        log.debug("Initialize PreferencesDialog.");
        this.layer = layer;
        BaseLayerPluginPreferences pref = layer.getLayerPluginPreferencesDialog();
        setLayerPluginPreferencesDialog(pref);
        setModal(true);
        setTitle(rmgr.getString("dialog.title"));

        Component[] comps = pref.getComponents();
        int h = 0;
        int w = 0;
        for (int i = 0; i < comps.length; i++)
        {
            Dimension d = comps[i].getPreferredSize();
            if (d.width > w)
            {
                w = d.width;
            }
//            h = h + d.height + 15;
            h = h + d.height;
        }
        w = w + 20;
        h = h + 50;
        pref.setPreferredSize(new Dimension(w, h));
        setSize(new Dimension(w + 40, 500));

        JPanel bottom = new JPanel();
        bottom.setLayout(new FlowLayout(FlowLayout.RIGHT));
        getContentPane().add("South", bottom);

        JButton button = new JButton();
        Action okAction = new AbstractAction()
        {
            private static final long serialVersionUID = 1L;

            public void actionPerformed(ActionEvent e)
            {
                PropertiesIF properties = LayerPreferencesDialog.this.layer.getDataset().getProperties();
                prep.saveForLayer(properties);
                properties.setChanged(true);
                store();
                refresh();
                LayerPreferencesDialog.this.setVisible(false);
                LayerPreferencesDialog.this.dispose();
            }
        };
        button.setText(rmgr.getString("ok.button"));
        button.addActionListener(okAction);
        bottom.add(button);

        button = new JButton();
        Action cancelAction = new AbstractAction()
        {
            private static final long serialVersionUID = 1L;

            public void actionPerformed(ActionEvent e)
            {
                LayerPreferencesDialog.this.setVisible(false);
                LayerPreferencesDialog.this.dispose();
            }
        };
        button.setText(rmgr.getString("cancel.button"));
        button.addActionListener(cancelAction);
        bottom.add(button);

        button = new JButton();
        Action applyAction = new AbstractAction()
        {
            private static final long serialVersionUID = 1L;

            public void actionPerformed(ActionEvent e)
            {
                setDefault();
                LayerPreferencesDialog.this.repaint();
                // repaint();
            }
        };
        button.setText(rmgr.getString("default.button"));
        button.addActionListener(applyAction);
        bottom.add(button);

        // tabbedPane = new JTabbedPane(JTabbedPane.LEFT);
        // getContentPane().add("Center", tabbedPane);

        // ダイアログの表示位置を設定。
        Toolkit tk = getToolkit();
        Dimension screenSize = tk.getScreenSize();
        int posX = (screenSize.width / 2) - (getWidth() / 2);
        int posY = (screenSize.height / 2) - (getHeight() / 2);

        // BaseLayerPluginPreferences base = new BaseLayerPluginPreferences();
        // tabbedPane.add(base);

        setLocation(posX, posY);
    }

    /**
     * T2Mapを設定します。
     * 
     * @param t2map T2Mapオブジェクト
     */
    public void setT2Map(T2Map t2map)
    {
        this.t2map = t2map;
    }

    /**
     * T2Mapオブジェクト
     */
    private T2Map t2map = null;

    /**
     * 再構築します。
     */
    private void refresh()
    {
        if (t2map != null && t2map.getTree() != null)
        {
            t2map.getTree().rebuildLayerList();
        }
    }

    /**
     * 適切なフォーマットで出力ストリームに書き込みます。
     */
    private void store()
    {
        layer.getDataset().applyStyleChanges(layer.getPluginName(), layer.getStyleName());
    }

    private void setLayerPluginPreferencesDialog(BaseLayerPluginPreferences prep)
    {
        this.prep = prep;
        JScrollPane sc = new JScrollPane(prep);
//        sc.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
//        sc.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        getContentPane().add(sc, BorderLayout.CENTER);
    }

    private void setDefault()
    {
        prep.setDefault(layer.getStyleName());
    }
}
