/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.preferences;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;

import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.plugin.PluginPreferences;
import jp.nihu.gttime.plugin.PluginResourceManager;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * メニューバーの「編集」→「設定...」の「一般」の参照ボタン押下時のダイアログを表示するクラスです。
 */
public class PreferencesDialog extends JDialog
{
    /**
     * シリアルバーションID
     */
    private static final long serialVersionUID = 1L;

    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(PreferencesDialog.class);

    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(PreferencesDialog.class, T2Map.LOCALE);

    /**
     * コンポーネント格納用の階層区画
     */
    private JTabbedPane tabbedPane;
    
    private ArrayList<PluginPreferences> pluginPreferences = new ArrayList<PluginPreferences>();

    /**
     * ダイアログを表示します。
     */
    public PreferencesDialog()
    {
    	pluginPreferences = new ArrayList<PluginPreferences>();
        log.debug("Initialize PreferencesDialog.");
        setModal(true);
        setTitle(rmgr.getString("dialog.title"));

        setSize(new Dimension(650, 520));

        JPanel bottom = new JPanel();
        bottom.setLayout(new FlowLayout(FlowLayout.RIGHT));
        getContentPane().add("South", bottom);

        JButton button = new JButton();
        Action okAction = new AbstractAction()
        {
            private static final long serialVersionUID = 1L;

            public void actionPerformed(ActionEvent e)
            {
                for (int i = 0; i < pluginPreferences.size(); i++)
                {
                	PluginPreferences p = pluginPreferences.get(i);
                	p.save();
                }
                store();
                refresh();
                PreferencesDialog.this.setVisible(false);
                PreferencesDialog.this.dispose();
            }
        };
        button.setText(rmgr.getString("ok.button"));
        button.addActionListener(okAction);
        bottom.add(button);

        button = new JButton();
        Action cancelAction = new AbstractAction()
        {
            private static final long serialVersionUID = 1L;

            public void actionPerformed(ActionEvent e)
            {
                PreferencesDialog.this.setVisible(false);
                PreferencesDialog.this.dispose();
            }
        };
        button.setText(rmgr.getString("cancel.button"));
        button.addActionListener(cancelAction);
        bottom.add(button);

        // button = new JButton();
        // Action applyAction = new AbstractAction()
        // {
        // private static final long serialVersionUID = 1L;
        //
        // public void actionPerformed(ActionEvent e)
        // {
        // Component[] components = tabbedPane.getComponents();
        // for (int i = 0; i < components.length; i++)
        // {
        // if (components[i] instanceof PluginPreferences)
        // {
        // ((PluginPreferences) components[i]).save();
        // }
        // }
        // store();
        // refresh();
        // }
        // };
        // button.setText(rmgr.getString("apply.button"));
        // button.addActionListener(applyAction);
        // bottom.add(button);

        tabbedPane = new JTabbedPane(JTabbedPane.LEFT);
        getContentPane().add("Center", tabbedPane);

        // ダイアログの表示位置を設定。
        Toolkit tk = getToolkit();
        Dimension screenSize = tk.getScreenSize();
        int posX = (screenSize.width / 2) - (getWidth() / 2);
        int posY = (screenSize.height / 2) - (getHeight() / 2);

        // BaseLayerPluginPreferences base = new BaseLayerPluginPreferences();
        // tabbedPane.add(base);

        setLocation(posX, posY);
    }

    /**
     * T2Mapを設定します。
     * 
     * @param t2map T2Mapオブジェクト
     */
    public void setT2Map(T2Map t2map)
    {
        this.t2map = t2map;
    }

    // /**
    // * T2Mapを取得します。
    // *
    // * @return T2Mapオブジェクト
    // */
    // public T2Map getT2Map()
    // {
    // return t2map;
    // }

    /**
     * T2Mapオブジェクト
     */
    private T2Map t2map = null;

    /**
     * 再構築します。
     */
    private void refresh()
    {
        if (t2map != null && t2map.getTree() != null)
        {
            t2map.getTree().rebuildLayerList();
        }
    }

    /**
     * 適切なフォーマットで出力ストリームに書き込みます。
     */
    private void store()
    {
        try
        {
            T2Map.properties.store();
        }
        catch (FileNotFoundException e)
        {
            e.printStackTrace();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
    }

    /**
     * プラグインディレクトリを追加します。
     * 
     * @param panel パネルオブジェクト
     */
    public void addPreference(PluginPreferences panel)
    {
    	pluginPreferences.add(panel);
        if (panel instanceof BaseLayerPluginPreferences && !panel.getName().endsWith(rmgr.getString("label.default")))
        {
            panel.setName(panel.getName() + rmgr.getString("label.default"));
        }
//        tabbedPane.add(panel);
        
        //panel.setSize(200, panel.getHeight());
        Component[] comps = panel.getComponents();
        int h = 0;
        int w = 0;
        for (int i = 0; i < comps.length; i++)
        {
            Dimension d = comps[i].getPreferredSize();
            if (d.width > w)
            {
                w = d.width;
            }
//            h = h + d.height + 15;
            h = h + d.height;
        }
        w = w + 20;
        h = h + 50;
        panel.setPreferredSize(new Dimension(w, h));
        
        JScrollPane sc = new JScrollPane(panel);
      sc.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
      sc.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
      //getContentPane().add(sc, BorderLayout.CENTER);
        tabbedPane.add(panel.getName(), sc);
        log.debug(panel.getLayout().toString());
    }
}
