/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.tree;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceDragEvent;
import java.awt.dnd.DragSourceDropEvent;
import java.awt.dnd.DragSourceEvent;
import java.awt.dnd.DragSourceListener;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Hashtable;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import jp.nihu.gttime.FileImportMenu;
import jp.nihu.gttime.FileSaveMenu;
import jp.nihu.gttime.T2Map;
import jp.nihu.gttime.T2MapMenuBar;
import jp.nihu.gttime.data.DatasetException;
import jp.nihu.gttime.data.DatasetIF;
import jp.nihu.gttime.data.ProjectException;
import jp.nihu.gttime.data.ProjectGts;
import jp.nihu.gttime.data.ProjectIF;
import jp.nihu.gttime.layer.LayerPopupMenu;
import jp.nihu.gttime.plugin.PluginManager;
import jp.nihu.gttime.plugin.PluginManagerException;
import jp.nihu.gttime.plugin.PluginResourceManager;
import jp.nihu.gttime.plugin.SavePlugin;
import jp.nihu.gttime.rdf.RDFSave;
import jp.nihu.gttime.time.TimeAxisIF;
import jp.nihu.gttime.time.utility.ISO8601Exception;
import jp.nihu.gttime.time.utility.JulianGregorianDate;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.semarglproject.rdf.ParseException;
import org.w3c.dom.Node;

/**
 * ツリー内のノードをドラック＆ドロップで移動させるための実装クラスである。
 */
public class TreeDragDrop extends JTree implements DragSourceListener, DropTargetListener, DragGestureListener, MouseListener
{// ルートノードをドラッグ＆ドロップできないようにする。
    /**
     * シリアルバーションID
     */
    private static final long serialVersionUID = 1L;

    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(TreeDragDrop.class);

    /**
     * リソースマネージャーオブジェクト
     */
    private static PluginResourceManager rmgr = PluginResourceManager.getManager(TreeDragDrop.class, T2Map.LOCALE);
    private static PluginResourceManager mbrmgr = PluginResourceManager.getManager(T2MapMenuBar.class, T2Map.LOCALE);

    /**
     * DataFlavorオブジェクト
     */
    private static DataFlavor flavorLocalObject;

    /**
     * DataFlavorを設定する
     */
    static
    {
        try
        {
            flavorLocalObject = new DataFlavor(DataFlavor.javaJVMLocalObjectMimeType);
        }
        catch (ClassNotFoundException e)
        {
            e.printStackTrace();
        }
    }

    /**
     * DataFlavor(データ転送用)
     */
    static DataFlavor[] supportFlavors = { flavorLocalObject };

    /**
     * DragSourceオブジェクト
     */
    private DragSource dragSrc = null;

    /**
     * TreeNodeオブジェクト
     */
    protected DefaultMutableTreeNode dropTargetNode = null;

    /**
     * TreeNodeオブジェクト
     */
    private DefaultMutableTreeNode draggedNode = null;

    /**
     * TreeNodeオブジェクト
     */
    private TreePath enterPath = null;

    /**
     * TreeViewPartオブジェクト
     */
    private TreeViewPart treeViewPart = null;

    /**
     * マウス押下時の動作を実装します。
     * 
     * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
     */
    public void mouseClicked(MouseEvent e)
    {
//    	TreePath path = getPathForLocation(e.getX(), e.getY());
//    	if (path != null) {
//    		TreeCheckNode node = (TreeCheckNode) path.getLastPathComponent();
//    		Object o = node.getUserObject();
//            if (o instanceof DatasetIF) {
//            	this.setSelectionPath(path);
//            }
//        }
    }

    /**
     * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
     */
    public void mousePressed(MouseEvent e)
    {
    }

    /**
     * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
     */
    public void mouseEntered(MouseEvent e)
    {
    }

    /**
     * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
     */
    public void mouseExited(MouseEvent e)
    {
    }

    /**
     * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
     */
    public void mouseReleased(MouseEvent e)
    {
        TreePath path = getPathForLocation(e.getX(), e.getY());
        if (path != null)
        {
            TreeCheckNode node = (TreeCheckNode) path.getLastPathComponent();
            if (SwingUtilities.isLeftMouseButton(e) && path != null)
            {
                if (node.isParentEnabled())
                {
                	Rectangle r = this.getPathBounds(path);
                    int m = e.getX()-r.x;
                    // boolean selected = !(node.isSelected());
                    if (3 < m && m < 17) {
                        node.setSelected(!node.isSelected());
                        treeViewPart.rebuildLayerList();
                        updateUI();
                    }
                }
            }
            else if (SwingUtilities.isRightMouseButton(e))
            {
                if (node.isParentEnabled())
                {
                    showPopup(e);
                }
            }
        }
    }

    /**
     * 右クリックで起動されるメニュー
     */
    private JPopupMenu getPopUpItem(final TreeCheckNode node)
    {
        // 名前変更機能、削除機能を追加する。
        JPopupMenu popup = new JPopupMenu();
        addRenameMenuItem(popup, node);
        
//        popup.add(new JMenuItem("【将来拡張】新規プロジェクト"));
//        popup.add(new JMenuItem("【将来拡張】切り取り"));
//        popup.add(new JMenuItem("【将来拡張】コピー"));
//        popup.add(new JMenuItem("【将来拡張】貼り付け"));
        
        addDeleteNodeMenuItem(popup, node);
        
        popup.addSeparator();
        
        final T2Map t2map = (T2Map) SwingUtilities.getRoot(this);
        
        JMenuItem item = new JMenuItem(rmgr.getString("menu.zoom.project"));
        item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				try {
					//log.debug("プロジェクト全体を表示");
					ProjectIF d = (ProjectIF)node.getUserObject();
					BigDecimal sd = d.getStartDatetime();
					BigDecimal ed = d.getEndDatetime();
					TimeAxisIF data = t2map.getTimeAxisData();
					data.setCurrentMin(sd);
	                data.setCurrentMax(ed);
	                data.fireValueChanged();
				} catch (ISO8601Exception e) {
					log.error(e.getMessage(), e);
				} catch (DatasetException e) {
					log.error(e.getMessage(), e);
				}
			}
		});
        popup.add(item);//【将来拡張】プロジェクト全体を表示
        
        popup.addSeparator();
        
        JMenu menux = new JMenu(rmgr.getString("menu.new"));
        item = new JMenuItem(rmgr.getString("menu.new.project"));
        item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				log.debug("New Project");
				// TODO: GET STARTDATE AND ENDDATE FROM CURRENT PROJECT
				try {
					BigDecimal sb = T2Map.getProject().getStartDatetime();
					String sartdate = JulianGregorianDate.toGregorian(sb, "Z");
					BigDecimal eb = T2Map.getProject().getEndDatetime();
					String enddate = JulianGregorianDate.toGregorian(eb, "Z");
					ProjectIF p = new ProjectGts(sartdate, enddate);
					((ProjectIF)node.getUserObject()).addProject(p);
				} catch (ISO8601Exception e) {
					log.error(e.getMessage(), e);
				} catch (DatasetException e) {
					log.error(e.getMessage(), e);
				} catch (ProjectException e) {
					log.error(e.getMessage(), e);
				}
				t2map.updateView();
			}
		});
        menux.add(item);
        popup.add(menux);
        
        FileImportMenu fim = new FileImportMenu(t2map, (ProjectIF)node.getUserObject(), false);
        popup.add(fim.getOpenImportMenu());
        popup.add(fim.getImportMenu());

        item = new JMenuItem(mbrmgr.getString("menu.file.save"));//"【将来拡張】上書き保存"
        final PluginManager pluginManager = T2Map.getPluginManager();
        ArrayList<Hashtable<String, String>> plugins = pluginManager.getPlugins("SavePlugin");
    	final String clazz = plugins.get(0).get("class");
    	item.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent e)
            {
            	log.debug(clazz);
                try
                {
                    SavePlugin plugin = (SavePlugin) pluginManager.newInstance(clazz);
                    ProjectIF p = (ProjectIF)node.getUserObject();
                    plugin.save(t2map, p);
                }
                catch (PluginManagerException ex)
                {
                	log.error(ex.getMessage(),ex);
                }
            }
        });
    	popup.add(item);
        
        FileSaveMenu fsm = new FileSaveMenu(t2map, (ProjectIF)node.getUserObject());
        JMenuItem menu = fsm.getSaveAsMenu();
        menu.setEnabled(true);
        popup.add(menu);
        
        JMenu outMenu = new JMenu(rmgr.getString("menu.data.export"));
        popup.add(outMenu);
        
        // RDF出力
        JMenuItem item4 = new JMenuItem(rmgr.getString("menu.rdf.output"));
        item4.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
            	try {
            		ProjectIF p = (ProjectIF)node.getUserObject();
//					RDFDataMgr.write(System.out, p.getModel(), Lang.TURTLE);
					RDFSave rdf = new RDFSave(t2map);
            		rdf.save(p.getModel(), p.getTitle());
				} catch (ParseException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				};
            }
        });
        outMenu.add(item4);
        
        return popup;
    }

    public void addRenameMenuItem(JPopupMenu popup, final TreeCheckNode node)
    {
        JMenuItem item = new JMenuItem(rmgr.getString("menu.change.name"));
        item.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                renameNode(node);
            }
        });
        popup.add(item);
    }

    public void addDeleteNodeMenuItem(JPopupMenu popup, final TreeCheckNode node)
    {
        if (node.getParent() != null)
        {
            JMenuItem item = new JMenuItem(rmgr.getString("menu.delete"));
            item.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    deleteNode(node);
                }
            });
            popup.add(item);
        }
    }

    /**
     * ノードを削除します。
     */
    private void deleteNode(TreeCheckNode node)
    {
        Object userObject = node.getUserObject();
        DefaultMutableTreeNode dmt = (DefaultMutableTreeNode) node.getParent();
        if (dmt != null)
        {
            Object parentUserObject = dmt.getUserObject();
            if (parentUserObject instanceof ProjectIF)
            {
                ProjectIF parentProject = (ProjectIF) parentUserObject;
                if (userObject instanceof ProjectIF)
                {
                    ProjectIF project = (ProjectIF) userObject;
                    if (doDelete(project.getTitle()))
                    {
                        parentProject.removeProject(parentProject.removeProject(project));
                        // node.setSelected(false);
                        ((DefaultTreeModel) getModel()).removeNodeFromParent(node);
                    }
                }
                else if (userObject instanceof DatasetIF)
                {
                    DatasetIF dataset = (DatasetIF) userObject;
                    if (doDelete(dataset.getTitle()))
                    {
                        parentProject.removeDataset(parentProject.removeDataset(dataset));
                        // node.setSelected(false);
                        ((DefaultTreeModel) getModel()).removeNodeFromParent(node);
                    }
                }
                else
                {
                    log.debug("userObject Error!!!!!!!!!!!!!!!!!");
                }
            }
            else
            {
                log.debug("parentUserObject Error!!!!!!!!!!!!!!!!!");
            }
            node = null;
            treeViewPart.rebuildLayerList();
//            updateUI();
        }
    }

    private boolean doDelete(String title)
    {
        int res = JOptionPane.showConfirmDialog(null, rmgr.format("menu.delete.message", title), rmgr.getString("menu.delete"), JOptionPane.YES_NO_OPTION);
        if (res == JOptionPane.YES_OPTION)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * ノードの名前を変更します。
     */
    private void renameNode(TreeCheckNode node)
    {
        Object o = node.getUserObject();
        if (o instanceof DatasetIF)
        {
            DatasetIF d = (DatasetIF) o;
            JTextField textField = new JTextField(d.getTitle());
            int res = showChangeNameDialog(textField);
            if (res == JOptionPane.YES_OPTION)
            {
                d.setTitle(textField.getText());
                treeViewPart.rebuildLayerList();
                updateUI();
            }
        }
        else if (o instanceof ProjectIF)
        {
            ProjectIF p = (ProjectIF) o;
            JTextField textField = new JTextField(p.getTitle());
            int res = showChangeNameDialog(textField);
            if (res == JOptionPane.YES_OPTION)
            {
                p.setTitle(textField.getText());
                treeViewPart.rebuildLayerList();
                updateUI();
            }
        }
    }

    private int showChangeNameDialog(JTextField textField)
    {
        return JOptionPane.showConfirmDialog(null, textField, rmgr.getString("menu.change.name"), JOptionPane.YES_NO_OPTION, JOptionPane.PLAIN_MESSAGE, null);
    }

    /**
     * メニューを表示
     */
    private void showPopup(MouseEvent e)
    {
        if (e.isPopupTrigger())
        {
            int row = getRowForLocation(e.getX(), e.getY());
            TreePath path = getPathForRow(row);
            if (path != null)
            {
                //

                // TreeCheckNode node = (TreeCheckNode)
                // path.getLastPathComponent();
                // LayerPopupMenu menu = node.getLayer().getMenu();
                // menu.addSeparator();
                // addRenameMenuItem(menu, node);
                // addDeleteNodeMenuItem(menu, node);
                // menu.show(e.getComponent(), e.getX(), e.getY());

                TreeCheckNode node = (TreeCheckNode) path.getLastPathComponent();
                if (node.getLayer() != null)
                {
                    LayerPopupMenu menu = node.getLayer().getMenu();
                    menu.fromTree = true;
                    menu.show(e.getComponent(), e.getX(), e.getY());
                }
                else
                {
                    getPopUpItem((TreeCheckNode) path.getLastPathComponent()).show(e.getComponent(), e.getX(), e.getY());
                }

            }
        }
    }

    /**
     * コンストラクター
     * 
     * @param treeViewPart TreeViewPartオブジェクト
     */
    public TreeDragDrop(final TreeViewPart treeViewPart)
    {
        super();
        TreeCheckRenderer renderer = new TreeCheckRenderer(this);
        setCellRenderer(renderer);
        this.setRowHeight(20);
        this.treeViewPart = treeViewPart;
        dragSrc = new DragSource();
        dragSrc.createDefaultDragGestureRecognizer(this, DnDConstants.ACTION_MOVE, this);
        new DropTarget(this, this);
        addMouseListener(this);
        setToggleClickCount(0);
    }

    /**
     * ドロップ先のノードを取得します。
     * 
     * @return ドロップ先のノード
     */
    public TreeNode getDropTargetNode()
    {
        return dropTargetNode;
    }

    /**
     * ドラッグ開始ジェスチャーを検出します。
     * 
     * @see java.awt.dnd.DragGestureListener#dragGestureRecognized(java.awt.dnd.DragGestureEvent)
     */
    public void dragGestureRecognized(DragGestureEvent e)
    {// DragGestureListener
        Point clickPoint = e.getDragOrigin();
        TreePath path = getPathForLocation(clickPoint.x, clickPoint.y);
        if (path == null)
        {
            log.debug("not on a node");
            return;
        }
        draggedNode = (DefaultMutableTreeNode) path.getLastPathComponent();
        Transferable trans = new DataTransferable(draggedNode);
        dragSrc.startDrag(e, Cursor.getDefaultCursor(), trans, this);
    }

    /**
     * ドラッグ＆ドロップ操作が完了時に呼び出されます。
     * 
     * @see java.awt.dnd.DragSourceListener#dragDropEnd(java.awt.dnd.DragSourceDropEvent)
     */
    public void dragDropEnd(DragSourceDropEvent e)
    {// DragSourceListener events
        dropTargetNode = null;
        draggedNode = null;
        repaint();
    }

    /**
     * 操作可能な部分に入る
     * 
     * @see java.awt.dnd.DragSourceListener#dragEnter(java.awt.dnd.DragSourceDragEvent)
     */
    public void dragEnter(DragSourceDragEvent e)
    {
    }

    /**
     * ドラッグが拒否された
     * 
     * @see java.awt.dnd.DragSourceListener#dragExit(java.awt.dnd.DragSourceEvent)
     */
    public void dragExit(DragSourceEvent dse)
    {
    }

    /**
     * dragEnter()の呼び出しに関連したドロップサイトの操作可能な部分とまだ交差している
     * 
     * @see java.awt.dnd.DragSourceListener#dragOver(java.awt.dnd.DragSourceDragEvent)
     */
    public void dragOver(DragSourceDragEvent e)
    {
    }

    /**
     * ユーザが使用している入力デバイスの状態が変更になった場合
     * 
     * @see java.awt.dnd.DragSourceListener#dropActionChanged(java.awt.dnd.DragSourceDragEvent)
     */
    public void dropActionChanged(DragSourceDragEvent e)
    {
    }

    /**
     * DropTargetListener events(ドロップを処理するため)
     * 現在のドラッグ操作に関連するデータは、getTransferable() を呼び出すことによって取得できる。 ドラッグ開始
     * 
     * @see java.awt.dnd.DropTargetListener#dragEnter(java.awt.dnd.DropTargetDragEvent)
     */
    public void dragEnter(DropTargetDragEvent e)
    {
        Point dropPoint = e.getLocation();
        enterPath = getPathForLocation(dropPoint.x, dropPoint.y);
        log.debug("drag path is " + enterPath);
        e.acceptDrag(DnDConstants.ACTION_COPY_OR_MOVE);
    }

    /**
     * 操作可能な部分からマウスポインタが出たとき
     * 
     * @see java.awt.dnd.DropTargetListener#dragExit(java.awt.dnd.DropTargetEvent)
     */
    public void dragExit(DropTargetEvent dte)
    {
    }

    /**
     * 操作可能な部分にマウスポインタがまだあるとき
     * 
     * @see java.awt.dnd.DropTargetListener#dragOver(java.awt.dnd.DropTargetDragEvent)
     */
    public void dragOver(DropTargetDragEvent e)
    {
        Point dragPoint = e.getLocation();
        TreePath path = getPathForLocation(dragPoint.x, dragPoint.y);
        if (path == null)
        {
            dropTargetNode = null;
        }
        else
        {
            dropTargetNode = (DefaultMutableTreeNode) path.getLastPathComponent();
        }

        // プロジェクトをデータセットの間に移動する場合はここをコメントアウトする。
        if (draggedNode != null)
        {
//            Object draggedUserObject = draggedNode.getUserObject();
//            if (dropTargetNode != null)
//            {
//                Object targetUserObject = dropTargetNode.getUserObject();
//                if (targetUserObject instanceof DatasetIF && draggedUserObject instanceof ProjectIF)
//                {
//                    dropTargetNode = null;
//                }
//            }
            // プロジェクトをデータセットの間に移動する場合はここをコメントアウトする。
            repaint();
        }
    }

    /**
     * ドロップでドラッグ操作が終了したとき
     * 
     * @see java.awt.dnd.DropTargetListener#drop(java.awt.dnd.DropTargetDropEvent)
     */
    public void drop(DropTargetDropEvent e)
    {
        Point dropPoint = e.getLocation();
        TreePath dropPath = getPathForLocation(dropPoint.x, dropPoint.y);
        log.debug("drop path is " + dropPath);
        if (dropPath != null && enterPath != null && !enterPath.isDescendant(dropPath))
        {
            boolean dropped = false;
            try
            {
                e.acceptDrop(DnDConstants.ACTION_MOVE);
                DefaultMutableTreeNode droppedNode = null;
                Node movedNode = null;
                Object userObject = null;

                droppedNode = (DefaultMutableTreeNode) enterPath.getLastPathComponent();

                // Project及びProjectのDOMからdroppedNodeに相当するProject又はDatasetを削除する。
                userObject = droppedNode.getUserObject();
                Object parentUserObject = ((DefaultMutableTreeNode) droppedNode.getParent()).getUserObject();
                if (parentUserObject instanceof ProjectIF)
                {
                    ProjectIF parentProject = (ProjectIF) parentUserObject;
                    if (userObject instanceof ProjectIF)
                    {
//                        if (!getDropTargetNode().isLeaf())
//                        {
                            ProjectIF project = (ProjectIF) userObject;
                            movedNode = parentProject.removeProject(project);
                            ((DefaultTreeModel) getModel()).removeNodeFromParent(droppedNode);
//                        }
                    }
                    else if (userObject instanceof DatasetIF)
                    {
                        DatasetIF dataset = (DatasetIF) userObject;
                        movedNode = parentProject.removeDataset(dataset);
                        ((DefaultTreeModel) getModel()).removeNodeFromParent(droppedNode);
                    }
                    else
                    {
                        log.debug("userObject Error!!!!!!!!!!!!!!!!!");
                    }
                }
                else
                {
                    log.debug("parentUserObject Error!!!!!!!!!!!!!!!!!");
                }

                DefaultMutableTreeNode parent = null;
                if (getDropTargetNode().isLeaf())
                {
                    parent = (DefaultMutableTreeNode) getDropTargetNode().getParent();
                    int index = parent.getIndex(getDropTargetNode());
                    if (userObject instanceof DatasetIF)
                    {
                        ((ProjectIF) parent.getUserObject()).moveDataset((DatasetIF) userObject, movedNode, index);
                        ((DefaultTreeModel) getModel()).insertNodeInto(droppedNode, parent, index);
                    }
                    else if (userObject instanceof ProjectIF)
                    {
                        ((ProjectIF)parent.getUserObject()).moveProject((ProjectIF)userObject, movedNode, index);
                        ((DefaultTreeModel)getModel()).insertNodeInto(droppedNode, parent, index);
                    }
                }
                else
                {
                    parent = (DefaultMutableTreeNode) getDropTargetNode();
                    int childCount = parent.getChildCount();
                    int index = childCount;
                    if (userObject instanceof DatasetIF)
                    {
//                        for (int i = 0; i < childCount; i++)
//                        {
//                            DefaultMutableTreeNode cnode = (DefaultMutableTreeNode) parent.getChildAt(i);
//                            Object cObject = cnode.getUserObject();
//                            if (cObject instanceof ProjectIF)
//                            {
//                                index = i;
//                                break;
//                            }
//                        }
                        ((ProjectIF) parent.getUserObject()).moveDataset((DatasetIF) userObject, movedNode, index);
                    }
                    else if (userObject instanceof ProjectIF)
                    {
                        ((ProjectIF) parent.getUserObject()).moveProject((ProjectIF) userObject, movedNode, index);
                    }
                    ((DefaultTreeModel) getModel()).insertNodeInto(droppedNode, parent, index);
                }

                // Project及びProjectのDOMにdroppedNodeに相当するProject又はDatasetを挿入する。
                // if (userObject instanceof Project)
                // {
                // ((Project) parent.getUserObject()).insertProject((Project)
                // userObject, movedNode, index);
                // }
                // else if (userObject instanceof Dataset)
                // {
                // ((Project) parent.getUserObject()).insertDataset((Dataset)
                // userObject, movedNode, index);
                // }
                // else
                // {
                // log.debug("userObject Error!!!!!!!!!!!!!!!!!");
                // }

                dropped = true;
            }
            catch (Exception err)
            {
                err.printStackTrace();
            }
            e.dropComplete(dropped);

            // ノードの移動が完了したらレイヤの再構築
            treeViewPart.rebuildLayerList();
            log.debug("Tree Move Complete");
        }
    }

    /**
     * ドロップジェスチャーを変更した場合に呼び出される
     * 
     * @see java.awt.dnd.DropTargetListener#dropActionChanged(java.awt.dnd.DropTargetDragEvent)
     */
    public void dropActionChanged(DropTargetDragEvent e)
    {
    }

    /**
     * 転送操作のためのデータを提供する
     */
    static class DataTransferable implements Transferable
    {
        /**
         * Object
         */
        Object object;

        /**
         * Objectを設定します。
         * 
         * @param o Object
         */
        public DataTransferable(Object o)
        {
            object = o;
        }

        /**
         * Objectを取得します。
         * 
         * @param df DataFlavorオブジェクト
         */
        public Object getTransferData(DataFlavor df) throws UnsupportedFlavorException, IOException
        {
            if (isDataFlavorSupported(df))
                return object;
            else
                throw new UnsupportedFlavorException(df);
        }

        /**
         * DataFlavorオブジェクトが設定されているものと等しいか判断します。
         * 
         * @param df DataFlavorオブジェクト
         */
        public boolean isDataFlavorSupported(DataFlavor df)
        {
            return (df.equals(flavorLocalObject));
        }

        /**
         * DataFlavor[]を取得します。
         * 
         * @return DataFlavor[]
         */
        public DataFlavor[] getTransferDataFlavors()
        {
            return supportFlavors;
        }
    }
}
