/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.tree;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;

import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.UIManager;
import javax.swing.plaf.ColorUIResource;

/**
 * ツリー内のノードをドラック＆ドロップで移動させるための実装クラスです。
 */
public class TreeDraw extends JLabel
{

    /**
     * シリアルバーションID
     */
    private static final long serialVersionUID = 1L;

    /**
     * 選択状態
     */
    private boolean isSelected;

    /**
     * フォーカス（色彩変化）を当てる
     */
    private boolean hasFocus;

    /**
     * 背景色を設定する
     * 
     * @param color 背景色
     */
    public void setBackground(Color color)
    {
        if (color instanceof ColorUIResource)
            color = null;
        super.setBackground(color);
    }

    /**
     * 描画する
     * 
     * @param g
     */
    public void paint(Graphics g)
    {
        String str;
        if ((str = getText()) != null)
        {
            if (0 < str.length())
            {
                if (isSelected)
                {
                    g.setColor(UIManager.getColor("Tree.selectionBackground"));
                }
                else
                {
                    g.setColor(UIManager.getColor("Tree.textBackground"));
                }
                Dimension d = getPreferredSize();
                int imageOffset = 0;
                Icon currentI = getIcon();
                if (currentI != null)
                {
                    imageOffset = currentI.getIconWidth() + Math.max(0, getIconTextGap() - 1);
                }
                g.fillRect(imageOffset, 0, d.width - 1 - imageOffset, d.height);
                if (hasFocus)
                {
                    g.setColor(UIManager.getColor("Tree.selectionBorderColor"));
                    g.drawRect(imageOffset, 0, d.width - 1 - imageOffset, d.height - 1);
                }
            }
        }
        super.paint(g);
    }

    /**
     * オブジェクトのサイズを取得する。
     * 
     * @see javax.swing.JComponent#getPreferredSize()
     */
    public Dimension getPreferredSize()
    {
        Dimension retDimension = super.getPreferredSize();
        if (retDimension != null)
        {
            retDimension = new Dimension(retDimension.width + 3, retDimension.height);
        }
        return retDimension;
    }

    /**
     * 選択状態を設定する
     * 
     * @param isSelected 選択状態
     */
    public void setSelected(boolean isSelected)
    {
        this.isSelected = isSelected;
    }

    /**
     * フォーカスを設定する。
     * 
     * @param hasFocus フォーカスを設定
     */
    public void setFocus(boolean hasFocus)
    {
        this.hasFocus = hasFocus;
    }
}