/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.utility;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.filechooser.FileFilter;

public class ExtensionFileFilter extends FileFilter
{

    private List<String> extList = new ArrayList<String>();

    private String description;

    public ExtensionFileFilter(String extension)
    {
        this(extension, null);
    }

    public ExtensionFileFilter(String[] extensions)
    {
        this(extensions, null);
    }

    public ExtensionFileFilter(String extension, String description)
    {
        String[] extensions = { extension };
        init(extensions, description);
    }

    public ExtensionFileFilter(String[] extensions, String description)
    {
        init(extensions, description);
    }

    private void init(String[] extensions, String description)
    {
        for (String ext : extensions)
        {
            extList.add(ext);
        }
        this.description = description;
    }

    @Override
    public boolean accept(File file)
    {
        boolean b = false;
        if (file.isDirectory())
        {
            b = true;
        }
        else
        {
            String fileExt = getFileExtension(file);
            if (fileExt != null)
            {
                for (String ext : extList)
                {
                    if (b = fileExt.equalsIgnoreCase(ext))
                    {
                        break;
                    }
                }
            }
        }
        return b;
    }

    @Override
    public String getDescription()
    {
        String s = description;
        if (s == null)
        {
            s = extList.get(0);
        }
        return s;
    }
    
    public String getExt() {
    	return extList.get(0);
    }

    private String getFileExtension(File file)
    {
        String s = null;
        if (file != null)
        {
            String name = file.getName();
            int i = name.lastIndexOf('.');
            if (i > 0)
            {
                s = name.substring(i + 1);
            }
        }
        return s;
    }
}
