/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.utility;

import java.awt.Color;
import java.awt.Font;
import java.io.IOException;
import java.util.Iterator;
import java.util.Map;

import jp.nihu.gttime.data.DatasetException;
import jp.nihu.gttime.data.DatasetIF;
import jp.nihu.gttime.data.property.LineIF;
import jp.nihu.gttime.data.property.PolygonIF;
import jp.nihu.gttime.data.property.PropertiesIF;
import jp.nihu.gttime.data.property.SymbolIF;
import jp.nihu.gttime.time.utility.JulianGregorianDate;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class GTM
{
    /**
     * ログ用オブジェクト
     */
    private static Log log = LogFactory.getLog(GTM.class);
    
    public static String getVersion() {
    	return "1.1";
    }

    /**
     * 現在のデータセットの状態をtmmファイルに記入する。
     * 
     * @param title データセット名
     * @param layertype レイヤタイプ
     * @param dataset Datasetオブジェクト
     * @return Documentオブジェクト
     */
    public static Document create(String title, String layertype, DatasetIF dataset, String format, String fileName)
    {
        String sDate = null;
        String eDate = null;
        Document domTmm = null;
        try
        {
            sDate = JulianGregorianDate.toGregorian(dataset.getStartDatetime(), "Z");
            eDate = JulianGregorianDate.toGregorian(dataset.getEndDatetime(), "Z");

            Document mdom = dataset.getDomDataset();

            Node n = mdom.getElementsByTagNameNS("http://www.h-gis.org/hgis", "title").item(0);
            n.setTextContent(title);
            n = mdom.getElementsByTagNameNS("http://www.h-gis.org/gt", "detaDefinition").item(0);
            n.setTextContent(format);
            n = mdom.getElementsByTagNameNS("http://www.h-gis.org/gt", "begin").item(0);
            n.setTextContent(sDate);
            n = mdom.getElementsByTagNameNS("http://www.h-gis.org/gt", "end").item(0);
            n.setTextContent(eDate);
            n = mdom.getElementsByTagNameNS("http://www.h-gis.org/gt", "filename").item(0);
            n.setTextContent(fileName);
            n = mdom.getElementsByTagNameNS("http://www.h-gis.org/hgis", "layerType").item(0);
            if (n != null)
            {
                n.setTextContent(layertype);
            }
            else
            {
//                String str = Utilities.toXmlString(mdom);
//                throw new DatasetException("No layerType Element: " + str);
            	log.warn("No layerType Element: " + Utilities.toXmlString(mdom));
            }

            domTmm = mdom;
        }
        catch (DatasetException e)
        {
            log.fatal(e.getMessage(), e);
        }
        catch (IOException e)
        {
            log.fatal(e.getMessage(), e);
        }
        return domTmm;
    }
    
    
    public static Document createForSaveAs(String title, String layertype, DatasetIF dataset, String format, String fileName)
    {
        String sDate = null;
        String eDate = null;
        Document domTmm = null;
        try
        {
            sDate = JulianGregorianDate.toGregorian(dataset.getStartDatetime(), "Z");
            eDate = JulianGregorianDate.toGregorian(dataset.getEndDatetime(), "Z");

            Document mdom = dataset.getDomDataset();

            Node n = mdom.getElementsByTagNameNS("http://www.h-gis.org/hgis", "title").item(0);
            n.setTextContent(title);
            n = mdom.getElementsByTagNameNS("http://www.h-gis.org/gt", "detaDefinition").item(0);
            n.setTextContent(format);
            n = mdom.getElementsByTagNameNS("http://www.h-gis.org/gt", "begin").item(0);
            n.setTextContent(sDate);
            n = mdom.getElementsByTagNameNS("http://www.h-gis.org/gt", "end").item(0);
            n.setTextContent(eDate);
            n = mdom.getElementsByTagNameNS("http://www.h-gis.org/gt", "filename").item(0);
            n.setTextContent(fileName);
//            n = mdom.getElementsByTagNameNS("http://www.h-gis.org/hgis", "layerType").item(0);
//            if (n != null)
//            {
//                n.setTextContent(layertype);
//            }
//            else
//            {
//            	log.warn("No layerType Element: " + Utilities.toXmlString(mdom));
//            }
            
            Node ocn = mdom.getElementsByTagNameNS("http://www.h-gis.org/hgis", "time").item(0);
            n = mdom.getElementsByTagNameNS("http://www.h-gis.org/hgis", "hgisApplications").item(0);
            n.replaceChild(getHgisTimeXML(mdom, dataset), ocn);

            domTmm = mdom;
        }
        catch (DatasetException e)
        {
            log.fatal(e.getMessage(), e);
        }
//        catch (IOException e)
//        {
//            log.fatal(e.getMessage(), e);
//        }
        return domTmm;
    }
    
    private static Node getHgisTimeXML(Document mdom, DatasetIF ds)
    {
    	Element time = mdom.createElementNS("http://www.h-gis.org/hgis", "time");
    	time.setAttribute("version", GTM.getVersion());
    	
    	Element layerType = mdom.createElementNS("http://www.h-gis.org/hgis", "layerType");
    	layerType.setTextContent(ds.getLayerType());
    	time.appendChild(layerType);
    	
    	
    	
//        StringBuffer sb = new StringBuffer();

        PropertiesIF properties = ds.getProperties();

//        sb.append("<hgis:time version=\""+GTM.getVersion()+"\" xmlns:hgis=\"http://www.h-gis.org/hgis\">\n");
//        sb.append("<hgis:layerType>").append(ds.getLayerType()).append("</hgis:layerType>\n");
        if (properties != null)
        {
            Map<String, LineIF> lines = properties.getLines();
            if (lines != null)
            {
                Iterator<String> it = lines.keySet().iterator();
                while (it.hasNext()) {
                	Element line = mdom.createElementNS("http://www.h-gis.org/hgis", "line");
                	String group = it.next();
                	if (!group.equals("")) {
                    	line.setAttribute("group", group);
                    }
                	
                	LineIF l = properties.getLine(group);
                	
                	Element thin = mdom.createElementNS("http://www.h-gis.org/hgis", "thin");
                	thin.setTextContent(Integer.toString(l.getThin()));
                    line.appendChild(thin);
                    
                    Element lineColor = mdom.createElementNS("http://www.h-gis.org/hgis", "lineColor");
                    lineColor.setTextContent(Utilities.getColor(l.getLineColor()));
                    line.appendChild(lineColor);
                    time.appendChild(line);
                    
//                    String group = it.next();
//                    sb.append("<hgis:line");
//                    if (!group.equals("")) {
//                        sb.append(" group=\"");
//                        sb.append(group);
//                        sb.append("\"");
//                    }
//                    sb.append(">\n");
//                    sb.append("<hgis:thin>1</hgis:thin>\n");
//                    sb.append("<hgis:lineColor>0,0,0,0</hgis:lineColor>\n");
//                    sb.append("</hgis:line>\n");
                }
            }
            Map<String, PolygonIF> polygons = properties.getPolygons();
            if (polygons != null)
            {
                Iterator<String> it = polygons.keySet().iterator();
                while (it.hasNext()) {
                	Element polygon = mdom.createElementNS("http://www.h-gis.org/hgis", "polygon");
                	String group = it.next();
                	if (!group.equals("")) {
                		polygon.setAttribute("group", group);
                    }
                	
                	PolygonIF p = properties.getPolygon(group);
                	
                	Element fillColor = mdom.createElementNS("http://www.h-gis.org/hgis", "fillColor");
                	fillColor.setTextContent(Utilities.getColor(p.getFillColor()));
                    polygon.appendChild(fillColor);
                    
                	Element thin = mdom.createElementNS("http://www.h-gis.org/hgis", "thin");
                	thin.setTextContent(Integer.toString(p.getThin()));
                    polygon.appendChild(thin);
                    
                    Element lineColor = mdom.createElementNS("http://www.h-gis.org/hgis", "lineColor");
                    lineColor.setTextContent(Utilities.getColor(p.getLineColor()));
                    polygon.appendChild(lineColor);
                    time.appendChild(polygon);
                	
//                    String group = it.next();
//                    sb.append("<hgis:polygon");
//                    if (!group.equals("")) {
//                        sb.append(" group=\"");
//                        sb.append(group);
//                        sb.append("\"");
//                    }
//                    sb.append(">\n");
//                    sb.append("<hgis:fillColor>255,200,0,0</hgis:fillColor>\n");
//                    sb.append("<hgis:thin>1</hgis:thin>\n");
//                    sb.append("<hgis:lineColor>0,0,0,0</hgis:lineColor>\n");
//                    sb.append("</hgis:polygon>\n");
                }
            }
            Map<String, SymbolIF> symbols = properties.getSymbols();
            if (symbols != null)
            {
                Iterator<String> it = symbols.keySet().iterator();
                while (it.hasNext()) {
                	Element symbol = mdom.createElementNS("http://www.h-gis.org/hgis", "symbol");
                	String group = it.next();
                	if (!group.equals("")) {
                		symbol.setAttribute("group", group);
                    }
                	
                	SymbolIF s = properties.getSymbol(group);
                	
                	Element symbola = mdom.createElementNS("http://www.h-gis.org/hgis", "symbol");
                	symbola.setTextContent(s.getSymbol());
                	symbol.appendChild(symbola);
                	
                	Element size = mdom.createElementNS("http://www.h-gis.org/hgis", "size");
                	size.setTextContent(Integer.toString(s.getSize()));
                	symbol.appendChild(size);
                	
                	Element fillColor = mdom.createElementNS("http://www.h-gis.org/hgis", "fillColor");
                	fillColor.setTextContent(Utilities.getColor(s.getFillColor()));
                	symbol.appendChild(fillColor);
                    
                	Element thin = mdom.createElementNS("http://www.h-gis.org/hgis", "thin");
                	thin.setTextContent(Integer.toString(s.getThin()));
                	symbol.appendChild(thin);
                    
                    Element lineColor = mdom.createElementNS("http://www.h-gis.org/hgis", "lineColor");
                    lineColor.setTextContent(Utilities.getColor(s.getLineColor()));
                    symbol.appendChild(lineColor);
                    time.appendChild(symbol);
                    
//                    String group = it.next();
//                    sb.append("<hgis:symbol");
//                        if (!group.equals("")) {
//                            sb.append(" group=\"");
//                            sb.append(group);
//                            sb.append("\"");
//                        }
//                        sb.append(">\n");
//                    sb.append("<hgis:symbol>Circle</hgis:symbol>\n");
//                    sb.append("<hgis:size>1</hgis:size>\n");
//                    sb.append("<hgis:fillColor>0,0,0,0</hgis:fillColor>\n");
//                    sb.append("<hgis:thin>1</hgis:thin>\n");
//                    sb.append("<hgis:lineColor>0,0,0,0</hgis:lineColor>\n");
//                    sb.append("</hgis:symbol>\n");
                }
            }
            Map<String, Font> fonts = properties.getFonts();
            if (fonts != null)
            {
                Iterator<String> it = fonts.keySet().iterator();
                while (it.hasNext()) {
                	Element font = mdom.createElementNS("http://www.h-gis.org/hgis", "font");
                	String group = it.next();
                	if (!group.equals("")) {
                		font.setAttribute("group", group);
                    }
                	
                	Font f = properties.getFont(group);
                	if (f != null) {
                    	
                    	Element fontFamily = mdom.createElementNS("http://www.h-gis.org/hgis", "font-family");
                    	fontFamily.setTextContent(f.getFamily());
                    	font.appendChild(fontFamily);
                    	
                    	Element fontName = mdom.createElementNS("http://www.h-gis.org/hgis", "font-name");
                    	fontName.setTextContent(f.getName());
                    	font.appendChild(fontName);
                    	
                    	Element fontSize = mdom.createElementNS("http://www.h-gis.org/hgis", "font-size");
                    	fontSize.setTextContent(f.getSize() + "pt");
                    	font.appendChild(fontSize);
                        
                    	if (properties.getFontColor(group) != null) {
    	                	Element fontColor = mdom.createElementNS("http://www.h-gis.org/hgis", "font-color");
    	                	fontColor.setTextContent(Utilities.getColor(properties.getFontColor(group)));
    	                	font.appendChild(fontColor);
                    	}
                        
                        Element fontStyle = mdom.createElementNS("http://www.h-gis.org/hgis", "font-style");
                        fontStyle.setTextContent(Utilities.getFontStyle(f.getStyle()));
                        font.appendChild(fontStyle);
                        time.appendChild(font);
                        
                	}
                	
//                    String group = it.next();
//                    sb.append("<hgis:font");
//                    if (!group.equals("")) {
//                        sb.append(" group=\"");
//                        sb.append(group);
//                        sb.append("\"");
//                    }
//                    sb.append(">\n");
//                    sb.append("<hgis:font-family>sans-serif</hgis:font-family>\n");
//                    sb.append("<hgis:font-name>MS-ゴシック</hgis:font-name>\n");
//                    sb.append("<hgis:font-size>10pt</hgis:font-size>\n");
//                    if (properties.getFontColor(group) != null)
//                    {
//                        sb.append("<hgis:font-color>0,0,0,0</hgis:font-color>\n");
//                    }
//                    sb.append("<hgis:font-style>normal</hgis:font-style>\n");
//                    sb.append("</hgis:font>\n");
                }
            }
            if (properties.getSelectedColor() != null)
            {
            	Element selectedColor = mdom.createElementNS("http://www.h-gis.org/hgis", "selectedColor");
            	selectedColor.setTextContent(Utilities.getColor(properties.getSelectedColor()));
                time.appendChild(selectedColor);
                
//                sb.append("<hgis:selectedColor>255,0,0,0</hgis:selectedColor>\n");
            }
            
            
            
            if (properties.getBackgroundColor() != null)
            {
            	Element backgroundColor = mdom.createElementNS("http://www.h-gis.org/hgis", "backgroundColor");
            	backgroundColor.setTextContent(Utilities.getColor(properties.getBackgroundColor()));
                time.appendChild(backgroundColor);
                
//                sb.append("<hgis:backgroundColor>255,0,0,0</hgis:backgroundColor>\n");
            }
            
            
            if (properties.getLayerHeight() != null && properties.getLayerHeight() > 70)
            {
            	Element layerHight = mdom.createElementNS("http://www.h-gis.org/hgis", "layerHight");
            	layerHight.setTextContent(properties.getLayerHeight().toString());
                time.appendChild(layerHight);
                
//                sb.append("<hgis:layerHight>").append(properties.getLayerHeight()).append("</hgis:layerHight>\n");
            }
            else
            {
            	Element layerHight = mdom.createElementNS("http://www.h-gis.org/hgis", "layerHight");
            	layerHight.setTextContent("100");
                time.appendChild(layerHight);
                
//                sb.append("<hgis:layerHight>100</hgis:layerHight>\n");
            }
            if (properties.getScaleMain() != null)
            {
            	Element scaleMain = mdom.createElementNS("http://www.h-gis.org/hgis", "scaleMain");
            	
            	Element fillColor = mdom.createElementNS("http://www.h-gis.org/hgis", "fillColor");
            	fillColor.setTextContent(Utilities.getColor(properties.getScaleMain().getFillColor()));
            	scaleMain.appendChild(fillColor);
                
            	Element thin = mdom.createElementNS("http://www.h-gis.org/hgis", "thin");
            	thin.setTextContent(Integer.toString(properties.getScaleMain().getThin()));
            	scaleMain.appendChild(thin);
                
                Element lineColor = mdom.createElementNS("http://www.h-gis.org/hgis", "lineColor");
                lineColor.setTextContent(Utilities.getColor(properties.getScaleMain().getLineColor()));
                scaleMain.appendChild(lineColor);
                
                time.appendChild(scaleMain);
            	
//                sb.append("<hgis:scaleMain>\n");
//                sb.append("<hgis:fillColor>255,200,0,0</hgis:fillColor>\n");
//                sb.append("<hgis:thin>1</hgis:thin>\n");
//                sb.append("<hgis:lineColor>0,0,0,0</hgis:lineColor>\n");
//                sb.append("</hgis:scaleMain>\n");
            }
            if (properties.getScaleSub() != null)
            {
            	Element scaleSub = mdom.createElementNS("http://www.h-gis.org/hgis", "scaleSub");
            	
            	Element fillColor = mdom.createElementNS("http://www.h-gis.org/hgis", "fillColor");
            	fillColor.setTextContent(Utilities.getColor(properties.getScaleSub().getFillColor()));
            	scaleSub.appendChild(fillColor);
                
            	Element thin = mdom.createElementNS("http://www.h-gis.org/hgis", "thin");
            	thin.setTextContent(Integer.toString(properties.getScaleSub().getThin()));
            	scaleSub.appendChild(thin);
                
                Element lineColor = mdom.createElementNS("http://www.h-gis.org/hgis", "lineColor");
                lineColor.setTextContent(Utilities.getColor(properties.getScaleSub().getLineColor()));
                scaleSub.appendChild(lineColor);
                
                time.appendChild(scaleSub);
                
//                sb.append("<hgis:scaleSub>\n");
//                sb.append("<hgis:fillColor>255,200,0,0</hgis:fillColor>\n");
//                sb.append("<hgis:thin>1</hgis:thin>\n");
//                sb.append("<hgis:lineColor>0,0,0,0</hgis:lineColor>\n");
//                sb.append("</hgis:scaleSub>\n");
            }
            if (lines != null || polygons != null || symbols != null || fonts != null) {
            	Element showGroupName = mdom.createElementNS("http://www.h-gis.org/hgis", "showGroupName");
            	Boolean b = properties.isShowGroupName();
            	if (b == null) {
            		b = true;
            	}
            	showGroupName.setTextContent(b.toString());
                time.appendChild(showGroupName);
                
//                sb.append("<hgis:showGroupName>").append(properties.isShowGroupName()).append("</hgis:showGroupName>\n");
            }
            if (properties.getDetailFontName() != null) {
            	Element detailFontName = mdom.createElementNS("http://www.h-gis.org/hgis", "detailFontName");
            	detailFontName.setTextContent(properties.getDetailFontName());
                time.appendChild(detailFontName);
                
//                sb.append("<hgis:detailFontName>").append(properties.getDetailFontName()).append("</hgis:detailFontName>\n");
            }
            if (properties.getDataReductionNum() != null) {
            	Element dataReductionEnableRecordNum = mdom.createElementNS("http://www.h-gis.org/hgis", "dataReductionEnableRecordNum");
            	dataReductionEnableRecordNum.setTextContent(properties.getDataReductionNum().toString());
                time.appendChild(dataReductionEnableRecordNum);
                
//                sb.append("<hgis:dataReductionEnableRecordNum>").append(properties.getDataReductionNum()).append("</hgis:dataReductionEnableRecordNum>\n");
            }
            Element dataReduction = mdom.createElementNS("http://www.h-gis.org/hgis", "dataReduction");
            dataReduction.setTextContent(properties.isDataReduction().toString());
            time.appendChild(dataReduction);
            
//            sb.append("<hgis:dataReduction>").append(properties.isDataReduction()).append("</hgis:dataReduction>\n");
        }

//        sb.append("</hgis:time>\n");
//        sb.append("</hgis:resourceStatus>\n");
//        sb.append("</hgis:layer>\n");

        return time;
    }
    
    
}
