/* ===========================================================
 * GT-Time
 * ===========================================================
 *
 * Copyright (C) 2010 National Institutes for the Humanities. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */

package jp.nihu.gttime.utility;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.net.MalformedURLException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.xerces.dom.DOMImplementationImpl;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.ls.DOMImplementationLS;
import org.w3c.dom.ls.LSSerializer;

/**
 * ユーティリティークラス
 */
public class Utilities
{
//    private static Log log = LogFactory.getLog(Utilities.class);
	
	public static String getUri(String path) throws MalformedURLException
    {
		if (path == null || path.equals("")) {
			return "";
		}
		if (path.startsWith("http://") || path.startsWith("https://")) {
			return path;
		}
		return  new File(path).toURI().toURL().toString();
    }

    public static String toXmlString(Document doc) throws IOException
    {
        DOMImplementation domImpl = new DOMImplementationImpl();
        DOMImplementationLS domImplLS = (DOMImplementationLS) domImpl.getFeature("LS", "3.0");
        LSSerializer lsSerializer = domImplLS.createLSSerializer();
        return lsSerializer.writeToString(doc);
    }

    public static Color getColor(String rgba)
    {
        Color color = null;
        int r = 0;
        int g = 0;
        int b = 0;
        int a = 0;
        String[] vals = rgba.split(",");
        if (vals.length > 3)
        {
            r = Integer.parseInt(vals[0].trim());
            g = Integer.parseInt(vals[1].trim());
            b = Integer.parseInt(vals[2].trim());
            a = Integer.parseInt(vals[3].trim());
            color = new Color(r, g, b, a);
        }
        else
        {
            r = Integer.parseInt(vals[0].trim());
            g = Integer.parseInt(vals[1].trim());
            b = Integer.parseInt(vals[2].trim());
            color = new Color(r, g, b);
        }
        return color;
    }

    public static String getColor(Color rgba)
    {
        int r = rgba.getRed();
        int g = rgba.getGreen();
        int b = rgba.getBlue();
        int a = rgba.getAlpha();
        return r + "," + g + "," + b + "," + a;
    }

    public static int getFontStyle(String style)
    {
        int s = Font.PLAIN;
        style = style.toLowerCase();
        if (style.equals("plain"))
        {
            s = Font.PLAIN;
        }
        else if (style.equals("bold"))
        {
            s = Font.BOLD;
        }
        else if (style.equals("italic"))
        {
            s = Font.ITALIC;
        }
        else if (style.equals("bold|italic"))
        {
            s = Font.BOLD | Font.ITALIC;
        }
        return s;
    }

    public static String getFontStyle(int style)
    {
        switch (style)
        {
            case Font.PLAIN:
                return "plain";
            case Font.BOLD:
                return "bold";
            case Font.ITALIC:
                return "italic";
            case Font.BOLD | Font.ITALIC:
                return "bold|italic";
            default:
                return "plain";
        }
    }

    /**
     * 一意な値を取得する。
     * 
     * @return 一意な文字列
     */
    public static String getUniqueString()
    {
        Date date = new Date();
        while (prevDate != null && date.equals(prevDate))
        {
            date = new Date();
        }
        prevDate = date;
        SimpleDateFormat uniqName = new SimpleDateFormat("yyyyMMddhhmmssSSS");
        return uniqName.format(date);
    }
    
    public static String escapeFileName(String str) {
    	return str.replaceAll("[/:\\*\\?\\\\\"<>\\|]", "_");
    }

    private static Date prevDate = null;

    /**
     * 描画される（フォントに依存した）文字列の長さを取得します。
     * 
     * @param str 日付関連の文字列
     * @param g 描画対象のグラフィックス
     * @return 文字列の長さ
     */
    public static Dimension stringSize(String str, Graphics g)
    {
        if (g instanceof Graphics2D)
        {
            Rectangle2D bounds = g.getFont().getStringBounds(str, ((Graphics2D) g).getFontRenderContext());
            return new Dimension((int) (bounds.getWidth() + .5), (int) (bounds.getHeight() + .5));
        }
        else
        {
            return new Dimension(g.getFontMetrics().stringWidth(str), g.getFontMetrics().getHeight());
        }
    }

    public static String getDurationVal(BigDecimal val)
    {
        if (val != null)
        {
//            log.debug(val.toPlainString());
            if (new BigDecimal(1).divide(new BigDecimal(24 * 60 * 60).multiply(new BigDecimal(1000000)), 30, BigDecimal.ROUND_DOWN).equals(val))
            {
                // 1/(24*60*60*1000000)
                return "0.000001second";
            }
            else if (new BigDecimal(1).divide(new BigDecimal(24 * 60 * 60).multiply(new BigDecimal(100000)), 30, BigDecimal.ROUND_DOWN).equals(val))
            {
                // 1/(24*60*60*100000)
                return "0.00001second";
            }
            else if (new BigDecimal(1).divide(new BigDecimal(24 * 60 * 60).multiply(new BigDecimal(10000)), 30, BigDecimal.ROUND_DOWN).equals(val))
            {
                // 1/(24*60*60*10000)
                return "0.0001second";
            }
            else if (new BigDecimal(1).divide(new BigDecimal(24 * 60 * 60).multiply(new BigDecimal(1000)), 30, BigDecimal.ROUND_DOWN).equals(val))
            {
                // 1/(24*60*60*1000)
                return "0.001second";
            }
            else if (new BigDecimal(1).divide(new BigDecimal(24 * 60 * 60).multiply(new BigDecimal(100)), 30, BigDecimal.ROUND_DOWN).equals(val))
            {
                // 1/(24*60*60*100)
                return "0.01second";
            }
            else if (new BigDecimal(1).divide(new BigDecimal(24 * 60 * 60).multiply(new BigDecimal(10)), 30, BigDecimal.ROUND_DOWN).equals(val))
            {
                // 1/(24*60*60*10)
                return "0.1second";
            }
            else if (new BigDecimal(1).divide(new BigDecimal(24 * 60 * 60), 30, BigDecimal.ROUND_DOWN).equals(val))
            {
                // 1/(24*60*60)
                return "second";
            }
            else if (new BigDecimal(1).divide(new BigDecimal(24 * 60), 30, BigDecimal.ROUND_DOWN).equals(val))
            {
                // 1/(24*60)
                return "minute";
            }
            else if (new BigDecimal(1).divide(new BigDecimal(24), 30, BigDecimal.ROUND_DOWN).equals(val))
            {
                return "hour";
            }
            else if (new BigDecimal(1).equals(val))
            {
                return "day";
            }
            else if (new BigDecimal(7).equals(val))
            {
                return "week";
            }
            else if (new BigDecimal(31).equals(val))
            {
                return "month";
            }
            else if (new BigDecimal(92).equals(val))
            {
                return "quarter";
            }
            else if (new BigDecimal(365).equals(val))
            {
                return "year";
            }
            else if (new BigDecimal(3650).equals(val))
            {
                return "10year";
            }
            else if (new BigDecimal(36500).equals(val))
            {
                return "100year";
            }
            else if (new BigDecimal(365000).equals(val))
            {
                return "1000year";
            }
            else if (new BigDecimal(3650000).equals(val))
            {
                return "10000year";
            }
            else if (new BigDecimal(36500000).equals(val))
            {
                return "100000year";
            }
            else if (new BigDecimal(365000000).equals(val))
            {
                return "1000000year";
            }
        }
        return null;
    }

    // 0.000001second -> 1/(24*60*60*1000000)
    // 0.00001second -> 1/(24*60*60*100000)
    // 0.0001second -> 1/(24*60*60*10000)
    // 0.001second -> 1/(24*60*60*1000)
    // 0.01second -> 1/(24*60*60*100)
    // 0.1second -> 1/(24*60*60*10)
    // second -> 1/(24*60*60)
    // minute -> 1/(24*60)
    // hour -> 1/24
    // day -> 1
    // week -> 7
    // month -> 31
    // quarter -> 92
    // year -> 365
    // 10year -> 3650
    // 100year -> 36500
    // 1000year -> 365000
    // 10000year -> 3650000
    // 100000year -> 36500000
    // 1000000year -> 365000000
    public static BigDecimal getDurationVal(String val)
    {
        if ("0.000001second".equals(val))
        {
            // 1/(24*60*60*1000000)
            return new BigDecimal(1).divide(new BigDecimal(24 * 60 * 60).multiply(new BigDecimal(1000000)), 30, BigDecimal.ROUND_DOWN);
        }
        else if ("0.00001second".equals(val))
        {
            // 1/(24*60*60*100000)
            return new BigDecimal(1).divide(new BigDecimal(24 * 60 * 60).multiply(new BigDecimal(100000)), 30, BigDecimal.ROUND_DOWN);
        }
        else if ("0.0001second".equals(val))
        {
            // 1/(24*60*60*10000)
            return new BigDecimal(1).divide(new BigDecimal(24 * 60 * 60).multiply(new BigDecimal(10000)), 30, BigDecimal.ROUND_DOWN);
        }
        else if ("0.001second".equals(val))
        {
            // 1/(24*60*60*1000)
            return new BigDecimal(1).divide(new BigDecimal(24 * 60 * 60).multiply(new BigDecimal(1000)), 30, BigDecimal.ROUND_DOWN);
        }
        else if ("0.01second".equals(val))
        {
            // 1/(24*60*60*100)
            return new BigDecimal(1).divide(new BigDecimal(24 * 60 * 60).multiply(new BigDecimal(100)), 30, BigDecimal.ROUND_DOWN);
        }
        else if ("0.1second".equals(val))
        {
            // 1/(24*60*60*10)
            return new BigDecimal(1).divide(new BigDecimal(24 * 60 * 60).multiply(new BigDecimal(10)), 30, BigDecimal.ROUND_DOWN);
        }
        else if ("second".equals(val))
        {
            // 1/(24*60*60)
            return new BigDecimal(1).divide(new BigDecimal(24 * 60 * 60), 30, BigDecimal.ROUND_DOWN);
        }
        else if ("minute".equals(val))
        {
            // 1/(24*60)
            return new BigDecimal(1).divide(new BigDecimal(24 * 60), 30, BigDecimal.ROUND_DOWN);
        }
        else if ("hour".equals(val))
        {
            return new BigDecimal(1).divide(new BigDecimal(24), 30, BigDecimal.ROUND_DOWN);
        }
        else if ("day".equals(val))
        {
            return new BigDecimal(1);
        }
        else if ("week".equals(val))
        {
            return new BigDecimal(7);
        }
        else if ("month".equals(val))
        {
            return new BigDecimal(31);
        }
        else if ("quarter".equals(val))
        {
            return new BigDecimal(92);
        }
        else if ("year".equals(val))
        {
            return new BigDecimal(365);
        }
        else if ("10year".equals(val))
        {
            return new BigDecimal(3650);
        }
        else if ("100year".equals(val))
        {
            return new BigDecimal(36500);
        }
        else if ("1000year".equals(val))
        {
            return new BigDecimal(365000);
        }
        else if ("10000year".equals(val))
        {
            return new BigDecimal(3650000);
        }
        else if ("100000year".equals(val))
        {
            return new BigDecimal(36500000);
        }
        else if ("1000000year".equals(val))
        {
            return new BigDecimal(365000000);
        }
        else if (val.matches("^[1-9][0-9]*(\\.[0-9]*[1-9])?$"))
        {
            return new BigDecimal(val);
        }
        return null;
    }
}
